##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################



from ALDefinition                  import  *
from ALSystem                      import  ALSystem
from ALString                      import  ALString
from ALVerbose                     import  ALVerbose
from ALTime                        import  ALTime
from ALDiskExplorer                import  ALDiskExplorer
from ALSlot                        import  ALSlot
from ALSignal                      import  ALSignal
from ALDebug                       import  ALDebug
from ALTrace                       import  ALTrace
from ALThreadControl               import  ALThreadControl
from ALException                   import  ALException
from ALExceptionSignal             import  ALExceptionSignal
from ALApplicationActionControl    import  ALApplicationActionControl


class ALAction( ALThreadControl ):
    """
    Parent class for all ALAction objects.

    #################################################################
    ALL Slot connection point avaiable for all ALAction Tree
    def connectInit( self, _oMethod ):
    def connectPause( self, _oMethod ):
    def connectResume( self, _oMethod ):
    def connectAbort( self, _oMethod ):
    def connectException( self, _oMethod ):
    def connectError( self, _oMethod ):
    def connectEnd( self, _oMethod ):
    def connectExecute( self, _oMethod ):
    def connectRun( self, _oMethod ):
    self.connectExecute( _oMethod )
    """
    def __init__( self, _oBoolPriority = False ):
        """
        Constructor for ALAction.

        """
        ALThreadControl.__init__( self )
        self.m_oalSlotPreProcess              = ALSlot()
        self.m_oalSlotProcess                 = ALSlot()
        self.m_oalSlotPostProcess             = ALSlot()
        self.m_oalSlotSUCCESS                 = ALSlot()
        self.m_oalSlotFAILURE                 = ALSlot()
        self.m_oalBoolHighPriority            = _oBoolPriority
        self.m_oalBoolSuccess                 = False
        self.m_oalStrWorkingDirectory         = ALDiskExplorer.getCurrentFolder()
        self.m_oalStrWorkingDirectoryOriginal = self.m_oalStrWorkingDirectory

        self.connectSUCCESS( self.traceSuccessStatusLog )
        self.connectFAILURE( self.traceFailureStatusLog )



    def setHighPriority( self, _oBoolPriority = True ):
        self.m_oalBoolHighPriority  = _oBoolPriority


    def getHighPriority( self ):
        return self.m_oalBoolHighPriority

    def isHighPriority( self ):
        return self.m_oalBoolHighPriority


    def getWorkingFolder( self ):
        return self.m_oalStrWorkingDirectory


    def getTerminateSuccess(self):
        return self.m_oalBoolSuccess


    def setWorkingFolder( self, _oalStrWorkingDirectory ):
        self.m_oalStrWorkingDirectoryOriginal = ALDiskExplorer.getCurrentFolder()
        self.m_oalStrWorkingDirectory = ALDiskExplorer.absolutePath( _oalStrWorkingDirectory )
        ALDiskExplorer.createFolder( self.m_oalStrWorkingDirectory )


    def traceSuccessStatusLog( self, _oalObject = None ):
        ALVerbose.logID( self.getID(), "[Thread] [ActionEndState] :: [SUCCESS]")


    def traceFailureStatusLog( self, _oalObject = None ):
        ALVerbose.logID( self.getID(), "[Thread] [ActionEndState] :: [FAILURE]")



    def __setRegisterOn(self):
        # Thread Register in the ALApplication
        if (self.getRegisterAction()==True):
            ALApplicationActionControl.startAction( self )


    def __setRegisterOff(self):
        # Thread Register in the ALApplication
        if (self.getRegisterAction()==True):
            ALApplicationActionControl.endAction( self )



    def run( self ):
        """
        This is the method started by the thread model.
        """
        self.executeKernel()


    def actionControl( self ):
        self.threadControl()


    def __raiseControl( self ):
        if (ALDebug.isDebugModeActionOn()==True or ALDebug.isDebugModeForceOn()==True):
            ALDebug.debugForce( self, False )
        else:
            oBoolStatus =  ALDebug.isDebugModeOn() # Here becsue is a bug in jython
            ALTrace.trace( self, oBoolStatus )



    def executeAction( self, _oalObject = None ):
        """
        This method starts the thread.
        Only here to remember the thread action in the framework
        """
        self.execute( _oalObject )


    def executeActionSynchronous( self, _oalObject = None ):
        """
        This method starts the Action in Synchronous  Mode.
        """
        self.executeSynchronous()



    def connectPreProcess( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotPreProcess.connect( _oMethod )
        self.synchronizeOff()


    def connectProcess( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotProcess.connect( _oMethod )
        self.synchronizeOff()


    def connectPostProcess( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotPostProcess.connect( _oMethod )
        self.synchronizeOff()


    def connectSUCCESS( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotSUCCESS.connect( _oMethod )
        self.synchronizeOff()


    def connectFAILURE( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotFAILURE.connect( _oMethod )
        self.synchronizeOff()



    def executeActionTimer( self ):
        """
        This is the main connection method to be used with Timer Plugins and methods.
        """
        self.executeKernel( )




    def executeKernel( self ):
        """
        This is the main kernel of the Action class.
        """
        try:
            try:
                # Controls the Succes of the excecute
                self.m_oalBoolSuccess = False

                self.m_oalTime = ALTime()
                ALVerbose.logID( self.getID(), "[Thread] [ThreadStart]    :: " + ALTime.timeString() )

                # Thread Register in the ALApplication
                self.__setRegisterOn()

                # Thread Init
                self.actionControl()
                self.m_oalSlotInit.call( self )

                # PreProcess Sequence
                self.actionControl()
                self.preProcess( self.m_oalObject )
                self.actionControl()
                self.m_oalSlotPreProcess.call( self )

                # Process Sequence
                self.actionControl()
                self.process( self.m_oalObject )
                self.actionControl()
                self.m_oalSlotProcess.call( self )

                # PostProcess Sequence
                self.actionControl()
                self.postProcess( self.m_oalObject )
                self.actionControl()
                self.m_oalSlotPostProcess.call( self )


                #Thread end
                self.actionControl()
                self.m_bIsEnded = True
                self.m_oalSlotEnd.call( self )

                # Controls the Succes of the excecute
                self.m_oalBoolSuccess = True

            except ALExceptionSignal, _oalExceptionSignal:
                oalSignal = _oalExceptionSignal.getSignal()

                if ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ABORT ):
                    self.m_oalSlotAbort.callSignal( ALSignal.abort() )
                elif ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ERROR ):
                    self.m_oalSlotError.callSignal( ALSignal.error() )
                    self.__raiseControl()
                else:
                    ALVerbose.DEBUGID( self.getID(), "[ALAction] Action stopped under Unknown signal: "  + ALString( oalSignal.getSignal() ) )


            except:
                # Priority control to avoid the Application interruption = complete stop
                if (self.isHighPriority()==True):
                    self.m_oalSlotException.callSignal( ALSignal.exception() )
                    ALVerbose.DEBUGID( self.getID(), "[ALAction] Action exception with HIGH Priority" )
                    ALVerbose.DEBUGID( self.getID(), "           Action: " + ALString( self.getID() ) )
                else:
                    ALVerbose.DEBUGID( self.getID(), "[ALAction] Action exception with LOW Priority" )
                    ALVerbose.DEBUGID( self.getID(), "           Action: " + ALString( self.getID() ) )
                    self.m_oalSlotError.callSignal( ALSignal.error() )
                self.__raiseControl()

        finally:
            #Thread end
            self.m_bIsEnded = True
            self.m_oalTimeDuration = self.m_oalTime.getDuration()
            ALVerbose.logID( self.getID(), "[ALAction] [ThreadEnd]      :: " + ALTime.timeString() )
            ALVerbose.logID( self.getID(), "[ALAction] [ThreadRuntime]  :: " + ALString( self.m_oalTimeDuration ) )

            if (self.getTerminateSuccess() == True):
                self.m_oalSlotSUCCESS.call( self )
            elif (self.getTerminateSuccess() == False):
                self.m_oalSlotFAILURE.call( self )

            # Thread Register in the ALApplication
            self.__setRegisterOff()






############################################################################
# To be modified by the user

    def preProcess( self, _oalObject = None ):
        """
        Any processing that must occur before the action should go into
        this method.
        """
        self.synchronizeOn()
        ALVerbose.DEBUGID( self.getID(),"Execution: ALAction.preProcess()" )
        self.synchronizeOff()


    def process( self, _oalObject = None ):
        """
        This is the main method of the action.
        """
        self.synchronizeOn()
        ALVerbose.DEBUGID( self.getID(),"Execution: ALAction.process()" )
        self.synchronizeOff()


    def postProcess( self, _oalObject = None ):
        """
        Similar to preProcess, any processing
        """
        self.synchronizeOn()
        ALVerbose.DEBUGID( self.getID(),"Execution: ALAction.postProcess()" )
        self.synchronizeOff()


##############################################################################

