##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition               import *
from ALVerbose                  import ALVerbose
from ALString                   import ALString
from ALList                     import ALList
from ALDiskExplorer             import ALDiskExplorer
from ALToken                    import ALToken
from ALCommandLineInterface     import ALCommandLineInterface
from ALAppDefinition            import ALAppDefinition


class ALAppDefinitionCommandLine( ALAppDefinition ):
    """
    Interface for ALAppDefinitionOutput objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinition.__init__( self, _oalString, _dversion, _obPlugin )



    def setCommandLineInterfaceFile( self, _oalStringFile ):
        oalStringFile = ALDiskExplorer.getAbsolutePath( _oalStringFile )
        ALAppDefinitionCommandLine.m_oalCommandLineInterface = ALCommandLineInterface( oalStringFile , ALAppDefinitionCommandLine.isFactoryPluginDynamic() )


    def getCommandLineArguments():
        ALAppDefinitionCommandLine.synchronizeOn()
        oalArguments = ALAppDefinitionCommandLine.m_oalCommandLine.getCommandLine()
        ALAppDefinitionCommandLine.synchronizeOff()
        return oalArguments
    getCommandLineArguments = staticmethod( getCommandLineArguments )


    def setCommandLine( _oalStringCommand ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalArguments = ALAppDefinitionCommandLine.m_oalCommandLine.setCommandLine( _oalStringCommand )
        ALAppDefinitionCommandLine.synchronizeOff()
        return oalArguments
    setCommandLine = staticmethod( setCommandLine )


#    def getCommandLine( _oalStringCommand ):
#        return ALAppDefinitionCommandLine.getCommandLineArguments()
#    getCommandLine = staticmethod( getCommandLine )




    def addCommand( self, _oalStringToken, _oalObjectMethod ):
        ALAppDefinitionCommandLine.synchronizeOn()
        ALAppDefinitionCommandLine.m_oalCommandLine.addCommand( _oalStringToken, _oalObjectMethod )
        ALAppDefinitionCommandLine.synchronizeOff()


    def addCommandLineArguments( _oalListArgument ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalListArgument = ALList( _oalListArgument )
        for i in oalListArgument:
            ALAppDefinitionCommandLine.m_oalCommandLine.addCommandArgument( i )
        ALAppDefinitionCommandLine.synchronizeOff()
    addCommandLineArguments = staticmethod( addCommandLineArguments )


    def addListArgument( self, _oalListArgument ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalListArgument = ALList( _oalListArgument )
        for i in oalListArgument:
            ALAppDefinitionCommandLine.m_oalCommandLine.addCommandArgument( i )
        ALAppDefinitionCommandLine.synchronizeOff()


    def addCommandLineArgument( _oalStringToken ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalStringToken = ALToken( _oalStringToken )
        ALAppDefinitionCommandLine.m_oalCommandLine.addCommandArgument( oalStringToken )
        ALAppDefinitionCommandLine.synchronizeOff()
    addCommandLineArgument = staticmethod( addCommandLineArgument )


    def getCommandLineArgument( _oalStringToken ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalCommandLineArg = ALAppDefinitionCommandLine.m_oalCommandLine.getArgument( _oalStringToken )
        ALAppDefinitionCommandLine.synchronizeOff()
        return oalCommandLineArg
    getCommandLineArgument = staticmethod( getCommandLineArgument )



    def getCommandLine():
        ALAppDefinitionCommandLine.synchronizeOn()
        oalCommandLine = ALAppDefinitionCommandLine.m_oalCommandLine.getCommand()
        ALAppDefinitionCommandLine.synchronizeOff()
        return oalCommandLine
    getCommandLine = staticmethod( getCommandLine )


    def existCommand( _oalStringCommand ):
        ALAppDefinitionCommandLine.synchronizeOn()
        bExist = ALAppDefinitionCommandLine.m_oalCommandLine.existCommand( _oalStringCommand )
        ALAppDefinitionCommandLine.synchronizeOff()
        return bExist
    existCommand = staticmethod( existCommand )



    def processCommandLine( self ):
        """
        This is the command line method of the ALAppDefinitionCommandLine.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCommandLine.processCommandLine()" )
        ALAppDefinitionCommandLine.m_oalCommandLine.executeCommand()
        ALAppDefinitionCommandLine.m_oalCommandLine.synchronize()




    def processCommandLineInterface( self ):
        """
        This is the command line Interface (from xml file definition) method of the ALAppDefinitionCommandLine.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCommandLine.processCommandLineInterface()" )
        if (ALAppDefinitionCommandLine.m_oalCommandLineInterface==None):
            return
        oalCommandLine = ALAppDefinitionCommandLine.m_oalCommandLine.getCommandLine()
        ALAppDefinitionCommandLine.m_oalCommandLineInterface.setCommandLine( oalCommandLine )
        ALAppDefinitionCommandLine.m_oalCommandLineInterface.executeCommand()
        ALAppDefinitionCommandLine.m_oalCommandLineInterface.synchronize()



##############################################################################
