##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition            import *
from ALVerbose               import ALVerbose
from ALString                import ALString
from ALDebug                 import ALDebug
from ALAssert                import ALAssert
from ALSystem                import ALSystem
from ALAppDefinitionControl  import ALAppDefinitionControl


class ALAppDefinitionInit( ALAppDefinitionControl ):
    """
    Interface for ALAppDefinitionInit objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionControl.__init__( self, _oalString, _dversion, _obPlugin )


    def init( self, _bInit = True ):
        """
        This is the Init method in the ALAppDefinitionInit.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionInit.init()" )
        oBoolStatus = True
        oalString = ALString( ALAppDefinitionInit.m_oalCommandLine.getCommandLine() )
        ALVerbose.log( oalString )
        ALVerbose.setVerboseOff()


        if (ALDebug.isDebugModeVerboseOn()==True):
            ALVerbose.setVerboseOn()


        if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--quiet" )):
            ALVerbose.setVerboseOff()


        if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--profile" )):
            ALAppDefinitionInit.setDebugProfileOn()


        if (ALDebug.isDebugModeOn()==True or ALDebug.isDebugModeForceOn()==True):
            ALAppDefinitionInit.setDebugMode()
            ALAppDefinitionInit.setDebugProfileOn()
            ALVerbose.DEBUG( "Debug Mode [ON]" )


        if (ALDebug.isDebugModeAssertOn()==True ):
            ALAssert.setAssertionModeOn()


        if (_bInit==True):
            ALVerbose.screen( self.m_oalApplicationName + "::Starting" )
            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--help" )
                or ALAppDefinitionInit.m_oalCommandLine.existCommand( "-h" )):
                ALVerbose.setVerboseOn()
                ALAppDefinitionInit.outputHelp()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--version" ) or
                ALAppDefinitionInit.m_oalCommandLine.existCommand( "-v" )):
                ALVerbose.setVerboseOn()
                ALAppDefinitionInit.outputVersion()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--man" ) or
                ALAppDefinitionInit.m_oalCommandLine.existCommand( "-m" )):
                ALVerbose.setVerboseOn()
                ALAppDefinitionInit.outputMan()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--no-log" )):
                ALVerbose.setLogFileOff()

            # workaround to avoid problem of the ALString.toXml cleaning function of jython
            if (ALSystem.getOSName()=="Linux" and ALSystem.getMachineVirtuel()=="Java"):
                pass
            else:
                oalObject = ALAppDefinitionInit.getResource( AL_FRAMEWORK_LAST_RUN )
                if (oalObject != None):
                    oalStringLastRun = ALString( oalObject );
                    ALVerbose.screen( "Last Run: " + oalStringLastRun )

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--import-resource" )):
                oalStrResourceFileName = ALAppDefinitionInit.m_oalCommandLine.getCommandArgument( "--import-resource" )
                ALAppDefinitionInit.importResource( oalStrResourceFileName )

        # The OK to continue the application
        return oBoolStatus





##############################################################################
