##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition                    import *
from ALVerbose                       import ALVerbose
from ALString                        import ALString
from ALDebug                         import ALDebug
from ALAppDefinitionCommandLine      import ALAppDefinitionCommandLine


class ALAppDefinitionOutput( ALAppDefinitionCommandLine ):
    """
    Interface for ALAppDefinitionOutput objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionCommandLine.__init__( self, _oalString, _dversion, _obPlugin )
        self.connectCommandLineBasis()


    def connectCommandLineBasis( self ):
        # Tricks for the command line help
        ALAppDefinitionOutput.m_oalCommandLine.addCommand( "--plugin", ALAppDefinitionOutput.outputScreenPlugin )
        # Tricks for the command line help - new standart comand Line
        ALAppDefinitionOutput.m_oalCommandLine.addCommand( "--plugin-group", ALAppDefinitionOutput.outputScreenPluginGroup )
        ALAppDefinitionOutput.m_oalCommandLine.addCommand( "--plugin-group-tree", ALAppDefinitionOutput.outputScreenPluginGroupTree )

        # To be deprecated
        ALAppDefinitionOutput.m_oalCommandLine.addCommand( "--pluginGroup", ALAppDefinitionOutput.outputScreenPluginGroup )
        ALAppDefinitionOutput.m_oalCommandLine.addCommand( "--pluginGroupTree", ALAppDefinitionOutput.outputScreenPluginGroupTree )


    def outputScreenPlugin( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
#        Bug in Jython version for this command - to verify ???
#        if (ALAppDefinitionOutput.m_oalFactoryPlugin!=None):
#            ALAppDefinitionOutput.m_oalFactoryPlugin.outputScreenPlugin()
        ALAppDefinitionOutput.m_oalFactoryPlugin.outputScreenPlugin()
        ALAppDefinitionOutput.synchronizeOff()
    outputScreenPlugin = staticmethod( outputScreenPlugin )


    def outputScreenPluginHelp( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
        bStatus = ALAppDefinitionOutput.m_oalFactoryPlugin.processPluginContainingStringName( "Help" )
        ALAppDefinitionOutput.synchronizeOff()
        return bStatus
    outputScreenPluginHelp = staticmethod( outputScreenPluginHelp )


    def outputScreenPluginMan( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
        bStatus = ALAppDefinitionOutput.m_oalFactoryPlugin.processPluginContainingStringName( "Man" )
        ALAppDefinitionOutput.synchronizeOff()
        return bStatus
    outputScreenPluginMan = staticmethod( outputScreenPluginMan )


    def outputScreenPluginGroup( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
        ALAppDefinitionOutput.m_oalFactoryPlugin.outputScreenPluginGroup()
        ALAppDefinitionOutput.synchronizeOff()
    outputScreenPluginGroup = staticmethod( outputScreenPluginGroup )


    def outputScreenPluginGroupTree( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
        ALAppDefinitionOutput.m_oalFactoryPlugin.outputScreenPluginGroupTree()
        ALAppDefinitionOutput.synchronizeOff()
    outputScreenPluginGroupTree = staticmethod( outputScreenPluginGroupTree )




    def outputVersion():
        ALAppDefinitionOutput.synchronizeOn()
        ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Version " + ALAppDefinitionOutput.m_oalApplicationVersion )
        ALAppDefinitionOutput.synchronizeOff()
    outputVersion = staticmethod( outputVersion )



    def ouputOption():
        ALAppDefinitionOutput.synchronizeOn()
        ALVerbose.screen( "" )
        ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Options" )
        ALVerbose.screen( "Standard Options:" )
        ALVerbose.screen( "" )
        ALVerbose.screen( "            --help or -h  [option] : to show help if avaiable" )
        ALVerbose.screen( "            --man  or -m  [option] : to show manual if avaiable" )
        ALVerbose.screen( "                      --help --all : it shows the main command list help" )
        ALVerbose.screen( "                       --man --all : it shows the main command list man" )
        ALVerbose.screen( "                         --verbose : to iteractive output" )
        ALVerbose.screen( "                           --quiet : NO output (default option)" )
        ALVerbose.screen( "                          --no-log : NO log file output" )
        ALVerbose.screen( "                         --profile : profile output for trace actions" )
        ALVerbose.screen( "                   --version or -v : executable version info" )
        ALVerbose.screen( "                          --plugin : show the plugins connected to the application" )
        ALVerbose.screen( "                    --plugin-group : show the plugin groups" )
        ALVerbose.screen( "               --plugin-group-tree : show the plugin group tree" )
        ALVerbose.screen( "                           --DEBUG : output debug info with no stops" )
        ALVerbose.screen( "                     --DEBUG-FORCE : output standart Python-Java debug info and" )
        ALVerbose.screen( "                                     FORCE the application to stop" )
        ALVerbose.screen( "              --DEBUG-FORCE-PLUGIN : output standart Python-Java debug only from a PLUGIN and " )
        ALVerbose.screen( "                                     FORCE the application to stop" )
        ALVerbose.screen( "                          --ASSERT : output assert info for debug" )
        ALVerbose.screen( "      --import-resource [filename] : import an existant application resource file" )
        ALVerbose.screen( "" )
        ALAppDefinitionOutput.synchronizeOff()
    ouputOption = staticmethod( ouputOption )



    def outputHelp():
        if (ALAppDefinitionOutput.m_oalHelp != None):
            ALAppDefinitionOutput.m_oalHelp.outputScreen()

        if( ALAppDefinitionOutput.getCommandLineArgument( "--help" )!=None ):
            oalString = ALAppDefinitionOutput.getCommandLineArgument( "--help" )
            oalStringHelp = ALAppDefinitionOutput.m_oalCommandLineInterface.getHelp( oalString )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Help option" )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalString + ": " + oalStringHelp ) )
        elif( ALAppDefinitionOutput.getCommandLineArgument( "-h" )!=None  ):
            oalString = ALAppDefinitionOutput.getCommandLineArgument( "-h" )
            oalStringHelp = ALAppDefinitionOutput.m_oalCommandLineInterface.getHelp( oalString )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Help option" )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalString + ": " + oalStringHelp ) )
        else:
            oalStringHelp = ALAppDefinitionOutput.m_oalCommandLineInterface.getHelp( )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalStringHelp ) )

        ALAppDefinitionOutput.ouputOption()
        ALAppDefinitionOutput.outputScreenPluginHelp()

    outputHelp = staticmethod( outputHelp )



    def outputMan():
        if (ALAppDefinitionOutput.m_oalMan != None):
            ALAppDefinitionOutput.m_oalMan.outputScreen()

        if( ALAppDefinitionOutput.getCommandLineArgument( "--man" )!=None ):
            oalString = ALAppDefinitionOutput.getCommandLineArgument( "--man" )
            oalStringManual = ALAppDefinitionOutput.m_oalCommandLineInterface.getManual( oalString )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Manual option" )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalString + ": " + oalStringManual ) )
        elif( ALAppDefinitionOutput.getCommandLineArgument( "-m" )!=None  ):
            oalString = ALAppDefinitionOutput.getCommandLineArgument( "-m" )
            oalStringManual = ALAppDefinitionOutput.m_oalCommandLineInterface.getManual( oalString )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Manual option" )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalString + ": " + oalStringManual ) )
        else:
            oalStringManual = ALAppDefinitionOutput.m_oalCommandLineInterface.getManual( )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalStringManual ) )

        ALAppDefinitionOutput.ouputOption()
        ALAppDefinitionOutput.outputScreenPluginMan()

    outputMan = staticmethod( outputMan )


    # Output control ==========================================================
    def setDebugMode():
        ALAppDefinitionOutput.synchronizeOn( )
        ALAppDefinitionOutput.m_oalDebug = True
        ALVerbose.setVerboseDebugOn()
        ALAppDefinitionOutput.synchronizeOff( )
    setDebugMode = staticmethod( setDebugMode )


    def setDebugModeON():
        ALAppDefinitionOutput.setDebugMode()
    setDebugModeON = staticmethod( setDebugModeON )


    def setDebugModeOFF():
        ALAppDefinitionOutput.synchronizeOn()
        ALAppDefinitionOutput.m_oalDebug = False
        ALVerbose.setVerboseDebugOff()
        ALAppDefinitionOutput.synchronizeOff()
    setDebugModeOFF = staticmethod( setDebugModeOFF )



    def isDebugMode():
        ALAppDefinitionOutput.synchronizeOn()
        bDebug = ALAppDefinitionOutput.m_oalDebug
        ALAppDefinitionOutput.synchronizeOff()
        return bDebug
    isDebugMode = staticmethod( isDebugMode )


    def setDebugProfileOn():
        ALAppDefinitionOutput.synchronizeOn( )
        ALVerbose.setProfileOn()
        ALAppDefinitionOutput.synchronizeOff( )
    setDebugProfileOn = staticmethod( setDebugProfileOn )


    def setDebugProfileOff():
        ALAppDefinitionOutput.synchronizeOn( )
        ALVerbose.setProfileOff()
        ALAppDefinitionOutput.synchronizeOff( )
    setDebugProfileOff = staticmethod( setDebugProfileOff )


    def setDebugVerboseOn():
        ALAppDefinitionOutput.synchronizeOn()
        ALDebug.setDebugModeVerboseOn()
        ALAppDefinitionOutput.synchronizeOff()
    setDebugVerboseOn = staticmethod( setDebugVerboseOn )


    def setDebugPluginOn():
        ALAppDefinitionOutput.synchronizeOn()
        ALDebug.setDebugModeActionOn()
        ALAppDefinitionOutput.synchronizeOff()
    setDebugPluginOn = staticmethod( setDebugPluginOn )

##############################################################################
