##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDictionary             import ALDictionary
from ALInformationAction      import ALInformationAction
from ALObject                 import ALObject
from ALSemaphore              import ALSemaphore
from ALString                 import ALString
from ALVerbose                import ALVerbose


class ALApplicationActionControl( ALObject ):
    """
    Parent class.
    """

    m_oalDictAction              = ALDictionary()
    m_oalDictActionFinished      = ALDictionary()
    m_oalSemaphoreActionControl  = ALSemaphore()
    m_oalNumberActions           = 0
    m_oalNumberActionsRunning    = 0
    m_oalNumberActionsFinished   = 0


    def __init__( self,  ):
        ALObject.__init__( self )


    def reset():
        ALApplicationActionControl.synchronizeOn()
        del ALApplicationActionControl.m_oalDictAction
        del ALApplicationActionControl.m_oalDictActionFinished
        ALApplicationActionControl.m_oalDictAction              = ALDictionary()
        ALApplicationActionControl.m_oalDictActionFinished      = ALDictionary()
        ALApplicationActionControl.m_oalNumberActions           = 0
        ALApplicationActionControl.m_oalNumberActionsRunning    = 0
        ALApplicationActionControl.m_oalNumberActionsFinished   = 0
        ALApplicationActionControl.synchronizeOff()
    reset = staticmethod( reset )


    def synchronizeOn():
        ALApplicationActionControl.m_oalSemaphoreActionControl.acquire()
    synchronizeOn = staticmethod( synchronizeOn )


    def synchronizeOff():
        ALApplicationActionControl.m_oalSemaphoreActionControl.release()
    synchronizeOff = staticmethod( synchronizeOff )


    def startAction( _oalAction ):
        ALApplicationActionControl.synchronizeOn()
        ALApplicationActionControl.m_oalNumberActions += 1
        oalInformationAction = ALInformationAction( _oalAction.getID(), _oalAction )
        ALApplicationActionControl.m_oalDictAction.addObject( oalInformationAction.getActionID(), oalInformationAction )
        ALApplicationActionControl.synchronizeOff()
    startAction = staticmethod( startAction )


    def endAction( _oalAction ):
        ALApplicationActionControl.synchronizeOn()
        oalInfoAction = ALApplicationActionControl.m_oalDictAction.getObject( _oalAction.getID() )
        if (oalInfoAction!=None):
            oalInfoAction.setActionEnd()
            ALApplicationActionControl.m_oalNumberActionsFinished += 1
            ALApplicationActionControl.m_oalDictActionFinished.addObject( _oalAction.getID(), oalInfoAction.getRunTime() )
            ALApplicationActionControl.m_oalDictAction.removeObject( _oalAction.getID() )
#        else:
#            ALVerbose.warning( "Object ALInformationAction doesn't exist" )
        ALApplicationActionControl.synchronizeOff()
    endAction = staticmethod( endAction )



    def getNumberActiveActions():
       ALApplicationActionControl.synchronizeOn()
       oiActive = ALApplicationActionControl.m_oalNumberActions
       oiActive = oiActive - ALApplicationActionControl.m_oalNumberActionsFinished
       ALApplicationActionControl.synchronizeOff()
       return oiActive
    getNumberActiveActions = staticmethod( getNumberActiveActions )


    def isRunningControl():
        obStatus = False
        if (ALApplicationActionControl.getNumberActiveActions()!=0):
            obStatus = True
        return obStatus
    isRunningControl = staticmethod( isRunningControl )



    def forceClose():
        ALApplicationActionControl.synchronizeOn()
        for iAction in ALApplicationActionControl.m_oalDictAction.ordered():
            oalInfo = ALApplicationActionControl.m_oalDictAction[ iAction ]
            oalInfo.forceActionClose()
        ALApplicationActionControl.synchronizeOff()
    forceClose = staticmethod( forceClose )



    def getStatusRunning():
        iActionsTotal = ALApplicationActionControl.m_oalNumberActions
        iActions      = ALApplicationActionControl.getNumberActiveActions()
        ALVerbose.DEBUG( "   Total Actions: " + ALString( iActionsTotal ) )
        ALVerbose.DEBUG( " Running Actions: " + ALString( iActions ) )
    getStatusRunning = staticmethod( getStatusRunning )


    def getActionInformation( _oalActionID ):
        ALApplicationActionControl.synchronizeOn()
        oalInfo = None
        try:
            oalInfo = ALApplicationActionControl.m_oalDictAction.getObject( _oalActionID )
        except:
            pass
        ALApplicationActionControl.synchronizeOff()
        return oalInfo
    getActionInformation = staticmethod( getActionInformation )




    def getActionLogInformation():
        ALApplicationActionControl.synchronizeOn()
        if(ALVerbose.isVerboseDebug()==True):
            ALVerbose.DEBUG( "                [Total Actions] - " + ALString( ALApplicationActionControl.m_oalNumberActions ) )
            for iAction in ALApplicationActionControl.m_oalDictActionFinished.ordered():
                ofTime = ALApplicationActionControl.m_oalDictActionFinished[ iAction ]
                ALVerbose.DEBUG( "                [" + iAction +"] - Run Time [s]: " + ALString( ofTime ) )
        else:
            ALVerbose.log(   "                [Total Actions] - " + ALString( ALApplicationActionControl.m_oalNumberActions ) )
            for iAction in ALApplicationActionControl.m_oalDictActionFinished.ordered():
                ofTime = ALApplicationActionControl.m_oalDictActionFinished[ iAction ]
                ALVerbose.log(   "                [" + iAction +"] - Run Time [s]: " + ALString( ofTime ) )
        ALApplicationActionControl.synchronizeOff()
        ALApplicationActionControl.reset()
    getActionLogInformation = staticmethod( getActionLogInformation )


##############################################################################


