##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALString                  import ALString
from ALSystem                  import ALSystem
from ALVerbose                 import ALVerbose
from ALTrace                   import ALTrace
from ALFile                    import ALFile
from ALList                    import ALList
from ALCommand                 import ALCommand
from ALDictionary              import ALDictionary
from ALCommandLineArgument     import ALCommandLineArgument
from ALDiskExplorer            import ALDiskExplorer


class ALCommandLine( ALCommand ):
    """
    Base class for command line objects.
    """
    def __init__( self, _oalCommandLineArgument = None ):
        ALCommand.__init__( self )
        self.m_oalDictCommand         = ALDictionary()
        self.m_oalCommandLineArgument = ALCommandLineArgument()
        if ( isinstance( _oalCommandLineArgument, ALString ) or isinstance( _oalCommandLineArgument, str )):
            self.m_oalStringCommand = _oalCommandLineArgument
            oalList = self.m_oalStringCommand.split()
            self.m_oalCommandLineArgument = ALCommandLineArgument( oalList )
        elif ( isinstance( _oalCommandLineArgument, ALCommandLineArgument ) ):
            self.m_oalCommandLineArgument = _oalCommandLineArgument



    def addCommandLine( self, _oalStringCommand ):
        oalList = _oalStringCommand.split( " " )
        for iElem in oalList:
            self.m_oalCommandLineArgument.add( iElem )



    def setCommandLine( self, _oalStringCommand ):
        self.setCommand( ALString( _oalStringCommand ) )
        self.m_oalCommandLineArgument = ALCommandLineArgument( ALString.empty())
        oalList = self.m_oalStringCommand.split( " " )
        for iElem in oalList:
            self.m_oalCommandLineArgument.add( iElem )




    def getCommand( self ):
        self.synchronizeOn()
        if ( self.m_oalCommandLineArgument != None ):
            self.m_oalStringCommand = ALString( "" )
            for cmdkey in self.m_oalCommandLineArgument:
                self.m_oalStringCommand += ALString( cmdkey ) + " "
        self.synchronizeOff()
        return self.m_oalStringCommand


    def getCommandLine( self ):
        return self.getCommand()


    def addCommand( self, _oalStringToken, _oalObjectMethod ):
        self.synchronizeOn()
        self.m_oalDictCommand.addObject( ALString( _oalStringToken ), _oalObjectMethod )
        self.synchronizeOff()


    def executeCommand( self ):
        self.executeAction()


    def process( self, _oalObject = None ):
        self.synchronizeOn()
        if ( self.m_oalCommandLineArgument != None ):
            for cmdkey in self.m_oalCommandLineArgument:
                if cmdkey in self.m_oalDictCommand:
                    oalObjectMethod = self.m_oalDictCommand[ cmdkey ]
                    oalObjectMethod( cmdkey )

#                    Maybe in the future???
#                    try:
#                        oalObjectMethod( cmdkey )
#                    except:
#                        ALTrace.trace( self, ALVerbose.isVerboseDebug() )
#                        ALVerbose.warning("ALCommand::process = Exception execution method with argument")
#                        ALVerbose.warning("Trying the execution with NO argument")
#                        try:
#                            oalObjectMethod()
#                        except:
#                            ALTrace.trace( self, ALVerbose.isVerboseDebug() )
        self.synchronizeOff()


    def existCommand( self, _oalString ):
#        ALVerbose.DEBUG( "Executing ALCommandLine::existCommand() = " + _oalString )
#        self.synchronizeOn( self )
        self.synchronizeOn()
        oalString = ALString( _oalString )
        bexist = self.m_oalCommandLineArgument.existArgument( oalString )
        self.synchronizeOff()
        return bexist


    def getArgument( self, _oalString ):
        self.synchronizeOn()
        oalString = ALString( _oalString )
        oalArgument = self.m_oalCommandLineArgument.getArgument( oalString )
        self.synchronizeOff()
        return oalArgument


    def getCommandArgument( self, _oalString ):
        return self.getArgument( _oalString )



    def addCommandArgument( self, _oalStrCommand, _oalStrArgument = None ):
        self.synchronizeOn()
        oalString = ALString( _oalStrCommand )
        self.m_oalCommandLineArgument.append( oalString )
        if ( _oalStrArgument != None ):
            oalString = ALString( _oalStrArgument )
            self.m_oalCommandLineArgument.append( oalString )
        self.synchronizeOff()


    def removeCommandArgument( self, _oalStringToken, _odtControl = False ):
        self.synchronizeOn()
        if ( self.m_oalCommandLineArgument != None ):
            i = 0
            for cmdkey in self.m_oalCommandLineArgument:
                if (cmdkey ==_oalStringToken):
                    self.m_oalCommandLineArgument.remove( i )
                    if (_odtControl==True):
                        self.m_oalCommandLineArgument.remove( i )
                        i = i-1
                    i = i-1
                i = i+1
        self.synchronizeOff()


    def isEmpty( self ):
        self.synchronizeOn()
        bEmpty = False
        if len(self.m_oalCommandLineArgument)<=1:
            bEmpty = True
        self.synchronizeOff()
        return bEmpty


    def synchronizeCommand( self, _oalObject = None ):
        self.waitActionEnd( 0.1 )



    def getArgumentNumber( self, _oValue ):
        self.synchronizeOn()
        oalObject = self.m_oalCommandLineArgument.getArgumentNumber( _oValue )
        self.synchronizeOff()
        return oalObject



    def getApplicationFolder( self ):
        self.synchronizeOn()
        oalStringFolder = self.__getApplicationPathFromCommandLine()
        #To be used by unix systems
        if (oalStringFolder==None):
           oalStringFolder = self.__getApplicationPath()
        #To be used by windows systems
        if (oalStringFolder==None):
           oalStringFolder = self.__getApplicationPath( ALString( ".exe" ) )
        #To be used by MacOSX systems
        if (oalStringFolder==None):
           oalStringFolder = self.__getApplicationPath( ALString( ".app" ) )
        self.synchronizeOff()
        return oalStringFolder


# These methods go to new ALPath class in the future
    def __getApplicationPath( self, _oStrExtension = None ):
        oalStringFolder = None
        oalStrFile = ALSystem.getProgramName()
        if ( _oStrExtension != None ):
            oalStrFile = ALFile.setExtension( oalStrFile, _oStrExtension )
        oalList    = ALSystem.getSystemPath()
        for iPath in oalList:
           oalStrFileName = ALDiskExplorer.getAbsolutePath( iPath ) + "/" + oalStrFile
           if ( ALDiskExplorer.existFile(oalStrFileName)==True):
                oalStringFolder = ALFile.getFolderName( oalStrFileName )
#        ALVerbose.DEBUG( "Application Folder from Path test: " +  ALString( oalStringFolder ) )
        return oalStringFolder


    def __getApplicationPathFromCommandLine( self ):
        oalStringFolder = self.m_oalCommandLineArgument.getArgumentNumber( 0 )
        if ( ALDiskExplorer.existFile(oalStringFolder)==True):
            oalStringFolder = ALDiskExplorer.getAbsolutePath( oalStringFolder )
            oalStringFolder = ALFile.getFolderName( oalStringFolder )
        else:
            oalStringFolder = None
#        ALVerbose.DEBUG("Application Folder from CommandLine test: " + ALString( oalStringFolder ) )
        return oalStringFolder


##############################################################################
