##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################



from ALImportPython            import PyOS
from ALObject                  import ALObject
from ALString                  import ALString
from ALList                    import ALList
from ALDiskExplorer            import ALDiskExplorer



class ALFile( ALObject ):
    """
    Interface for AL thread objects.
    """
    def __init__( self, _oalString = "None"  ):
        pass


    def exist( _oalString ):
        return PyOS.path.isfile( ALDiskExplorer.normalize( _oalString ) )
    exist = staticmethod( exist )


    def getExtension( _oalString ):
        oalList = ALList( PyOS.path.splitext( ALDiskExplorer.normalize( _oalString ) ) )
        iLenght = len( oalList )
        oalString = ALString( oalList[iLenght-1] )
        return oalString
    getExtension = staticmethod( getExtension )


    def setExtension( _oalStrFile, _oalStrExt ):
        """
        You must provide a complete extension name
        """
        oalList = ALList( PyOS.path.splitext( ALDiskExplorer.normalize( _oalStrFile ) ) )
    	oalString2 = ALString()
    	for i in range( len( oalList )-1 ):
                oalString2 = oalString2 + ALString( oalList[i] )
    	#oalString2 = oalString2 + "." + _oalStrExt
    	oalString2 = oalString2 + _oalStrExt
    	return oalString2
    setExtension = staticmethod( setExtension )


    def isExtension( _oalStrFileName, _oalStrExtension ):
        #if ALFile.exist( _osStrFileName )==False:
        #    return False
        #
        if ALFile.getExtension( _oalStrFileName )==_oalStrExtension:
            return True
        else:
            return False
    isExtension = staticmethod( isExtension )



    # Dir List + FileName with extension
    def getListName( _oalString ):
        oalString = PyOS.path.dirname( _oalString )
        if (oalString == ".." ):
            oalString = ALString( PyOS.getcwd() )
            oalString = ALDiskExplorer.normalize( PyOS.path.dirname( oalString ) )
            oalString = oalString + "/" + ALFile.getFileName( _oalString )
        else:
            oalString = ALDiskExplorer.normalize( _oalString )

        oalList = ALList( oalString.split( "/" ) )
        return oalList
    getListName = staticmethod( getListName )


    # Without extension
    def getBaseName( _oalString ):
        oalString = ALString( PyOS.path.basename( ALDiskExplorer.normalize( _oalString ) ) )
        oalList = ALList( PyOS.path.splitext( ALDiskExplorer.normalize( oalString ) ) )
        oalString2 = ALString()
        for i in range( len( oalList )-1 ):
            oalString2 = oalString2 + ALString( oalList[i] )
        return oalString2
    getBaseName = staticmethod( getBaseName )


    # File name + extesion = NO PATH
    def getFileName( _oalString ):
       return ALString( PyOS.path.basename( ALDiskExplorer.normalize( _oalString ) ) )
    getFileName = staticmethod( getFileName )



    def getFolderName( _oalString ):
        oalString = PyOS.path.dirname( _oalString )
        return ALDiskExplorer.normalize( oalString )
    getFolderName = staticmethod( getFolderName )



##############################################################################
#  http://en.wikipedia.org/wiki/File_name:
#  Filename
#  From Wikipedia, the free encyclopedia
#  (Redirected from File name)
#  Jump to: navigation, search
#
#  A filename is a special kind of string used to uniquely identify a file stored on the file system of a computer.
#  Different operating systems impose different restrictions regarding length and allowed characters on filenames.
#  The address (unique resource identifier - URI) of a file includes at least one or more of these components:
#
#    * host (or network-ID) ? host name, IP address, domain name, or LAN network name (e.g., wikipedia.org, 207.142.131.206, \\MYCOMPUTER, PS:, etc.)
#    * device (or node) ? drive, root mountpoint, disc, volume (e.g., C:, /, SYSLIB, etc.)
#    * directory (or path) ? directory tree (e.g., /usr/bin, \TEMP, [USR.LIB.SRC], etc.)
#    * file ? base name of the file
#    * type (or extension) ? indicates the content type of the file (e.g., .txt, .exe, .dir, etc.)
#    * version ? revision number of the file
