##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython        import PyOS
from ALDefinition          import *
from ALSystem              import ALSystem
from ALLog                 import ALLog
from ALString              import ALString
from ALTime                import ALTime
from ALFile                import ALFile
from ALStreamFile          import ALStreamFile
from ALDictionary          import ALDictionary
from ALDiskExplorer        import ALDiskExplorer
#from ALDebug               import ALDebug



class ALLogFile( ALLog ):
    """
    Base class to Log output in a file
    This class control the information to log in a file
    """
    m_oalLogFileName  = None
    m_bIsInit         = False
    m_oalLockFileName = False


    def __init__( self, _oalString = AL_FRAMEWORK_LOG_NAME, _bIsInit = False ):
        ALLog.__init__( self )
        self.__createFile( _oalString,  _bIsInit )


    def __versionName(self, _oalString ):
        oalString = None
        if (ALSystem.getOSName()=="Java" and _oalString==AL_FRAMEWORK_LOG_NAME):
            oalString = "Jy" + _oalString
        elif (_oalString==AL_FRAMEWORK_LOG_NAME):
            oalString = "Py" + _oalString
        else :
            oalString = _oalString
        return ALString( oalString )



    def __fromIndex( self, _oalString ):
        oList = ALString( _oalString ).split( AL_TOKEN_SEPARATOR )
        oalStrTime = ALTime.timeStringValue( float( oList[0] ) )
        oalStrInfo = ALString( oList[1] )
        return oalStrTime, oalStrInfo


    def __createFile( self, _oalString, _bIsInit = False ):
        if (ALLogFile.m_oalLockFileName==True):
            return
        ALLogFile.m_bIsInit = _bIsInit
        oalStringFilePath = ALSystem.getCurrentFolder()
        oalStringFileName = ALString( oalStringFilePath + ALString( "/" ) + ALTime.getDate()+ "-" + self.__versionName( _oalString ) )
        ALLogFile.m_oalLogFileName = PyOS.path.abspath( oalStringFileName )
        if (ALLogFile.m_bIsInit==True) :
            self.initFile()



    def initFile( self ):
        if (ALLogFile.m_bIsInit) :
            return
        else:
            self.synchronizeOn()
            ALLogFile.m_bIsInit = True
            oalLogFile = ALStreamFile( ALLogFile.m_oalLogFileName, "a" )
            oalLogFile.write( "\n\n\n" )
            oalLogFile.write( "<!-- ################################################################################## -->\n" )
            oalLogFile.write( "<logLine>\n" )
            oalLogFile.write( "<logSystem>\n" )
            oalLogFile.write( "     Project        :: Asynchronous Action Library Project\n" )
            oalLogFile.write( "     Version        :: " + ALString( AL_VERSION ) + " - Python::Jython\n" )
            oalLogFile.write( "     VitualMachine  :: " + ALSystem.getMachineVirtuel() + "\n" )
            oalLogFile.write( "     Log File       :: " + ALTime.timeString( ) + "\n" )
            oalLogFile.write( "     OS             :: " + ALSystem.getOSName() + "\n" )
            oalLogFile.write( "     OSType         :: " + ALSystem.getOSType()  + "\n" )
            oalLogFile.write( "     OSVersion      :: " + ALSystem.getOSVersion() + "\n" )
            oalLogFile.write( "     Machine        :: " + ALSystem.getMachineVersion() + "\n" )
            oalLogFile.write( "     Host           :: " + ALSystem.getHost()  + "\n" )
            oalLogFile.write( "     HostIP         :: " + ALSystem.getHostIP()  + "\n" )
            oalLogFile.write( "     UserName       :: " + ALSystem.getUserName()  + "\n" )
            oalLogFile.write( "     Compiler       :: " + ALSystem.getCompilerName()  + "\n" )
            oalLogFile.write( "     Executable     :: " + ALSystem.getProgramName()  + "\n" )
            oalLogFile.write( "     ExecFolder     :: " + ALSystem.getCwd()  + "\n" )
            oalLogFile.write( "</logSystem>\n" )
            oalLogFile.write( "</logLine>\n" )
            oalLogFile.write( "<!-- ################################################################################## -->\n\n" )
            oalLogFile.close()
            self.synchronizeOff()



    def log( self, _oalString = None ):
        if (_oalString != None ):
            self.initFile()
        self.synchronizeOn()
        if ( _oalString != None ):
            oalString  = ALString( " <logLine> " + ALTime.timeString() +   ALString.setLogXmlToString( ALString( _oalString ) ) + "</logLine>" )
            oalLogFile = ALStreamFile( ALLogFile.m_oalLogFileName, "a" )
            oalLogFile.write( oalString )
            oalLogFile.close()
        self.synchronizeOff()


    def logVerboseTree( self, _oalDictionary ):
        self.synchronizeOn()
        oalDict = ALDictionary()
        for i in _oalDictionary.ordered():
            oalDict.addObject( _oalDictionary[i], _oalDictionary[i] )

        oalLogFile = ALStreamFile( ALLogFile.m_oalLogFileName, "a" )
        oalLogFile.write( "\n\n" )
        oalLogFile.write( "<!-- [ Actions Log List ]################################################################ -->\n" )
        oalLogFile.write( "<logLine>\n" )
        oalLogFile.write( "<logActionList>\n" )
        try:
            for i in oalDict.ordered():
                oalLogFile.write( "  <!-- ################################################################################## -->\n" )
                oalLogFile.write( "  <logAction>\n" )
                oalLogFile.write( "            " + ALTime.timeString( ) + "  [ Action log :: " + i + "]\n" )

                for j in _oalDictionary.ordered():
                    if ( _oalDictionary[j]==i):
                        oalStrTime, oalStrInfo = self.__fromIndex( j )
                        oalStringFiltered =  ALString.setLogXmlToString( ALString( oalStrInfo ) )

                        oalLogFile.write( "            " + ALTime.timeString( ) +"        ==>" + oalStrTime + " " + oalStringFiltered + "\n" )

                oalLogFile.write( "  </logAction>\n" )
                oalLogFile.write( "  <!-- ################################################################################## -->\n" )

        except:
            oalLogFile.write("<logActionListException>\n" )
            oalLogFile.write( ALTime.timeString( ) +"        ==> Exception in the Jython dictionary version \n" )
            oalLogFile.write("</logActionListException>\n" )

        oalLogFile.write( "</logActionList>\n" )
        oalLogFile.write( "</logLine>\n" )
        oalLogFile.write( "<!-- ################################################################################## -->\n" )
        oalLogFile.write( "\n\n" )
        oalLogFile.close()

        self.synchronizeOff()


    def setFileName( self, _oalString ):
        self.synchronizeOn()
        self.__createFile( _oalString )
        self.synchronizeOff()


    def setUniqueFileName( self, _oalString ):
        self.synchronizeOn()
        self.__createFile( _oalString )
        self.synchronizeOff()
        self.setLockFileName()


    def setLockFileName( self ):
        self.synchronizeOn()
        ALLogFile.m_oalLockFileName = True
        self.synchronizeOff()


    def getFileName( self ):
        return ALLogFile.m_oalLogFileName


    def getProfileFile( self ):
        if ( ALDiskExplorer.existFile( self.getFileName() )==False):
            return
        self.synchronizeOn()
#        oalStrFileName = ALFile.setExtension( ALString( self.getFileName() ), ".profile" )
        oalStrFileName = ALFile.setExtension( ALString( self.getFileName() ), "-Profile.xml" )

        oalLogFile = ALStreamFile( self.getFileName(), "r" )
        oalList =  oalLogFile.getLines()
        oalLogFile.close()


        oalProfileFile = ALStreamFile( oalStrFileName, "w" )
        oalProfileFile.write( "<?xml version=\"1.0\"?>\n" )
        oalProfileFile.write( "<!-- Generated Log File Profile :: PyAALib-JyAALib -->\n" )
        oalProfileFile.write( "<logProfile>\n" )
        oalProfileFile.write( "<version>" + self.getVersion() +"</version>\n" )
        oalProfileFile.write( "<logLineList>\n" )

        for iLine in oalList:
            oalProfileFile.write( iLine )

        oalProfileFile.write( "</logLineList>\n" )
        oalProfileFile.write( "\n\n</logProfile>\n" )
        oalProfileFile.close()

        self.synchronizeOff()
        return oalStrFileName


##############################################################################


