##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################



from ALImportPython     import PySys
from ALString           import ALString
from ALSystem           import ALSystem
from ALVerbose          import ALVerbose
from ALList             import ALList
from ALManager          import ALManager
from ALSemaphore        import ALSemaphore



class ALManagerTest( ALManager ):
    """
    Parent class for all ALManagerTest objects.
    """
    m_oalListTest                  = ALList()
    m_oalListTestTotal             = ALList()
    m_oalSemaphore                 = ALSemaphore()
    m_oalRunningTest               = None
    m_oiCumulativeTotalTest        = 0
    m_oiCumulativeTotalTestMethod  = 0
    m_oiCumulativeTotalTestSuccess = 0
    m_oiCumulativeTotalTestFailure = 0

    def __init__( self ):
        ALManager.__init__( self )


    def synchronizeOn():
        ALManagerTest.m_oalSemaphore.synchronizeOn()
    synchronizeOn = staticmethod( synchronizeOn )


    def synchronizeOff():
        ALManagerTest.m_oalSemaphore.synchronizeOff()
    synchronizeOff = staticmethod( synchronizeOff )


    def create():
        ALManagerTest.m_oiCumulativeTotalTest        += ALManagerTest.getNumberTest()
        ALManagerTest.m_oiCumulativeTotalTestMethod  += ALManagerTest.getNumberTestMethod()
        ALManagerTest.m_oiCumulativeTotalTestSuccess += ALManagerTest.getNumberTestSuccess()
        ALManagerTest.m_oiCumulativeTotalTestFailure += ALManagerTest.getNumberTestFailure()
        ALManagerTest.synchronizeOn()
        for iTest in ALManagerTest.m_oalListTest:
            ALManagerTest.m_oalListTestTotal.addObject( iTest )
        del ALManagerTest.m_oalListTest
        ALManagerTest.m_oalListTest  = ALList()
        ALManagerTest.synchronizeOff()
    create = staticmethod( create )



    def outputTable( _oalStream = PySys.stdout ):
        ALManagerTest.synchronizeOn()
        _oalStream.write( "\n" )
        _oalStream.write( "  =====================================================================\n" )
        _oalStream.write( "  Final Report - Test Manager\n\n" )
        _oalStream.write( "                        Compiler: [ " + ALString( ALSystem.getCompilerName() ) +" ]\n" )
        _oalStream.write( "                 Virtual Machine: [ " + ALString( ALSystem.getMachineVirtuel() ) +" ]\n" )
        _oalStream.write( "           Total Cumulated Tests: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTest ) +" ]\n" )
        _oalStream.write( "    Total Cumulated Test SUCCESS: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTestSuccess ) +" ]\n" )
        _oalStream.write( "   Total Cumulated Test FAILLURE: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTestFailure ) +" ]\n" )
        _oalStream.write( "    Total Cumulated Test Methods: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTestMethod ) +" ]\n" )
        for iTest in ALManagerTest.m_oalListTestTotal:
            iTest.outputTable( _oalStream )
        ALManagerTest.synchronizeOff()
    outputTable = staticmethod( outputTable )




    def outputScreen():
        ALManagerTest.synchronizeOn()
        ALVerbose.screen( "" )
        ALVerbose.screen( "  =====================================================================" )
        ALVerbose.screen( "  Final Report - Test Manager" )
        ALVerbose.screen( "                        Compiler: [ " + ALString( ALSystem.getCompilerName() ) +" ]" )
        ALVerbose.screen( "                 Virtual Machine: [ " + ALString( ALSystem.getMachineVirtuel() ) +" ]" )
        ALVerbose.screen( "           Total Cumulated Tests: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTest ) +" ]" )
        ALVerbose.screen( "    Total Cumulated Test SUCCESS: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTestSuccess ) +" ]" )
        ALVerbose.screen( "   Total Cumulated Test FAILLURE: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTestFailure ) +" ]" )
        ALVerbose.screen( "    Total Cumulated Test Methods: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTestMethod ) +" ]" )
        ALVerbose.screen( "" )
        for iTest in ALManagerTest.m_oalListTestTotal:
            iTest.outputScreen()
        ALManagerTest.synchronizeOff()
    outputScreen = staticmethod( outputScreen )


    def getAcidIndex():
        ovAcidIndex = ( float (ALManagerTest.m_oiCumulativeTotalTestSuccess) / float( ALManagerTest.m_oiCumulativeTotalTest ) )
        ovAcidIndex = ovAcidIndex * 100.0
        return int( ovAcidIndex )
    getAcidIndex = staticmethod( getAcidIndex )


    def getNumberTestCumulative():
        return ALManagerTest.m_oiCumulativeTotalTest
    getNumberTestCumulative = staticmethod( getNumberTestCumulative )


    def getNumberTestMethodCumulative():
        return ALManagerTest.m_oiCumulativeTotalTestMethod
    getNumberTestMethodCumulative = staticmethod( getNumberTestMethodCumulative )



    def getNumberTestSuccessCumulative():
        return ALManagerTest.m_oiCumulativeTotalTestSuccess
    getNumberTestSuccessCumulative = staticmethod( getNumberTestSuccessCumulative )


    def getNumberTestFailureCumulative():
        return ALManagerTest.m_oiCumulativeTotalTestFailure
    getNumberTestFailureCumulative = staticmethod( getNumberTestFailureCumulative )




    def addInformationTest( _oalInformationTest ):
        ALManagerTest.synchronizeOn()
        ALManagerTest.m_oalRunningTest = _oalInformationTest
        ALManagerTest.m_oalListTest.addObject( _oalInformationTest )
        ALManagerTest.synchronizeOff()
    addInformationTest = staticmethod( addInformationTest )




    def addInformationTestMethod( _oalInformationTest ):
        ALManagerTest.synchronizeOn()
        if (ALManagerTest.m_oalRunningTest!=None):
            ALManagerTest.m_oalRunningTest.addInformationTest( _oalInformationTest )
        ALManagerTest.synchronizeOff()
    addInformationTestMethod = staticmethod( addInformationTestMethod )




    def getNumberTest():
        ALManagerTest.synchronizeOn()
        oiNumberTestTotal = ALManagerTest.m_oalListTest.getNumberObjects()
        ALManagerTest.synchronizeOff()
        return oiNumberTestTotal
    getNumberTest = staticmethod( getNumberTest )


    def getNumberTestMethod():
        ALManagerTest.synchronizeOn()
        oiNumberTestTotal = 0
        for iTest in ALManagerTest.m_oalListTest:
            oiNumberTestTotal += iTest.getNumberTestMethod()
        ALManagerTest.synchronizeOff()
        return oiNumberTestTotal
    getNumberTestMethod = staticmethod( getNumberTestMethod )



    def getNumberTestSuccess():
        ALManagerTest.synchronizeOn()
        oiNumberTestTotal = 0
        for iTest in ALManagerTest.m_oalListTest:
            if (iTest.isSuccess()==True):
                oiNumberTestTotal += 1
        ALManagerTest.synchronizeOff()
        return oiNumberTestTotal
    getNumberTestSuccess = staticmethod( getNumberTestSuccess )


    def getNumberTestFailure():
        ALManagerTest.synchronizeOn()
        oiNumberTestTotal = 0
        for iTest in ALManagerTest.m_oalListTest:
            if (iTest.isSuccess()==False):
                oiNumberTestTotal += 1
        ALManagerTest.synchronizeOff()
        return oiNumberTestTotal
    getNumberTestFailure = staticmethod( getNumberTestFailure )





##############################################################################
