##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################



from ALActionController    import ALActionController
from ALToken               import ALToken
from ALVerbose             import ALVerbose
from ALInformation         import ALInformation
from ALString              import ALString


class ALPlugin( ALActionController ):
    """
    Interface for plugin classes.

    # To be modified by the user
    def preProcess( self, _oalObject = None ):
    def process( self, _oalObject = None ):
    def postProcess( self, _oalObject = None ):
    """
    def __init__( self, _oalPluginName = None, _oBoolPriority = False  ):
        ALActionController.__init__( self, _oBoolPriority )
        self.m_oalInformation = ALInformation()
        if ( _oalPluginName is not None ):
            self.m_oalPluginName = ALToken( _oalPluginName )
        else:
            self.m_oalPluginName = ALToken( self.getClassName() )
        self.setGroup(  self.m_oalPluginName )
        self.setSource( self.m_oalPluginName + ALToken( ".py" ) )
        self.m_oalStringMethod = ALString()


    # Only here to be used with XMLRPC servers = to define the interfaces of the server
    # as a Plugin
    def _listMethods(self):
        # implement this method so that method.listMethods
        # knows to advertise the strings methods
        return list_public_methods(self) + \
                ['method.' + method for method in list_public_methods( self.m_oalStringMethod )]


    def outputScreenInformation( self ):
        self.synchronizeOn()
        ALVerbose.screen( "Plugin        Name: " + self.getPluginName() )
        ALVerbose.screen( "                ID: " + self.getID() )
        ALVerbose.screen( "             Group: " + self.m_oalInformation.getGroupName() )
        ALVerbose.screen( "            Source: " + self.m_oalInformation.getFileName() )
        ALVerbose.screen( "            Author: " + self.m_oalInformation.getAuthor() )
        ALVerbose.screen( "           Contact: " + self.m_oalInformation.getContact() )
        ALVerbose.screen( "           Version: " + self.m_oalInformation.getVersion() )
        ALVerbose.screen( "       Information: " + self.m_oalInformation.getInformation() )
        ALVerbose.screen( "         Copyright: " + self.m_oalInformation.getCopyright() )
        self.synchronizeOff()


    def outputScreenSignature( self ):
        self.outputScreenInformation()


    def setSource( self, _oFileName ):
        self.synchronizeOn()
        self.m_oalInformation.setFileName( _oFileName )
        self.synchronizeOff()


    def getSource( self ):
        self.synchronizeOn()
        oFileName = self.m_oalInformation.getFileName( )
        self.synchronizeOff()
        return oFileName


    def setGroup( self, _oGroupName ):
        self.synchronizeOn()
        self.m_oalInformation.setGroupName( _oGroupName )
        self.synchronizeOff()


    def getGroup( self ):
        self.synchronizeOn()
        oGroupName = self.m_oalInformation.getGroupName( )
        self.synchronizeOff()
        return oGroupName


    def setInformation( self, _oalInformation ):
        self.synchronizeOn()
        self.m_oalInformation = _oalInformation
        self.synchronizeOff()


    def getInformation( self ):
        self.synchronizeOn()
        oalInfo = self.m_oalInformation
        self.synchronizeOff()
        return oalInfo


    def setToken( self, _oalTokenName ):
        self.synchronizeOn()
        self.m_oalPluginName = ALToken( _oalTokenName )
        self.synchronizeOff()


    def getToken( self ):
        oalToken = ALToken( self.m_oalPluginName )
        return oalToken

    def setPluginName( self, _oalTokenName ):
        self.setToken( _oalTokenName )

    def getPluginName( self ):
        return self.getToken()


    def pluginControl(self):
        self.actionControl()


    def synchronizePlugin( self, _oalObject = None ):
        self.synchronize( )


    def pluginControl( self ):
        self.actionControl()


    def executePlugin( self, _oalObject = None ):
        """
        This must be called to start the Threaded Action Plugin
        """
        self.executeAction( _oalObject )


    def executePluginTimer( self ):
        """
        This is the mian connection method to be used with Timer Plugins and methods.
        """
        self.executeActionTimer()



    def executePluginSynchronous( self, _oalObject = None ):
        """
        This method starts the Action in Synchronous  Mode.
        """
        self.executeActionSynchronous()





##############################################################################
