##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALSystem                import  ALSystem
from ALObject                import  ALObject
from ALString                import  ALString
from ALToken                 import  ALToken
from ALDictionary            import  ALDictionary
from ALXml                   import  ALXml
from ALXmlNode               import  ALXmlNode
from ALStreamFile            import  ALStreamFile
from ALStreamSerial          import  ALStreamSerial
from ALStreamXml             import  ALStreamXml
from ALDiskExplorer          import  ALDiskExplorer
from ALFile                  import  ALFile
from ALVerbose               import  ALVerbose



class ALResource( ALObject ):
    """
    Parent class for all ALResource objects.
    """


    def __init__( self, _oalString = "resource.res", _bUserPath = True, _bRead = True ):
        ALObject.__init__( self )
        self.m_oalDict = ALDictionary()
        self.m_oResourceFilePath = ALDiskExplorer.getCurrentFolder()
        self.m_oResourceFileName = ALString( self.m_oResourceFilePath + ALString( "/" ) + ALString( _oalString ) )

        oalStringFolder = ALString( "/Library/" )
        if (ALSystem.getOSName()=="Linux"):
            ALString( "/.Library/" )

        if (_bUserPath == True):
            self.m_oResourceFilePath = ALSystem.getUserPath() + oalStringFolder + ALFile.getBaseName( _oalString )
            self.m_oResourceFileName = ALString( self.m_oResourceFilePath + ALString( "/" ) + ALString( _oalString ) )
        if (_bRead == True):
            try:
                self.input()
            except:
                ALVerbose.warning("Exception reading the resource file: " + self.m_oResourceFileName )


    def getResourceDictionary(self):
        self.synchronizeOn()
        oalDic = ALDictionary( self.m_oalDict )
        self.synchronizeOff()
        return oalDic


    def getResource( self, _oalToken ):
        self.synchronizeOn()
        oalObject = self.m_oalDict.getValue( ALToken( _oalToken ) )
        self.synchronizeOff()
        return oalObject


    def setResource( self, _oalToken, _oalTokenObject ):
        self.synchronizeOn()
        self.__setResource( _oalToken, _oalTokenObject )
        self.synchronizeOff()


    def importResource( self, _oalString ):
        oalResource = ALResource( _oalString, False )
        oalDictionary = oalResource.getResourceDictionary()
        for iToken in oalDictionary.ordered():
            self.setResource( iToken, oalDictionary[iToken])



    def __setResource( self, _oalToken, _oalTokenObject ):
        self.m_oalDict.add( ALToken( _oalToken ), _oalTokenObject )



    def reset( self ):
        self.synchronizeOn()
        self.m_oalDict = ALDictionary()
        self.synchronizeOff()
        self.output()


    def exist( self ):
        self.synchronizeOn()
        obExist = False
        if ALDiskExplorer.existFile( self.m_oResourceFileName ) :
            obExist = True
        self.synchronizeOff()
        return obExist



    def output( self ):
        self.synchronizeOn()
        ALDiskExplorer.createFolder( self.m_oResourceFilePath )

        oalXml = ALXml()
        oalXml.addComment( "AALib Resource File::DO NOT EDIT" )
        self.synchronizeOff()

        oalXml.addObject( self )

        self.synchronizeOn()
        if (ALSystem.getOSName()!="Linux"):
            oalXml.output( self.m_oResourceFileName, True )
        else:
            oalXml.output( self.m_oResourceFileName ) # using to bypass error on Jython linux version
        oalXml.close()

        ALVerbose.log( "Resource::Saved   = " + self.m_oResourceFileName  )
        self.synchronizeOff()



    def input( self ):
        self.synchronizeOn()
        if ( ALDiskExplorer.existFile( self.m_oResourceFileName ) ) :

            oalXml = ALXml()
            oalXml.input( self.m_oResourceFileName )
            oalXmlElement = oalXml.getElement( self.getMagicNumber() )

            if (oalXmlElement!=None):
                self.inputXml( oalXmlElement, oalXml )
            oalXml.close()

            ALVerbose.log( "Resource::Loaded  = " + self.m_oResourceFileName )
        else :
            ALVerbose.log( "Resource::It will be CREATED = " + self.m_oResourceFileName )
        self.synchronizeOff()



    def outputXml( self, _oalXml, _oalXmlElement ):
        oalXmlElementObject = self.outputXmlObject( _oalXml, _oalXmlElement )
        self.m_oalDict.outputXml( _oalXml, oalXmlElementObject )



    def inputXml( self, _oalXmlElement, _oalXml  ):
        self.inputXmlObject( _oalXmlElement )
        oalXmlElementDictionary = _oalXml.getElementObject( _oalXmlElement, self.m_oalDict.getMagicNumber() )
        if (oalXmlElementDictionary!=None):
            self.m_oalDict.inputXml( oalXmlElementDictionary )




    ###########################################################################
    # Serial Output - TO BE DEPRECATED
    def outputSerial( self ):
        oalStreamSerial = ALStreamSerial( self.m_oResourceFileName, "wb" )
        oalStreamSerial.output( self.m_oalDict )
        oalStreamSerial.close()



    def inputSerial( self ):
        if ALDiskExplorer.existFile( self.m_oResourceFileName ) :
            oalStreamSerial = ALStreamSerial( self.m_oResourceFileName, "rb" )
            self.m_oalDict = oalStreamSerial.input()
            oalStreamSerial.close()
            ALVerbose.log( "Resource::Loaded  = " + self.m_oResourceFileName )
        else :
            ALVerbose.log( "Resource::CREATED = " + self.m_oResourceFileName )


    # Only to import old resouce files - STRINGS - To Be Deprecated ==========
    def outputSerialASCII( self, _oalObject = None ):
        self.synchronizeOn()
        oalStreamFile = ALStreamFile( "ascii-resource.res", "w" )
        for ikey in self.m_oalDict.ordered():
            oalStreamFile.write( ALString( ikey ) + ' ' + ALString( self.m_oalDict[ikey] ) + '\n')
        oalStreamFile.close()
        ALVerbose.log( "Resource::Saved   = " + "ascii-resource.res"  )
        self.synchronizeOff()


    def inputSerialASCII( self, _oalObject = None ):
        self.synchronizeOn()
        if ALDiskExplorer.existFile( "ascii-resource.res" ) :
            oalStreamFile = ALStreamFile( "ascii-resource.res", "r" )
            #self.m_oalDict = oalStreamFile.read()
            for iline in oalStreamFile.readLines():
                key, token  = ALString.split( iline )
                self.setResource( ALString( key ), ALString( token ) )
            oalStreamFile.close()
            ALVerbose.log( "Resource::Loaded  = " + "ascii-resource.res" )
        else :
            ALVerbose.log( "Resource::CREATED = " + "ascii-resource.res" )
        self.synchronizeOff()


##############################################################################
