##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython      import PyXmlRpcServer
from ALImportPython      import PySimpleXMLRPCServer
from ALImportPython      import PySerializer
from ALImportPython      import PyXmlRpcLib
from ALServer      import ALServer
from ALString      import ALString


class ALServerXmlRpc( ALServer ):
    """
    Parent class for all AL XML-RPC Server objects.
    """
    
    
    def __init__( self, _oalHostName = "localhost", _oalPortNumber = 8000 ):
        ALServer.__init__( self )
        self.m_oalServerXmlRpc = PySimpleXMLRPCServer( ( _oalHostName, _oalPortNumber ) )
#        self.m_oalServerXmlRpc.register_introspection_functions()
    
    
    def executeServer( self ):
        self.m_oalServerXmlRpc.serve_forever()

   
    def addMethod( self, _oalMethod ):
#        self.m_oalServerXmlRpc.register_introspection_functions()
        self.m_oalServerXmlRpc.register_function( _oalMethod )
    
    
    def addObject( self, _oalObject ):
#        self.m_oalServerXmlRpc.register_introspection_functions()
        self.m_oalServerXmlRpc.register_instance( _oalObject )
    
    
    
    def connectServer( _oalStringHostAndSocket = "http://localhost:8000" ):
        return PyXmlRpcServer( _oalStringHostAndSocket )
    connectServer = staticmethod( connectServer )
     
     
    def encoder( _oalObject ):
        return PySerializer.dumps( _oalObject )
    encoder = staticmethod( encoder )
    
    
    def decoder( _oalStringIO ):
        return PySerializer.loads( _oalStringIO )
    decoder = staticmethod( decoder )
    
     
    def response( _oalString = None ):
        if ( _oalString==None ):
            return PyXmlRpcLib.True
        else:
            return ALString( _oalString )
    response = staticmethod( response )
    
    
    
#
#
#1. Install functions:
#
#server = SimpleXMLRPCServer(("localhost", 8000))
#server.register_function(pow)
#server.register_function(lambda x,y: x+y, 'add')
#server.serve_forever()
#
#2. Install an instance:
#
#class MyFuncs:
#    def __init__(self):
#        # make all of the string functions available through
#        # string.func_name
#        import string
#        self.string = string
#    def _listMethods(self):
#        # implement this method so that system.listMethods
#        # knows to advertise the strings methods
#        return list_public_methods(self) + \
#                ['string.' + method for method in list_public_methods(self.string)]
#    def pow(self, x, y): return pow(x, y)
#    def add(self, x, y) : return x + y
#
#server = SimpleXMLRPCServer(("localhost", 8000))
#server.register_introspection_functions()
#server.register_instance(MyFuncs())
#server.serve_forever()
#
#3. Install an instance with custom dispatch method:
#
#class Math:
#    def _listMethods(self):
#        # this method must be present for system.listMethods
#        # to work
#        return ['add', 'pow']
#    def _methodHelp(self, method):
#        # this method must be present for system.methodHelp
#        # to work
#        if method == 'add':
#            return "add(2,3) => 5"
#        elif method == 'pow':
#            return "pow(x, y[, z]) => number"
#        else:
#            # By convention, return empty
#            # string if no help is available
#            return ""
#    def _dispatch(self, method, params):
#        if method == 'pow':
#            return pow(*params)
#        elif method == 'add':
#            return params[0] + params[1]
#        else:
#            raise 'bad method'
#
#server = SimpleXMLRPCServer(("localhost", 8000))
#server.register_introspection_functions()
#server.register_instance(Math())
#server.serve_forever()

##############################################################################
