##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython        import *
from ALString              import ALString
from ALSystem              import ALSystem
from ALManager             import ALManager
from ALList                import ALList
from ALSignal              import ALSignal
from ALVerbose             import ALVerbose
from ALDebug               import ALDebug
from ALTrace               import ALTrace
from ALExceptionSignal     import ALExceptionSignal



class ALSlot( ALManager ):
    """
    Class for managing signals.
    """


    def __init__( self ):
        ALManager.__init__( self )
        self.m_oalListConnection = ALList()



    def connect(self, _oalClassMethod, _oalSignal = None ):
        """
        Connects a signal between an object and a method in an other object.
        """
        if ( _oalClassMethod == None ):
            ALVerbose.warning( "[Slot] ALSlot.connect : _oalClassMethod is None" )
            return
        self.synchronizeOn()    # Block the thread
        self.m_oalListConnection.append( [ _oalSignal, _oalClassMethod ] )
        self.synchronizeOff()   # Release the thread


    def __raiseControl( self ):
        oBoolStatus =  ALDebug.isDebugModeOn()
        ALDebug.controlDebug( self, oBoolStatus )


    # Blocked by self.call()
    def executeMethod( self, _method, _oalListArguments = None ):
        try:
            if (_oalListArguments == None):
                _method()
            else:
                _method( _oalListArguments )
        except:
            ALVerbose.DEBUG( "[Slot] EXCEPTION - ALSlot::executeMethod - parameter: " + ALString( _oalListArguments ) )
            ALVerbose.DEBUG( "[Slot] EXCEPTION - -------------------------- method: " + ALString( _method ) )
            self.__raiseControl()



    def call( self, _oalListArguments = None, _oalSignal = None ):
        self.synchronizeOn()    # Block the thread
        for connection in self.m_oalListConnection:
            oalSignal = connection[ 0 ]
            omethod   = connection[ 1 ]
            if (_oalSignal == None):
                if (oalSignal == None):
                    if (omethod!=None):
                        self.executeMethod( omethod, _oalListArguments )
            else:
                if (oalSignal != None):
                    oalStringSignalExt = _oalSignal.getSignal()
                    oalStringSignal    =  oalSignal.getSignal()
                    if (oalStringSignal.equals( oalStringSignalExt )):
                        if (_oalListArguments == None and omethod!=None ):
                            self.executeMethod( omethod, _oalSignal )
                        else :
                            self.executeMethod( omethod, _oalListArguments )
        self.synchronizeOff()   # Release the thread


    def callSignal( self, _oalSignal ):
        self.call( None, _oalSignal )

##############################################################################



