##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from  ALImportPython import  PyString
from  ALMetaObject   import  ALMetaObject


class ALString( ALMetaObject, str ):
    """
    Interface for AL string objects.
    """


    def __init__( self, _oString = None ):
        ALMetaObject.__init__( self )
        oString = ""
        if (oString!=None):
            oString = _oString
        str.__init__( _oString )


    def getPythonString( self ):
        """
        #Workaround to Jython and java bugs for multi inheritance
        it allows convert the ALString for jython string!!!!
        """
        return PyString.join( self, "" )


    def duplicate( self ):
        """
        This method is normally provided by the ESObject base class but
        since we don't inherit from ESObject we must define this method
        explicitly.
        """
        return ALString( self )


    def equals( self, _ostr ):
        """
        a.equals(b) corresponds to (a == b) in Python.
        """
        return ( str( self ) == str( _ostr ) )



    def atoi( self ):
        """
        Returns an integer.
        """
        try:
            oValue = float( str( self ) )
        except:
            oValue = int( str( self ) )
        return int( oValue )


    def atof( self ):
        """
        Returns a float.
        """
        try:
            oValue = float( str( self ) )
        except:
            oValue = int( str( self ) )
        return float( oValue )



    def replace( self, _ostr1, _ostr2, _iMaxcount = -1 ):
        """
        Overrides the str replace method.
        """
        return ALString( str.replace( str( self ), _ostr1, _ostr2, _iMaxcount ) )


    def cleanXml( self ):
        """
        Returns a string with no espaces
        """
        #bug on jython linux version the time in teh format [999] is cutted
        # workaround
        oString2 = ALString( self )
        oString2 = oString2.replace( "\n", "" )
        oString2 = oString2.replace( "\t", "" )
        return ALString( oString2 )



    def split( self, _oCharater = " " ):
        """
        """
        oList = str.split( self, _oCharater  )
        return oList


    def getToken( self, _oPosition = 0,  _oCharater = " " ):
        oList = self.split( _oCharater )
        oalString = None
        if ( len( oList) >_oPosition+1 ):
            oalString = ALString( oList[ _oPosition ] )
        return oalString


    def getNextToken( self, _oRefWord = " ",  _oCharater = " " ):
        oList = self.split( _oCharater )
        oalString = None

        if _oRefWord in oList:
            refWordIndex = oList.index( _oRefWord )
            wordIndex = refWordIndex + 1
            if ( len( oList) > refWordIndex+1 ):
                oalString = ALString( oList[ wordIndex ] )

        return oalString


    def existSubString( self, _oString ):
        result = self.rfind( _oString )
        if (result==-1):
            return False
        else:
            return True


    def contains( self, _oString ):
        return self.existSubString( _oString )



    def empty():
        return ALString( "" )
    empty = staticmethod( empty )


   # only to fix xml log stuffs
    def setLogXmlToString(  _oalString ):
        oalString = ALString( _oalString )
        oalString = oalString.replace( "<", "[" )
        return oalString.replace( ">", "]" )
    setLogXmlToString = staticmethod( setLogXmlToString )



    def setFormat( _oString, _oiSize = 25 ):
        oalStr = ALString( _oString )
        iSizeSpace  = 1
        iSizeString = len( oalStr )

        if (iSizeString<_oiSize):
            iSizeSpace = _oiSize - iSizeString

        oalString = ALString()
        for i in range( 0, iSizeSpace):
            oalString = oalString + " "
        for i in range( 0, iSizeString):
            oalString = oalString + oalStr[i]

        return oalString

    setFormat = staticmethod( setFormat )

##############################################################################
