##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#
#   Author(s): Romeu A. Pieritz (originally ALUnitTest)
#   Contact  : aalib_info@yahoo.com
#   Date:      2005/02/02
#
#
#   Refactoring: Marie-Francoise Incardona & Romeu A. Pieritz
#   Date:        2007/10/05
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython    import  PySys
from ALTime            import  ALTime
from ALVerbose         import  ALVerbose
from ALSystem          import  ALSystem
from ALString          import  ALString
from ALAssert          import  ALAssert
from ALTrace           import  ALTrace
from ALDebug           import  ALDebug
from ALLogFile         import  ALLogFile
from ALTest            import  ALTest
from ALManagerTest     import  ALManagerTest
from ALInformationTest import  ALInformationTest



class ALTestSuite( ALTest ):
    """
    Interface for Tests Suite

    # To be modified by the user with its methods
    def process( self ):
    def preProcess( self ):
    def postProcess( self ):

    """
    def __init__( self, _oalStringTestName = None ):
        ALTest.__init__( self,  _oalStringTestName )
        self.m_oNumberTest        = 0
        self.m_oNumberTestTotal   = 0
        self.m_oNumberTestSuccess = 0
        self.m_oNumberTestFailure = 0
        if (_oalStringTestName != None ):
            oalStringNameLog = self.m_oalStringTestName + ALString( ".log" )
            ALDebug.setDebugLogFile( oalStringNameLog )
        self.controlCommandLine()
        self.oalTime = ALTime()
        ALAssert.setAssertionModeOn()



    def __patchEclipseConsolePipe( self ):
        """
        Fix to Eclipse/Python console output not close the pipe quickly
        """
        ALTime.wait(1.0)



    def outputReport(self):
        ALVerbose.screen(   " " )
        ALVerbose.screen(   " " )
        ALVerbose.unitTest( "###################################################################" )
        ALVerbose.unitTest( self.getTestName() + " - END Status [OK]" )
        ALVerbose.unitTest( self.getTestName() + " - Test Report:" )
        ALVerbose.unitTest( "                            Compiler: [ " + ALString( ALSystem.getCompilerName() ) +" ]" )
        ALVerbose.unitTest( "                     Virtual Machine: [ " + ALString( ALSystem.getMachineVirtuel() ) +" ]" )
        ALVerbose.unitTest( "                     Total TestCases: " + ALString( self.m_oNumberTestTotal ) )
        ALVerbose.unitTest( "                  Executed TestCases: " + ALString( self.m_oNumberTest ) )
        ALVerbose.unitTest( "                 TestCases [SUCCESS]: " + ALString( self.m_oNumberTestSuccess ) )
        ALVerbose.unitTest( "                    TestCases [FAIL]: " + ALString( self.m_oNumberTestFailure ) )
        ALVerbose.unitTest( "                 [Total TestMethods]: " + ALString( ALManagerTest.getNumberTestMethod() ) )
        ofTime = self.oalTime.getDuration()
        ALVerbose.unitTest( "                             Runtime: " + ALString( ofTime ) + " [s]" )
        ALVerbose.unitTest( "                                 Run: " + ALTime.getTimeStringPSP( ofTime )  )



    def outputReportCumulate(self):
        ALVerbose.outputActionLog()
        ALManagerTest.create()
        ALManagerTest.outputScreen()
        ALVerbose.screen(   " " )
        ALVerbose.screen(   " " )
        ALVerbose.unitTest( "###################################################################" )
        ALVerbose.unitTest( "###################################################################" )
        ALVerbose.unitTest( "###################################################################" )
        ALVerbose.unitTest( "Final Summary Report:" )
        ALVerbose.unitTest( "                            Compiler: [ " + ALString( ALSystem.getCompilerName() ) +" ]" )
        ALVerbose.unitTest( "                     Virtual Machine: [ " + ALString( ALSystem.getMachineVirtuel() ) +" ]" )
        ALVerbose.unitTest( "                     Total TestCases: " + ALString( ALManagerTest.getNumberTestCumulative() ) )
        ALVerbose.unitTest( "           Total TestCases [SUCCESS]: " + ALString( ALManagerTest.getNumberTestSuccessCumulative() ) )
        ALVerbose.unitTest( "              Total TestCases [FAIL]: " + ALString( ALManagerTest.getNumberTestFailureCumulative() ) )
        ALVerbose.unitTest( "                 [Total TestMethods]: " + ALString( ALManagerTest.getNumberTestMethodCumulative() ) )
        ALVerbose.unitTest( "           Acid Index [Succes Ratio]: [ " + ALString( ALManagerTest.getAcidIndex() ) +"% ]" )

        ofTime = self.oalTime.getDuration()
        ALVerbose.unitTest( "                             Runtime: " + ALString( ofTime ) + " [s]" )
        ALVerbose.unitTest( "                                 Run: " + ALTime.getTimeStringPSP( ofTime )  )
        ALVerbose.screen(   " " )
        ALVerbose.screen(   " " )




    def processKernel( self ):
        ALVerbose.DEBUG( "Execution: ALTestSuite.processKernel()" )
        ALManagerTest.create()
        self.m_oNumberTestTotal   = self.m_oalListTest.getNumberObjects( )
        self.m_oNumberTestSuccess = 0
        self.m_oNumberTestFailure = 0
        self.m_oNumberTest        = 0
        for oTestMethod in self.m_oalListTest:
            self.m_oNumberTest += 1
            oalInformationTest = ALInformationTest( self.patchMethodName( oTestMethod ), self.m_oNumberTest )
            ALVerbose.screen( " " )
            ALVerbose.unitTest( "###################################################################" )
            ALVerbose.unitTest( "TEST [ " + ALString( oalInformationTest.getTestNumber() ) + " ] : [ " + oalInformationTest.getTestName() + " ]" )
            ALVerbose.unitTest( " ")
            ALManagerTest.addInformationTest( oalInformationTest )
            try:
                try:
                    oTestMethod()
                    oalInformationTest.setTerminate()
                    ALVerbose.unitTest( "TEST [" + ALString(  oalInformationTest.getTestNumber() ) + " ] : [SUCCESS]" )
                except Exception, opyException:
                    oalInformationTest.setTerminate()
                    oalInformationTest.setFailure()
                    oalInformationTest.setException( ALString(opyException) )
                    ALVerbose.unitTest( "Test Failure = [ " + oalInformationTest.getException() + " ] Raised"  )
                    ALVerbose.unitTest( "TEST [ " + ALString(  oalInformationTest.getTestNumber() ) + " ] : [ " + oalInformationTest.getTestName() + " ]: [FAILURE]" )
                    self.raiseControl()

            finally:
                ALVerbose.unitTest( "###################################################################" )
                ofTime = oalInformationTest.getRunTime()
                ALVerbose.unitTest( "Test - " + ALString( oalInformationTest.getTestNumber() ) + " : Module Runtime: " + ALString( ofTime ) + " [s]" )
                ALVerbose.screen( " " )

        self.m_oNumberTestSuccess = ALManagerTest.getNumberTestSuccess()
        self.m_oNumberTestFailure = ALManagerTest.getNumberTestFailure()



    def execute( self ):
        ALVerbose.unitTest( "STARTING::" + self.getTestName() )
        ALVerbose.unitTest( "###################################################################" )
        try:
            try:
                self.executeKernel()
            except:
                self.raiseControl()
        finally:
            self.outputReport()
            self.outputReportCumulate()
            self.outputProfile()
            self.__patchEclipseConsolePipe()


##############################################################################