##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython            import PyThread
from ALString                  import ALString
from ALObject                  import ALObject
from ALVerbose                 import ALVerbose
from ALDebug                   import ALDebug
from ALTrace                   import ALTrace


class ALThread( ALObject, PyThread ):
    """
    Interface for ALThread objects.

    To be modified by the user
    def run( self ):
    """
    def __init__( self ):
        ALObject.__init__( self )
        PyThread.__init__( self )
        self.m_oalObject = None


    def __controlRaise( self ):
        oBoolStatus =  ALDebug.isDebugModeOn()
        ALDebug.controlDebug( self, oBoolStatus )


    def executeKernel( self ):
        """
        This is the method started by the thread model in Synchronous
        It must be modified by the user
        """
        pass



    def execute( self, _oalObject = None ):
        """
        This method starts the thread.
        """
        ALVerbose.DEBUGID( self.getID(), "[Thread] Starting instance: " + ALString.setLogXmlToString( ALString( self ) ) )
        ALVerbose.DEBUGID( self.getID(), "[Thread]         parameter: " + ALString.setLogXmlToString( ALString(  _oalObject )) )

        self.m_oalObject = _oalObject
        try:
            self.start()
        except:
            ALVerbose.errorID( self.getID(), "Exception starting the thread: " + ALString( self.getID() ) )
            ALVerbose.DEBUGID( self.getID(), "   [Thread] Starting instance: " + ALString.setLogXmlToString( ALString( self ) ) )
            ALVerbose.DEBUGID( self.getID(), "   [Thread]         parameter: " + ALString.setLogXmlToString( ALString(  _oalObject ) ) )
            self.__controlRaise()




    def executeSynchronous( self, _oalObject = None ):
        """
        This method starts the Synchronous Thread.
        """
        ALVerbose.DEBUGID( self.getID(), "[Synchronous Thread] Starting instance: " + ALString.setLogXmlToString( ALString( self ) ) )
        ALVerbose.DEBUGID( self.getID(), "[Synchronous Thread]         parameter: " + ALString.setLogXmlToString(ALString(  _oalObject ) ) )

        self.m_oalObject = _oalObject
        try:
            self.executeKernel()
        except:
            ALVerbose.errorID( self.getID(), "Exception starting the Synchronous thread: " + ALString( self.getID() ) )
            ALVerbose.DEBUGID( self.getID(), "   [Synchronous Thread] Starting instance: " + ALString.setLogXmlToString( ALString( self ) ) )
            ALVerbose.DEBUGID( self.getID(), "   [Synchronous Thread]         parameter: " + ALString.setLogXmlToString( ALString( _oalObject )) )
            self.__controlRaise()



##############################################################################