##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython            import PyTime
from ALString                  import ALString
from ALObject                  import ALObject


class ALTime( ALObject ):
    """
    Interface for AL time objects.

    """
    m_fTimeInit = 0.0
    m_fTimeEnd  = 1.0

    def __init__( self ):
        ALObject.__init__( self )
        self.m_oFloatTimeInit = 0.0
        self.m_oFloatTimeEnd  = 1.0
        self.initTime()


    def initTime( self ):
        """
        Set the time in seconds
        """
        self.m_oFloatTimeInit = ALTime.time()
        return self.m_oFloatTimeInit



    def endTime( self ):
        """
        Set the time in seconds
        """
        self.m_oFloatTimeEnd = ALTime.time()
        return self.m_oFloatTimeEnd


    def getDuration( self ):
        """
        Return the time in seconds
        """
        self.endTime()
        return self.m_oFloatTimeEnd - self.m_oFloatTimeInit



    # Static methods

    def time():
        return PyTime.time()
    time = staticmethod( time )


    def clock():
        return PyTime.clock()
    clock = staticmethod( clock )


    def date():
        osltimestring = ALString( PyTime.strftime( "%Y%m%d", PyTime.localtime( PyTime.time() ) ) )
        return osltimestring
    date = staticmethod( date )


    def getDate():
        return ALTime.date()
    getDate = staticmethod( getDate )


    def hour():
        oslTimeString = ALString( PyTime.strftime( "%H:%M:%S", PyTime.localtime( PyTime.time() ) ) )
        return oslTimeString
    hour = staticmethod( hour )


    def getTimeStringPSP( _ofTime ):
     #   oslTimeString = ALString(  PyTime.strftime( "%dd:%Hh:%Mm:%Ss",  PyTime.localtime(_ofTime) )  )
        ofTime       = _ofTime
        oTotalHour   = 0
        oiHour       = 0
        oiMinute     = 0
        oiDay        = 0
        oiSecond     = 0
        oiMiliSecond = 0
        if (ofTime>3600.0):
            oTotalHour = int(ofTime/3600.0)
            if (oTotalHour>24):
                oiDay = int(oTotalHour/24.0)
                oiHour = int( oTotalHour -(oiDay*24.0) )
            else:
                oiHour = int( oTotalHour )

        ofSecond = ofTime - (3600.0*oTotalHour)
        if (ofSecond>60.0):
            oiMinute = int( ofSecond/60.0)
        ofSecond = ofSecond - (oiMinute*60.0)
        oiSecond = int( ofSecond )
        oiMiliSecond = int( (ofSecond - oiSecond)*1000 )

        if (oiDay<10):
            osDay = ALString( "0" + ALString(oiDay) + "d:" )
        else:
            osDay = ALString( ALString(oiDay) + "d:" )
        if (oiHour<10):
            osHour = ALString( "0" + ALString(oiHour) + "h:" )
        else:
            osHour = ALString( ALString(oiHour) + "h:" )

        if (oiMinute<10):
            osMinute = ALString( "0" + ALString(oiMinute) + "m:" )
        else:
            osMinute = ALString( ALString(oiMinute) + "m:" )

        if (oiSecond<10):
            osSecond = ALString( "0" + ALString(oiSecond) + "s:" )
        else:
            osSecond = ALString( ALString(oiSecond) + "s:" )

        if (oiMiliSecond<10):
            osMiliSecond = ALString( "00" + ALString(oiMiliSecond) + "ms" )
        elif (oiMiliSecond<100):
            osMiliSecond = ALString( "0" + ALString(oiMiliSecond) + "ms" )
        else:
            osMiliSecond = ALString( ALString(oiMiliSecond) + "ms" )

        oslTimeString = osDay + osHour + osMinute + osSecond + osMiliSecond

        return oslTimeString
    getTimeStringPSP = staticmethod( getTimeStringPSP )



    def timeString():
        osltimestring = ALString( PyTime.strftime( "[%Y/%m/%d-%H:%M:%S]", PyTime.localtime( PyTime.time() ) ) )
        return osltimestring
    timeString = staticmethod( timeString )



    def timeStringValue( _fvalue ):
        osltimestring = ALString( PyTime.strftime( "[%Y/%m/%d-%H:%M:%S]", PyTime.localtime( _fvalue ) ) )
        return osltimestring
    timeStringValue = staticmethod( timeStringValue )


    def getTimeString():
        return ALTime.timeString()
    getTimeString = staticmethod( getTimeString )


    def getTimeName():
        osltimestring = ALString( PyTime.strftime( "%Y%m%d-%H%M%S", PyTime.localtime( PyTime.time() ) ) )
        return osltimestring
    getTimeName = staticmethod( getTimeName )


    def sleep( _fTimeInSeconds = 1 ):
        PyTime.sleep( _fTimeInSeconds )
    sleep = staticmethod( sleep )


    def wait( _fTimeInSeconds = 1 ):
        ALTime.sleep( _fTimeInSeconds )
    wait = staticmethod( wait )


    def init():
        """
        Set the time in seconds
        """
        ALTime.m_fTimeInit = ALTime.time()
        return ALTime.m_fTimeInit
    init = staticmethod( init )


    def end():
        """
        Set the time in seconds
        """
        ALTime.m_fTimeEnd = ALTime.time()
        return ALTime.m_fTimeEnd
    end = staticmethod( end )


    def duration():
        """
        Return the time in seconds
        """
        ALTime.end()
        return ALTime.m_fTimeEnd - ALTime.m_fTimeInit
    duration = staticmethod( duration )


    def dif():
        """
        Return the time in seconds
        """
        ALTime.end()
        fdif = ALTime.m_fTimeEnd - ALTime.m_fTimeInit
        ALTime.init()
        return fdif
    dif = staticmethod( dif )



##############################################################################
