##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################

##############################################################################
#
#  Revised by: Marie-Francoise Incardona
#
##############################################################################



from ALImportPython          import  PySys
from ALDefinition            import  *
from ALObject                import  ALObject
from ALString                import  ALString
from ALDictionary            import  ALDictionary
from ALThreadingCondition    import  ALThreadingCondition
from ALTime                  import  ALTime
from ALLogFile               import  ALLogFile



class ALVerbose( ALObject ):
    """
    Base class to Verbose
    This class control the information flux between the
    application and the outside world (screen, log files, error, std,
    stderror, etc. )

    Note: Designed to be a static class
    Note: Trying to be Thread Safety!!!!
    """

    m_oalThreadingConditionError    = ALThreadingCondition()
    m_oalThreadingConditionUnitTest = ALThreadingCondition()
    m_oalThreadingConditionScreen   = ALThreadingCondition()
    m_oalThreadingConditionLog      = ALThreadingCondition()
    m_oalThreadingConditionLogFile  = ALThreadingCondition()
    m_oalThreadingConditionWrite    = ALThreadingCondition()
    m_oalThreadingConditionProfile  = ALThreadingCondition()
    m_bIsProfile      = False
    m_oalDictionary   = ALDictionary()
    m_oalLogFile      = ALLogFile( AL_FRAMEWORK_LOG_NAME )
    m_bIsLogFile      = True
    m_bIsVerbose      = False
    m_bIsVerboseDebug = False
    m_iindex = 0;


    def __init__( self ):
        ALObject.__init__( self )



    def __writeError( _oalString ):
        if (ALVerbose.m_bIsVerbose == True or ALVerbose.m_bIsVerboseDebug==True ):
            PySys.stderr.write( ALString( _oalString ) )
    __writeError = staticmethod( __writeError )


    def __write( _oalString ):
        if (ALVerbose.m_bIsVerbose == True or  ALVerbose.m_bIsVerboseDebug==True ):
            PySys.stdout.write( ALString( _oalString ) )
    __write = staticmethod( __write )



    def __toIndex( _oalObject ):
        return ALString( ALString( ALTime.time() ) + AL_TOKEN_SEPARATOR + ALString( _oalObject ) )
    __toIndex = staticmethod( __toIndex )



    def __fromIndex( _oalstring ):
        oList = ALString( _oalstring ).split( AL_TOKEN_SEPARATOR )
        oalStrTime = ALTime.timeStringValue( float( oList[0] ) )
        oalStrInfo = ALString( oList[1] )
        return oalStrTime, oalStrInfo
    __fromIndex = staticmethod( __fromIndex )



    def errorID(  _oalStrID, _oalString ):
        ALVerbose.m_oalThreadingConditionError.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " [ERROR]: " + _oalString, _oalStrID )
        ALVerbose.m_oalThreadingConditionError.synchronizeOff()
        ALVerbose.error( _oalString )
    errorID = staticmethod( errorID )


    def error( _oalString ):
        ALVerbose.m_oalThreadingConditionError.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  [ERROR]: " + oalString + "\n"  )
        # Force the ERROR output in the screen
        btmp = ALVerbose.isVerbose()
        ALVerbose.setVerboseOn()
        ALVerbose.__writeError( oalString )
        if btmp ==False:
            ALVerbose.setVerboseOff()
        # Force the ERROR output in the screen
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionError.synchronizeOff()
    error = staticmethod( error )


    def warningID(  _oalStrID, _oalString ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " [Warning]: " + _oalString, _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.warning( _oalString )
    warningID = staticmethod( warningID )



    def warning( _oalString ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  [Warning]: " + oalString + "\n" )
        ALVerbose.__write( oalString )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
    warning = staticmethod( warning )


    def unitTestID( _oalStrID, _oalString = ALString( "[ ok ]" ) ):
        ALVerbose.m_oalThreadingConditionUnitTest.synchronizeOn()
        ALVerbose.__addObjectIDInformation( "  [UnitTest]: " + _oalString, _oalStrID )
        ALVerbose.m_oalThreadingConditionUnitTest.synchronizeOff()
        ALVerbose.unitTest( _oalString )
    unitTestID = staticmethod( unitTestID )

    def unitTest( _oalString = ALString( "[ ok ]" ) ):
        ALVerbose.m_oalThreadingConditionUnitTest.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  [UnitTest]: " + oalString + "\n" )
        ALVerbose.__write( oalString )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionUnitTest.synchronizeOff()
    unitTest = staticmethod( unitTest )


    def screenID( _oalStrID, _oalString = ALString( "[ ok ]" ) ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " " + _oalString, _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.screen( _oalString )
    screenID = staticmethod( screenID )

    def screen( _oalString = ALString( "[ ok ]" ) ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  " + oalString + "\n" )
        ALVerbose.__write( oalString )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
    screen = staticmethod( screen )



    def screenIndexID( _oalStrID, _fvalue, _fref ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " " + ALString( _fvalue) + " " + ALString( _fref ), _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.screenIndex( _fvalue, _fref )
    screenIndexID = staticmethod( screenIndexID )



    def screenIndex( _fvalue, _fref ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        index = ( float(_fvalue)/ float(_fref) )*100.0
        if ( index >= ALVerbose.m_iindex+1 ):
            ALVerbose.m_iindex = int( index )
            oalString = ALString( ALVerbose.m_iindex )
            oalString = ALString( "  [ " + oalString + "% ]" + "\n" )
            ALVerbose.__write( oalString )
            ALVerbose.logFile( oalString )
        if (index>99.75):  #bug to fix
            ALVerbose.m_iindex = 0
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
    screenIndex = staticmethod( screenIndex )





    def DEBUGID( _oalStrID, _oalString = ALString( " DEBUG mode " ) ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " [DEBUG]: "  + _oalString, _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.DEBUG( _oalString )
    DEBUGID = staticmethod( DEBUGID )


    def DEBUG( _oalString = ALString( " DEBUG mode " ) ):
        if ALVerbose.isVerboseDebug():
            ALVerbose.screen( ALString( "[DEBUG]: " + _oalString ) )
    DEBUG = staticmethod( DEBUG )


    def DEBUGINDEXID( _oalStrID, _fvalue, _fref ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " [DEBUG]: "  + ALString( _fvalue) + " " + ALString( _fref ), _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.DEBUGINDEX( _fvalue, _fref )
    DEBUGINDEXID = staticmethod( DEBUGINDEXID )


    def DEBUGINDEX( _fvalue, _fref ):
        if ALVerbose.isVerboseDebug():
            ALVerbose.screenIndex( _fvalue, _fref )
    DEBUGINDEX = staticmethod( DEBUGINDEX )



    def ASSERT( _oalString ):
        if ALVerbose.isVerbose():
            ALVerbose.screen( "[ASSERT]:   "  + ALString( _oalString ) )
    ASSERT = staticmethod( ASSERT )



    # Private Method
    def logFile( _oalString ):
        if (ALVerbose.m_bIsLogFile == False):
            return
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOn()
        ALVerbose.m_oalLogFile.log( ALString(  _oalString ) )
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOff()
    logFile = staticmethod( logFile )



    def log( _oalString ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  " + oalString + "\n" )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    log = staticmethod( log )


    def logID( _oalStrID, _oalString ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  " + oalString + "\n" )
        ALVerbose.__addObjectIDInformation( " " + _oalString, _oalStrID )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    logID = staticmethod( logID )



    def isVerbose():
        bisVerbose = ALVerbose.m_bIsVerbose
        return bisVerbose
    isVerbose = staticmethod( isVerbose )



    def isLogFile( ):
        return ALVerbose.m_bIsLogFile
    isLogFile = staticmethod( isLogFile )


    def setLogFileOn( ):
        ALVerbose.m_bIsLogFile = True
    setLogFileOn = staticmethod( setLogFileOn )


    def setLogFileOff( ):
        ALVerbose.m_bIsLogFile = False
    setLogFileOff = staticmethod( setLogFileOff )


    def setLogFileName( _oalString ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        ALVerbose.m_oalLogFile.setFileName( _oalString )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    setLogFileName = staticmethod( setLogFileName )


    def setLogFileNameUnique( _oalString ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        ALVerbose.m_oalLogFile.setUniqueFileName( _oalString )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    setLogFileNameUnique = staticmethod( setLogFileNameUnique )




    def getLogFileName():
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        oalString = ALVerbose.m_oalLogFile.getFileName()
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
        return oalString
    getLogFileName = staticmethod( getLogFileName )


    def isVerbose():
        bisVerbose = ALVerbose.m_bIsVerbose
        return bisVerbose
    isVerbose = staticmethod( isVerbose )


    def setVerboseOn( ):
        ALVerbose.m_bIsVerbose = True
    setVerboseOn = staticmethod( setVerboseOn )


    def setVerboseOff( ):
        ALVerbose.m_bIsVerbose = False
    setVerboseOff = staticmethod( setVerboseOff )


    def isVerboseDebug():
        bisVerboseDebug = ALVerbose.m_bIsVerboseDebug
        return bisVerboseDebug
    isVerboseDebug = staticmethod( isVerboseDebug )


    def setVerboseDebugOn( ):
        ALVerbose.m_bIsVerboseDebug = True
    setVerboseDebugOn = staticmethod( setVerboseDebugOn )


    def setVerboseDebugOff( ):
        ALVerbose.m_bIsVerboseDebug = False
    setVerboseDebugOff = staticmethod( setVerboseDebugOff )



    def outputObjectsDEBUG( _oalFile = PySys.stdout ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        for i in ALVerbose.m_oalDictionary.ordered():
            oalStrTime, oalStrInfo = ALVerbose.__fromIndex( i )
            _oalFile.write( oalStrTime + " " + oalStrInfo + " " +  ALString( ALVerbose.m_oalDictionary[i] )+ "\n" )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    outputObjectsDEBUG = staticmethod( outputObjectsDEBUG )


    def outputActionLog():
        if (ALVerbose.isLogFile()==False or ALVerbose.isProfile()==False):
            return
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOn()
        ALVerbose.m_oalLogFile.logVerboseTree( ALVerbose.m_oalDictionary )
        del ALVerbose.m_oalDictionary
        ALVerbose.m_oalDictionary = ALDictionary()
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOff()
    outputActionLog = staticmethod( outputActionLog )



    # Profile Tool
    def setProfileOn( ):
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
        ALVerbose.m_bIsProfile = True
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
    setProfileOn = staticmethod( setProfileOn )


    def setProfileOff( ):
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
        ALVerbose.m_bIsProfile = False
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
    setProfileOff = staticmethod( setProfileOff )


    def isProfile():
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
        bIsProfile = ALVerbose.m_bIsProfile
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
        return bIsProfile
    isProfile = staticmethod( isProfile )


    def getProfileFile():
        if (ALVerbose.isProfile()==True):
            ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
            ALVerbose.m_oalLogFile.getProfileFile()
            ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
    getProfileFile = staticmethod( getProfileFile )


    def __addObjectIDInformation( _oalString, _oalStringID ):
        if (ALVerbose.m_bIsProfile==True):
            ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
            ALVerbose.m_oalDictionary.addObject( ALVerbose.__toIndex( _oalString  ), _oalStringID )
            ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
    __addObjectIDInformation = staticmethod( __addObjectIDInformation )

##############################################################################


