##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition       import *
from ALVerbose          import ALVerbose
from ALTime             import ALTime
from ALActionMethod     import ALActionMethod


class ALActionMethodTimer( ALActionMethod ):
    """
    Interface for ALActionMethodTimer classes.

    The methods to be modified by the user (overload):
        def preProcess( self, _oalObject = None )
        def postProcess( self, _oalObject = None )

    """
    def __init__( self, _oalClassMethod = None,
                 _ofMethodTimeDelay = 0.1, _ofMethodTimeDuration = AL_TIME_INFINITE, _obRegisterAction = False ):
        ALActionMethod.__init__( self, _oalClassMethod )
        self.m_oalMethodTimeDelay      = _ofMethodTimeDelay
        self.m_oalMethodTimeDuration   = _ofMethodTimeDuration

        if (_obRegisterAction == False):
            self.setNoRegisterAction()


    def getMethodTimeDelay( self ):
        return self.m_oalMethodTimeDelay

    def getMethodTimeDuration( self ):
        return self.m_oalMethodTimeDuration


    def setMethodTimeDelay( self, _oalMethodTimeDelay ):
        self.m_oalMethodTimeDelay = _oalMethodTimeDelay

    def setMethodTimeDuration( self, _oalMethodTimeDuration ):
        self.m_oalMethodTimeDuration = _oalMethodTimeDuration


    # NOTE: avoid to set methods started on the thread.run() method
    # It will generate an error starting the run() method on the loop
    def process( self, _oalObject = None ):
        ALVerbose.DEBUGID( self.getID(), "Execution: ALActionMethodTimer.process()" )
        oalTime = ALTime()
        while ( oalTime.getDuration()<self.getMethodTimeDuration() ):
            self.actionControl()
            ALActionMethod.process( self, _oalObject )
            self.actionControl()
            self.threadControlTimer( self.getMethodTimeDelay() )


##############################################################################
