##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALList                import  ALList
from ALSignal              import  ALSignal
from ALActionController    import  ALActionController


class ALActionSet( ALActionController ):
    """
    Parent class for all action set objects.

    The methods to be modified by the user (overload):
        def preProcess( self, _oalObject = None )
        def process( self, _oalObject = None )
        def postProcess( self, _oalObject = None )


    NOTE: this class is ASynchronous
    """
    def __init__( self ):
        """
        Constructor for ALActionSet.

        """
        ALActionController.__init__( self  )
        self.m_oalListAction = ALList()


    def deleteActions(self):
        """
        Delete all actions - Warning: it deletes all actions!!!
        """
        self.synchronizeOn()
        if ( self.m_oalListAction!=None ):
            self.m_oalListAction.deleteObjects()
        self.synchronizeOff()



    def addAction( self, _oalAction ):
        """
        Add actions
        """
        self.synchronizeOn()
        if ( _oalAction!=None ):
            self.m_oalListAction.append( _oalAction )
            self.m_oalSlotAbort.connect( _oalAction.abort, ALSignal.abort() )
            self.m_oalSlotException.connect( _oalAction.abort, ALSignal.exception() )
        self.synchronizeOff()



    def abort( self, _oalObject = None ):
        ALActionController.abort( self )
        for action in self.m_oalListAction :
            action.abort()


    def pause( self, _oalObject = None ):
        ALActionController.pause( self )
        for action in self.m_oalListAction :
            action.pause()


    def resume( self, _oalObject = None ):
        ALActionController.resume( self )
        for action in self.m_oalListAction :
            action.resume()



##############################################################################


