##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################

from ALDefinition        import  *
from ALTime              import  ALTime
from ALActionSet         import  ALActionSet


class ALActionSetSynchronous( ALActionSet ):
    """
    NOTE: this class is Synchronous

    The methods to be modified by the user (overload):
        def preProcess( self, _oalObject = None )
        def postProcess( self, _oalObject = None )

    """
    def __init__( self ):
        """
        Constructor for ALActionSetSynchronous.
        """
        ALActionSet.__init__( self )




    def __controlTimeOutAction( self, _oalAction, _oalTimeIteration = AL_TIME_MINIMUM * 2 ):
        oalTime = ALTime()
        obStatus = True
        while (obStatus==True):
            self.threadControlTimer( _oalTimeIteration )
            if ( oalTime.getDuration()>_oalAction.getTimeOut() ):
                obStatus = False
                _oalAction.stop()
            if ( _oalAction.isRunning()!=True ):
                obStatus = False
        self.actionControl()
        if ( _oalAction.isRunning()==True ):
           _oalAction.stop()




    def process( self, _oalObject = None ):
        """
        This is the main method of the action.
        """

        for action in self.m_oalListAction :
            self.actionControl()

            self.synchronizeOn()
            action.execute( _oalObject )
            self.synchronizeOff()

            self.actionControl()

            self.synchronizeOn()
            self.__controlTimeOutAction( action )
            self.synchronizeOff()



