##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition               import *
from ALDebug                    import ALDebug
from ALSystem                   import ALSystem
from ALString                   import ALString
from ALTime                     import ALTime
from ALTrace                    import ALTrace
from ALVerbose                  import ALVerbose
from ALAppDefinitionOutput      import ALAppDefinitionOutput
from ALApplicationActionControl import ALApplicationActionControl
from ALApplicationCommunicator  import ALApplicationCommunicator



class ALAppDefinitionControl( ALAppDefinitionOutput ):
    """
    Interface for ALAppDefinitionControl objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionOutput.__init__( self, _oalString, _dversion, _obPlugin )



    def signalControl( self ):
        """
        This ALAppDefinitionControl Signal control.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionControl.signalControl()" )
        ALAppDefinitionControl.synchronizeOn( )
        oalSignalExt = ALApplicationCommunicator.getSignal()
        if (oalSignalExt != None):
            oalSignal = oalSignalExt.clone()
            ALAppDefinitionControl.m_oalSlotSignal.callSignal( oalSignal )
        ALAppDefinitionControl.synchronizeOff( )



    def __patchBugPythonThread(self):
        # workaround to avoid problem of the thread starting
        ALVerbose.DEBUG( "Execution: ALAppDefinitionControl.__patchBugPythonThread()" )
        ALTime.wait( 0.5 )



    def __actionControlExecutionSpeed( self, _fTimeOut ):
        """
        This ALAppDefinitionControl Action __actionControlExecutionSpeed.
        The Standard time out corresponds to: 11d:13h:46m:39s:000ms = AL_TIME_INFINITE
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionControl.__actionControlExecutionSpeed()" )
        # It must add the timeout control....in the future
        oalTime = ALTime()
        obStatusActionControl = True
        while (obStatusActionControl):
            if ( ALApplicationActionControl.isRunningControl()==True ):
                ALTime.wait( AL_TIME_MINIMUM )
                if (ALVerbose.isVerboseDebug()==True):
                    ALApplicationActionControl.getStatusRunning()
            else:
                obStatusActionControl = False
            # Controls the ALApplication ALAction time out
            # The end will be executed by the end method of the next class
            if (oalTime.getDuration()>_fTimeOut):
                obStatusActionControl = False
                ALVerbose.DEBUG( "ALApplication:: ALAction-ALPlugin control time out - REACHED: " + ALString( oalTime.getDuration() ) )



    def actionControlExecution( self ):
        """
        This ALAppDefinitionControl Action Control.
        The Standard time out corresponds to: 11d:13h:46m:39s:000ms = AL_TIME_INFINITE
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionControl.actionControl()" )
        self.__patchBugPythonThread()
        self.__actionControlExecutionSpeed( self.getTimeOutPlugins() )




    def raiseControl( self ):
        """
        This ALAppDefinitionControl Raise Control.
        """
        oBoolStatus =  ALDebug.isDebugModeOn()
        ALDebug.controlDebug( self, oBoolStatus )



##############################################################################
