##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition            import *
from ALVerbose               import ALVerbose
from ALString                import ALString
from ALSignal                import ALSignal
from ALDebug                 import ALDebug
from ALException             import ALException
from ALExceptionSignal       import ALExceptionSignal
from ALAppDefinitionEnd      import ALAppDefinitionEnd


class ALAppDefinitionCore( ALAppDefinitionEnd ):
    """
    Interface for ALAppDefinitionCore objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionEnd.__init__( self, _oalString, _dversion, _obPlugin )



    def __executeCoreInit( self, _bInit ):
        """
        This is the Initial Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.__executeCoreInit()" )
        oBoolStatus = self.init( _bInit )
        if (oBoolStatus==False):
            return oBoolStatus
        self.signalControl()
        self.callbackSlotInit( self )
        return oBoolStatus


    def __executeCoreCommandLine( self ):
        """
        This is the CommandLine Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.__executeCoreCommandLine()" )
        # Process Sequence
        self.signalControl()
        self.processCommandLine()
        self.signalControl()
        self.processCommandLineInterface()



    def __executeCorePreProcess( self ):
        """
        This is the PreProcessing Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.__executeCorePreProcess()" )
        # PreProcess Sequence
        self.signalControl()
        self.preProcess()
        self.signalControl()
        self.callbackSlotPreProcess( self )



    def __executeCoreProcess( self ):
        """
        This is the Processing Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.__executeCoreProcess()" )
        # Process Sequence
        self.signalControl()
        self.process()
        self.signalControl()
        self.callbackSlotProcess( self )


    def __executeCorePostProcess( self ):
        """
        This is the Post Processing Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.__executeCorePostProcess()" )
        # PostProcess Sequence
        self.signalControl()
        self.postProcess()
        self.signalControl()
        self.callbackSlotPostProcess( self )


    def __executeCoreEnd( self ):
        """
        This is the Final Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.__executeCoreEnd()" )
        # End
        self.signalControl()
        self.actionControlExecution()
        self.callbackSlotEnd( self )


    def __executeCoreClose( self ):
        """
        This is the Final Close Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.__executeCoreClose()" )
        # Closing
        self.end()


    def __executeCoreRaise( self ):
        """
        This is the Raise Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.__executeCoreRaise()" )
        oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
        ALDebug.controlDebug( self, oBoolStatus )


    def __executeCore( self ):
        """
        This is the Main Core Element of the ALAppDefinitionCore.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.__executeCore()" )
        self.__executeCoreCommandLine()
        self.__executeCorePreProcess()
        self.__executeCoreProcess()
        self.__executeCorePostProcess()
        self.__executeCoreEnd()



############################################################################

    def preProcess( self ):
        """
        Any processing that must occur before the main ALAppDefinitionCore loop
        """
        ALAppDefinitionCore.synchronizeOn()
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.preProcess()" )
        ALAppDefinitionCore.synchronizeOff()


    def process( self ):
        """
        This is the main method of the ALAppDefinitionCore.
        """
        ALAppDefinitionCore.synchronizeOn()
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.process()" )
        ALAppDefinitionCore.synchronizeOff()


    def postProcess( self ):
        """
        This is the final method executed by the ALAppDefinitionCore
        """
        ALAppDefinitionCore.synchronizeOn()
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.postProcess()" )
        ALAppDefinitionCore.synchronizeOff()


##############################################################################


    def execute( self, _bInit = True ):
        """
        This is the Main ALAppDefinitionCore Loop Control.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.execute()" )
        try:
            try:
                if (self.__executeCoreInit( _bInit )==True):
                    self.__executeCore()


            except ALExceptionSignal, _oalExceptionSignal:
                oalSignal = _oalExceptionSignal.getSignal()
                ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.execute() - AL_FRAMEWORK_FORCE_CLOSE_APPLICATION" )
                if ( oalSignal!=None ):
                    ALVerbose.DEBUG("ALAppDefinitionCore.execute() - stopped by Signal: "  + ALString( oalSignal.getSignal() ) )
                else:
                    ALVerbose.DEBUG("ALAppDefinitionCore.execute() - stopped under Unknown signal: "  + ALString( oalSignal.getSignal() ) )
                self.__executeCoreRaise()


            except:
                ALVerbose.DEBUG( "Execution: ALAppDefinitionCore.execute() - UNEXPECTED CONTROL RAISED" )
                self.__executeCoreRaise()

        finally:
            self.__executeCoreClose()


##############################################################################
