##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition               import *
from ALVerbose                  import ALVerbose
from ALString                   import ALString
from ALTime                     import ALTime
from ALSystem                   import ALSystem
from ALSignal                   import ALSignal
from ALExceptionSignal          import ALExceptionSignal
from ALAppDefinitionInit        import ALAppDefinitionInit
from ALApplicationActionControl import ALApplicationActionControl


class ALAppDefinitionEnd( ALAppDefinitionInit ):
    """
    Interface for ALAppDefinitionEnd objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionInit.__init__( self, _oalString, _dversion, _obPlugin )



    def __forceExit( self, _iArg = 1 ):
        """
        This ALAppDefinitionEnd __forceExit
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionEnd.__forceExit()" )
        raise ALExceptionSignal, ALSignal.exit()




    def __forceCloseTimerActions( self ):
        """
        This ALAppDefinitionEnd force Close Timer Actions.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionEnd.__forceCloseTimerActions()" )
        ALAppDefinitionEnd.synchronizeOn()
        for iAction in ALAppDefinitionEnd.m_oalListTimerActions:
            iAction.close()
        ALAppDefinitionEnd.synchronizeOff()


    def __forceCloseActions( self ):
        """
        This ALAppDefinitionEnd force Close All Actions.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionEnd.__forceCloseActions()" )
        ALAppDefinitionEnd.synchronizeOn()
        ALApplicationActionControl.forceClose()
        ALAppDefinitionEnd.synchronizeOff()



    def exit( self, _oalObject = None ):
        """
        This ALAppDefinitionEnd calling force exit.
        """
        ALVerbose.DEBUG( "Execution: ALAppDefinitionEnd.exit() - calling  __forceExit" )
        if (_oalObject != None and isinstance( _oalObject, ALSignal )):
            oalSignal = _oalObject.getSignal()
            oalString = _oalObject.getInformation()
            if ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ERROR ) :
                ALVerbose.error( "Application::" + ALString( oalSignal.getSignal() ) + "::" + oalString )
            elif ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ABORT  ) :
                ALVerbose.error( "Application::" + ALString( oalSignal.getSignal() ) + "::" + oalString )
            else:
                ALVerbose.log( "Application::" + ALString( oalSignal.getSignal() ) + "::" + oalString )
        self.__forceExit()




    def __getActionLog( self ):
        ALApplicationActionControl.getActionLogInformation()


    def end( self ):
        """
        End The application framework...closing and registering the resources
        """
        self.__forceCloseActions()
        self.__forceCloseTimerActions()
        ALAppDefinitionEnd.setResource( AL_FRAMEWORK_LAST_RUN, ALTime.timeString() )
        ALAppDefinitionEnd.outputResource()
        self.__getActionLog()


        ofAppRunTime     = self.getRunTime()
        oalStrAppName    = ALAppDefinitionEnd.getApplicationName()
        oalStrAppRunTime = ALString( ofAppRunTime )
        ALVerbose.outputActionLog()
        ALVerbose.screen( oalStrAppName + "::Closing" )
        ALVerbose.screen( oalStrAppName + "::Runtime: " + oalStrAppRunTime + " [s]" )
        ALVerbose.screen( oalStrAppName + "::Runtime: " + ALTime.getTimeStringPSP( ofAppRunTime ) )


        # Used to generate a XML profile log file to analise the performance
        ALAppDefinitionEnd.outputProfile()

        # To be safety and close external pipes on remote control
        #ALSystem.exit( 0 ) #Bug in Python 2.4 - it raises an exception !!!!!!!!!!



##############################################################################
