##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition                import *
from ALVerbose                   import ALVerbose
from ALApplication               import ALApplication
from ALServerXmlRpc              import ALServerXmlRpc



class ALApplicationServerXmlRpc( ALApplication ):
    """
    Interface for Application objects.

    """


    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _oalHost = "localhost",
                 _oalSocket = 8888, _dversion = AL_VERSION ):
        ALApplication.__init__( self, _oalString, _dversion )
        self.m_oalServerXmlRpc = ALServerXmlRpc( _oalHost, _oalSocket )


    def addObject( self, _oalObject ):
        self.m_oalServerXmlRpc.addObject( _oalObject )


    def addMethod( self, _oalMethod ):
        self.m_oalServerXmlRpc.addMethod( _oalMethod )


    def addServerControlInterface( self, _oalObject ):
        self.addObject( _oalObject )


    def process( self ):
        """
        This is the main method of the application.
        """
        ALApplication.synchronizeOn()
        ALVerbose.DEBUG( "Execution: ALApplicationServerXmlRpc.process()" )
        self.m_oalServerXmlRpc.executeServer()
        ALApplication.synchronizeOff()



##############################################################################
