#!/usr/bin/env python


##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      Tue Dec 11 16:19:52 2007
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################
#
#
# Generated by ALExternalPackGenerateDS.py.
#
# This file is generated by a modified version of the original file:
# generateDS.py, proposed by Dave Kuhlman: 
#
## LICENSE
#
## Copyright (c) 2003 Dave Kuhlman
#
## Permission is hereby granted, free of charge, to any person obtaining
## a copy of this software and associated documentation files (the
## "Software"), to deal in the Software without restriction, including
## without limitation the rights to use, copy, modify, merge, publish,
## distribute, sublicense, and/or sell copies of the Software, and to
## permit persons to whom the Software is furnished to do so, subject to
## the following conditions:
#
## The above copyright notice and this permission notice shall be
## included in all copies or substantial portions of the Software.
#
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
## EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
## MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
## IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
## CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
## SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
##############################################################################

import sys
import getopt
from xml.dom import minidom
from xml.dom import Node


#
# Support/utility functions.
#

def showIndent(outfile, level):
    for idx in range(level):
        outfile.write('    ')


def showIndentHtml(outfile, level):
    for idx in range(level):
        outfile.write(' &nbsp; &nbsp; &nbsp; &nbsp;')

        
def quote_xml(inStr):
    s1 = inStr
    s1 = s1.replace('&', '&amp;')
    s1 = s1.replace('<', '&lt;')
    s1 = s1.replace('"', '&quot;')
    return s1

def quote_python(inStr):
    s1 = inStr
    if s1.find("'") == -1:
        if s1.find('\n') == -1:
            return "'%s'" % s1
        else:
            return "'''%s'''" % s1
    else:
        if s1.find('"') != -1:
            s1 = s1.replace('"', '\\"')
        if s1.find('\n') == -1:
            return '"%s"' % s1
        else:
            return '"""%s"""' % s1


class MixedContainer(object):
    # Constants for category:
    CategoryNone = 0
    CategoryText = 1
    CategorySimple = 2
    CategoryComplex = 3
    # Constants for content_type:
    TypeNone = 0
    TypeText = 1
    TypeString = 2
    TypeInteger = 3
    TypeFloat = 4
    TypeDecimal = 5
    TypeDouble = 6
    TypeBoolean = 7
    def __init__(self, category, content_type, name, value):
        self.category = category
        self.content_type = content_type
        self.name = name
        self.value = value
    def getCategory(self):
        return self.category
    def getContenttype(self, content_type):
        return self.content_type
    def getValue(self):
        return self.value
    def getName(self):
        return self.name
    def export(self, outfile, level, name):
        if self.category == MixedContainer.CategoryText:
            outfile.write(self.value)
        elif self.category == MixedContainer.CategorySimple:
            self.exportSimple(outfile, level, name)
        else:    # category == MixedContainer.CategoryComplex
            self.value.export(outfile, level, name)
    def exportSimple(self, outfile, level, name):
        if self.content_type == MixedContainer.TypeString:
            outfile.write('<%s>%s</%s>' % (self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeInteger or \
                self.content_type == MixedContainer.TypeBoolean:
            outfile.write('<%s>%d</%s>' % (self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeFloat or \
                self.content_type == MixedContainer.TypeDecimal:
            outfile.write('<%s>%f</%s>' % (self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeDouble:
            outfile.write('<%s>%g</%s>' % (self.name, self.value, self.name))
    def exportLiteral(self, outfile, level, name):
        if self.category == MixedContainer.CategoryText:
            showIndent(outfile, level)
            outfile.write('MixedContainer(%d, %d, "%s", "%s"),\n' % \
                (self.category, self.content_type, self.name, self.value))
        elif self.category == MixedContainer.CategorySimple:
            showIndent(outfile, level)
            outfile.write('MixedContainer(%d, %d, "%s", "%s"),\n' % \
                (self.category, self.content_type, self.name, self.value))
        else:    # category == MixedContainer.CategoryComplex
            showIndent(outfile, level)
            outfile.write('MixedContainer(%d, %d, "%s",\n' % \
                (self.category, self.content_type, self.name,))
            self.value.exportLiteral(outfile, level + 1)
            showIndent(outfile, level)
            outfile.write(')\n')



TEMPLATE_PAGE_P1 = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
    <title>AALib Plugin Data Description</title>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
    <meta name="description" content="AALib Plugin generation from XSD files">
    <meta name="keywords" content="AALib, xsd, plugin">
    <meta name="author" content="Romeu Pieritz(romeu_pieritz@yahoo.fr)">
    <link href="style.css" rel="stylesheet" type="text/css">
    <base target="main">
</head>
<body>
    <div id="main">
        <div id="block">
          %s
    </div>
"""

TEMPLATE_PAGE_P2 = """
    </div>
</body>
</html>
"""



#
# Class definition
#

class ALDataProcess(object):
    subclass = None
    def __init__(self, command='', imageName='', pid=-1, sessionName='', session='', memoryUsage='', status='', userName='', cpuTime='', windowTitle=''):
        self.command = command
        self.imageName = imageName
        self.pid = pid
        self.sessionName = sessionName
        self.session = session
        self.memoryUsage = memoryUsage
        self.status = status
        self.userName = userName
        self.cpuTime = cpuTime
        self.windowTitle = windowTitle
    def factory(*args_, **kwargs_):
        if ALDataProcess.subclass:
            return ALDataProcess.subclass(*args_, **kwargs_)
        else:
            return ALDataProcess(*args_, **kwargs_)
    factory = staticmethod(factory)
    def getCommand(self): return self.command
    def setCommand(self, command): self.command = command
    def getImageName(self): return self.imageName
    def setImageName(self, imageName): self.imageName = imageName
    def getPid(self): return self.pid
    def setPid(self, pid): self.pid = pid
    def getSessionName(self): return self.sessionName
    def setSessionName(self, sessionName): self.sessionName = sessionName
    def getSession(self): return self.session
    def setSession(self, session): self.session = session
    def getMemoryUsage(self): return self.memoryUsage
    def setMemoryUsage(self, memoryUsage): self.memoryUsage = memoryUsage
    def getStatus(self): return self.status
    def setStatus(self, status): self.status = status
    def getUserName(self): return self.userName
    def setUserName(self, userName): self.userName = userName
    def getCpuTime(self): return self.cpuTime
    def setCpuTime(self, cpuTime): self.cpuTime = cpuTime
    def getWindowTitle(self): return self.windowTitle
    def setWindowTitle(self, windowTitle): self.windowTitle = windowTitle
    def export(self, outfile, level = 0, name_='ALDataProcess'):
        showIndent(outfile, level)
        outfile.write('<%s>\n' % name_)
        self.exportChildren(outfile, level + 1, name_)
        showIndent(outfile, level)
        outfile.write('</%s>\n' % name_)
    def exportAttributes(self, outfile, level, name_='ALDataProcess'):
        pass
    def exportChildren(self, outfile, level, name_='ALDataProcess'):
        showIndent(outfile, level)
        outfile.write('<command>%s</command>\n' % quote_xml(self.getCommand()))
        showIndent(outfile, level)
        outfile.write('<imageName>%s</imageName>\n' % quote_xml(self.getImageName()))
        showIndent(outfile, level)
        outfile.write('<pid>%d</pid>\n' % self.getPid())
        showIndent(outfile, level)
        outfile.write('<sessionName>%s</sessionName>\n' % quote_xml(self.getSessionName()))
        showIndent(outfile, level)
        outfile.write('<session>%s</session>\n' % quote_xml(self.getSession()))
        showIndent(outfile, level)
        outfile.write('<memoryUsage>%s</memoryUsage>\n' % quote_xml(self.getMemoryUsage()))
        showIndent(outfile, level)
        outfile.write('<status>%s</status>\n' % quote_xml(self.getStatus()))
        showIndent(outfile, level)
        outfile.write('<userName>%s</userName>\n' % quote_xml(self.getUserName()))
        showIndent(outfile, level)
        outfile.write('<cpuTime>%s</cpuTime>\n' % quote_xml(self.getCpuTime()))
        showIndent(outfile, level)
        outfile.write('<windowTitle>%s</windowTitle>\n' % quote_xml(self.getWindowTitle()))

    #Only to export the entire XML tree to a file stream on disk
    def outputFile( self, _outfileName ):
        outfile = open( _outfileName, "w" )
        outfile.write("<?xml version=\"1.0\" ?>\n")
        self.export( outfile, 0, name_='ALDataProcess' )
        outfile.close()


    #Only to export the entire XML tree to a Html Page in a file stream on disk
    def outputPageHtml( self, _outfileName ):
        odtStrTitle = "    <span class=\"title\">ALDataProcess:</span> <br> <br> <br>\n" 
        outfile = open( _outfileName, "w" )
        outfile.write( TEMPLATE_PAGE_P1 % ( odtStrTitle ) )
        self.exportLiteralHtml( outfile, 0, name_='ALDataProcess' )
        outfile.write( TEMPLATE_PAGE_P2  )
        outfile.close()


    def outputFileHtml( self, _outfileName ):
        self.outputPageHtml( _outfileName )


    def exportLiteral(self, outfile, level = 0, name_='ALDataProcess'):
        level += 1
        self.exportLiteralAttributes(outfile, level, name_)
        self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        showIndent(outfile, level)
        outfile.write('command = %s,\n' % quote_python(self.getCommand()))
        showIndent(outfile, level)
        outfile.write('imageName = %s,\n' % quote_python(self.getImageName()))
        showIndent(outfile, level)
        outfile.write('pid = %d,\n' % self.getPid())
        showIndent(outfile, level)
        outfile.write('sessionName = %s,\n' % quote_python(self.getSessionName()))
        showIndent(outfile, level)
        outfile.write('session = %s,\n' % quote_python(self.getSession()))
        showIndent(outfile, level)
        outfile.write('memoryUsage = %s,\n' % quote_python(self.getMemoryUsage()))
        showIndent(outfile, level)
        outfile.write('status = %s,\n' % quote_python(self.getStatus()))
        showIndent(outfile, level)
        outfile.write('userName = %s,\n' % quote_python(self.getUserName()))
        showIndent(outfile, level)
        outfile.write('cpuTime = %s,\n' % quote_python(self.getCpuTime()))
        showIndent(outfile, level)
        outfile.write('windowTitle = %s,\n' % quote_python(self.getWindowTitle()))
    def exportLiteralHtml(self, outfile, level = 0, name_='ALDataProcess'):
        level += 1
        self.exportLiteralAttributesHtml(outfile, level, name_)
        self.exportLiteralChildrenHtml(outfile, level, name_)
    def exportLiteralAttributesHtml(self, outfile, level, name_):
        pass
    def exportLiteralChildrenHtml(self, outfile, level, name_):
        showIndentHtml(outfile, level)
        outfile.write('command = <a  href:"#">%s</a><br>\n' % quote_python(self.getCommand()))
        showIndentHtml(outfile, level)
        outfile.write('imageName = <a  href:"#">%s</a><br>\n' % quote_python(self.getImageName()))
        showIndentHtml(outfile, level)
        outfile.write('pid = <a  href:"#">%d</a><br>\n' % self.getPid())
        showIndentHtml(outfile, level)
        outfile.write('sessionName = <a  href:"#">%s</a><br>\n' % quote_python(self.getSessionName()))
        showIndentHtml(outfile, level)
        outfile.write('session = <a  href:"#">%s</a><br>\n' % quote_python(self.getSession()))
        showIndentHtml(outfile, level)
        outfile.write('memoryUsage = <a  href:"#">%s</a><br>\n' % quote_python(self.getMemoryUsage()))
        showIndentHtml(outfile, level)
        outfile.write('status = <a  href:"#">%s</a><br>\n' % quote_python(self.getStatus()))
        showIndentHtml(outfile, level)
        outfile.write('userName = <a  href:"#">%s</a><br>\n' % quote_python(self.getUserName()))
        showIndentHtml(outfile, level)
        outfile.write('cpuTime = <a  href:"#">%s</a><br>\n' % quote_python(self.getCpuTime()))
        showIndentHtml(outfile, level)
        outfile.write('windowTitle = <a  href:"#">%s</a><br>\n' % quote_python(self.getWindowTitle()))
    def build(self, node_):
        attrs = node_.attributes
        self.buildAttributes(attrs)
        for child_ in node_.childNodes:
            nodeName_ = child_.nodeName.split(':')[-1]
            self.buildChildren(child_, nodeName_)
    def buildAttributes(self, attrs):
        pass
    def buildChildren(self, child_, nodeName_):
        if child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'command':
            command_ = ''
            for text__content_ in child_.childNodes:
                command_ += text__content_.nodeValue
            self.command = command_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'imageName':
            imageName_ = ''
            for text__content_ in child_.childNodes:
                imageName_ += text__content_.nodeValue
            self.imageName = imageName_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'pid':
            if child_.firstChild:
                sval_ = child_.firstChild.nodeValue
                try:
                    ival_ = int(sval_)
                except ValueError:
                    raise ValueError('requires integer -- %s' % child_.toxml())
                self.pid = ival_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'sessionName':
            sessionName_ = ''
            for text__content_ in child_.childNodes:
                sessionName_ += text__content_.nodeValue
            self.sessionName = sessionName_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'session':
            session_ = ''
            for text__content_ in child_.childNodes:
                session_ += text__content_.nodeValue
            self.session = session_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'memoryUsage':
            memoryUsage_ = ''
            for text__content_ in child_.childNodes:
                memoryUsage_ += text__content_.nodeValue
            self.memoryUsage = memoryUsage_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'status':
            status_ = ''
            for text__content_ in child_.childNodes:
                status_ += text__content_.nodeValue
            self.status = status_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'userName':
            userName_ = ''
            for text__content_ in child_.childNodes:
                userName_ += text__content_.nodeValue
            self.userName = userName_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'cpuTime':
            cpuTime_ = ''
            for text__content_ in child_.childNodes:
                cpuTime_ += text__content_.nodeValue
            self.cpuTime = cpuTime_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'windowTitle':
            windowTitle_ = ''
            for text__content_ in child_.childNodes:
                windowTitle_ += text__content_.nodeValue
            self.windowTitle = windowTitle_
# end class ALDataProcess




def parse(inFileName):
    doc = minidom.parse(inFileName)
    rootNode = doc.documentElement
    rootObj = ALDataProcess.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc.unlink()
#    doc = None
    return rootObj


def parseString(inString):
    doc = minidom.parseString(inString)
    rootNode = doc.documentElement
    rootObj = ALDataProcess.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc.unlink()
#    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout, 0, name_="ALDataProcess")
    return rootObj


# AAlib Modifications for marshalling and unmarshalling
# Creating program data constructs from XML
def unMarshalling( _inFileName ):
    return parse(  _inFileName )
    

# Serializing program data constructs to XML   
def marshalling( _oObject, _inFileName ):
    oFile = open( _inFileName, "w" )
    oFile.write('<?xml version="1.0" ?>\n')
    _oObject.export( oFile, 0, name_="ALDataProcess" )
    oFile.close()
    
