##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython            import PySys
from ALObject                  import ALObject
from ALSystem                  import ALSystem
from ALString                  import ALString
from ALSemaphore               import ALSemaphore
from ALTrace                   import ALTrace
from ALVerbose                 import ALVerbose



class ALDebug( ALObject ):
    """
    Interface for ALDebug control objects.
    """

    m_obStateDebugMode        = False
    m_obStateDebugModeForce   = False
    m_obStateDebugModeAssert  = False
    m_obStateDebugModeVerbose = False
    m_obStateDebugModeLogFile = True
    m_obStateDebugModeAction  = False
    m_obStateDebugModeProfile = False
    m_obStateDebugLogFile     = False
    m_oalStringLogFileName    = None
    m_oalSemaphoreDebug       = ALSemaphore()


    def __init__( self ):
        ALObject.__init__( self )


    def __analyseCommandLine():
        ALDebug.synchronizeOn()
        for cmdkey in PySys.argv:
            if ( cmdkey == "--DEBUG-FORCE" or
                 cmdkey == "--DEBUGECLIPSE" or
                 cmdkey == "--DEBUGFORCE"   or
                 cmdkey == "--DEBUGTRACE"   or
                 cmdkey == "--DEBUGPYTHON"  or
                 cmdkey == "--DEBUGJYTHON"  or
                 cmdkey == "--DEBUGJAVA"):
                ALDebug.m_obStateDebugModeForce  = True

            if ( cmdkey == "--DEBUG" ):
                ALDebug.m_obStateDebugMode = True

            if ( cmdkey == "--ASSERT" ):
                ALDebug.m_obStateDebugModeAssert = True

            if ( cmdkey == "--verbose" ):
                ALDebug.m_obStateDebugModeVerbose = True

            if ( cmdkey == "--profile" ):
                ALDebug.m_obStateDebugModeProfile = True

            if ( cmdkey == "--log" ):
                ALDebug.m_obStateDebugModeLogFile = False

            if ( cmdkey == "--DEBUG-FORCE-PLUGIN" or
                 cmdkey == "--DEBUG-PLUGIN" or
                 cmdkey == "--DEBUGFORCEPLUGIN" or
                 cmdkey == "--DEBUGPLUGIN" ):
                ALDebug.m_obStateDebugModeAction = True
        ALDebug.synchronizeOff()

    __analyseCommandLine = staticmethod( __analyseCommandLine )




    def synchronizeOn():
        """
        This method must be used in together with the method synchronizeOff().
        This method makes the code threadsafe until the method synchronizeOff
        is called.
        """
        ALDebug.m_oalSemaphoreDebug.acquire()
    synchronizeOn = staticmethod( synchronizeOn )


    def synchronizeOff():
        """
        This method must be used in together with the method synchronizeOn().
        """
        ALDebug.m_oalSemaphoreDebug.release()
    synchronizeOff = staticmethod( synchronizeOff)



    def controlDebug( _oalObject = None, _oBoolScreen = True ):
        if (ALDebug.isDebugModeForceOn()==True):
            ALDebug.debugForce( _oalObject, False )
        else:
            ALTrace.trace( _oalObject, _oBoolScreen )
    controlDebug = staticmethod( controlDebug )



    def debugForce( _oalObject = None, _oBoolScreen = True ):
        ALTrace.trace( _oalObject, _oBoolScreen )
        raise
    debugForce = staticmethod( debugForce )



    def isDebugModeOn():
        if (ALDebug.m_obStateDebugMode==True):
            return ALDebug.m_obStateDebugMode
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugMode
    isDebugModeOn = staticmethod( isDebugModeOn )


    def isDebugModeCommand():
        return ALDebug.isDebugModeOn()
    isDebugModeCommand = staticmethod( isDebugModeCommand )


    def isDebugModeForceOn():
        if (ALDebug.m_obStateDebugModeForce==True):
            return ALDebug.m_obStateDebugModeForce
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeForce
    isDebugModeForceOn = staticmethod( isDebugModeForceOn )


    def isDebugModeVerboseOn():
        if (ALDebug.m_obStateDebugModeVerbose==True):
            return ALDebug.m_obStateDebugModeVerbose
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeVerbose
    isDebugModeVerboseOn = staticmethod( isDebugModeVerboseOn )


    def isDebugModeLogFileOn():
        if (ALDebug.m_obStateDebugModeLogFile==False):
            return ALDebug.m_obStateDebugModeLogFile
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeLogFile
    isDebugModeLogFileOn = staticmethod( isDebugModeLogFileOn )


    def isDebugModeAssertOn():
        if (ALDebug.m_obStateDebugModeAssert==True):
            return ALDebug.m_obStateDebugModeAssert
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeAssert
    isDebugModeAssertOn = staticmethod( isDebugModeAssertOn )


    def isDebugModeActionOn():
        if (ALDebug.m_obStateDebugModeAction==True):
            return ALDebug.m_obStateDebugModeAction
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeAction
    isDebugModeActionOn = staticmethod( isDebugModeActionOn )



    def isDebugModeProfileOn():
        if (ALDebug.m_obStateDebugModeProfile==True):
            return ALDebug.m_obStateDebugModeProfile
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeProfile
    isDebugModeProfileOn = staticmethod( isDebugModeProfileOn )


    def isDebugModePluginOn():
        return ALDebug.isDebugModeActionOn()
    isDebugModePluginOn = staticmethod( isDebugModePluginOn )


###################################################################

    def setDebugModeVerboseOn():
        ALDebug.synchronizeOn()
        ALDebug.m_obStateDebugModeVerbose = True
        ALDebug.synchronizeOff()
    setDebugModeVerboseOn = staticmethod( setDebugModeVerboseOn )


    def setDebugModeActionOn():
        ALDebug.synchronizeOn()
        ALDebug.m_obStateDebugModeAction = True
        ALDebug.synchronizeOff()
    setDebugModeActionOn = staticmethod( setDebugModeActionOn )


    def setDebugModePluginOn():
        ALDebug.setDebugModeActionOn()
    setDebugModePluginOn = staticmethod( setDebugModePluginOn )


    def setDebugModeProfileOn():
        ALDebug.synchronizeOn()
        ALDebug.m_obStateDebugModeProfile = True
        ALDebug.synchronizeOff()
    setDebugModeProfileOn = staticmethod( setDebugModeProfileOn )


###################################################################


    def setDebugModeOn():
        ALDebug.synchronizeOn()
        ALVerbose.setVerboseDebugOn()
        ALDebug.synchronizeOff()
        ALDebug.setDebugModeVerboseOn()
    setDebugModeOn = staticmethod( setDebugModeOn )


    def setDebugModeForceOn():
        ALDebug.setDebugModeOn()
        ALDebug.synchronizeOn()
        ALDebug.m_obStateDebugModeForce=True
        ALDebug.synchronizeOff()
    setDebugModeForceOn = staticmethod( setDebugModeForceOn )


    def setDebugModeOff():
        ALDebug.synchronizeOn()
        ALVerbose.setVerboseDebugOff()
        ALDebug.synchronizeOff()
    setDebugModeOff = staticmethod( setDebugModeOff )


    def setDebugProfileOn():
        ALDebug.synchronizeOn()
        ALVerbose.setProfileOn()
        ALDebug.synchronizeOff()
    setDebugProfileOn = staticmethod( setDebugProfileOn )


    def setDebugProfileOff():
        ALDebug.synchronizeOn()
        ALVerbose.setProfileOff()
        ALDebug.synchronizeOff()
    setDebugProfileOff = staticmethod( setDebugProfileOff )


    def setDebugVerboseOn():
        ALDebug.setDebugModeVerboseOn()
        ALDebug.synchronizeOn()
        ALVerbose.setVerboseOn()
        ALDebug.synchronizeOff()
    setDebugVerboseOn = staticmethod( setDebugVerboseOn )


    def setDebugPluginOn():
        ALDebug.setDebugModeActionOn()
    setDebugPluginOn = staticmethod( setDebugPluginOn )

##############################################################################

    def setDebugLogFile( _oalString = "DebugLogFile.log" ):
        """
        You can lock the log file only one time during the runtime.
        That means: if the log file is locked, it returns the locked name
        """
        ALDebug.synchronizeOn()
        ALDebug.m_oalStringLogFileName = ALString( _oalString )
        ALDebug.m_obStateDebugLogFile = True
        ALVerbose.setLogFileNameUnique( ALDebug.m_oalStringLogFileName )
        ALDebug.m_oalStringLogFileName = ALVerbose.getLogFileName()
        ALDebug.synchronizeOff()
    setDebugLogFile = staticmethod( setDebugLogFile )


    def isDebugLogFile():
        ALDebug.synchronizeOn()
        oObject = ALDebug.m_obStateDebugLogFile
        ALDebug.synchronizeOff()
        return oObject
    isDebugLogFile = staticmethod( isDebugLogFile )


    def getDebugLogFileName():
        ALDebug.synchronizeOn()
        oObject = ALDebug.m_oalStringLogFileName
        ALDebug.synchronizeOff()
        return oObject
    getDebugLogFileName = staticmethod( getDebugLogFileName )

##############################################################################