##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################



from ALImportPython            import PyGlob
from ALImportPython            import PyOS
from ALImportPython            import PyShutil
from ALList                    import ALList
from ALObject                  import ALObject
from ALString                  import ALString
from ALSystem                  import ALSystem
from ALThreadingCondition      import ALThreadingCondition


class ALDiskExplorer( ALObject ):
    """
    Interface for AL disk folder and file manager.
    """

    m_oalThrCondDiskExplorer = ALThreadingCondition()


    def __init__( self ):
        ALObject.__init__( self )


    def existFile( _oalString ):
        """
        Verify the existence of the file
        """
        return ALDiskExplorer.isFile( _oalString  )
    existFile = staticmethod( existFile )


    def isFile( _oalString ):
        """
        Verify the file
        """
#        print _oalString
        oalFilePath = ALDiskExplorer.normPath( _oalString )
        return PyOS.path.isfile( oalFilePath )
    isFile = staticmethod( isFile )


    def isDirectory( _oalString ):
        """
        Verify the directory
        """
        return PyOS.path.isdir( ALDiskExplorer.normPath( _oalString ) )
    isDirectory  = staticmethod( isDirectory )


    def isFolder( _oalString ):
        return ALDiskExplorer.isDirectory( _oalString )
    isFolder  = staticmethod( isFolder )


    def existFolder( _oalString ):
        return ALDiskExplorer.isDirectory( _oalString )
    existFolder  = staticmethod( existFolder )


    def createFolder( _oalString ):
        if (_oalString != None):
            if (ALDiskExplorer.existFolder( _oalString ) == False):
                oalString = ALDiskExplorer.absolutePath( _oalString )
                oalList = oalString.split( "/" )
                iFolder = ALString( "" )
                for i in oalList:
                    iFolder = iFolder + ALString( i )
                    if (ALSystem.getOSName()=="Windows"):
                        if (iFolder[1]==":" and len(iFolder)>2 ):
                            if (ALDiskExplorer.existFolder( iFolder ) == False):
                                PyOS.mkdir( iFolder )
                    else:
                        if (iFolder!=ALString.empty()):
                            if (ALDiskExplorer.existFolder( iFolder ) == False):
                                PyOS.mkdir( iFolder )
                    iFolder = iFolder + "/"
    createFolder  = staticmethod( createFolder )


    def deleteFolder( _oalString ):
        if _oalString != None:
            if ( ALDiskExplorer.existFolder( _oalString ) == True ):
                PyOS.rmdir( ALDiskExplorer.normPath( _oalString ) )
    deleteFolder  = staticmethod( deleteFolder )


    def deleteFile( _oalString ):
        if _oalString != None:
            if ( ALDiskExplorer.existFile( _oalString ) == True ):
                PyOS.remove( ALDiskExplorer.normPath( _oalString ) )
    deleteFile  = staticmethod( deleteFile )


    def isLink( _oalString ):
        """
        Verify the link
        """
        return PyOS.path.islink(  ALDiskExplorer.normPath( _oalString ) )
    isLink  = staticmethod( isLink )


    def getFileList( _oalStringPath = "./", _oalStringMask = "*" ):
        """
        This method get the file list of a folder by
        a mask
        """
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOn()
        oalStringFind = PyOS.path.join( _oalStringPath, _oalStringMask )
        oalListFile = ALList( PyGlob.glob( oalStringFind ) )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOff()
        for i in range( 0, len( oalListFile )):
            oalListFile[i] = ALDiskExplorer.normPath( oalListFile[i] )
        return oalListFile
    getFileList = staticmethod( getFileList )


    def getFolderList( _oalStringPath = "./", _oalStringMask = "*" ):
        return ALDiskExplorer.getFileList( _oalStringPath, _oalStringMask )
    getFolderList = staticmethod( getFolderList )


    def folderName( _oalStringPath ):
        return PyOS.path.dirname( ALDiskExplorer.normPath( _oalStringPath ) )
    folderName = staticmethod( folderName )


    def getFolderName( _oalStringPath ):
        return PyOS.path.dirname( ALDiskExplorer.normPath( _oalStringPath ) )
    getFolderName = staticmethod( getFolderName )


    # FIle name manipulation ###########################################################
    # File name + extesion = NO PATH
    def getFileName( _oalString ):
        return ALString( PyOS.path.basename( ALDiskExplorer.normalize( _oalString ) ) )
    getFileName = staticmethod( getFileName )

    # Without extension
    def getFileBaseName( _oalString ):
        oalString = ALString( PyOS.path.basename( ALDiskExplorer.normalize( _oalString ) ) )
        oalList = ALList( PyOS.path.splitext( ALDiskExplorer.normalize( oalString ) ) )
        oalString2 = ALString()
        for i in range( len( oalList )-1 ):
            oalString2 = oalString2 + ALString( oalList[i] )
        return oalString2
    getFileBaseName = staticmethod( getFileBaseName )


    def getFileExtension( _oalString ):
        oalList = ALList( PyOS.path.splitext( ALDiskExplorer.normalize( _oalString ) ) )
        iLenght = len( oalList )
        oalString = ALString( oalList[iLenght-1] )
        return oalString
    getFileExtension = staticmethod( getFileExtension )


    def setFileExtension(  _oalStrFile, _oalStrExt ):
        """
        You must provide a complete extension name
        """
        oalList = ALList( PyOS.path.splitext( ALDiskExplorer.normalize( _oalStrFile ) ) )
        oalString2 = ALString()
        for i in range( len( oalList )-1 ):
                oalString2 = oalString2 + ALString( oalList[i] )
        oalString2 = oalString2 + _oalStrExt
        return oalString2
    setFileExtension = staticmethod( setFileExtension )


    def getFileFolderName( _oalString ):
        oalString = PyOS.path.dirname( _oalString )
        return ALDiskExplorer.normalize( oalString )
    getFileFolderName = staticmethod( getFileFolderName )




    def absPath( _oalStringPath ):
        oalStringPath = ALString( _oalStringPath )
        oalAbsPath = PyOS.path.abspath( oalStringPath )
        return ALDiskExplorer.normPath( oalAbsPath )
    absPath = staticmethod( absPath )


    def absolutePath( _oalStringPath ):
        return ALDiskExplorer.absPath( _oalStringPath )
    absolutePath = staticmethod( absolutePath )


    def getAbsolutePath( _oalStringPath ):
        return ALDiskExplorer.absPath( _oalStringPath )
    getAbsolutePath = staticmethod( getAbsolutePath )


    def normPath( _oalStringPath ):
        # Fix for Jython
        try:
            oalStrPath = PyOS.path.normpath( _oalStringPath )
        except:
            oalStrPath = _oalStringPath
        return ALDiskExplorer.normDrive( oalStrPath )
    normPath = staticmethod( normPath )



    def normalize( _oalStringPath ):
        return ALDiskExplorer.normPath( ALString( _oalStringPath ) )
    normalize = staticmethod( normalize )



    def normDrive( _oalStringPath ):
        oalStringPath = ALString( _oalStringPath )
        oalStringPath = oalStringPath.replace( "\\", "/" )
        if ALSystem.getOSType()==ALString( "Cygwin" ):
            if oalStringPath.count("cygdrive") == True :
                lstr = oalStringPath[10]
                oalStringPath = ALString.replace( oalStringPath, "/cygdrive/" + lstr + "/", lstr + ":/" )
        return oalStringPath
    normDrive = staticmethod( normDrive )



    def iteractPath( _oalPath, _oalObjectMethod, _oalArgument) :
        """
        Execute the object method iteractivetily with alll elements
        in the folder
        """
        PyOS.path.walk( _oalPath, _oalObjectMethod, _oalArgument )
    iteractPath = staticmethod( iteractPath )


    def mergePath( _oalPath0, _oalPath1 ):
        oalPath0 = ALDiskExplorer.normPath( _oalPath0 )
        oalPath1 = ALDiskExplorer.normPath( _oalPath1 )
        return ALDiskExplorer.normPath( PyOS.path.join( oalPath0, oalPath1 ) )
    mergePath = staticmethod( mergePath )


    def addPath( _oalPath0, _oalPath1 ):
        oalPath0 = ALDiskExplorer.normPath( _oalPath0 )
        oalPath1 = ALDiskExplorer.normPath( _oalPath1 )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOn()
        oalPath = oalPath0 + "/" + oalPath1
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOff()
        return oalPath
    addPath = staticmethod( addPath )


    def getCwd():
        return ALDiskExplorer.normPath( PyOS.getcwd() )
    getCwd = staticmethod( getCwd )


    def getCurrentFolder():
        return ALDiskExplorer.getCwd()
    getCurrentFolder = staticmethod( getCurrentFolder )


    def setCurrentFolder( _oalPath ):
        oalStrPath = ALDiskExplorer.normPath( _oalPath )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOn()
        PyOS.chdir( oalStrPath )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOff()
    setCurrentFolder = staticmethod( setCurrentFolder )


    def setMode( _oalPath, _pyMode ):
        oalStrPath = ALDiskExplorer.normPath( _oalPath )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOn()
        PyOS.chmod( oalStrPath, _pyMode )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOff()
    setMode = staticmethod( setMode )


    # Force the remove!!!
    def removeFolder( _oalStrSource, _oalIgnoreErrors = 0, _oalControlErrors = None ):
        return PyShutil.rmtree( _oalStrSource, _oalIgnoreErrors, _oalControlErrors  )
    removeFolder = staticmethod( removeFolder )



    # TO IMPROVE!!! a lot errors from the original python version = NOT INTELIGENT
    # we must do it with inteligence
    def copyFile( _oalStrSource, _oalStrDestination ):
        try:
            return PyShutil.copyfile( _oalStrSource, _oalStrDestination )
        except:
#            ALVerbose.warning( "Error creating or coping file+folder: " + _oalStrDestination )
            print " [Warning] ALDiskExplorer::copyFile() - Error creating or coping file+folder: " + _oalStrDestination
    copyFile = staticmethod( copyFile )


    def copyFolder( _oalStrSource, _oalStrDestination, _oalSimbolicLinks = 0 ):
        try:
            return PyShutil.copytree( _oalStrSource, _oalStrDestination, _oalSimbolicLinks  )
        except:
#            ALVerbose.warning( "The folder exists: " + _oalStrDestination )
            print " [Warning] ALDiskExplorer::copyFile() - The folder exists: " + _oalStrDestination
    copyFolder = staticmethod( copyFolder )






##############################################################################