##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDebug           import ALDebug
from ALDictionary      import ALDictionary
from ALDiskExplorer    import ALDiskExplorer
from ALFactory         import ALFactory
from ALFile            import ALFile
from ALList            import ALList
from ALPlugin          import ALPlugin
from ALString          import ALString
from ALSystem          import ALSystem
from ALTrace           import ALTrace
from ALVerbose         import ALVerbose
from ALXsd             import ALXsd


class ALFactoryPlugin( ALFactory ):
    """
    Interface for plugin factory classes.
    """
    def __init__( self ):
        ALFactory.__init__( self )
        self.m_oalPluginFolder    = None
        self.m_oalPluginFile      = None
        self.m_oalDictPluginGroup = ALDictionary()


    def __raiseControl( self ):
        """
        This ALFactoryPlugin Raise Control.
        """
        oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
        ALDebug.controlDebug( self, oBoolStatus )


    def __addPluginGroup( self, _oGroup ):
        oGroup = ALString( _oGroup )
        self.m_oalDictPluginGroup.addObject( oGroup, oGroup )



    def __getGroupName( self, _oFile,  _oalStrFolderPluginRoot ):
        oGroupName = _oalStrFolderPluginRoot
        oFile = _oFile
        if (oGroupName!=None):
            oFile = oFile.replace( oGroupName, "" )
        oGroupList = oFile.split( "/" )
        oGroupName = oGroupList[1]
        if ALFile.isExtension( oGroupName.upper(), ".PY" ):
            oGroupName = ALFile.getBaseName( oGroupName )
        return oGroupName




    def __addPlugin( self, _oalString, _oalStrFolderPluginRoot = None):
        ifile = ALString( _oalString )
        if ALDiskExplorer.isFile( ifile ):
            if ALFile.isExtension( ifile.upper(), ".PY" ):
                oNameModule = ALFile.getBaseName( ifile )
                try:
                    oModule = __import__( oNameModule )
                    oBoolStatus = False
                    for iKey in oModule.__dict__:
                        if (iKey==oNameModule):
                            oBoolStatus = True
                    if (oBoolStatus==True):
                        try:
                            oPlugin = oModule.__dict__[ oNameModule ]()
                            oGroupName = self.__getGroupName( ifile, _oalStrFolderPluginRoot )
                            self.__addPluginGroup( oGroupName )
                            self.synchronizeOff()
                            self.addObject( oNameModule, oModule )
                            self.synchronizeOn()
                        except:
                            self.__raiseControl()
                    else:
                        ALVerbose.DEBUG( "It's not an ALLib Standard Plugin :: [" + ifile + "]" )
                        ALVerbose.DEBUG( " The standard class was not found :: [" + oNameModule + "]" )
                except:
                    ALVerbose.DEBUG( "Problem Compiling and Loading Plugin :: [" + ifile + "]" )
                    ALVerbose.DEBUG( "Please DEBUG the plugin code to fix the problem." )
                    self.__raiseControl()
        else:
           ALVerbose.DEBUG( "ALFactoryPlugin - Error loading plugin :: [" + ifile + "]" )





    def __addPluginXsd( self, _oalString, _oalStrFolderPluginRoot = None):
        ifile = ALString( _oalString )
        if ALDiskExplorer.isFile( ifile ):
            if ALFile.isExtension( ifile.upper(), ".XSD" ):
                try:
                    oNameModule = ALFile.getBaseName( ifile )

                    oalStrPyOutputFile = ALFile.setExtension( ifile, ".py" )
                    oalStrXsdInputFile = ifile

                    try:
                        ALXsd().generateModuleBuffer( oalStrPyOutputFile, oalStrXsdInputFile )
                    except:
                        self.__raiseControl()

                    oModule = __import__( oNameModule )
                    self.synchronizeOff()
                    self.addObject( oNameModule, oModule )
                    self.synchronizeOn()
                except:
                    ALVerbose.DEBUG( "ALFactoryPluginXSD - Error importing module :: [" + ifile + "]" )
                    self.__raiseControl()
        else:
            ALVerbose.DEBUG( "ALFactoryPluginXSD - Error loading plugin :: [" + ifile + "]" )



    def __addPluginInteract( self,  _pyArg, _pyFolder, _pyFiles ):
        oalStrCurFolder       = ALDiskExplorer.normPath( _pyFolder )
        oalListFile           = ALList( _pyFiles )
        oalStrFolderPluginRoot= ALString( _pyArg )
        for ifile in oalListFile:
            if ALFile.isExtension( ifile.upper(), ".PY" ):
                if ( self.m_oalPluginFolder != oalStrCurFolder ):
                    self.m_oalPluginFolder = oalStrCurFolder
#                    ALSystem.setPath( self.m_oalPluginFolder )
                oFile = ALDiskExplorer.mergePath( oalStrCurFolder, ifile )
                self.__addPlugin( oFile, oalStrFolderPluginRoot )
            if ALFile.isExtension( ifile.upper(), ".XSD" ):
                if ( self.m_oalPluginFolder != oalStrCurFolder ):
                    self.m_oalPluginFolder = oalStrCurFolder
#                    ALSystem.setPath( self.m_oalPluginFolder )
                oFile = ALDiskExplorer.mergePath( oalStrCurFolder, ifile )
                self.__addPluginXsd( oFile, oalStrFolderPluginRoot )



    def __addPluginPathModuleInteract( self,  _pyArg, _pyFolder, _pyFiles ):
        oalStrCurFolder       = ALDiskExplorer.normPath( _pyFolder )
        oalListFile           = ALList( _pyFiles )
        oalStrFolderPluginRoot= ALString( _pyArg )
        for ifile in oalListFile:
            if ALFile.isExtension( ifile.upper(), ".PY" ):
                if ( self.m_oalPluginFolder != oalStrCurFolder ):
                    self.m_oalPluginFolder = oalStrCurFolder
                    ALSystem.setPath( self.m_oalPluginFolder )
            if ALFile.isExtension( ifile.upper(), ".XSD" ):
                if ( self.m_oalPluginFolder != oalStrCurFolder ):
                    self.m_oalPluginFolder = oalStrCurFolder
                    ALSystem.setPath( self.m_oalPluginFolder )



    def addPluginFolder( self, _oalString ):
        self.synchronizeOn()
        oalStatus = True
        if (ALDiskExplorer.existFolder( _oalString )):
            # Separate procedure to import modules from other folders in the plugins
            # directory...
            # First add all folders to the modules import PATH
            ALDiskExplorer.iteractPath( ALDiskExplorer.normPath( _oalString ),
                                   self.__addPluginPathModuleInteract, ALDiskExplorer.normPath( _oalString ) )
            # and loads the plugins
            ALDiskExplorer.iteractPath( ALDiskExplorer.normPath( _oalString ),
                                   self.__addPluginInteract, ALDiskExplorer.normPath( _oalString ) )
        else:
            ALVerbose.DEBUG( "[NO] Standard Plugin Folder NOT Found :: [" + ALString( _oalString ) +"]" )
            oalStatus = False
        self.synchronizeOff()
        return oalStatus



    def addPluginFolderFromPath( self, _oalString ):
        self.synchronizeOn()
        oalStatus = False
        oalList =  ALSystem.getSystemPath()
        oalAppName = ALFile.getFileName( _oalString )
        for iFolder in oalList:
            oalFolder = ALDiskExplorer.normPath( iFolder )
            oalFile = oalFolder + "/" + oalAppName
            if (ALDiskExplorer.existFile( oalFile )):
                oalFolder = oalFolder + "/plugins"
                self.synchronizeOff()
                self.addPluginFolder( oalFolder )
                self.synchronizeOn()
                oalStatus = True
            elif (ALDiskExplorer.existFile( oalFile + ".exe" )):
                oalFolder = oalFolder + "/plugins"
                self.synchronizeOff()
                self.addPluginFolder( oalFolder )
                self.synchronizeOn()
                oalStatus = True

        if ( oalStatus==False ):
            ALVerbose.DEBUG( "[NO] Standard Plugin Folder NOT Found :: [*/plugins]" )
        self.synchronizeOff()
        return oalStatus



    def addPluginFile( self, _oalString ):
        self.synchronizeOn()
        oalFolder = ALFile.getFolderName( _oalString )
        if ( self.m_oalPluginFolder != oalFolder ):
            self.m_oalPluginFolder = oalFolder
            ALSystem.setPath( self.m_oalPluginFolder )
        self.m_oalPluginFile = ALString( _oalString )
        self.__addPlugin( _oalString )
        self.synchronizeOff()




    def getPlugin( self, _oalToken, _oalXmlFileName = None, _oBoolXsd = False ):
        self.synchronizeOn()
        oalPlugin = None
        if ( _oalToken!=None ):
            try:
                if (_oalXmlFileName == None  and  _oBoolXsd == False):
                    # Workaround to bypass instance plugin problem = Must to be fixed for modules import only
                    try:
                        oalModule = self.getObject( _oalToken )
                        oalPlugin = oalModule.__dict__[ _oalToken ]()
                    except:
                        oalPlugin = self.getObject( _oalToken )

                else:
                    # Try to generate an instance of the object based on XSD and an XML file
                    if (_oalXmlFileName != None and  _oBoolXsd == False):
                        try:
                            oalModule = self.getObject( _oalToken )

                            oalStrXmlFileName = ALDiskExplorer.normalize( _oalXmlFileName )
                            if (ALDiskExplorer.existFile( oalStrXmlFileName )!=False):
                                oalPlugin = oalModule.unMarshalling( oalStrXmlFileName )
                            else:
                                ALVerbose.warning( "Plugin Factory XML file inconscistance: getPlugin() - " + ALString(_oalToken ) + ", " + ALString( _oalXmlFileName )  )
                        except:
                            self.__raiseControl()
                    # Try to generate an empty instance of the object based on XSD
                    elif (_oalXmlFileName == None and  _oBoolXsd == True):
                        try:
                            oalModule = self.getObject( _oalToken )
                            oalPlugin = oalModule.__dict__[ _oalToken ]()
                        except:
                            self.__raiseControl()
            except:
                ALVerbose.DEBUG( "Plugin Factory inconsistence: getPlugin() - " + ALString(_oalToken ) + ", " + ALString( _oalXmlFileName )  )
                self.__raiseControl()
        self.synchronizeOff()
        return oalPlugin





    def addPlugin( self, _oalPluginInstance ):
        self.synchronizeOn()
        if ( _oalPluginInstance!=None ):
            oalToken  = _oalPluginInstance.getPluginName()
            self.synchronizeOff()
            self.addObject( oalToken, _oalPluginInstance )
            self.synchronizeOn()
        self.synchronizeOff()


    def outputScreenPlugin( self ):
       self.synchronizeOn()
       for ikey in self.getSequence():
            ALVerbose.screen( "" )
            iPlugin = self[ ikey ]
            if (iPlugin!=None):
                try:
                    iPlugin.outputScreenSignature()
                except:
                    try:
                        oalModule = iPlugin
                        oalPlugin = oalModule.__dict__[ ikey ]()
                        oalPlugin.outputScreenSignature()
                    except:
                        ALVerbose.DEBUG( "No information available: a XSD plugin = " + ALString( ikey ) )
                        self.__raiseControl()

       ALVerbose.screen( "" )
       self.synchronizeOff()


    def outputScreenPluginGroup( self ):
       self.synchronizeOn()
       ALVerbose.screen( "" )
       for ikey in self.m_oalDictPluginGroup:
            ALVerbose.screen( "Plugin GROUP: " + self.m_oalDictPluginGroup[ ikey ] )
       ALVerbose.screen( "" )
       self.synchronizeOff()


    def outputScreenPluginGroupTree( self ):
       self.synchronizeOn()
       for ikey in self.m_oalDictPluginGroup:
           oalPluginGroup = self.m_oalDictPluginGroup[ ikey ]
           ALVerbose.screen( "" )
           ALVerbose.screen( "" )
           ALVerbose.screen( "#####################################" )
           ALVerbose.screen( "####################### Plugin GROUP: " + oalPluginGroup )
           for jkey in self:
               jPlugin = self[ jkey ]
               try:
                   try:
                       if (jPlugin.getGroup()==oalPluginGroup):
                           ALVerbose.screen( "" )
                           jPlugin.outputScreenSignature()
                   except:
                       try:
                           jPlugin = jPlugin.__dict__[ jkey ]()
                           if (jPlugin.getGroup()==oalPluginGroup):
                               ALVerbose.screen( "" )
                               jPlugin.outputScreenSignature()
                       except:
                           ALVerbose.DEBUG( "The group information is not available: " + ALString( jPlugin) )
                           self.__raiseControl()
               except:
                    ALVerbose.DEBUG( "It's not a AALib standard plugin: " + ALString( jPlugin) )
                    self.__raiseControl()
           ALVerbose.screen( "" )
           ALVerbose.screen( "#####################################" )
       ALVerbose.screen( "" )
       self.synchronizeOff()


    def processPluginContainingStringName( self, _oalString = "Help", _bSyncronization = True ):
       self.synchronizeOn()
       bStatus = False
       oalString = ALString( _oalString )


       for ikey in self.getSequence():
            iPlugin = self[ ikey ]
            try:
                ostrPluginName = iPlugin.getPluginName()
                if (ostrPluginName.existSubString( oalString )):
                    iPlugin.executePlugin()
                    if (_bSyncronization==True):
                        iPlugin.synchronizePlugin()
                    bStatus = True
            except:
#                 iPlugin   = iPlugin.__dict__[ ikey ]()
#                 try: # to be improved...
                 try:
                     iPlugin   = iPlugin.__dict__[ ikey ]()
                     ostrPluginName = iPlugin.getPluginName()
                     if (ostrPluginName.existSubString( oalString )):
                        iPlugin.executePlugin()
                        if (_bSyncronization==True):
                            iPlugin.synchronizePlugin()
                        bStatus = True
                 except:
                     pass

       self.synchronizeOff()
       return bStatus


##############################################################################
