##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition    import *
from ALString        import ALString
from ALObject        import ALObject
from ALTime          import ALTime


class ALInformation( ALObject ):
    """
    Interface for Information Classes.
    """
    def __init__( self, _oAuthor = "PLEASE sign your Plugin-Action-Code-Test", _oVersion = AL_VERSION,
                 _oInfo = "Developed with the PyAALib-JyAALib Framework :: 2005-2008",
                 _oContact = "http://jyaalib.sourceforge.net",
                 _oCopyright = "PyAALib-JyAALib Framework is a Free Software" ):
        ALObject.__init__( self )
        self.m_oalAuthor      = ALString( _oAuthor )
        self.m_oalVersion     = ALString( _oVersion )
        self.m_oalInformation = ALString( _oInfo )
        self.m_oalContact     = ALString( _oContact )
        self.m_oalCopyright   = ALString( _oCopyright )
        self.m_oalFileName    = ALString( self.getClassName() )
        self.m_oalGroupName   = ALString( self.getClassName() )
        self.m_oalTimeInit    = 0.0
        self.m_oalTimeEnd     = 0.0
        self.setTimeInit()


    def setTimeInit( self ):
        self.synchronizeOn()
        self.m_oalTimeInit = ALTime.time()
        self.m_oalTimeEnd  = self.m_oalTimeInit
        self.synchronizeOff()


    def setTimeEnd( self ):
        self.synchronizeOn()
        self.m_oalTimeEnd  = ALTime.time()
        self.synchronizeOff()


    def getRunTime( self ):
        self.synchronizeOn()
        oTime = self.m_oalTimeEnd - self.m_oalTimeInit
        self.synchronizeOff()
        return oTime


    def setAuthor( self, _oAuthor ):
        self.synchronizeOn()
        self.m_oalAuthor      = ALString( _oAuthor )
        self.synchronizeOff()

    def setVersion( self, _oVersion ):
        self.synchronizeOn()
        self.m_oalVersion      = ALString( _oVersion )
        self.synchronizeOff()

    def setInformation( self, _oInfo ):
        self.synchronizeOn()
        self.m_oalInformation = ALString( _oInfo )
        self.synchronizeOff()

    def setContact( self, _oContact ):
        self.synchronizeOn()
        self.m_oalContact     = ALString( _oContact )
        self.synchronizeOff()

    def setCopyright( self, _oCopyright ):
        self.synchronizeOn()
        self.m_oalCopyright   = ALString( _oCopyright )
        self.synchronizeOff()

    def setFileName( self, _oFileName ):
        self.synchronizeOn()
        self.m_oalFileName   = ALString( _oFileName )
        self.synchronizeOff()

    def setGroupName( self, _oGroupName ):
        self.synchronizeOn()
        self.m_oalGroupName   = ALString( _oGroupName )
        self.synchronizeOff()


    def getAuthor( self ):
        return self.m_oalAuthor

    def getVersion( self ):
        return self.m_oalVersion

    def getInformation( self ):
        return self.m_oalInformation

    def getContact( self ):
        return self.m_oalContact

    def getCopyright( self ):
        return self.m_oalCopyright

    def getFileName( self ):
        return self.m_oalFileName

    def getGroupName( self ):
        return self.m_oalGroupName


##############################################################################
