##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition         import *
from ALImportPython       import PySys
from ALVerbose            import ALVerbose
from ALInformation        import ALInformation
from ALList               import ALList
from ALString             import ALString



class ALInformationTest( ALInformation ):
    """
    Interface for Information Classes.
    """
    def __init__( self, _oalStringNameTest = "Test", _oiNumberTest = 0 ):
        ALInformation.__init__( self )
        self.m_oalStringNameTest  = _oalStringNameTest
        self.m_oalStringException = ALString.empty()
        self.m_oiNumberTest       = _oiNumberTest
        self.m_oboolSuccess       = False
        self.m_oalListTest        = ALList()


    def isSuccess(self):
        self.synchronizeOn()
        oboolSuccess = True
        if(self.m_oboolSuccess==True):
            for iTest in self.m_oalListTest:
                if (iTest.isSuccess()==False):
                    oboolSuccess = False
        else:
            oboolSuccess = False
        self.synchronizeOff()
        return oboolSuccess


    def getTestNumber(self):
        return self.m_oiNumberTest


    def getTestName(self):
        return self.m_oalStringNameTest



    def setTerminate( self, _oboolSuccess = True):
        self.synchronizeOn()
        self.m_oboolSuccess = _oboolSuccess
        self.synchronizeOff()
        self.setTimeEnd()


    def setFailure( self ):
        self.synchronizeOn()
        self.m_oboolSuccess = False
        self.synchronizeOff()


    def setSuccess( self ):
        self.synchronizeOn()
        self.m_oboolSuccess = True
        self.synchronizeOff()



    def addInformationTest( self, _oalInformationTest ):
        self.synchronizeOn()
        self.m_oalListTest.addObject( _oalInformationTest )
        self.synchronizeOff()



    def setException( self, _oalString ):
        self.synchronizeOn()
        self.m_oalStringException = ALString( _oalString  )
        self.synchronizeOff()
        self.setFailure()



    def getException( self ):
        return self.m_oalStringException



    def getNumberTestMethod( self ):
        return self.m_oalListTest.getNumberObjects()




    def outputTable( self, _oalStream = PySys.stdout, _oalString = "  " ):
        ofTime = self.getRunTime()
        obSuccess = self.isSuccess()
        self.synchronizeOn()

        _oalStream.write( "\n" )
        if (obSuccess==True):
            _oalStream.write( _oalString + "[SUCCESS]" )
        else:
            _oalStream.write( _oalString + "[FAILURE]" )
        _oalStream.write( " [ "+ALString( self.m_oiNumberTest )+" ][ " + self.m_oalStringNameTest +" ]["+ ALString( ofTime )+"]" )
        if (obSuccess==False):
            if (self.m_oalStringException!=ALString.empty()):
                _oalStream.write( "\n" + _oalString + "          [Exception]: " + self.m_oalStringException )

        for iTest in self.m_oalListTest:
            iTest.outputTable( _oalStream, "            " )
        self.synchronizeOff()




    def outputScreen( self, _oalString = "  " ):
        ofTime = self.getRunTime()
        obSuccess = self.isSuccess()
        self.synchronizeOn()

        if (obSuccess==True):
            ALVerbose.screen( _oalString + "[SUCCESS] [ " +ALString( self.m_oiNumberTest )+" ][ " + self.m_oalStringNameTest +" ]["+ ALString( ofTime )+"]" )
        else:
            ALVerbose.screen( _oalString + "[FAILURE] [ " +ALString( self.m_oiNumberTest )+" ][ " + self.m_oalStringNameTest +" ]["+ ALString( ofTime )+"]" )
        if (obSuccess==False):
            if (self.m_oalStringException!=ALString.empty()):
                ALVerbose.screen(  _oalString + "          [Exception]: " + self.m_oalStringException )

        for iTest in self.m_oalListTest:
            iTest.outputScreen( "            " )
        self.synchronizeOff()






##############################################################################
