##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython             import PySubProcess
from ALImportPython             import PyOS
from ALImportPython             import PySignal
from ALImportPython             import PyThreading
from ALImportPython             import PySelect
from ALObject                   import ALObject
from ALString                   import ALString
from ALSystem                   import ALSystem
from ALDebug                    import ALDebug
from ALDiskExplorer             import ALDiskExplorer
from ALList                     import ALList
from ALTrace                    import ALTrace
from ALVerbose                  import ALVerbose



class ALPythonSubProcess( ALObject, PySubProcess.Popen ):
    """
    Interface for ALSubProcess objects.
    Inherits all developments from Python 2.4 subprocess module
    """
    def __init__( self, _oStrCommandArgs, _oStringFolderName = None ):
        ALObject.__init__( self )
        shell = False
        self.m_oReturnCode   = -1
        self.m_oalStrCommand = ALString( _oStrCommandArgs )
        self.m_oListOutput = None
        self.m_oListError  = None
        self.m_oStringFolderName         = ALSystem.getCurrentFolder()
        self.m_oStringFolderNameOriginal = ALSystem.getCurrentFolder()

        # Fix Problem in Linux to call softwares by the system config
        if ( ALSystem.getOSName()!="Windows" ):
            shell = True

        if (_oStringFolderName!=None):
            self.m_oStringFolderName = ALDiskExplorer.getAbsolutePath( _oStringFolderName )
            ALDiskExplorer.createFolder( self.m_oStringFolderName )

      # Verify how I can change the subprocess folder with no change my main software shell currect folder
      # I must find some scheme on python to change it here on the pipe creation
      #  ALSystem.setCurrentFolder( self.m_oStringFolderName )
        PySubProcess.Popen.__init__( self, self.m_oalStrCommand, 0, None,
                 PySubProcess.PIPE, PySubProcess.PIPE, PySubProcess.PIPE,
                 None, False, shell,
                 None, None, True,
                 None, 0 )
      #  ALSystem.setCurrentFolder( self.m_oStringFolderNameOriginal )



    def setWorkingDirectory( self, _oStringFolderName ):
        self.setFolder( _oStringFolderName )


    def setDirectory( self, _oStringFolderName ):
        self.setFolder( _oStringFolderName )


    def setWorkingFolder( self, _oStringFolderName ):
        self.setFolder( _oStringFolderName )


    def setFolder( self, _oStringFolderName ):
        self.synchronizeOn()
        self.m_oStringFolderName = ALDiskExplorer.getAbsolutePath( _oStringFolderName )
        ALSystem.setCurrentFolder( self.m_oStringFolderName )
        self.synchronizeOff()


    def getInput( self ):
        self.synchronizeOn()
        oListStdInput = ALList()
        oListStdInput.add( self.m_oalStrCommand )
        self.synchronizeOff()
        return oListStdInput



    def getOutput( self ):
        self.synchronizeOn()
        oalList = ALList()
        try:
            if (self.m_oListOutput!=None):
                oalString = ALString(  self.m_oListOutput  )
                oalList = oalString.split("\n")
        except:
            self.__raiseControl()
        self.synchronizeOff()
        return oalList



    def getOutputError( self ):
        self.synchronizeOn()
        oalList = ALList()
        try:
            if (self.m_oListError!=None):
                oalString = ALString(  self.m_oListError  )
                oalList = oalString.split("\n")
        except:
            self.__raiseControl()
        self.synchronizeOff()
        return oalList



    def getReturnCode( self ):
        self.synchronizeOn()
        self.m_oReturnCode = self.returncode
        self.synchronizeOff()
        return self.m_oReturnCode



    def __raiseControl( self ):
        oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
        ALDebug.controlDebug( self, oBoolStatus )



    def executeProcess( self, _oalObject = None ):
        return self.getCommunication( _oalObject )


#
#    def getCommunication( self, _oalObject = None ):
#        self.synchronizeOn()
#        ALSystem.setCurrentFolder( self.m_oStringFolderName )
#        self.m_oListOutput, self.m_oListError = self.communicate()
#        self.wait()
#        ALSystem.setCurrentFolder( self.m_oStringFolderNameOriginal )
#        self.synchronizeOff()



    def getCommunication( self, _oalObject = None ):
        self.synchronizeOn()
        self.m_oListOutput, self.m_oListError = self.communicate()
        self.wait()
        self.synchronizeOff()


##############################################################################