##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython   import PySys
from ALImportPython   import PyOS
from ALImportPython   import PySocket
from ALDefinition     import *
from ALObject         import ALObject
from ALString         import ALString


class ALSysInfo( ALObject ):
    """
    Used to identify diferent parameters from the OS System
    For each plataform and interpreter the basic layer extracts the main information
    http://java.sun.com/javase/6/docs/api/java/lang/System.html
    """
    def __init__( self ):
       ALObject.__init__( self )
       self.m_interpreter    = None
       self.m_machineVirtuel = None
       self.m_OSName         = None
       self.m_typeOS         = None
       self.m_userName       = None
       self.m_hostName       = None
       self.m_pythonVersion  = None
       self.m_OSVersion      = None
       self.m_machineVersion = None
       self.m_hostIP         = None

       self.__init()


    def __init( self ):
       oSystem = PySys.platform[:4]
       if (oSystem=="java"):
           self.m_interpreter    = ALString( "Jython" )
           self.m_machineVirtuel = ALString( "Java" )
       else:
           self.m_interpreter    = ALString( "Python" )
           self.m_machineVirtuel = ALString( "Python" )
       self.m_pythonVersion  = ALString( PySys.version )
       self.__setOSParameters()



    def __setOSParameters( self ):
       self.__getOSControl()
       self.__setJava()
       self.__setWindows()
       self.__setSun()
       self.__setMacOSX()
       self.__setLinux()
       self.__setBSD()
       self.__setCygwin()
       self.__setUnknown()



    def __setJava( self ):
        if (self.m_typeOS==AL_SYSTEM_JAVA):
           from     ALImportJython   import JySystem
           self.m_OSName         = ALString( JySystem.getProperty( "os.name" ) )
           if (self.m_OSName.contains("Windows")):
               self.m_OSName = "Windows"
           self.m_OSType         = ALString( JySystem.getProperty( "os.name" ) )
           self.m_userName       = ALString( JySystem.getProperty( "user.name" ) )
           self.m_hostName       = ALString( "NoMachineName" )
           self.m_OSVersion      = ALString( JySystem.getProperty( "os.version" ) )
           self.m_machineVersion = ALString( JySystem.getProperty( "os.arch" ) )
           try:
               self.m_hostName       = ALString( PySocket.gethostname() )
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )


    def __setWindows( self ):
        if (self.m_typeOS==AL_SYSTEM_WINDOWS):
           from ALImportPython   import PyPlatform
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "Windows" )
           self.m_OSType         = ALString( PyPlatform.uname()[2] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyPlatform.uname()[3] )
           self.m_machineVersion = ALString( "i686" )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )


    def __setSun( self ):
        if (self.m_typeOS==AL_SYSTEM_SOLARIS):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "Sun" )
           self.m_OSType         = ALString( PyOS.uname()[2] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )



    def __setMacOSX( self ):
        if (self.m_typeOS==AL_SYSTEM_MACOSX):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "MacOSX" )
           self.m_OSType         = ALString( PyOS.uname()[0] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )




    def __setLinux( self ):
        if (self.m_typeOS==AL_SYSTEM_LINUX):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "Linux" )
           self.m_OSType         = ALString( PyOS.uname()[2] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )



    def __setBSD( self ):
        if (self.m_typeOS==AL_SYSTEM_BSD):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "BSD" )
           self.m_OSType         = ALString( PyOS.uname()[2] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )



    def __setCygwin( self ):
        if (self.m_typeOS==AL_SYSTEM_WINCYGWIN):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "Cygwin" )
           self.m_OSType         = ALString( "Cygwin" )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )



    def __setUnknown( self ):
        if (self.m_typeOS==AL_SYSTEM_UNKNOWN):
           try:
               from ALImportPython   import PyGetPass
               self.m_OSName         = ALString( AL_SYSTEM_UNKNOWN )
               self.m_OSType         = ALString( PyOS.uname()[2] )
               self.m_userName       = ALString( PyGetPass.getuser() )
               self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
               self.m_OSVersion      = ALString( PyOS.uname()[2] )
               self.m_machineVersion = ALString( PyOS.uname()[4] )
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               print " [Warnning] PyAALib::NOT TESTED AND VALIDATED in this Unknown Operational System\n"


    def __getOSControl( self ):
        oalString = AL_SYSTEM_UNKNOWN
        if (PyOS.name =="java"):
            oalString = ALString( AL_SYSTEM_JAVA )
        elif (PyOS.name =="nt"):
            oalString = ALString( AL_SYSTEM_WINDOWS )
        elif ( PyOS.name == "posix" ):
            oalStrSystem = PyOS.uname()
            if ( oalStrSystem[0] == "Linux" ):
                oalString = ALString( AL_SYSTEM_LINUX )
            elif ( oalStrSystem[0] == "SunOS" ):
                oalString = ALString( AL_SYSTEM_SOLARIS )
            elif ( oalStrSystem[0] == "Darwin" ):
                oalString = ALString( AL_SYSTEM_MACOSX )
            elif ( oalStrSystem[0] == "FreeBSD" ):
                oalString = ALString( AL_SYSTEM_BSD )
            elif ( oalStrSystem[0] != "Linux" ):
                oalStrPath = PyOS.getcwd()
                if ( oalStrPath.count("cygdrive") == True ):
                    oalString = ALString( AL_SYSTEM_WINCYGWIN )
                else :
                    print " [Warnning] PyAALib::NOT TESTED AND VALIDATED in this Unknown Operational System\n"
        self.m_typeOS = oalString

##############################################################################
