##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython   import PySys
from ALImportPython   import PyOS
from ALImportPython   import PySocket
from ALImportPython   import PyStat
from ALImportPython   import PyPipe3
from ALObject         import ALObject
from ALString         import ALString
from ALSysInfo        import ALSysInfo


class ALSystem( ALObject ):
    """
    Interface to the System parameters
    """

    m_oalSysInfo = ALSysInfo()


    def __init__( self ):
       ALObject.__init__( self )


    def getByteOrder():
        return  PySys.byteorder
    getByteOrder = staticmethod( getByteOrder )

    def getInterpreter():
        return ALSystem.m_oalSysInfo.m_interpreter
    getInterpreter = staticmethod( getInterpreter )


    def getCompiler():
        return ALSystem.getMachineVirtuel()
    getCompiler = staticmethod( getCompiler )


    def getMachineVirtuel():
        return ALSystem.m_oalSysInfo.m_machineVirtuel
    getMachineVirtuel = staticmethod( getMachineVirtuel )


    def getMachineVirtual():
        return ALSystem.getMachineVirtuel()
    getMachineVirtual = staticmethod( getMachineVirtual )


    def getPid():
        return PyOS.getpid()
    getPid = staticmethod( getPid )


    def getVersion():
        return ALSystem.m_oalSysInfo.m_pythonVersion
    getVersion = staticmethod( getVersion )


    def exit( _iArg = 0 ):
        PySys.exit( _iArg )
    exit = staticmethod( exit )


    def killProcess( _oPid, _oSignal = 1 ):
        try:
            if ( ALSystem.getOSName()=="Windows" ):
                """kill function for Win32"""
                from ALPackage   import  ALPackWin32Api
                handle = ALPackWin32Api.OpenProcess(1, 0, self.getPid() )
                return (0 != ALPackWin32Api.TerminateProcess(handle, 0))
            else:
                PyOS.kill( _oPid, _oSignal  )
        except:
            pass
#            print " [Warning] ALSystem.killProcess() = Exception raised"
    killProcess = staticmethod( killProcess )


    def exc_info():
        return PySys.exc_info()
    exc_info = staticmethod( exc_info )


    def executeProcess(  _pathExe, _arg = " " , _mode = 0 ):
        return ALSystem.openPipe3( _pathExe )
    executeProcess= staticmethod( executeProcess )



    def openPipe( _command ):
        return PyOS.popen( _command )
    openPipe= staticmethod( openPipe )



    def openPipe3( _command ):
        return PyPipe3( _command )
    openPipe3= staticmethod( openPipe3 )



    def waitPid( _pid, _option ):
        return PyOS.waitpid( _pid, _option )
    waitPid= staticmethod( waitPid )


    def setExecuteMode( _oalString ):
        PyOS.chmod(  _oalString, PyStat.S_IRWXU )
    setExecuteMode= staticmethod( setExecuteMode )


    def execute( _oalString ):
        PyOS.system( _oalString )
    execute= staticmethod( execute )


    def getOSControl( ):
        return ALSystem.m_oalSysInfo.m_typeOS
    getOSControl = staticmethod( getOSControl )


    def getOSName():
        return ALSystem.m_oalSysInfo.m_OSName
    getOSName = staticmethod( getOSName )


    def getOSType():
        return ALSystem.m_oalSysInfo.m_OSType
    getOSType = staticmethod( getOSType )


    def getHostName( ):
        return ALSystem.m_oalSysInfo.m_hostName
    getHostName = staticmethod( getHostName )


    def getOSVersion():
        return ALSystem.m_oalSysInfo.m_OSVersion
    getOSVersion = staticmethod( getOSVersion )


    def getMachineVersion():
        return ALSystem.m_oalSysInfo.m_machineVersion
    getMachineVersion = staticmethod( getMachineVersion )


    def getUserName():
        return ALSystem.m_oalSysInfo.m_userName
    getUserName = staticmethod( getUserName )


    def getHostIP():
        return ALSystem.m_oalSysInfo.m_hostIP
    getHostIP = staticmethod( getHostIP )


    def getHost():
        return ALSystem.m_oalSysInfo.m_hostName
    getHost = staticmethod( getHost )


    def getCwd():
        oalString = ALString( PyOS.getcwd() )
        oalString = oalString.replace( "\\", "/" )
        return oalString
    getCwd = staticmethod( getCwd )


    def getCurrentFolder():
        return ALSystem.getCwd()
    getCurrentFolder = staticmethod( getCurrentFolder )


    def getProgramName( ):
        oalString = ALString( PySys.argv[0] )
        oalString = oalString.replace( "\\", "/" )
        return oalString
    getProgramName = staticmethod( getProgramName )


    def getCompilerName():
        oalString = ALString( "Python " + ALString( PySys.version ) )
        oalString = oalString.replace( "\n", " " )
        return oalString
    getCompilerName = staticmethod( getCompilerName )


    def getUserPath( ):
        oalString = ALString( PyOS.path.expanduser("~") )
        oalString = oalString.replace( "\\", "/" )
        if ALSystem.getOSControl() == ALString( "WindowsCygwinSystem" ):
            if oalString.count("cygdrive") == True :
                lstr = oalString[10]
                oalStringPath = ALString.replace( oalString, "/cygdrive/" + lstr + "/", lstr + ":/" )
                oalString = oalStringPath
        return oalString;
    getUserPath = staticmethod( getUserPath )


    def setPath( _oalString ):
        oalListString = PySys.path
#        print "string1:  " + _oalString
        oalString = PyOS.path.abspath( _oalString )
#        print "string2:  " + oalString
        oBoolStatus = True
        for iString in oalListString:
            if (iString==oalString):
                oBoolStatus = False
        if (oBoolStatus==True):
#            print oalString
            return PySys.path.append( oalString )
        else:
            return False
    setPath = staticmethod( setPath )



    def getSystemPath( _oalString = "PATH" ):
        oalPath = ALString( PyOS.getenv( _oalString ) )
        oalList = oalPath.split(";")
        return oalList
    getSystemPath = staticmethod( getSystemPath )


    def setSystemPath( _oalString ):
        return ALSystem.setPath( _oalString )
    setSystemPath = staticmethod( setSystemPath )


    def addSystemPath( _oalString ):
        return ALSystem.setPath( _oalString )
    addSystemPath = staticmethod( addSystemPath )


    def setCurrentFolder( _oalString ):
        if ALSystem.getOSControl() == ALString( "JavaSystem" ):
          # print "ALSystem::setCurrentFolder() - Not Supported in Jython yet"
            return
        return PyOS.chdir( _oalString )
    setCurrentFolder = staticmethod( setCurrentFolder )


    def isCommandLine( _oalString ):
        oalString = ALString( _oalString )
        for cmdkey in PySys.argv:
            if (cmdkey == oalString):
                return True
        return False
    isCommandLine = staticmethod( isCommandLine )



##############################################################################
