##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz & Marie-Francoise Incardona
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2007/10/05
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALObject          import  ALObject
from ALDebug           import  ALDebug
from ALList            import  ALList
from ALTrace           import  ALTrace
from ALTime            import  ALTime
from ALString          import  ALString
from ALVerbose         import  ALVerbose
from ALTrace           import  ALTrace
from ALAssert          import  ALAssert


class ALTest( ALObject ):
    """
    Interface for AL Tests Object
    """
    def __init__( self, _oalStringTestName = None ):
        ALObject.__init__( self )
        self.m_oalStringTestName  = _oalStringTestName
        self.m_oalListTest        = ALList()
        self.m_oBoolIsVerbose     = False
        self.m_oBoolIsDebug       = False
        self.m_oBoolIsAssert      = False
        self.m_oBoolIsLogFile     = False
        self.m_oBoolIsProfile     = False
        self.m_obForceDebug       = False


    def controlCommandLine( self ):
        if ( ALDebug.isDebugModeOn()==True or ALDebug.isDebugModeForceOn()==True ):
            ALVerbose.setVerboseDebugOn()
            ALVerbose.setProfileOn()
        if ( ALDebug.isDebugModeVerboseOn()==True ):
            ALVerbose.setVerboseOn()
        if ( ALDebug.isDebugModeLogFileOn()==False ):
            ALVerbose.setLogFileOff()
        if ( ALDebug.isDebugModeAssertOn()==True):
            ALAssert.setAssertionModeOn()
        if ( ALDebug.isDebugModeProfileOn()==True):
           ALVerbose.setProfileOn()


    def outputProfile(self):
        ALVerbose.getProfileFile()


    def getTestName( self ):
        oalString = ALString("Test")
        if (self.m_oalStringTestName!=None):
            oalString = self.m_oalStringTestName
        return oalString


    def addTestMethod( self, _oClassMethod ):
        self.m_oalListTest.append( _oClassMethod )


    def patchMethodName(self, _oalStringText ):
        """
        Fix for Jython name scheme of methods = <method xxxxx....
        and Python is = <bound method xxxx....
        """
        oalStringTest = ALString ( _oalStringText )
        oalStringTest2 = oalStringTest.replace( "<", " " )
        oalString = oalStringTest2.getNextToken("method")
        if (oalString==None):
            oalString = ALString ( "None Object" )
        return oalString


    def __preProcessKernel( self ):
        self.m_oBoolIsVerbose = ALVerbose.isVerbose()
        self.m_oBoolIsDebug   = ALVerbose.isVerboseDebug()
        self.m_oBoolIsLogFile = ALVerbose.isLogFile()
        self.m_oBoolIsAssert  = ALAssert.isAssertionMode()
        self.m_oBoolIsProfile = ALVerbose.isProfile()


    def __postProcessKernel( self ):
        if (self.m_oBoolIsVerbose==True):
            ALVerbose.setVerboseOn()
        else:
            ALVerbose.setVerboseOff()

        if (self.m_oBoolIsDebug==True):
            ALVerbose.setVerboseDebugOn()
        else:
            ALVerbose.setVerboseDebugOff()

        if (self.m_oBoolIsLogFile==True):
            ALVerbose.setLogFileOn()
        else:
            ALVerbose.setLogFileOff()

        if (self.m_oBoolIsAssert==True):
            ALAssert.setAssertionModeOn()
        else:
            ALAssert.setAssertionModeOff()

        if (self.m_oBoolIsProfile==True):
            ALVerbose.setProfileOn()
        else:
            ALVerbose.setProfileOff()



    def processKernel( self ):
        ALVerbose.DEBUG( "Execution: ALTest.processKernel()" )



    def raiseControl( self ):
        oBoolStatus =  ALDebug.isDebugModeOn()
        ALDebug.controlDebug( self, oBoolStatus )


    def executeKernel( self ):
        ALVerbose.DEBUG( "Execution: ALTest.executeKernel()" )
        self.__preProcessKernel()
        self.preProcess()

        self.process()
        self.processKernel()

        self.postProcess()
        self.__postProcessKernel()



    def execute( self ):
        ALVerbose.DEBUG( "Execution: ALTest.execute()" )
        try:
            self.executeKernel()
        except:
            self.raiseControl()




    ##########################################################################
    # To be modified by the user in the child classes
    def preProcess( self ):
        ALVerbose.DEBUG( "Execution: ALTest.preProcess()" )


    def process( self ):
        ALVerbose.DEBUG( "Execution: ALTest.process()" )


    def postProcess( self ):
        ALVerbose.DEBUG( "Execution: ALTest.postProcess()" )



##############################################################################