##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDebug                   import ALDebug
from ALImportPython            import PyMinidom
from ALImportPython            import PyXmlNode
from ALList                    import ALList
from ALObject                  import ALObject
from ALSlot                    import ALSlot
from ALStreamFile              import ALStreamFile
from ALString                  import ALString
from ALThreadingCondition      import ALThreadingCondition
from ALToken                   import ALToken
from ALTrace                   import ALTrace
from ALVerbose                 import ALVerbose
from ALXmlNode                 import ALXmlNode


class ALXml( ALObject ):
    """
    Interface for ALXml objects.
    ATTENTION:: must be transformed to thread safety
    """

    m_oalThreadingConditionXml = ALThreadingCondition()
    

    def __init__( self, _oalString = "document" ):
        ALObject.__init__( self )
        self.m_oPyXmlDocument = None
        oPyImplementation = PyMinidom.getDOMImplementation()
        self.m_oPyXmlDocument = oPyImplementation.createDocument(None, _oalString, None)
        self.m_oPyXmlDocumentRoot = self.m_oPyXmlDocument.documentElement


    def __controlRaise( self ):
        oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
        ALDebug.controlDebug( self, oBoolStatus )


    def getXmlDocument( self ):
        return self.m_oPyXmlDocument


    def output( self, _oalStringName, _bPrintType = False, _oalStringEncodingType = "ISO-8859-1" ): # or using "utf-8"
        if (self.m_oPyXmlDocument==None):
            return
        self.synchronizeOn()
        oalStreamFile = ALStreamFile( _oalStringName, "w" )
        # Here to bypass error in the python xml module when compiled with py2exe
        try:
            if (_bPrintType == False ) :
                oalStreamFile.write( self.m_oPyXmlDocument.toxml( _oalStringEncodingType )  )
            else:
                oalStreamFile.write( self.m_oPyXmlDocument.toprettyxml( "\t", "\n", _oalStringEncodingType )  ) # it creates a bug in komodo kpf file - tests
        except:
            oalStreamFile.write( self.m_oPyXmlDocument.toxml() )

        oalStreamFile.close()
        self.synchronizeOff()



    def input( self,  _oalStrFileName ):
        self.synchronizeOn()
        self.m_oPyXmlDocument = ALXml.__parseXmlFile( _oalStrFileName )
        self.synchronizeOff()


    def createXml( _oalStrFileName ):
        return ALXml.__parseXmlFile( _oalStrFileName )
    createXml = staticmethod( createXml )


    def createXmlString( _oalString ):
        return ALXml.__parseXmlString( _oalString )
    createXmlString = staticmethod( createXmlString )



    def __parseXmlFile( _oalStrFileName ):
        oXml = None
        ####################################################################
#        # Nasty error on Jython  Java sax implementation....It waits a str pure string!!!!
#          [ERROR]: File  :: C:/Program-Files/Automation/win32/Program-Files/jython-2.2rc3/Lib/xml/sax/drivers2/drv_javasax.py
#          [ERROR]: Method:: parse
#          [ERROR]: Line  :: 141
#          [ERROR]: Object:: self._parser.parse(JyInputSourceWrapper(source))
#          [ERROR]: java.net.MalformedURLException: java.net.MalformedURLException: unknown protocol: c
#
        oalString = ALString( _oalStrFileName )
        oPyString = oalString.getPythonString()
        #####################################################################
        try:
            oXml = PyMinidom.parse( oPyString )
        except:
            ALVerbose.error( "ALXml::__parseXmlFile() = parser Failed" )
            ALVerbose.error( "File Parser Error :: " + ALString( _oalStrFileName ) )
            self.__controlRaise()
        return oXml
    __parseXmlFile = staticmethod( __parseXmlFile )


    def __parseXmlString( _oalString ):
        oXml = None
        try:
            oXml = PyMinidom.parseString( _oalString )
        except:
            ALVerbose.error( "ALXml::__parseXmlString() = parser Failed" )
            self.__controlRaise()
        return oXml
    __parseXmlString = staticmethod( __parseXmlString )


    def addComment( self, _oalString ):
        self.synchronizeOn()
        oPyXmlComment = self.m_oPyXmlDocument.createComment( ALString( _oalString ) )
        self.m_oPyXmlDocumentRoot.appendChild(oPyXmlComment)
        self.synchronizeOff()



    def addText( self, _oalString ):
        self.synchronizeOn()
        oPyXmlText = self.m_oPyXmlDocument.createTextNode( ALString( _oalString ) )
        self.m_oPyXmlDocumentRoot.appendChild(oPyXmlText)
        self.synchronizeOff()



    def close( self ):
        self.synchronizeOn()
        if (self.m_oPyXmlDocument!=None):
            self.m_oPyXmlDocument.unlink()
        self.synchronizeOff()


    def destroy( self ):
        self.close()



    def addObject( self, _oalObject ):
        self.synchronizeOn()
        _oalObject.outputXml( self, self.m_oPyXmlDocumentRoot )
        self.synchronizeOff()

    # Clonage of methods from Document() xml class
    def createElement( self, _oToken ):
        return self.m_oPyXmlDocument.createElement( _oToken )

    def createTextNode( self, _oTextNode ):
        return self.m_oPyXmlDocument.createTextNode( _oTextNode )


    def rootNode( self ):
        return self.m_oPyXmlDocumentRoot





    def addTokenValue( self, _oalToken, _oalString ):
        self.synchronizeOn()
        if ( _oalString!=None ):
            oalXmlNodeString = self.m_oPyXmlDocument.createTextNode( ALString( _oalString ) )
            oalXmlNode = self.m_oPyXmlDocument.createElement( ALToken( _oalToken ) )
            oalXmlNode.appendChild( oalXmlNodeString )
            self.m_oPyXmlDocumentRoot.appendChild( oalXmlNode  )
        self.synchronizeOff()




    # Only in the main branch of the tree
    # WARNING: returns every time the first object!!!
    def getTokenValue( self, _oalToken ):
        return ALXml.getElementTokenValue( self.m_oPyXmlDocumentRoot, _oalToken )




    # Looking for ONE object XmlElements in the main branch of the tree
    # WARNING: returns every time the first object!!!
    def getElement( self, _oalToken ):
        self.synchronizeOn()
        oalXmlElement = None

        # Trick to bypass bad Xml documents
        if ALToken( self.m_oPyXmlDocumentRoot.tagName ) == ALToken( _oalToken ):
            oalXmlElement = self.m_oPyXmlDocumentRoot
            self.synchronizeOff()
            return oalXmlElement
        else :
            self.synchronizeOff()
            return ALXml.getElementObject( self.m_oPyXmlDocument, _oalToken )

        return oalXmlElement




    def removeElement( self, _oalToken ):
        oalXmlElement = self.getElement( _oalToken )
        self.synchronizeOn()
        if (oalXmlElement != None):
            self.m_oPyXmlDocumentRoot.removeChild( oalXmlElement  )
        self.synchronizeOff()



    # Looking for ONE XmlElement from any start node (sub-points)
    # WARNING: returns every time the first object!!!
    def getElementObject( _oalXmlElement, _oalToken ):
        ALXml.m_oalThreadingConditionXml.synchronizeOn()
        oalXmlElement = None
        for inode in _oalXmlElement.getElementsByTagName( ALToken( _oalToken ) ):
            oalXmlElement = inode

            # Bad....
            ALXml.m_oalThreadingConditionXml.synchronizeOff()
            return oalXmlElement
        ALXml.m_oalThreadingConditionXml.synchronizeOff()
        return oalXmlElement
    getElementObject = staticmethod( getElementObject )






    # Looking for a list of XmlElements in the main branch of the tree
    def getElementList( self, _oalToken ):
        self.synchronizeOn()
        oalListXmlElement = None
        # Trick to bypass bad Xml documents
        if ALToken( self.m_oPyXmlDocumentRoot.tagName ) == ALToken( _oalToken ):
            oalListXmlElement = ALList()
            oalListXmlElement.add( self.m_oPyXmlDocumentRoot )
        else :
            self.synchronizeOff()
            return ALXml.getElementListObject( self.m_oPyXmlDocumentRoot, _oalToken )
        self.synchronizeOff()
        return oalListXmlElement




    # Looking for VALUE of XmlElement from any start node (sub-points)
    # WARNING: returns every time the first object!!!
    def getElementTokenValue( _oalXmlElement, _oalToken ):
        ALXml.m_oalThreadingConditionXml.synchronizeOn()
        oalStringValue = None
        for inode in _oalXmlElement.childNodes:
            if inode.nodeType==PyXmlNode.ELEMENT_NODE:
                if ( ALToken( inode.nodeName ) == ALToken( _oalToken ) ):
                    for jnode in inode.childNodes:
                        if jnode.nodeType==PyXmlNode.TEXT_NODE:
                            oalStringValue = ALString( jnode.data )
                            # cleaning the xml errors from the python code!!!!!!
                            oalStringValue = oalStringValue.cleanXml()
                            # BAD....but runs
                            ALXml.m_oalThreadingConditionXml.synchronizeOff()
                            return oalStringValue
        ALXml.m_oalThreadingConditionXml.synchronizeOff()
        return oalStringValue
    getElementTokenValue = staticmethod( getElementTokenValue )


    # Looking for a list of XmlElements from any start node (sub-points)
    def getElementListObject( _oalXmlElement, _oalToken ):
        ALXml.m_oalThreadingConditionXml.synchronizeOn()
        oalListXmlElement = None
        for inode in _oalXmlElement.childNodes:
            if inode.nodeType==ALXmlNode.ELEMENT_NODE:
                if ( ALToken( inode.nodeName ) == ALToken( _oalToken ) ):
                    if ( oalListXmlElement==None ):
                        oalListXmlElement = ALList()
                    oalListXmlElement.add( inode )
        ALXml.m_oalThreadingConditionXml.synchronizeOff()
        return oalListXmlElement
    getElementListObject = staticmethod( getElementListObject )




    #Used to fix the "m_"  of AALib class atributte squeme
    def getAtributeToken( _oalToken ):
        oalToken = ALToken( _oalToken )
        if oalToken[0]=="m" and oalToken[1]=="_" :
            #oalToken.replace( "m_", "" ) # It can generate a problem in complex names
            oalTokenNew = oalToken[2:]
            oalToken = oalTokenNew
        return oalToken
    getAtributeToken = staticmethod( getAtributeToken )






##############################################################################
