##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALObject        import ALObject
from ALVerbose       import ALVerbose
from ALString        import ALString
from ALTime          import ALTime
from ALDiskExplorer  import ALDiskExplorer
from ALStreamFile    import ALStreamFile
from ALList          import ALList

import ALExternalPackGenerateDS  as ALPackXsd


class ALXsd( ALObject ):
    """
    Interface for Xsd objects.
    """

    def __init__( self ):
        ALObject.__init__( self )


    def generateModule( self, _oalPyFileNameOutput, _oalXsdFileNameInput, _nameSpace = "xs:" ):
        oalXsdFileNameTemp = self.setSchemaLocation( _oalXsdFileNameInput, _nameSpace )
        ALPackXsd.generateClassDefinition( _oalPyFileNameOutput, oalXsdFileNameTemp, "xsd:" )
        ALDiskExplorer.deleteFile( oalXsdFileNameTemp )




    def generateModuleBuffer( self, _oalPyFileNameOutput, _oalXsdFileNameInput, _nameSpace = "xs:" ):
        """
        Not yet Full Compatible with Jython2.2
        """
        try:
            oalXsdStringTemp = self.setSchemaLocationBuffer( _oalXsdFileNameInput, _nameSpace )
            #Path to fix possible bug on Java-Jython: some lazy process or disk writing???
            ALTime.wait(0.1)
            ALPackXsd.generateClassDefinitionBuffer( _oalPyFileNameOutput, oalXsdStringTemp, "xsd:" )
        except:
            ALVerbose.DEBUG( "[Warning] ALXsd.generateModuleBuffer() - Jython2.2 Limitation" )
            self.generateModule( _oalPyFileNameOutput, _oalXsdFileNameInput, _nameSpace )






    def setSchemaLocationBuffer( self, _oalXsdFileName, _nameSpace ):
        oalXsdString = ALString.empty()

        oalFolder = ALDiskExplorer.getFolderName( _oalXsdFileName )

        oalStream = ALStreamFile( _oalXsdFileName, "r" )
        oalList = oalStream.getLines()
        oalStream.close()

        oalListList = ALList()

        oalListLine = ALList()
        for i in oalList:
            istr = ALString( i ).replace( "xs:", "xsd:" )
            if ( istr.existSubString("xml version=")==False and istr.existSubString("schema")==False and istr.existSubString("schemaLocation")==False):
                oalListLine.append( istr )
            if (istr.existSubString("schemaLocation")==True):
                oalListList.append( self.__getSchemeLocationData( istr, oalFolder ) )


        oalXsdString = ALString( "<?xml version=\"1.0\"?>\n" )
        oalXsdString = oalXsdString + ALString( "<xsd:schema xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n\n" )
        for i in oalListLine:
            oalXsdString = oalXsdString + ALString( i )
        for iList in oalListList:
           oalXsdString = oalXsdString + ALString( "\n\n" )
           for j in iList:
               oalXsdString = oalXsdString + ALString(  j )
        oalXsdString = oalXsdString + ALString( "\n" )
        oalXsdString = oalXsdString + ALString( "</xsd:schema>" )

        return oalXsdString




    def setSchemaLocation( self, _oalXsdFileName, _nameSpace ):
        oalXsdFileName = ALTime.getTimeName() + ALString( "-xsdTemp.xsd" )

        oalFolder = ALDiskExplorer.getFolderName( _oalXsdFileName )

        oalStream = ALStreamFile( _oalXsdFileName, "r" )
        oalList = oalStream.getLines()
        oalStream.close()

        oalListList = ALList()

        oalListLine = ALList()
        for i in oalList:
            istr = ALString( i ).replace( "xs:", "xsd:" )
            if ( istr.existSubString("xml version=")==False and istr.existSubString("schema")==False and istr.existSubString("schemaLocation")==False):
                oalListLine.append( istr )
            if (istr.existSubString("schemaLocation")==True):
                oalListList.append( self.__getSchemeLocationData( istr, oalFolder ) )

        oalStream = ALStreamFile( oalXsdFileName )
        oalStream.write( "<?xml version=\"1.0\"?>\n" )
        oalStream.write( "<xsd:schema xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n\n" )
        for i in oalListLine:
            oalStream.write( i )
        for iList in oalListList:
           oalStream.write( "\n\n" )
           for j in iList:
               oalStream.write( j )
        oalStream.write( "\n" )
        oalStream.write( "</xsd:schema>" )
        oalStream.close()

        return oalXsdFileName




    def __getSchemeLocationData( self, _oalXsdLine, _oalFolder ):
        oalString = ALString( _oalXsdLine )
        oalList = oalString.split( "\"" )
        if (_oalFolder==ALString.empty()):
            _oalFolder = "."

        oalListLine = ALList()
        oalString = _oalFolder + "/" + ALString( oalList[1] )

        if (ALDiskExplorer.existFile(oalString)==True):
            oalStream = ALStreamFile( oalString, "r" )
            oalList = oalStream.getLines()
            oalStream.close()
            oalListList = ALList()
            for i in oalList:
                istr = ALString( i ).replace( "xs:", "xsd:" )
                if ( istr.existSubString("xml version=")==False and istr.existSubString("schema")==False and istr.existSubString("schemaLocation")==False):
                    oalListLine.append( istr )
                if (istr.existSubString("schemaLocation")==True):
                    oalListList.append( self.__getSchemeLocationData( istr, _oalFolder ) )

            for iList in oalListList:
               for j in iList:
                   oalListLine.append( j )
        else:
            ALVerbose.warning( "ALXsd::__getSchemeLocationData = File not found: ", oalString )

        return oalListLine



##############################################################################