##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALObject        import ALObject
from ALVerbose       import ALVerbose
from ALString        import ALString
from ALTime          import ALTime
from ALDebug         import ALDebug
from ALDiskExplorer  import ALDiskExplorer
from ALFile          import ALFile
from ALXsd           import ALXsd


class ALXsdDataBinding( ALXsd ):
    """
    Interface for Xsd objects.
    """

    def __init__( self, _oalStrXsdFileName, _oalStrModuleFileName = None ):
        ALXsd.__init__( self )
        self.m_oalStrXsdFileName    = ALDiskExplorer.normalize(  _oalStrXsdFileName )
        if (_oalStrModuleFileName==None):
            oalNameModule = ALFile.getBaseName( self.m_oalStrXsdFileName )
            self.m_oalStrModuleFileName = ALFile.setExtension( oalNameModule, ".py" )
        else:
            self.m_oalStrModuleFileName =  ALFile.setExtension( _oalStrModuleFileName, ".py" )



    def generateModuleEngine( self, _oalStrModuleFileName, _oalStrXsdFileName ):
        self.generateModuleBuffer( _oalStrModuleFileName, _oalStrXsdFileName )



    def getObject( self, _oalStrXmlFileName = None, _oalStrModuleFileName = None, _oStringNameClass = None ):
        """
        inputs: ( xml file name, module.py file name, class to import )
        """
        return self.getObjectParser( _oalStrXmlFileName, _oalStrModuleFileName, _oStringNameClass )



    def getObjectClass( self, _oStringNameClass ):
        """
        inputs: ( xml file name, module.py file name, class to import )
        """
        return self.getObjectParser( None, None, _oStringNameClass )




    def getObjectParser( self, _oalStrXmlFileName = None,
                         _oalStrModuleFileName = None, _oStringNameClass = None ):
        """
        inputs: ( xml file name, module.py file name, class to import )
        """

        oalObject = None

        if (_oalStrModuleFileName==None):
            oalStrModuleFileName = self.m_oalStrModuleFileName
        else:
            oalNameModule = ALFile.getBaseName( _oalStrModuleFileName )
            oalStrModuleFileName = ALFile.setExtension( oalNameModule, ".py" )

        if (ALDiskExplorer.existFile( oalStrModuleFileName )==True):
           # print oalStrModuleFileName
            ALDiskExplorer.deleteFile( oalStrModuleFileName )


        try:
            self.generateModuleEngine( oalStrModuleFileName, self.m_oalStrXsdFileName )
            oNameModule = ALFile.getBaseName( oalStrModuleFileName )
            oModule = __import__( oNameModule )

            if (_oStringNameClass is not None):
                oStringNameClass = _oStringNameClass
            else:
                oStringNameClass = oNameModule


            if (_oalStrXmlFileName!=None):
                oalStrXmlFileName = ALDiskExplorer.normalize( _oalStrXmlFileName )
                if (ALDiskExplorer.existFile( oalStrXmlFileName )==True):
                    oalObject = oModule.unMarshalling( oalStrXmlFileName )
                else:
                    oalObject = oModule.__dict__[ oStringNameClass ]()
            else:
                oalObject = oModule.__dict__[ oStringNameClass ]()

        except:
            raise
            ALVerbose.warning("ALXsdDataBinding::getObject = problem to generate the module: " + ALString( oalStrModuleFileName ) )
            oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
            ALDebug.controlDebug( self, oBoolStatus )



        return oalObject


##############################################################################