#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDHandlerXSDataBestv01.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                    import EDObject
from EDImport                    import EDDictionary
from XSDataBestv01               import XSDataBestv01Input
from XSDataBestv01               import XSDataFloat
from XSDataCharacterisationv01   import XSDataStrategyResult
from XSDataCharacterisationv01   import XSDataCollection
from XSDataCharacterisationv01   import XSDataSubWedge
from XSDataCharacterisationv01   import XSDataExperimentalCondition
from XSDataCharacterisationv01   import XSDataAngle


class EDHandlerXSDataBestv01( EDObject ):
    """
    """
    def getXSDataBestv01Input( self, _xsDataBeam, _xsDataSample, _xsDataDetector, _xsDataGoniostat, _xsDataDiffractionPlan,
                               _oedStrBestFileContentDat, _oedStrBestFileContentPar, _oedListBestFileContentHKL ):
        """
        """
        xsDataBestInput = XSDataBestv01Input()
        
        # Sample      
        xdDataAbsorbedDose = None
        xsDataSusceptibility = None
        
        # Could be None if sample has not been set
        # It could be not None in case Raddose has calculated an absorbed dose with default sample values
        if( _xsDataSample is not None ):
            xdDataAbsorbedDose = _xsDataSample.getAbsorbedDoseRate()
            xsDataSusceptibility = _xsDataSample.getSusceptibility()

        # Could be None if Raddose failed to calculate the absorbed dose
        if( xdDataAbsorbedDose is not None):
            xsDataBestInput.setCrystalAbsorbedDoseRate( _xsDataSample.getAbsorbedDoseRate() )
            
        xsDataBestInput.setCrystalSusceptibility( xsDataSusceptibility )
        
        # crystalShape
        # Default value is 1 (We assume that Xtal is smaller than beam)
        xsDataFloatCrystalShape = _xsDataSample.getShape()
        if( xsDataFloatCrystalShape is None ):
            xsDataBestInput.setCrystalShape( XSDataFloat( 1 ) )
        else:
            xsDataBestInput.setCrystalShape( xsDataFloatCrystalShape )
        
        # Detector
        xsDataBestInput.setDetectorType( _xsDataDetector.getType() )
        
        # Beam
        xsDataBestInput.setBeamExposureTime( _xsDataBeam.getExposureTime() )
        xsDataBestInput.setBeamMinExposureTime( _xsDataBeam.getMinExposureTimePerImage() )
        
        # Goniostat
        xsDataBestInput.setGoniostatMaxRotationSpeed( _xsDataGoniostat.getMaxOscillationSpeed() )
        xsDataBestInput.setGoniostatMinRotationWidth( _xsDataGoniostat.getMinOscillationWidth() )
        
        # Diffraction plan
        xsDataBestInput.setAimedResolution( _xsDataDiffractionPlan.getAimedResolution() )
        xsDataBestInput.setAimedRedundancy( _xsDataDiffractionPlan.getAimedMultiplicity() )
        xsDataBestInput.setAimedCompleteness( _xsDataDiffractionPlan.getAimedCompleteness() )
        xsDataBestInput.setAimedIOverSigma( _xsDataDiffractionPlan.getAimedIOverSigmaAtHighestResolution() )
        xsDataBestInput.setBeamMaxExposureTime( _xsDataDiffractionPlan.getMaxExposureTimePerDataCollection() )
        
        # Best Files
        xsDataBestInput.setBestFileContentDat( _oedStrBestFileContentDat )
        xsDataBestInput.setBestFileContentPar( _oedStrBestFileContentPar )
        xsDataBestInput.setBestFileContentHKL( _oedListBestFileContentHKL )
            
        return xsDataBestInput
    
    
    def getXSDataStrategyResult( self, _xsDataBestOutput, _xsDataExperimentalCondition, _xsDataSample ):
        """
        """
        xsDataStrategyResult = XSDataStrategyResult()
        xsDataCollectionStrategy = XSDataCollection()
        xsDataCollectionRuns = _xsDataBestOutput.getCollectionRun()
        
        for xsDataCollectionRun in xsDataCollectionRuns:
            xsDataSubWedge = XSDataSubWedge()
            strXmlStringDataExperimentalCondition = _xsDataExperimentalCondition.marshal()
            xsDataExperimentalCondition = XSDataExperimentalCondition.parseString( strXmlStringDataExperimentalCondition )
            xsDataExperimentalCondition.getBeam().setExposureTime( xsDataCollectionRun.getExposureTime() )
            xsDataExperimentalCondition.getDetector().setDistance( xsDataCollectionRun.getDistance() )
            xsDataExperimentalCondition.getGoniostat().setRotationAxisStart( xsDataCollectionRun.getPhiStart() )
            xsDataExperimentalCondition.getGoniostat().setOscillationWidth( xsDataCollectionRun.getPhiWidth() )
            fRotationAxisEnd = xsDataCollectionRun.getPhiStart().getValue() + xsDataCollectionRun.getNumberOfImages().getValue() * xsDataCollectionRun.getPhiWidth().getValue()
            xsDataExperimentalCondition.getGoniostat().setRotationAxisEnd( XSDataAngle( fRotationAxisEnd ) )
            xsDataSubWedge.setExperimentalCondition( xsDataExperimentalCondition )
            xsDataCollectionStrategy.addSubWedge( xsDataSubWedge )
            
        xsDataCollectionStrategy.setSample( _xsDataSample )
        
        xsDataStrategyResult.setCollectionStrategy( xsDataCollectionStrategy )
        xsDataStrategySummary = _xsDataBestOutput.getStrategySummary()
        xsDataStrategyResult.setStrategySummary( xsDataStrategySummary )
        
        xsDataStatistics = _xsDataBestOutput.getStatisticalPrediction()
        xsDataStrategyResult.setStatistics( xsDataStatistics )
        
        return xsDataStrategyResult