#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginBestv01.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                     import EDPluginExecProcessScript
from EDImport                     import EDVerbose
from EDImport                     import EDString
from EDImport                     import EDDiskExplorer
from EDImport                     import EDList
from EDImport                     import EDDict
from EDImport                     import EDMessage
from EDImport                     import EDUtilsFile
from EDImport                     import EDUtilsTable
from EDImport                     import EDConfiguration
from XSDataBestv01                import *
from XSDataDnaTables              import dna_tables


class EDPluginBestv01( EDPluginExecProcessScript ):
    """
    """
    
    def __init__( self ):
        """
        """
        EDPluginExecProcessScript.__init__( self )
        
        self.m_oedStrCONF_STRATEGY_COMPLEXITY_LABEL = "complexity"
        # Default value of strategy complexity
        self.m_oedStringConfigComplexity = EDString( "min" )
        
        self.m_oedStrBestHome = None
        self.m_oedStrCommandBestHome = None
        self.m_oedStrCommandBest = None
        
        self.m_oedStrExposureTime = None
        self.m_oedStrDoseRate = None
        self.m_oedStrDetectorType = None
        
        self.m_oedFileBestDat = None
        self.m_oedFileBestPar = None
        self.m_oedListFileBestHKL = EDList()
        
        
    def getConfigComplexity( self ):
        """
        """
        return self.m_oedStringConfigComplexity
    
    
    def setConfigComplexity( self, _oedStringConfigComplexity ):
        """
        """
        self.m_oedStringConfigComplexity = _oedStringConfigComplexity

    
    def getBestHome( self ):
        """
        """
        return self.m_oedStrBestHome
    
    
    def setBestHome( self, oedStrBestHome ):
        """
        """
        self.m_oedStrBestHome = oedStrBestHome
        
        
    def getCommandBestHome( self ):
        """
        """
        return self.m_oedStrCommandBestHome
    
    
    def setCommandBestHome( self, _oedStrCommandBestHome ):
        """
        """
        self.m_oedStrCommandBestHome = _oedStrCommandBestHome
    
    
    def getCommandBest( self ):
        """
        """
        return self.m_oedStrCommandBest
    
    
    def setCommandBest( self, _oedStrCommandBest ):
        """
        """
        self.m_oedStrCommandBest = _oedStrCommandBest
        
        
    def getFileBestDat( self ):
        """
        """
        return self.m_oedFileBestDat
    
        
    def setFileBestDat( self, _oedFileBestDat ):
        """
        """
        self.m_oedFileBestDat = _oedFileBestDat
        
        
    def getFileBestPar( self ):
        """
        """
        return self.m_oedFileBestPar
    
        
    def setFileBestPar( self, _oedFileBestPar ):
        """
        """
        self.m_oedFileBestPar = _oedFileBestPar
        
        
    def getListFileBestHKL( self ):
        """
        """
        return self.m_oedListFileBestHKL
    
        
    def setListFileBestHKL( self, _oedListFileBestHKL ):
        """
        """
        self.m_oedListFileBestHKL = _oedListFileBestHKL
    
    
    def configure( self ):
        """
        """
        EDPluginExecProcessScript.configure( self )
        EDVerbose.DEBUG( "*** EDPluginBestv01.configure")
        self.setRequireCCP4( True )
        oedStrScriptExecutable = self.getScriptExecutable()
        EDVerbose.DEBUG("*** EDPluginBestv01.configure: Script Executable: " + oedStrScriptExecutable )
        oedStrBestHome = EDDiskExplorer.getFolderName( oedStrScriptExecutable )
        self.setBestHome( oedStrBestHome )
        self.setCommandBestHome( "export besthome=" + self.getBestHome() )
        
        pluginConfiguration = self.getConfiguration()
        
        if( pluginConfiguration == None):
            warningMessage = EDMessage.WARNING_NO_PLUGIN_CONFIGURATION_ITEM_FOUND_02 % ('EDPluginBestv01.configure', self.getPluginName() )
            EDVerbose.warning( warningMessage )           
        else:
            oedStrConnfigComplexity = EDConfiguration.getStringParamValue( pluginConfiguration, self.m_oedStrCONF_STRATEGY_COMPLEXITY_LABEL )
            if( oedStrConnfigComplexity == None ):
                warningMessage = EDMessage.WARNING_NO_PARAM_CONFIGURATION_ITEM_FOUND_03 % ('EDPluginBestv01.configure', self.m_oedStrCONF_STRATEGY_COMPLEXITY_LABEL, self.getPluginName() )
                EDVerbose.warning( warningMessage )
            else:
                self.setConfigComplexity( EDString( oedStrConnfigComplexity ) )
        
                
    def preProcess( self, _oedObject = None ):
        """
        """
        EDPluginExecProcessScript.preProcess( self )
        
        self.setFileBestDat( EDDiskExplorer.mergePath( self.getWorkingDirectory(), "bestfile.dat" ))
        self.setFileBestPar( EDDiskExplorer.mergePath( self.getWorkingDirectory(), "bestfile.par" ))
        
        EDUtilsFile.writeFile( self.getFileBestDat(), self.m_oxsDataInput.getBestFileContentDat().getValue() )
        EDUtilsFile.writeFile( self.getFileBestPar(), self.m_oxsDataInput.getBestFileContentPar().getValue())
        
        listBestFileContentHKL = self.m_oxsDataInput.getBestFileContentHKL() 
        
        iterator = 0       
        for bestFileContentHKL in listBestFileContentHKL:
            iterator = iterator + 1
            bestFileHKL = EDDiskExplorer.mergePath( self.getWorkingDirectory(), "bestfile" + EDString( iterator ) + ".hkl" )
            self.m_oedListFileBestHKL.add( bestFileHKL )
            EDUtilsFile.writeFile( bestFileHKL, bestFileContentHKL.getValue() )

        self.initializeCommands()
        
        
    def initializeCommands( self ):
        """
        """
        self.addListCommandPreExecution( self.m_oedStrCommandBestHome )
        
        listFileBestHKL = self.getListFileBestHKL()
        listFileBestHKLCommand = ""
        
        for fileBestHKL in listFileBestHKL:
            listFileBestHKLCommand = listFileBestHKLCommand + fileBestHKL + " "
            
        oedStrDetectorName = EDString( self.m_oxsDataInput.getDetectorType().getValue() )
        oedStrExposureTime = EDString( self.m_oxsDataInput.getBeamExposureTime().getValue() )
        oedStrMaxExposureTime = EDString( self.m_oxsDataInput.getBeamMaxExposureTime().getValue() )
        
        self.m_oedStrCommandBest = EDString( "-f " + oedStrDetectorName + " " + \
                                   "-t " + oedStrExposureTime  + " " )
        
        if( self.m_oxsDataInput.getBeamMinExposureTime() is not None ):
            oedStrBeamMinExposureTime = EDString( self.m_oxsDataInput.getBeamMinExposureTime().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-M " + oedStrBeamMinExposureTime + " " )
            
        if( self.m_oxsDataInput.getGoniostatMaxRotationSpeed() is not None ):
            oedStrGoniostatMaxRotationSpeed = EDString( self.m_oxsDataInput.getGoniostatMaxRotationSpeed().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-S " + oedStrGoniostatMaxRotationSpeed + " " )
            
        if( self.m_oxsDataInput.getGoniostatMinRotationWidth() is not None ):
            oedStrGoniostatMinRotationWidth = EDString( self.m_oxsDataInput.getGoniostatMinRotationWidth().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-w " + oedStrGoniostatMinRotationWidth + " " )
        
        if( self.m_oxsDataInput.getAimedResolution() is not None ):
            oedStrAimedResolution = EDString( self.m_oxsDataInput.getAimedResolution().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-r " + oedStrAimedResolution + " " )
            
        if( self.m_oxsDataInput.getAimedRedundancy() is not None ):
            oedStrAimedRedundancy = EDString( self.m_oxsDataInput.getAimedRedundancy().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-R " + oedStrAimedRedundancy + " " )
            
        if( self.m_oxsDataInput.getAimedCompleteness() is not None ):
            oedStrAimedCompleteness = EDString( self.m_oxsDataInput.getAimedCompleteness().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-C " + oedStrAimedCompleteness + " " )
            
        if( self.m_oxsDataInput.getAimedIOverSigma() is not None ):
            oedStrAimedIOverSigma = EDString( self.m_oxsDataInput.getAimedIOverSigma().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-i2s " + oedStrAimedIOverSigma + " " )
                                   
        if( self.m_oxsDataInput.getCrystalAbsorbedDoseRate() is not None ):
            oedStrCrystalAbsorbedDoseRate = EDString( self.m_oxsDataInput.getCrystalAbsorbedDoseRate().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-GpS " + oedStrCrystalAbsorbedDoseRate + " " )
            
        if( self.m_oxsDataInput.getCrystalShape() is not None ):
            oedStrCrystalShape = EDString( self.m_oxsDataInput.getCrystalShape().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-sh " + oedStrCrystalShape + " " )
            
        if( self.m_oxsDataInput.getCrystalSusceptibility() is not None ):
            oedStrCrystalSusceptibility = EDString( self.m_oxsDataInput.getCrystalSusceptibility().getValue() )
            self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-su " + oedStrCrystalSusceptibility + " " )
            
                                              
        self.m_oedStrCommandBest = self.m_oedStrCommandBest + EDString( "-T " + oedStrMaxExposureTime + " " + \
                                     "-dna " + self.getScriptBaseName() + "_dnaTables.xml" + " " + \
                                     "-e " + self.getConfigComplexity() + " " + \
                                     "-mos " + EDString( self.getFileBestDat() + " " + self.getFileBestPar() + " " + listFileBestHKLCommand ))


                                              
        self.setScriptCommandline( self.m_oedStrCommandBest )
    
    
    def postProcess( self, _oedObject = None ):
        """
        """
        oedStringError = self.readProcessErrorLogFile()
        if( (oedStringError is not None) and ( oedStringError != EDString.empty() ) ):
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginBestv01.postProcess', 'EDPluginBestv01', oedStringError )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 

        outputData = self.getOutputDataFromDNATableFile( EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getScriptBaseName() + "_dnaTables.xml" ))
        self.setDataOutput( outputData )   
    
    
    def getOutputDataFromDNATableFile( self, _oedStringFileName ):
        """
        """
        xsDataBestv01Output = XSDataBestv01Output()
        
        oedStringDnaTablesXML = self.readProcessFile( _oedStringFileName )
        xsDataDnaTables = dna_tables.parseString( oedStringDnaTablesXML )
        # Loop through all the tables and fill in the relevant parts of xsDataBestOutput
        
        # SubWedges
        xsTableCollectionStrategy = EDUtilsTable.getTableFromTables( xsDataDnaTables, "data_collection_strategy" )
        xsCollectionRunList = EDUtilsTable.getListsFromTable( xsTableCollectionStrategy, "collection_run" )
        
        for xsCollectionRunItemList in xsCollectionRunList:
            xsDataCollectionRun = self.collectionRunItemListToCollectionRun( xsCollectionRunItemList )
            xsDataBestv01Output.addCollectionRun( xsDataCollectionRun )
            
        # Strategy Summary
        xsStrategySummaryItemList = EDUtilsTable.getListsFromTable( xsTableCollectionStrategy, "summary" )
        xsDataStrategySummary = self.strategySummaryItemListToStrategySummary( xsStrategySummaryItemList[0] )
        # Ranking Resolution
        # Not part of strategySummaryItemListToStrategySummary method since it is in the general_form part
        xsTableGeneralInform = EDUtilsTable.getTableFromTables( xsDataDnaTables, "general_inform" )
        xsRankingResolutionItemList = EDUtilsTable.getListsFromTable( xsTableGeneralInform, "ranking_resolution" )
        xsItemRankingResolution = EDUtilsTable.getItemFromList( xsRankingResolutionItemList[0], "dmin" )
        fRankingResolution = EDString( xsItemRankingResolution.getValueOf_() ).atof()
        xsDataStrategySummary.setRankingResolution( XSDataFloat( fRankingResolution ) )
        
        xsDataBestv01Output.setStrategySummary( xsDataStrategySummary )
        
        # Satistics
        xsTableStatisticalPrediction = EDUtilsTable.getTableFromTables( xsDataDnaTables, "statistical_prediction" )
        xsResolutionBinList = EDUtilsTable.getListsFromTable( xsTableStatisticalPrediction, "resolution_bin" )
        
        xsDataStatisticalPrediction = XSDataStatisticalPrediction()
        for xsResolutionBinItemList in xsResolutionBinList:
            xsDataResolutionBin = self.resolutionBinItemListToResolutionBin( xsResolutionBinItemList )
            xsDataStatisticalPrediction.addResolutionBin( xsDataResolutionBin )
            
        xsDataBestv01Output.setStatisticalPrediction( xsDataStatisticalPrediction )
        
        return xsDataBestv01Output
            
            
    def collectionRunItemListToCollectionRun( self, _xsCollectionRunItemList ):
        """
        """
        xsDataCollectionRun = XSDataCollectionRun()
        
        xsDataBeam = XSDataBeam()
        xsItemExposureTime = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "exposure_time" )
        fExposureTime = EDString( xsItemExposureTime.getValueOf_() ).atof()
        xsDataCollectionRun.setExposureTime( XSDataTime( fExposureTime ))
        
        xsDataDetector = XSDataDetector()
        xsItemDistance = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "distance" )
        fDistance = EDString( xsItemDistance.getValueOf_() ).atof()
        xsDataCollectionRun.setDistance( XSDataLength( fDistance ) )
        
        xsItemRotationAxisStart = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "phi_start" )
        fRotationAxisStart = EDString( xsItemRotationAxisStart.getValueOf_() ).atof()
        xsDataCollectionRun.setPhiStart( XSDataAngle( fRotationAxisStart ))
        
        xsNumberOfImages = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "number_of_images" )
        iNumberOfImages = EDString( xsNumberOfImages.getValueOf_() ).atoi()
        xsDataCollectionRun.setNumberOfImages( XSDataInteger( iNumberOfImages ))
        
        xsItemPhiWidth = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "phi_width" )
        fPhiWidth = EDString( xsItemPhiWidth.getValueOf_() ).atof()
        xsDataCollectionRun.setPhiWidth( XSDataAngle( fPhiWidth ))
        
        xsItemOverlaps = EDUtilsTable.getItemFromList(_xsCollectionRunItemList, "overlaps" )
        oedStringOverlaps = EDString( xsItemOverlaps.getValueOf_() )
        xsDataCollectionRun.setOverlaps( XSDataString( oedStringOverlaps ) )

        return xsDataCollectionRun
    
    
    def strategySummaryItemListToStrategySummary( self, _xsStrategySummaryItemList ):
        """
        """
        xsDataStrategySummary = XSDataStrategySummary()
        strItemAttenuation = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "attenuation" )
        fAttenuation = EDString( strItemAttenuation.getValueOf_() ).atof()
        xsDataStrategySummary.setAttenuation( XSDataFloat( fAttenuation ))
        
        strItemCompleteness = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "completeness" )
        # For homegeneity concerns, EDNA data model should store all the completeness value in fraction
        # ( DNA table xml file stores the summary strategy completeness in percentage whereas
        # the resolution bin completeness are in fraction )
        fCompleteness = EDString( strItemCompleteness.getValueOf_() ).atof() / 100
        xsDataStrategySummary.setCompleteness( XSDataFloat( fCompleteness ))
        
        strItemISigma = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "i_sigma" )
        fISigma = EDString( strItemISigma.getValueOf_() ).atof()
        xsDataStrategySummary.setISigma( XSDataFloat( fISigma ))
        
        strItemRedundancy = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "redundancy" )
        fRedundancy = EDString( strItemRedundancy.getValueOf_() ).atof()
        xsDataStrategySummary.setRedundancy( XSDataFloat( fRedundancy ))
        
        strItemResolution = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "resolution" )
        fResolution = EDString( strItemResolution.getValueOf_() ).atof()
        xsDataStrategySummary.setResolution( XSDataFloat( fResolution ))
        
        strItemResolutionReasoning = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "resolution_reasoning" )
        oedStringResolutionReasoning = EDString( strItemResolutionReasoning.getValueOf_())
        xsDataStrategySummary.setResolutionReasoning( XSDataString( oedStringResolutionReasoning ))
        
        strItemTotalDataCollectionTime = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "total_data_collection_time" )
        fTotalDataCollectionTime = EDString( strItemTotalDataCollectionTime.getValueOf_() ).atof()
        xsDataStrategySummary.setTotalDataCollectionTime( XSDataTime( fTotalDataCollectionTime ))
        
        strItemTotalExposureTime = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "total_exposure_time" )
        fTotalExposureTime = EDString( strItemTotalExposureTime.getValueOf_() ).atof()
        xsDataStrategySummary.setTotalExposureTime( XSDataTime( fTotalExposureTime ))
        
        return xsDataStrategySummary
    
    
    def resolutionBinItemListToResolutionBin( self, _xsResolutionBinItemList ):
        """
        """
        xsDataResolutionBin = XSDataResolutionBin()
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "min_resolution" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setMinResolution( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "max_resolution" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setMaxResolution( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "completeness" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setCompleteness( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "redundancy" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setRedundancy( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "average_intensity" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setAverageIntensity( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "average_error" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setAverageSigma( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "average_i_over_sigma" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setIOverSigma( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "R_factor" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setRFactor( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "Chi**2" )
        if ( strItem is not None ):
            fItem = EDString( strItem.getValueOf_() ).atof()
            xsDataResolutionBin.setChi2( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "average_i_over_sigma_Chi**2" )
        if ( strItem is not None ):
            fItem = EDString( strItem.getValueOf_() ).atof()
            xsDataResolutionBin.setIOverSigmaChi( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "fract_overload" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setPercentageOverload( XSDataFloat( fItem ))
        
        return xsDataResolutionBin
          
        
    def checkParameters( self ):
        """
        Checks the data input object
        """
        # Checks the mandatory parameters:
        self.checkMandatoryParameters( self.getDataInput().getBeamExposureTime(), "beamExposureTime")
        self.checkMandatoryParameters( self.getDataInput().getBeamMaxExposureTime(), "beamMaxExposureTime")
        self.checkMandatoryParameters( self.getDataInput().getDetectorType(), "detectorType")
        
        self.checkImportantParameters( self.getDataInput().getCrystalAbsorbedDoseRate(), "crystalDoseRate")
        self.checkImportantParameters( self.getDataInput().getCrystalShape(), "crystalShape")
            

    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPluginBestv01.setDataInput" )
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginBestv01.setDataInput: Input Data is of EDString " )
            self.m_oxsDataInput = XSDataBestv01Input.parseString( _dataInput )
        elif ( isinstance( _dataInput, XSDataBestv01Input ) ):
            self.m_oxsDataInput = _dataInput
        else:
            errorMessage = EDMessage.ERROR_WRONG_DATA_TYPE_02 % ('EDPluginBestv01.setDataInput', "XMLString or XSDataBestv01Input expected" )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )  
            raise RuntimeError, errorMessage
        
        
    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginBestv01.generateExecutiveSummary" )
        if ( self.getStringVersion() is not None ):
            self.addExecutiveSummaryLine( self.getStringVersion() )
            
        xsDataBestv01Output = self.getDataOutput()
        self.addExecutiveSummaryLine( "" )
        self.addExecutiveSummaryLine( "Plan for data collection:" )
        self.addExecutiveSummaryLine( " N |  Phi_start |  N.of.images | Rot.width |  Exposure | Distance |  Overlap |" )
        iIndex = 1
        iTotalImages = 0
        fTotalRotationRange = 0.0
        for xsDataCollectionRun in xsDataBestv01Output.getCollectionRun():
            fDistance = xsDataCollectionRun.getDistance().getValue()
            fExposureTime = xsDataCollectionRun.getExposureTime().getValue()
            iNumberOfImages = xsDataCollectionRun.getNumberOfImages().getValue()
            oedStringOverlaps = ""
            if ( xsDataCollectionRun.getOverlaps() is not None ):
                oedStringOverlaps = xsDataCollectionRun.getOverlaps().getValue()
            fPhiStart = xsDataCollectionRun.getPhiStart().getValue()
            fPhiWidth = xsDataCollectionRun.getPhiWidth().getValue()
            self.addExecutiveSummaryLine( "%2d | %8.2f   | %7d      |%9.2f  | %9.2f | %7.1f  | %7s  |" % \
                                          ( iIndex, fPhiStart, iNumberOfImages,\
                                            fPhiWidth, fExposureTime, fDistance, oedStringOverlaps ) )
            iIndex += 1
            iTotalImages += iNumberOfImages
            fTotalRotationRange += fPhiWidth * iNumberOfImages
        xsDataStrategySummary = xsDataBestv01Output.getStrategySummary()
        self.addExecutiveSummaryLine( "" )
        self.addExecutiveSummaryLine( "Attenuation                   : %6.1f" % xsDataStrategySummary.getAttenuation().getValue() )
        self.addExecutiveSummaryLine( "Total rotation range          : %6.1f [degrees]" % fTotalRotationRange )
        self.addExecutiveSummaryLine( "Total number of images        : %6d"   % iTotalImages )
        self.addExecutiveSummaryLine( "Total exposure time           : %6.1f [s]" % xsDataStrategySummary.getTotalExposureTime().getValue() )
        self.addExecutiveSummaryLine( "Total data collection time    : %6.1f [s]" % xsDataStrategySummary.getTotalDataCollectionTime().getValue() )
        self.addExecutiveSummaryLine( "Resolution                    : %6.1f [A]" % xsDataStrategySummary.getResolution().getValue() )
        self.addExecutiveSummaryLine( "Resolution reasoning          : %s"    % xsDataStrategySummary.getResolutionReasoning().getValue() )
        self.addExecutiveSummaryLine( "Ranking resolution            : %6.1f [A]" % xsDataStrategySummary.getRankingResolution().getValue() )
        self.addExecutiveSummaryLine( "" )
        self.addExecutiveSummaryLine( "Predicted statistics:" )
        self.addExecutiveSummaryLine( "Completeness                  : %6.1f [%%]" % ( xsDataStrategySummary.getCompleteness().getValue() * 100 ))
        self.addExecutiveSummaryLine( "I/sigma at highest resolution : %6.1f" % xsDataStrategySummary.getISigma().getValue() )
        self.addExecutiveSummaryLine( "Multiplicity                  : %6.1f" % xsDataStrategySummary.getRedundancy().getValue() )

        self.addExecutiveSummaryLine( "" )
        self.addExecutiveSummaryLine( "Statistics according to the plan:" )
        
        # Retrieve first resolution bin to check if Chi2 is present
        # Could arrive if Radiation damage is not estimated or double sweep strategy
        bChi2Present = False
        xsDataResolutionBin = xsDataBestv01Output.getStatisticalPrediction().getResolutionBin()[0]
        if( xsDataResolutionBin.getChi2() is not None ):
            bChi2Present = True
        
        if( bChi2Present == True ):
            self.addExecutiveSummaryLine( " Resolution  Compl.       Average     I/Sigma  I/Sigma Chi**2  R-fact  Overload")
            self.addExecutiveSummaryLine( "Lower Upper     %   Intensity  Sigma            /Chi             %        %")
            self.addExecutiveSummaryLine( "-------------------------------------------------------------------------------" )
        else:
            self.addExecutiveSummaryLine( " Resolution  Compl.       Average     I/Sigma   R-fact  Overload")
            self.addExecutiveSummaryLine( "Lower Upper     %   Intensity  Sigma               %        %")
            self.addExecutiveSummaryLine( "-----------------------------------------------------------------")
        
        for xsDataResolutionBin in xsDataBestv01Output.getStatisticalPrediction().getResolutionBin():
            fMinResolution =  xsDataResolutionBin.getMinResolution().getValue()
            fMaxResolution =  xsDataResolutionBin.getMaxResolution().getValue()
            fCompleteness = xsDataResolutionBin.getCompleteness().getValue()
            fAverageIntensity = xsDataResolutionBin.getAverageIntensity().getValue()
            fAverageSigma = xsDataResolutionBin.getAverageSigma().getValue()
            fIOverSigma = xsDataResolutionBin.getIOverSigma().getValue()
            if( xsDataResolutionBin.getIOverSigmaChi() is not None ):
                fIOverSigmaChi = xsDataResolutionBin.getIOverSigmaChi().getValue()
            if( xsDataResolutionBin.getChi2() is not None ):
                fChi2 = xsDataResolutionBin.getChi2().getValue()
            fRFactor = xsDataResolutionBin.getRFactor().getValue()
            fPercentageOverload = xsDataResolutionBin.getPercentageOverload().getValue()
            
            if( bChi2Present == True ):
                self.addExecutiveSummaryLine( "%5.2f %5.2f %6.1f %7.1f %9.1f %7.1f %7.1f %7.2f %7.1f %7.2f" % \
                                            ( fMinResolution, fMaxResolution, ( fCompleteness * 100 ), fAverageIntensity, fAverageSigma, fIOverSigma, \
                                              fIOverSigmaChi, fChi2, \
                                              fRFactor, fPercentageOverload) )
            else:
                self.addExecutiveSummaryLine( "%5.2f %5.2f %6.1f %7.1f %9.1f %7.1f %9.1f %9.2f" % \
                                            ( fMinResolution, fMaxResolution, ( fCompleteness * 100 ), fAverageIntensity, fAverageSigma, fIOverSigma, \
                                              fRFactor, fPercentageOverload) )    
        
            
            
            
            