#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDHandlerXSDataMOSFLMv01.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDObject
from EDImport import EDDictionary
from EDImport import EDUtilsImage
from EDImport import EDString
from EDImport import EDVerbose
from EDImport import EDDiskExplorer
from EDImport import EDMessage

from XSDataMOSFLMv01 import XSDataMOSFLMInputIndexing
from XSDataMOSFLMv01 import XSDataMOSFLMBeamPosition
from XSDataMOSFLMv01 import XSDataMOSFLMImage
from XSDataMOSFLMv01 import XSDataMOSFLMInputIntegration
from XSDataMOSFLMv01 import XSDataMOSFLMNewmat
from XSDataMOSFLMv01 import XSDataMOSFLMMissettingsAngles
from XSDataMOSFLMv01 import XSDataMOSFLMDetector
from XSDataMOSFLMv01 import XSDataMOSFLMInputGeneratePrediction

from XSDataCommonv01 import XSDataMatrix
from XSDataCommonv01 import XSDataCell
from XSDataCommonv01 import XSDataLength
from XSDataCommonv01 import XSDataAngle
from XSDataCommonv01 import XSDataInteger
from XSDataCommonv01 import XSDataFloat
from XSDataCommonv01 import XSDataString
from XSDataCommonv01 import XSDataCrystal
from XSDataCommonv01 import XSDataSpaceGroup
from XSDataCommonv01 import XSDataDetector
from XSDataCommonv01 import XSDataWavelength
from XSDataCommonv01 import XSDataImage

from XSDataCharacterisationv01 import XSDataIndexingResult
from XSDataCharacterisationv01 import XSDataIndexingSolution
from XSDataCharacterisationv01 import XSDataIndexingSolutionSelected
from XSDataCharacterisationv01 import XSDataOrientation
from XSDataCharacterisationv01 import XSDataStatisticsIndexing
from XSDataCharacterisationv01 import XSDataExperimentalCondition
from XSDataCharacterisationv01 import XSDataIntegrationSubWedgeResult
from XSDataCharacterisationv01 import XSDataStatisticsIntegration


class EDHandlerXSDataMOSFLMv01( EDObject ):
    """
    """
    def generateXSDataMOSFLMInputIndexing( _xsDataIndexingInput ):
        """
        """
        EDVerbose.DEBUG( "*** EDHandlerXSDataMOSFLMv01.generateXSDataMOSFLMInputIndexing")

        xsDataCollection            = _xsDataIndexingInput.getDataCollection()
        xsDataExperimentalCondition = _xsDataIndexingInput.getExperimentalCondition()
        xsDataCrystal               = _xsDataIndexingInput.getCrystal()
        xsDataSubWedgeList          = xsDataCollection.getSubWedge()
        xsDataMOSFLMInputIndexing   = XSDataMOSFLMInputIndexing()
                        
        xsDataBeam       = xsDataExperimentalCondition.getBeam()
        xsDataDetector   = xsDataExperimentalCondition.getDetector()
        xsDataGoniostat  = xsDataExperimentalCondition.getGoniostat()
        
        dWavelength           = xsDataBeam.getWavelength().getValue()
        dDistance             = xsDataDetector.getDistance().getValue()
        dBeamPositionX        = xsDataDetector.getBeamPositionX().getValue()
        dBeamPositionY        = xsDataDetector.getBeamPositionY().getValue()
          
        xsDataMOSFLMBeamPosition = XSDataMOSFLMBeamPosition()
        xsDataMOSFLMBeamPosition.setX( XSDataLength( dBeamPositionX ) )
        xsDataMOSFLMBeamPosition.setY( XSDataLength( dBeamPositionY ) )
        xsDataMOSFLMInputIndexing.setBeam( xsDataMOSFLMBeamPosition )
        
        xsDataMOSFLMDetector = EDHandlerXSDataMOSFLMv01.getXSDataMOSFLMDetector( xsDataDetector )
        xsDataMOSFLMInputIndexing.setDetector( xsDataMOSFLMDetector )
        
        xsDataMOSFLMInputIndexing.setWavelength( XSDataWavelength( dWavelength ) )
        xsDataMOSFLMInputIndexing.setDistance( XSDataLength( dDistance ) )

        xsDataSubWedgeFirst = xsDataSubWedgeList[0]
        xsDataImageFirst = xsDataSubWedgeFirst.getImage()[0]
        oedStringPath = EDString( xsDataImageFirst.getPath().getValue() )
        oedStringFileName = EDDiskExplorer.getFileName( oedStringPath )
        oedStringDirectory = EDDiskExplorer.getFolderName( oedStringPath )

        oedStringMOSFLMTemplate = EDUtilsImage.getTemplate( oedStringFileName, "#" )
        xsDataMOSFLMInputIndexing.setTemplate( XSDataString( oedStringMOSFLMTemplate ) )
        xsDataMOSFLMInputIndexing.setDirectory( XSDataString( oedStringDirectory ) )
                
        if ( xsDataCrystal is not None ):
            xsDataSpaceGroup = xsDataCrystal.getSpaceGroup()
            if ( xsDataSpaceGroup is not None ):
                xsDataStringName = xsDataSpaceGroup.getName()
                if ( xsDataStringName is not None ):
                    xsDataMOSFLMInputIndexing.setSymmetry( XSDataString( xsDataStringName.getValue() ) )      
                       
        # Loop through the list of sub wedges 
        
        for xsDataSubWedge in xsDataSubWedgeList:
            
            xsDataImageList = xsDataSubWedge.getImage()
            xsDataGoniostat = xsDataSubWedge.getExperimentalCondition().getGoniostat()
            fGonioStatOscillationStart = xsDataGoniostat.getRotationAxisStart().getValue()
            fGonioStatOscillationRange = xsDataGoniostat.getOscillationWidth().getValue()
            
            # First find the lowest image number
            iLowestImageNumber = None
            for xsDataImage in xsDataImageList:
                iImageNumber = xsDataImage.getNumber().getValue()
                if ( iLowestImageNumber is None ):
                    iLowestImageNumber = iImageNumber
                elif ( iImageNumber < iLowestImageNumber ):
                    iLowestImageNumber = iImageNumber
                                        
            # Loop through the list of images
            
            for xsDataImage in xsDataImageList:
                # Create the MOSFLM image object
                
                xsDataMOSFLMImage = XSDataMOSFLMImage()
                                                
                iImageNumber = xsDataImage.getNumber().getValue()
                xsDataMOSFLMImage.setNumber( XSDataInteger( iImageNumber ) )

                fImageOscillationStart = fGonioStatOscillationStart + ( iImageNumber - iLowestImageNumber )*fGonioStatOscillationRange
                xsDataMOSFLMImage.setRotationAxisStart( XSDataAngle( fImageOscillationStart ) )
                xsDataMOSFLMImage.setRotationAxisEnd( XSDataAngle( fImageOscillationStart + fGonioStatOscillationRange ) )
                
                xsDataMOSFLMInputIndexing.addImage( xsDataMOSFLMImage )
            
        #print xsDataMOSFLMInputIndexing.marshal()
        return xsDataMOSFLMInputIndexing
                
    generateXSDataMOSFLMInputIndexing = staticmethod( generateXSDataMOSFLMInputIndexing )
        
        
        
    def generateXSDataIndexingResult( _xsDataMOSFLMIndexingOutput, _xsDataExperimentalCondition=None ):
        EDVerbose.DEBUG( "*** EDHandlerXSDataMOSFLMv01.generateXSDataIndexingOutput")
        #print _xsDataMOSFLMIndexingOutput.marshal()
        xsDataMOSFLMBeamPositionRefined  = _xsDataMOSFLMIndexingOutput.getRefinedBeam()
        xsDataMOSFLMBeamPositionShift    = _xsDataMOSFLMIndexingOutput.getBeamShift()
        dDeviationAngular                = _xsDataMOSFLMIndexingOutput.getDeviationAngular().getValue()
        dDeviationPositional             = _xsDataMOSFLMIndexingOutput.getDeviationPositional().getValue()
        dMosaicityEstimation             = _xsDataMOSFLMIndexingOutput.getMosaicityEstimation().getValue()
        xsDataMOSFLMNewmat               = _xsDataMOSFLMIndexingOutput.getRefinedNewmat()
        dDistanceRefined                 = _xsDataMOSFLMIndexingOutput.getRefinedDistance().getValue()
        iSelectedSolution                = _xsDataMOSFLMIndexingOutput.getSelectedSolutionNumber().getValue()
        iSpotsTotal                      = _xsDataMOSFLMIndexingOutput.getSpotsTotal().getValue()
        iSpotsUsed                       = _xsDataMOSFLMIndexingOutput.getSpotsUsed().getValue()
        xsDataCellRefined                = _xsDataMOSFLMIndexingOutput.getRefinedNewmat().getRefinedCell()
        xsDataMatrixA                    = _xsDataMOSFLMIndexingOutput.getRefinedNewmat().getAMatrix()
        xsDataMatrixU                    = _xsDataMOSFLMIndexingOutput.getRefinedNewmat().getUMatrix()
        oedStringSelectedSpaceGroupName  = EDString( _xsDataMOSFLMIndexingOutput.getSelectedSolutionSpaceGroup().getValue() )
        iSelectedSpaceGroupNumber        = _xsDataMOSFLMIndexingOutput.getSelectedSolutionSpaceGroupNumber().getValue()

        xsDataIndexingResult = XSDataIndexingResult()
        xsDataIndexingSolutionSelected = None
        
        for possibleSolutions in _xsDataMOSFLMIndexingOutput.getPossibleSolutions():
            xsDataCrystal = XSDataCrystal()
            xsDataSpaceGroup = XSDataSpaceGroup()
            xsDataSpaceGroup.setName( XSDataString( possibleSolutions.getLattice().getValue() ) )
            xsDataCrystal.setSpaceGroup( xsDataSpaceGroup )
            xsDataCrystal.setCell( possibleSolutions.getCell() )
            xsDataIndexingSolution = XSDataIndexingSolution()
            xsDataIndexingSolution.setCrystal( xsDataCrystal )
            iIndex = possibleSolutions.getIndex().getValue()
            xsDataIndexingSolution.setNumber( XSDataInteger( iIndex ) )
            xsDataIndexingSolution.setPenalty( XSDataFloat( possibleSolutions.getPenalty().getValue() ) )
            xsDataIndexingResult.addSolution( xsDataIndexingSolution )
            if ( iIndex == iSelectedSolution ):
                xsDataIndexingSolutionSelected = XSDataIndexingSolutionSelected()
                xsDataIndexingSolutionSelected.setNumber( XSDataInteger( iIndex ) )
                xsDataIndexingSolutionSelected.setPenalty( XSDataFloat( possibleSolutions.getPenalty().getValue() ) )
          
        xsDataCrystalSelected = XSDataCrystal()
        xsDataSpaceGroupSelected = XSDataSpaceGroup()
        xsDataSpaceGroupSelected.setName( XSDataString( oedStringSelectedSpaceGroupName ) )
        xsDataSpaceGroupSelected.setITNumber( XSDataInteger( iSelectedSpaceGroupNumber ) )
        xsDataCrystalSelected.setSpaceGroup( xsDataSpaceGroupSelected )
        xsDataCrystalSelected.setCell( xsDataCellRefined ) 
        xsDataCrystalSelected.setMosaicity( XSDataFloat( dMosaicityEstimation ) )
        xsDataIndexingSolutionSelected.setCrystal( xsDataCrystalSelected )
        
        xsDataOrientation = XSDataOrientation()
        xsDataOrientation.setMatrixA( xsDataMatrixA )
        xsDataOrientation.setMatrixU( xsDataMatrixU )
        xsDataIndexingSolutionSelected.setOrientation( xsDataOrientation )
        
        xsDataStatisticsIndexing = XSDataStatisticsIndexing()
        
        xsDataStatisticsIndexing.setBeamPositionShiftX( XSDataLength( xsDataMOSFLMBeamPositionShift.getX().getValue() ) )
        xsDataStatisticsIndexing.setBeamPositionShiftY( XSDataLength( xsDataMOSFLMBeamPositionShift.getY().getValue() ) )
        xsDataStatisticsIndexing.setSpotDeviationAngular( XSDataAngle( dDeviationAngular ) )
        xsDataStatisticsIndexing.setSpotDeviationPositional( XSDataLength( dDeviationPositional ) )
        xsDataStatisticsIndexing.setSpotsUsed( XSDataInteger( iSpotsUsed ) )
        xsDataStatisticsIndexing.setSpotsTotal( XSDataInteger( iSpotsTotal ) )
        xsDataIndexingSolutionSelected.setStatistics( xsDataStatisticsIndexing)
        
        xsDataExperimentalConditionRefined = None
        if ( _xsDataExperimentalCondition is None):
            xsDataExperimentalConditionRefined = XSDataExperimentalCondition()
        else:
            # Copy the incoming experimental condition
            xmlExperimentalCondition = _xsDataExperimentalCondition.marshal()
            xsDataExperimentalConditionRefined = XSDataExperimentalCondition.parseString( xmlExperimentalCondition )
        
        xsDataDetector = xsDataExperimentalConditionRefined.getDetector()
        if ( xsDataDetector is None ):
            xsDataDetector = XSDataDetector()

        xsDataDetector.setBeamPositionX( XSDataLength( xsDataMOSFLMBeamPositionRefined.getX().getValue() ) )
        xsDataDetector.setBeamPositionY( XSDataLength( xsDataMOSFLMBeamPositionRefined.getY().getValue() ) )
        xsDataDetector.setDistance( XSDataLength( dDistanceRefined ) )

        xsDataExperimentalConditionRefined.setDetector( xsDataDetector )
        xsDataIndexingSolutionSelected.setExperimentalConditionRefined( xsDataExperimentalConditionRefined )
        
        xsDataIndexingResult.setSelectedSolution( xsDataIndexingSolutionSelected )
        
        return xsDataIndexingResult

    generateXSDataIndexingResult = staticmethod( generateXSDataIndexingResult )
        
        
    def generateXSDataMOSFLMInputIntegration( _xsDataIntegrationInput ):
        EDVerbose.DEBUG( "*** EDHandlerXSDataMOSFLMv01.generateXSDataMOSFLMInputIntegration")

        xsDataCollection                     = _xsDataIntegrationInput.getDataCollection()
        xsDataCrystalRefined                 = _xsDataIntegrationInput.getCrystalRefined()
        xsDataIndexingSolutionSelected       = _xsDataIntegrationInput.getSelectedIndexingSolution()
        xsDataExperimentalConditionRefined   = _xsDataIntegrationInput.getExperimentalConditionRefined()

        if ( xsDataCrystalRefined is None ):
            xsDataCrystalRefined = xsDataIndexingSolutionSelected.getCrystal()
                
        xsDataCollection             = _xsDataIntegrationInput.getDataCollection()
        xsDataSubWedge               = xsDataCollection.getSubWedge()[0]
        xsDataImageList              = xsDataSubWedge.getImage()
        xsDataImageFirst             = xsDataImageList[0]
        xsDataGoniostat              = xsDataSubWedge.getExperimentalCondition().getGoniostat()

        xsDataCrystal                = xsDataIndexingSolutionSelected.getCrystal()
        xsDataOrientation            = xsDataIndexingSolutionSelected.getOrientation()
        xsDataExperimentalCondition  = xsDataIndexingSolutionSelected.getExperimentalConditionRefined()
        xsDataDetector               = xsDataExperimentalCondition.getDetector()
        xsDataBeam                   = xsDataExperimentalCondition.getBeam()
        xsDataMatrixA                = xsDataOrientation.getMatrixA()
        xsDataMatrixU                = xsDataOrientation.getMatrixU()
        xsDataCell                   = xsDataCrystal.getCell()

        xsDataMOSFLMInputIntegration = XSDataMOSFLMInputIntegration()
        xsDataMOSFLMNewmat           = XSDataMOSFLMNewmat()
        xsDataMOSFLMNewmat.setRefinedCell( xsDataCell )
        xsDataMOSFLMNewmat.setAMatrix( xsDataMatrixA )
        xsDataMOSFLMNewmat.setUMatrix( xsDataMatrixU )
        xsDataMOSFLMInputIntegration.setMatrix( xsDataMOSFLMNewmat )

        xsDataMOSFLMMissettingsAngles = XSDataMOSFLMMissettingsAngles()
        xsDataMOSFLMMissettingsAngles.setPhix( XSDataAngle( 0.0 ) )
        xsDataMOSFLMMissettingsAngles.setPhiy( XSDataAngle( 0.0 ) )
        xsDataMOSFLMMissettingsAngles.setPhiz( XSDataAngle( 0.0 ) )
        xsDataMOSFLMNewmat.setMissettingAngles( xsDataMOSFLMMissettingsAngles )

        xsDataMOSFLMBeamPosition = XSDataMOSFLMBeamPosition()
        xsDataMOSFLMBeamPosition.setX( xsDataDetector.getBeamPositionX() )
        xsDataMOSFLMBeamPosition.setY( xsDataDetector.getBeamPositionY() )
        xsDataMOSFLMInputIntegration.setBeam( xsDataMOSFLMBeamPosition )

        xsDataMOSFLMInputIntegration.setMosaicity( xsDataCrystal.getMosaicity() )        
        xsDataMOSFLMInputIntegration.setSymmetry( xsDataCrystal.getSpaceGroup().getName() )
        
        oedStringPathFirst      = EDString( xsDataImageFirst.getPath().getValue() )
        oedStringDirectoryFirst = EDDiskExplorer.getFolderName( oedStringPathFirst )
        oedStringFilenameFirst  = EDDiskExplorer.getFileName( oedStringPathFirst )
        dOscillationRange       = xsDataGoniostat.getOscillationWidth().getValue()

        xsDataMOSFLMInputIntegration.setWavelength( xsDataBeam.getWavelength() )
        xsDataMOSFLMInputIntegration.setDistance( xsDataDetector.getDistance() )
        xsDataMOSFLMInputIntegration.setDirectory( XSDataString( oedStringDirectoryFirst ) )
        xsDataMOSFLMInputIntegration.setOscillationWidth( XSDataAngle( dOscillationRange ) )
        xsDataMOSFLMDetector = EDHandlerXSDataMOSFLMv01.getXSDataMOSFLMDetector( xsDataDetector )
        xsDataMOSFLMInputIntegration.setDetector( xsDataMOSFLMDetector )
        
        oedStringMOSFLMTemplate = EDUtilsImage.getTemplate( oedStringFilenameFirst, "#" )
        xsDataMOSFLMInputIntegration.setTemplate( XSDataString( oedStringMOSFLMTemplate ) )

        iImageStart = None
        iImageEnd = None
        for xsDataImage in xsDataImageList:   
            iImageNumber = xsDataImage.getNumber().getValue()
            if ( iImageStart is None ):
                iImageStart = iImageNumber
            elif ( iImageStart > iImageNumber ):
                iImageStart = iImageNumber
            if ( iImageEnd is None ):
                iImageEnd = iImageNumber
            elif ( iImageEnd < iImageNumber ):
                iImageEnd = iImageNumber

        xsDataMOSFLMInputIntegration.setImageStart( XSDataInteger( iImageStart ) )
        xsDataMOSFLMInputIntegration.setImageEnd( XSDataInteger( iImageEnd ) )                
        xsDataMOSFLMInputIntegration.setRotationAxisStart( xsDataGoniostat.getRotationAxisStart() )

        #print xsDataMOSFLMInputIntegration.marshal()
        
        return xsDataMOSFLMInputIntegration
      
    generateXSDataMOSFLMInputIntegration = staticmethod( generateXSDataMOSFLMInputIntegration )
    

    def generateXSDataMOSFLMInputGeneratePrediction( xsDataGeneratePredictionInput ):
        EDVerbose.DEBUG( "*** EDHandlerXSDataMOSFLMv01.generateXSDataMOSFLMInputGeneratePrediction")

        xsDataCollection                     = xsDataGeneratePredictionInput.getDataCollection()
        xsDataIndexingSolutionSelected       = xsDataGeneratePredictionInput.getSelectedIndexingSolution()
        xsDataExperimentalConditionRefined   = xsDataIndexingSolutionSelected.getExperimentalConditionRefined()
        
        xsDataCollection             = xsDataGeneratePredictionInput.getDataCollection()
        xsDataSubWedge               = xsDataCollection.getSubWedge()[0]
        xsDataImageList              = xsDataSubWedge.getImage()
        xsDataImageFirst             = xsDataImageList[0]
        xsDataGoniostat              = xsDataSubWedge.getExperimentalCondition().getGoniostat()

        xsDataCrystal                = xsDataIndexingSolutionSelected.getCrystal()
        xsDataOrientation            = xsDataIndexingSolutionSelected.getOrientation()
        xsDataExperimentalCondition  = xsDataIndexingSolutionSelected.getExperimentalConditionRefined()
        xsDataDetector               = xsDataExperimentalCondition.getDetector()
        xsDataBeam                   = xsDataExperimentalCondition.getBeam()
        xsDataMatrixA                = xsDataOrientation.getMatrixA()
        xsDataMatrixU                = xsDataOrientation.getMatrixU()
        xsDataCell                   = xsDataCrystal.getCell()

        xsDataMOSFLMInputGeneratePrediction = XSDataMOSFLMInputGeneratePrediction()
        xsDataMOSFLMNewmat           = XSDataMOSFLMNewmat()
        xsDataMOSFLMNewmat.setRefinedCell( xsDataCell )
        xsDataMOSFLMNewmat.setAMatrix( xsDataMatrixA )
        xsDataMOSFLMNewmat.setUMatrix( xsDataMatrixU )
        xsDataMOSFLMInputGeneratePrediction.setMatrix( xsDataMOSFLMNewmat )

        xsDataMOSFLMMissettingsAngles = XSDataMOSFLMMissettingsAngles()
        xsDataMOSFLMMissettingsAngles.setPhix( XSDataAngle( 0.0 ) )
        xsDataMOSFLMMissettingsAngles.setPhiy( XSDataAngle( 0.0 ) )
        xsDataMOSFLMMissettingsAngles.setPhiz( XSDataAngle( 0.0 ) )
        xsDataMOSFLMNewmat.setMissettingAngles( xsDataMOSFLMMissettingsAngles )

        xsDataMOSFLMBeamPosition = XSDataMOSFLMBeamPosition()
        xsDataMOSFLMBeamPosition.setX( xsDataDetector.getBeamPositionX() )
        xsDataMOSFLMBeamPosition.setY( xsDataDetector.getBeamPositionY() )
        xsDataMOSFLMInputGeneratePrediction.setBeam( xsDataMOSFLMBeamPosition )

        xsDataMOSFLMInputGeneratePrediction.setMosaicity( xsDataCrystal.getMosaicity() )        
        xsDataMOSFLMInputGeneratePrediction.setSymmetry( xsDataCrystal.getSpaceGroup().getName() )
        
        oedStringPathFirst      = EDString( xsDataImageFirst.getPath().getValue() )
        oedStringDirectoryFirst = EDDiskExplorer.getFolderName( oedStringPathFirst )
        oedStringFilenameFirst  = EDDiskExplorer.getFileName( oedStringPathFirst )
        dOscillationRange       = xsDataGoniostat.getOscillationWidth().getValue()

        xsDataMOSFLMInputGeneratePrediction.setWavelength( xsDataBeam.getWavelength() )
        xsDataMOSFLMInputGeneratePrediction.setDistance( xsDataDetector.getDistance() )
        xsDataMOSFLMInputGeneratePrediction.setDirectory( XSDataString( oedStringDirectoryFirst ) )
        xsDataMOSFLMDetector = EDHandlerXSDataMOSFLMv01.getXSDataMOSFLMDetector( xsDataDetector )
        xsDataMOSFLMInputGeneratePrediction.setDetector( xsDataMOSFLMDetector )
        
        oedStringMOSFLMTemplate = EDUtilsImage.getTemplate( oedStringFilenameFirst, "#" )
        xsDataMOSFLMInputGeneratePrediction.setTemplate( XSDataString( oedStringMOSFLMTemplate ) )

        # The MOSFLM plugin can only handle one image 
        
        xsDataImage = xsDataSubWedge.getImage()[0]
        xsDataGoniostat = xsDataSubWedge.getExperimentalCondition().getGoniostat()
            
        xsDataMOSFLMImage = XSDataMOSFLMImage()
                                        
        iImageNumber = xsDataImage.getNumber().getValue()
        xsDataMOSFLMImage.setNumber( XSDataInteger( iImageNumber ) )
        
        dOscillationStart = xsDataGoniostat.getRotationAxisStart().getValue()
        dOscillationRange = xsDataGoniostat.getOscillationWidth().getValue()
        xsDataMOSFLMImage.setRotationAxisStart( XSDataAngle( dOscillationStart ) )
        xsDataMOSFLMImage.setRotationAxisEnd( XSDataAngle( dOscillationStart + dOscillationRange ) )
                
        xsDataMOSFLMInputGeneratePrediction.setImage( xsDataMOSFLMImage )

        #print xsDataMOSFLMInputGeneratePrediction.marshal()
        
        return xsDataMOSFLMInputGeneratePrediction
      
    generateXSDataMOSFLMInputGeneratePrediction = staticmethod( generateXSDataMOSFLMInputGeneratePrediction )



    def generateXSDataIntegrationSubWedgeResult( _xsDataMOSFLMOutputIntegration, _xsDataExperimentalCondition=None ):
        EDVerbose.DEBUG( "*** EDHandlerXSDataMOSFLMv01.generateXSDataIntegrationInput")
        xsDataIntegrationSubWedgeResult = XSDataIntegrationSubWedgeResult()
        #print _xsDataMOSFLMOutputIntegration.marshal()
        xsDataExperimentalConditionRefined = None
        if ( _xsDataExperimentalCondition is None):
            xsDataExperimentalConditionRefined = XSDataExperimentalCondition()
        else:
            # Copy the incoming experimental condition
            xmlExperimentalCondition = _xsDataExperimentalCondition.marshal()
            xsDataExperimentalConditionRefined = XSDataExperimentalCondition.parseString( xmlExperimentalCondition )

        xsDataDetector = xsDataExperimentalConditionRefined.getDetector()
        if ( xsDataDetector is None ):
            xsDataDetector = XSDataDetector()
            
        xsDataIntegrationSubWedgeResult.setBestfilePar( XSDataString( _xsDataMOSFLMOutputIntegration.getBestfilePar().getValue() ) )
        xsDataIntegrationSubWedgeResult.setBestfileDat( XSDataString( _xsDataMOSFLMOutputIntegration.getBestfileDat().getValue() ) )
        xsDataIntegrationSubWedgeResult.setBestfileHKL( XSDataString( _xsDataMOSFLMOutputIntegration.getBestfileHKL().getValue() ) )
        
        oxsDataLengthRefinedDistance = _xsDataMOSFLMOutputIntegration.getRefinedDistance()
        if ( oxsDataLengthRefinedDistance is not None ):
            xsDataDetector.setDistance( oxsDataLengthRefinedDistance )

        xsDataMOSFLMBeamPositionRefined = _xsDataMOSFLMOutputIntegration.getRefinedBeam()
        if ( xsDataMOSFLMBeamPositionRefined is not None ):
            xsDataDetector.setBeamPositionX( XSDataLength( xsDataMOSFLMBeamPositionRefined.getX().getValue() ) )
            xsDataDetector.setBeamPositionY( XSDataLength( xsDataMOSFLMBeamPositionRefined.getY().getValue() ) )
            
        xsDataExperimentalConditionRefined.setDetector( xsDataDetector )
        xsDataIntegrationSubWedgeResult.setExperimentalConditionRefined( xsDataExperimentalConditionRefined )
                    
        xsDataStatisticsIntegration = XSDataStatisticsIntegration()
        xsDataStatisticsIntegration.setRMSSpotDeviation( XSDataLength( _xsDataMOSFLMOutputIntegration.getRMSSpotDeviation().getValue() ) )
        
        xsDataIntegrationSubWedgeResult.setStatistics( xsDataStatisticsIntegration )
        xsDataIntegrationSubWedgeResult.setExperimentalConditionRefined( xsDataExperimentalConditionRefined )
        return xsDataIntegrationSubWedgeResult
    generateXSDataIntegrationSubWedgeResult = staticmethod( generateXSDataIntegrationSubWedgeResult )
        

    def areInstrumentParametersOK( _xsDataImageList ):
        EDVerbose.DEBUG( "*** EDHandlerXSDataMOSFLMv01.areInstrumentParametersOK" )
        """
        Helper method for checking that the instrument parameters for a list of
        XSDataImage images haven't changed
        """
        
        returnValue = True

        xsDataImageFirst = _xsDataImageList[0]   
        
        xsDataInstrumentFirst = xsDataImageFirst.getInstrument()
        xsDataBeamFirst       = xsDataInstrumentFirst.getBeam()
        xsDataDetectorFirst   = xsDataInstrumentFirst.getDetector()
        xsDataGoniostatFirst  = xsDataInstrumentFirst.getGoniostat()
        
        dWavelengthFirst           = xsDataBeamFirst.getWavelength().getValue()
        dDistanceFirst             = xsDataDetectorFirst.getDistance().getValue()
        dBeamPositionXFirst        = xsDataDetectorFirst.getBeamPositionX().getValue()
        dBeamPositionYFirst        = xsDataDetectorFirst.getBeamPositionY().getValue()
        oedStringDetectorTypeFirst = xsDataDetectorFirst.getType().getValue()
        
        for xsDataImage in _xsDataImageList:
            # Check that the Instrument parameters hasn't changed
            xsDataInstrument = xsDataImage.getInstrument()
            xsDataBeam       = xsDataInstrument.getBeam()
            xsDataDetector   = xsDataInstrument.getDetector()
            xsDataGoniostat  = xsDataInstrument.getGoniostat()
            
            dWavelength           = xsDataBeam.getWavelength().getValue()
            dDistance             = xsDataDetector.getDistance().getValue()
            dBeamPositionX        = xsDataDetector.getBeamPositionX().getValue()
            dBeamPositionY        = xsDataDetector.getBeamPositionY().getValue()
            oedStringDetectorType = xsDataDetector.getType().getValue()
            
            if  ( EDHandlerXSDataMOSFLMv01.isDifferent( dWavelengthFirst, dWavelength ) ):
                print "Error : different wavelengths!"
                returnValue = False
            elif ( EDHandlerXSDataMOSFLMv01.isDifferent( dDistanceFirst, dDistance ) ):
                print "Error : different distances!"
                returnValue = False
            elif ( EDHandlerXSDataMOSFLMv01.isDifferent( dBeamPositionXFirst, dBeamPositionX ) ):
                print "Error : different beam position X!"
                returnValue = False
            elif ( EDHandlerXSDataMOSFLMv01.isDifferent( dBeamPositionYFirst, dBeamPositionY ) ):
                print "Error : different beam position Y!"
                returnValue = False
            elif ( EDHandlerXSDataMOSFLMv01.isDifferent( oedStringDetectorTypeFirst, oedStringDetectorType ) ):
                print "Error : different detectors!"
                returnValue = False
                
        return returnValue
        
    areInstrumentParametersOK = staticmethod( areInstrumentParametersOK )
        
        
    def isDifferent( _oedObject1, _oedObject2 ):
        EDVerbose.DEBUG( "*** EDHandlerXSDataMOSFLMv01.isDifferent")
        returnValue = True
        opyTypeObject1 = type( _oedObject1 )
        opyTypeObject2 = type( _oedObject2 )
        if ( opyTypeObject1 == opyTypeObject1 ):
            if ( opyTypeObject1 == int ):
                if ( _oedObject1 != _oedObject2 ):
                    returnValue = False
            elif ( opyTypeObject1 == str ) or ( opyTypeObject1 == type ( EDString( "a" ) ) ) or ( opyTypeObject1 == unicode ):
                if (  EDString( _oedObject1 ).equals( EDString( _oedObject2 ) ) ):
                    returnValue = False
            elif ( opyTypeObject1 == float ):
                if (  abs( _oedObject1 - _oedObject2 ) < 0.01 ) :
                    returnValue = False
        return returnValue
    isDifferent = staticmethod( isDifferent )
    

    def getXSDataMOSFLMDetector( _xsDataDetector ):
        xsDataMOSFLMDetector = XSDataMOSFLMDetector()
        oedStringDetectorType = EDString( _xsDataDetector.getType().getValue() )        
        if ( oedStringDetectorType.equals( "q4") or \
             oedStringDetectorType.equals( "q4-2x") or \
             oedStringDetectorType.equals( "q210") or \
             oedStringDetectorType.equals( "q210-2x") or \
             oedStringDetectorType.equals( "q315") or \
             oedStringDetectorType.equals( "q315-2x") ):
            xsDataMOSFLMDetector.setType( XSDataString( "ADSC" ) )
        elif ( oedStringDetectorType.equals( "mar165") or \
               oedStringDetectorType.equals( "mar225") ):
            xsDataMOSFLMDetector.setType( XSDataString( "MARCCD" ) )
        else:
            # This is a temporary solution for the exception problem pointed out in bug #43.
            # Instead of raising an exception with a known type we send the error message as a string.
            oedStringErrorMessage = "EDHandlerXSDataMOSFLMv01.getXSDataMOSFLMDetector: Unknown detector type : " + oedStringDetectorType
            raise Exception, oedStringErrorMessage          
        if ( _xsDataDetector.getNumberPixelX() is not None ):
            xsDataMOSFLMDetector.setNumberPixelX( _xsDataDetector.getNumberPixelX() )
        if ( _xsDataDetector.getNumberPixelY() is not None ):
            xsDataMOSFLMDetector.setNumberPixelY( _xsDataDetector.getNumberPixelY() )
        if ( _xsDataDetector.getPixelSizeX() is not None ):
            xsDataMOSFLMDetector.setPixelSizeX( _xsDataDetector.getPixelSizeX() )
        if ( _xsDataDetector.getPixelSizeY() is not None ):
            xsDataMOSFLMDetector.setPixelSizeY( _xsDataDetector.getPixelSizeY() )
        return xsDataMOSFLMDetector
    getXSDataMOSFLMDetector = staticmethod( getXSDataMOSFLMDetector )
    