#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginMOSFLMv01GeneratePrediction.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDUtilsTable
from EDImport import EDDiskExplorer
from EDImport import EDMessage
 
from EDPluginMOSFLMv01 import EDPluginMOSFLMv01

from XSDataMOSFLMv01 import XSDataMOSFLMInputGeneratePrediction
from XSDataMOSFLMv01 import XSDataMOSFLMOutputGeneratePrediction
from XSDataMOSFLMv01 import XSDataMOSFLMBeamPosition

from XSDataDnaTables import dna_tables

from XSDataCommonv01 import XSDataAngle
from XSDataCommonv01 import XSDataLength
from XSDataCommonv01 import XSDataInteger
from XSDataCommonv01 import XSDataFloat
from XSDataCommonv01 import XSDataString
from XSDataCommonv01 import XSDataCell
from XSDataCommonv01 import XSDataImage
from XSDataCommonv01 import XSDataString

class EDPluginMOSFLMv01GeneratePrediction( EDPluginMOSFLMv01 ):


    def __init__( self ):
        EDPluginMOSFLMv01.__init__( self )
        self.m_oxsdStringPredictionImageFileName = EDString( "EDPluginMOSFLMv01GeneratePrediction.jpg" )


    def process( self, _oedObject = None ):
        EDPluginMOSFLMv01.process( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.process")

    
    def preProcess( self, _oedObject = None ):
        EDPluginMOSFLMv01.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.preProcess")
        self.generateMOSFLMCommands()


    def postProcess( self, _oedObject = None ):
        EDPluginMOSFLMv01.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.postProcess")
        xsDataMOSFLMOutputGeneratePrediction = self.createDataMOSFLMOutputGeneratePrediction()
        self.setDataOutput( xsDataMOSFLMOutputGeneratePrediction )


    def configure( self ):
        EDPluginMOSFLMv01.configure( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.configure")   
        oxsPluginItem = self.getConfiguration()
        if ( oxsPluginItem == None ):
            EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.configure: oxsPluginItem is None")                 
        self.createPredictionImageFileName()


    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.setDataInput" )
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.setDataInput: Input Data is of EDString " )
            self.m_oxsDataInput = XSDataMOSFLMInputGeneratePrediction.parseString( _dataInput )
        elif ( isinstance( _dataInput, XSDataMOSFLMInputGeneratePrediction ) ):
            self.m_oxsDataInput = _dataInput
        else:
            errorMessage = EDMessage.ERROR_WRONG_DATA_TYPE_02 % ('EDPluginMOSFLMv01GeneratePrediction.setDataInput', "XMLString or XSDataMOSFLMInputGeneratePrediction expected" )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage


    def checkParameters( self ):
        """
        Checks the mandatory parameters for MOSLFM indexing
        """
        EDPluginMOSFLMv01.checkParameters( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.checkParameters")
        self.checkMandatoryParameters( self.getDataInput().getImage(), "image")

    
    def getPredictionImageFileName( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.getPredictionImageFileName" )
        return EDString( self.m_oxsdStringPredictionImageFileName )
        
        
    def setPredictionImageFileName( self, _oedStringFileName ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.setPredictionImageFileName : " + _oedStringFileName )
        self.m_oxsdStringPredictionImageFileName = EDString( _oedStringFileName )

    
    def createPredictionImageFileName( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.createPredictionImagePath" )
        oedStringImageFileName = self.getBaseName() + "_image.jpg" 
        self.setPredictionImageFileName( oedStringImageFileName )

        
    def generateMOSFLMCommands( self ):
        """
        This method creates a list of MOSFLM integration commands given a valid
        XSDataMOSFLMInputGeneratePrediction as self.m_oxsDataInput
        """
        EDPluginMOSFLMv01.generateMOSFLMCommands( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.generateMOSFLMCommands")

        oxsDataMOSFLMInputGeneratePrediction = self.getDataInput()

        if ( oxsDataMOSFLMInputGeneratePrediction is not None):
                                    
            xsDataMOSFLMImage  = oxsDataMOSFLMInputGeneratePrediction.getImage()
            iImageNumber       = xsDataMOSFLMImage.getNumber().getValue()
            fRotationAxisStart = xsDataMOSFLMImage.getRotationAxisStart().getValue()
            fRotationAxisEnd   = xsDataMOSFLMImage.getRotationAxisEnd().getValue()
            self.addListCommandExecution( "XGUI ON" )
            self.addListCommandExecution( "IMAGE %d PHI %f TO %f" % ( iImageNumber, fRotationAxisStart, fRotationAxisEnd ) )
            self.addListCommandExecution( "GO" )
            self.addListCommandExecution( "PREDICT_SPOTS" )
            self.addListCommandExecution( "CREATE_IMAGE PREDICTION ON BINARY TRUE FILENAME %s"  % ( self.getPredictionImageFileName() ) )
            self.addListCommandExecution( "RETURN" )
            self.addListCommandExecution( "EXIT" )
            
        EDVerbose.DEBUG( "*** Finished EDPluginMOSFLMv01GeneratePrediction.generateMOSFLMCommands")

            
    def createDataMOSFLMOutputGeneratePrediction( self ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01GeneratePrediction.createDataMOSFLMOutputIntegration")
        xsDataMOSFLMInputGeneratePrediction = self.getDataInput()
        xsDataMOSFLMImage  = xsDataMOSFLMInputGeneratePrediction.getImage()
        iImageNumber = xsDataMOSFLMImage.getNumber().getValue()
        xsDataMOSFLMOutputGeneratePrediction = XSDataMOSFLMOutputGeneratePrediction()
        xsDataImage = XSDataImage()
        xsDataImage.setNumber( XSDataInteger( iImageNumber ) )
        xsDataImage.setPath( XSDataString( EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getPredictionImageFileName() ) ) )
        xsDataMOSFLMOutputGeneratePrediction.setPredictionImage( xsDataImage )
        return xsDataMOSFLMOutputGeneratePrediction
    