#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginMessage.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport       import EDPlugin
from EDImport       import EDVerbose
from EDImport       import EDString
from EDImport       import EDConfiguration
from XSDataBasicv01 import *


class EDPluginMessage( EDPlugin ):
    """
    A Configurable Plugin class that prints a Message on the screen
    """
    
    def __init__ ( self  ):
        """
        """
        EDPlugin.__init__( self )
        self.m_oedStrMESSAGE_LABEL = "message"
        self.m_oedStrCONF_WORKING_DIR_LABEL = "workingDir"
        
        self.m_oedStrMessage = "This is the default EDPluginMessage Message"
        self.m_oedStrWorkingDir = "Default location"
        self.m_oedStrResponse = "Sorry... Try Again!!!"
        
        
    def setResponse( self, oedStrResponse):
        """
        """
        self.m_oedStrResponse = oedStrResponse 
        
        
    def getResponse( self ):
        """
        """
        return self.m_oedStrResponse
        
    
    def setMessage( self, oedStrMessage):
        """
        """
        self.m_oedStrMessage = oedStrMessage
        
    
    def getMessage( self ):
        """
        """
        return self.m_oedStrMessage
    
        
    def setWorkingDir( self, oedStrWorkingDir):
        """
        """
        self.m_oedStrWorkingDir = oedStrWorkingDir
        

    def getWorkingDir( self ):
        """
        """
        return self.m_oedStrWorkingDir
        
        
    def preProcess( self, _oalObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPlugin.preProcess( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginMessage.preProcess...")
                
            
    def process( self, _oalObject = None ):
        """
        Prints a Message on the screen
        """
        EDPlugin.process( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginMessage.process...")
        EDVerbose.screen( "*** EDPluginMessage.process: Configuration working dir: " + self.m_oedStrWorkingDir )
        EDVerbose.screen( "*** EDPluginMessage.process: Data Message: " + self.m_oedStrMessage )
        
        if( self.getMessage() == "This is the secret key" ):
            self.setResponse("OK!!! Got your Message")
            self.addWarningMessage( "*** EDPluginMessage.process: Someone Found the Secret Key" )
            
    
        
    def postProcess( self, _oalObject = None ):
        """
        Prints a Message on the screen
        """
        EDPlugin.postProcess( self, _oalObject )
#        self.addErrorMessage("PluginMessage: Big Problem" )
#        raise "PluginMessage: Big Problem"
        EDVerbose.DEBUG( "*** EDPluginMessage.postProcess...")
        xsParamItem = XSParamItem( "response", self.getResponse())
        xsParamItems = []
        xsParamItems.append(xsParamItem)
        xsDataBasic = XSDataBasic( XSParamList( xsParamItems ))
        self.setDataOutput( xsDataBasic )
        
        EDVerbose.DEBUG( "*** EDPluginMessage.postProcess...: " + EDPluginMessage.getStringParamValueData( xsDataBasic, "response"))
        
        
    def configure( self ):
        EDPlugin.configure(self)
        EDVerbose.DEBUG( "*** EDPluginMessage.configure...")
        pluginConfiguration = self.getConfiguration()
        
        if( pluginConfiguration == None):
            EDVerbose.warning( "!!! No configuration plugin item found for: " + self.getPluginName() + "... letting default..." )
        else:
            oedStrWorkingDir = EDConfiguration.getStringParamValue( pluginConfiguration, self.m_oedStrCONF_WORKING_DIR_LABEL )
            if( oedStrWorkingDir == None ):
                EDVerbose.warning( "!!! No configuration parameter found for: " + self.getPluginName() + "... letting default..." )
            else:
                self.setWorkingDir( oedStrWorkingDir )
                
                
    def checkParameters( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginMessage.checkParameters...")
        if(( self.getMessage() == None ) | ( len( self.getMessage())==0 )):
            EDVerbose.warning( "!!! The Message None or Empy ")
    
        
    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPluginMessage.setDataInput " )
        
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginMessage.setDataInput: Input Data is of EDString" )
            self.m_oxsDataInput = XSDataBasic.parseString( _dataInput )
        elif( isinstance( _dataInput, XSDataBasic ) ):
            EDVerbose.DEBUG( "*** EDPluginMessage.setDataInput: Input Data is of XSDataBasic" )
        else:
            EDVerbose.ERROR( "*** EDPluginMessage.setDataInput: Unexpected type for Input Data" )
            
        if( _dataInput == None):
            EDVerbose.warning( "!!! No data found for: " + self.getPluginName() )
        else:
            oedStrMessage = EDPluginMessage.getStringParamValueData( self.getDataInput(), self.m_oedStrMESSAGE_LABEL )
            if( oedStrMessage == None ):
                EDVerbose.warning( "!!! No data found for: " + self.getPluginName() + self.m_oedStrMESSAGE_LABEL )
            else:
                self.setMessage( oedStrMessage )
            
            
    def getStringParamValueData( _xsDataBasic, _strParamName ):
        """
        Returns the parameter value in a string format
        Should place in a proper Data Handler Class
        """
        oxsParamList = _xsDataBasic.getXSParamList()
        oxsParamItems = oxsParamList.getXSParamItem()
        paramValue = None
        
        for oxsParamItem in oxsParamItems:
            if ( oxsParamItem.getName() == _strParamName ):
                paramValue = oxsParamItem.getValue()
                
        return paramValue
    getStringParamValueData = staticmethod( getStringParamValueData )

            

        
        