#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginRaddosev01.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                     import EDPluginExecProcessScript
from EDImport                     import EDVerbose
from EDImport                     import EDString
from EDImport                     import EDList
from EDImport                     import EDDict
from EDImport                     import EDMessage
from EDImport                     import EDUtilsFile
from EDImport                     import EDDiskExplorer
from XSDataRaddosev01             import *


class EDPluginRaddosev01( EDPluginExecProcessScript ):
    """
    """
    
    m_oedStrABSORBED_DOSE = "Dose in Grays"
    m_oedStrSOLVENT = "Solvent Content (%)"
    
    def __init__( self ):
        """
        """
        EDPluginExecProcessScript.__init__( self )
        self.m_oedStrCommandBeamSize = None
        self.m_oedStrCommandBeamFlux = None
        self.m_oedStrCommandBeamWavelength = None
        
        self.m_oedStrCommandCrystalSize = None
        self.m_oedStrCommandCrystalCell = None
        self.m_oedStrCommandCrystalNRES = None
        self.m_oedStrCommandCrystalNMON = None
        self.m_oedStrCommandCrystalNDNA = None
        self.m_oedStrCommandCrystalNRNA = None
        self.m_oedStrCommandCrystalPATM = None
        self.m_oedStrCommandCrystalSATM = None
        
        self.m_oedStrCommandExposureTime = None
        self.m_oedStrCommandImages = None
        
        self.m_oedListCommandsRaddose = EDList()
        
        self.m_oedDictResults = EDDict()
        self.m_oedDictResults.add( EDPluginRaddosev01.m_oedStrABSORBED_DOSE, None )
        self.m_oedDictResults.add( EDPluginRaddosev01.m_oedStrSOLVENT, None )
        
        self.m_fSolvent = None
        self.m_fTimeToReachHendersonLimit = None


    def getCommandCrystalSize( self ):
        """
        """
        return self.m_oedStrCommandCrystalSize
    
    
    def getCommandCrystalCell( self ):
        """
        """
        return self.m_oedStrCommandCrystalCell
    
    
    def getCommandCrystalNRES( self ):
        """
        """
        return self.m_oedStrCommandCrystalNRES
    
    
    def getCommandCrystalNMON( self ):
        """
        """
        return self.m_oedStrCommandCrystalNMON
    
    
    def getCommandCrystalNDNA( self ):
        """
        """
        return self.m_oedStrCommandCrystalNDNA
    
    
    def getCommandCrystalNRNA( self ):
        """
        """
        return self.m_oedStrCommandCrystalNRNA
          
        
    def getCommandCrystalPATM( self ):
        """
        """
        return self.m_oedStrCommandCrystalPATM
    
    
    def getCommandCrystalSATM( self ):
        """
        """
        return self.m_oedStrCommandCrystalSATM
    
    
    def getCommandBeamSize( self ):
        """
        """
        return self.m_oedStrCommandBeamSize
    
    
    def getCommandBeamWavelength( self ):
        """
        """
        return self.m_oedStrCommandBeamWavelength


    def getCommandBeamFlux( self ):
        """
        """
        return self.m_oedStrCommandBeamFlux
    
    
    def getCommandBeamWavelength( self ):
        """
        """
        return self.m_oedStrCommandBeamWavelength
    
    
    def getCommandExposureTime( self ):
        """
        """
        return self.m_oedStrCommandExposureTime
    
    
    def getCommandImages( self ):
        """
        """
        return self.m_oedStrCommandImages
    
    
    def getListCommandsRaddose( self ):
        """
        """
        return self.m_oedListCommandsRaddose
    
    
    def configure( self ):
        EDPluginExecProcessScript.configure( self )
        EDVerbose.DEBUG( "*** EDPluginRaddosev01.configure")
        self.setRequireCCP4( True )
    
    
    def preProcess( self, _oedObject = None ):
        """
        """
        EDPluginExecProcessScript.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginRaddosev01.preProcess")
        self.initializeCommandExecution()
        
        
    def initializeCommandExecution( self ):
        """
        """
        self.m_oedStrCommandBeamSize = self.sizeToCommand( self.m_oxsDataInput.getBeamSize(), "BEAM")
        self.addListCommandExecution (self.m_oedStrCommandBeamSize )
        self.m_oedStrCommandBeamFlux = self.singleValueToCommand( self.m_oxsDataInput.getBeamFlux(), "PHOSEC")
        self.addListCommandExecution( self.m_oedStrCommandBeamFlux )
        self.m_oedStrCommandBeamWavelength = self.singleValueToCommand( self.m_oxsDataInput.getBeamWavelength(), "WAVELENGTH")
        self.addListCommandExecution( self.m_oedStrCommandBeamWavelength )
        
        self.m_oedStrCommandCrystalSize = self.sizeToCommand( self.m_oxsDataInput.getCrystalSize(), "CRYSTAL")
        self.addListCommandExecution( self.m_oedStrCommandCrystalSize )
        self.m_oedStrCommandCrystalNRES = self.singleValueToCommand( self.m_oxsDataInput.getCrystalNRES(), "NRES")
        self.addListCommandExecution( self.m_oedStrCommandCrystalNRES )
        self.m_oedStrCommandCrystalNMON = self.singleValueToCommand( self.m_oxsDataInput.getCrystalNMON(), "NMON")
        self.addListCommandExecution( self.m_oedStrCommandCrystalNMON )
        self.m_oedStrCommandCrystalNDNA = self.singleValueToCommand( self.m_oxsDataInput.getCrystalNDNA(), "NDNA")
        self.addListCommandExecution( self.m_oedStrCommandCrystalNDNA )
        self.m_oedStrCommandCrystalNRNA = self.singleValueToCommand( self.m_oxsDataInput.getCrystalNRNA(), "NRNA")
        self.addListCommandExecution( self.m_oedStrCommandCrystalNRNA )
        self.m_oedStrCommandCrystalPATM = self.atomCompositionToCommand( self.m_oxsDataInput.getCrystalPATM(), "PATM")
        self.addListCommandExecution( self.m_oedStrCommandCrystalPATM )
        self.m_oedStrCommandCrystalSATM = self.atomCompositionToCommand( self.m_oxsDataInput.getCrystalSATM(), "SATM")
        self.addListCommandExecution( self.m_oedStrCommandCrystalSATM )
        self.m_oedStrCommandCrystalCell = self.cellToCommand( self.m_oxsDataInput.getCrystalCell(), "CELL")
        self.addListCommandExecution( self.m_oedStrCommandCrystalCell )
        
        self.m_oedStrCommandExposureTime = self.singleValueToCommand( self.m_oxsDataInput.getBeamExposureTime(), "EXPOSURE")
        self.addListCommandExecution( self.m_oedStrCommandExposureTime )
        self.m_oedStrCommandImages = self.singleValueToCommand( self.m_oxsDataInput.getNumberOfImages(), "IMAGES")
        self.addListCommandExecution( self.m_oedStrCommandImages )
        
        self.addListCommandExecution( EDString("END") )
        
            
    
    def postProcess( self, _oedObject = None ):
        """
        """
        
        oedStringRaddoseError = self.readProcessErrorLogFile()
        if( (oedStringRaddoseError is not None) and ( oedStringRaddoseError != EDString.empty() ) ):
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginRaddosev01.postProcess', 'EDPluginRaddosev01', oedStringRaddoseError )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 
        
        resultDict = self.analyseScriptLogFileName( EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getScriptLogFileName() ) )

        oedStringSolvent = self.m_oedDictResults.getValue( EDPluginRaddosev01.m_oedStrSOLVENT )
        oedStringAborbedDose = self.m_oedDictResults.getValue( EDPluginRaddosev01.m_oedStrABSORBED_DOSE )
        
        if( oedStringSolvent is None or oedStringAborbedDose is None):
            errorMessage = EDMessage.ERROR_RADDOSEv01_02 % ('EDPluginRaddosev01.postProcess', "No Result Keyword found in Raddose Log see: " + self.getScriptLogFileName() )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 
        
        self.m_fSolvent = float( oedStringSolvent )
        if( self.m_fSolvent < 25 or self.m_fSolvent > 75 ):
            warningMessage = EDMessage.WARNING_RADDOSEv01_CONSISTENCY_PROBLEM_02 % ('EDPluginRaddosev01.postProcess', "Inconsistent solvent percentage value: " + oedStringSolvent )
            EDVerbose.warning( warningMessage )
            self.addWarningMessage( warningMessage )
            
        oedStringAborbedDose = self.m_oedDictResults.getValue( EDPluginRaddosev01.m_oedStrABSORBED_DOSE )
        
        iNumberOfImages = self.m_oxsDataInput.getNumberOfImages().getValue()
        fExposureTimePerImageInSec = self.m_oxsDataInput.getBeamExposureTime().getValue()
        fTotalExposureTime = iNumberOfImages * fExposureTimePerImageInSec
        fAbsorbedDoseSpeed = float( oedStringAborbedDose ) / fTotalExposureTime
        xsDataAbsorbedDoseRate = XSDataAbsorbedDoseRate( fAbsorbedDoseSpeed )
        
        xsDataAbsorbedDose = XSDataFloat( float( oedStringAborbedDose ))
        
        self.m_fTimeToReachHendersonLimit = 2e7 / fAbsorbedDoseSpeed
        
        xsDataRaddosev01Output = XSDataRaddosev01Output()
        xsDataRaddosev01Output.setAbsorbedDose( xsDataAbsorbedDose )
        xsDataRaddosev01Output.setAbsorbedDoseRate( xsDataAbsorbedDoseRate )

        self.setDataOutput( xsDataRaddosev01Output )
        
        
    def analyseScriptLogFileName( self, _oedStringFileName ):
        """
        """
        resultDict = None
        
        f = None
        try:
            f = open( _oedStringFileName )
        except:
            errorMessage = EDMessage.ERROR_CANNOT_READ_FILE_02 % ('EDPluginRaddosev01.analyseScriptLogFileName', self.getScriptLogFileName() )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )    
            raise RuntimeError, errorMessage     
        
        for line in f.xreadlines():
            edStringLine = EDString( line ).replace('\n', "")
            resultDict = self.addResultToDict( edStringLine )   
            
        return resultDict
            
                    
    def addResultToDict( self, _oedStringLine ):
        """
        """
        for key in self.m_oedDictResults.keys():
            if ( _oedStringLine.contains( key )) :
                tokenList = _oedStringLine.split(" ")
                value = EDString( tokenList[-1] )
                self.m_oedDictResults.setValue( EDString( key ), value )
                
        return self.m_oedDictResults
                
        
    def checkParameters( self ):
        """
        Checks the data input object
        """
        # Checks the mandatory parameters:

        self.checkMandatoryParameters( self.getDataInput(), "Data Input is None")
        self.checkMandatoryParameters( self.getDataInput().getCrystalCell(), "crystalCell")
        self.checkMandatoryParameters( self.getDataInput().getBeamWavelength(), "beamWavelength")
        self.checkMandatoryParameters( self.getDataInput().getBeamFlux(), "beamFlux")
        self.checkMandatoryParameters( self.getDataInput().getBeamSize(), "beamSize")
        
        #self.checkMandatoryParameters( self.getDataInput().getCrystalSize(), "crystalSize")
        # Default is 0.1 0.1 0.1

        # Default is 0.1 0.1
        # self.checkMandatoryParameters( self.getDataInput().getBeamExposureTime(), "beamExposureTime")
        # self.checkMandatoryParameters( self.getDataInput().getNumberOfImages(), "numberOfImages")

        self.checkImportantParameters( self.getDataInput().getCrystalNMON(), "NMON (Check your Crystal chemical composition consistency)" )
              

    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPluginRaddosev01.setDataInput" )
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginRaddosev01.setDataInput: Input Data is of EDString "  )
            self.m_oxsDataInput = XSDataRaddosev01Input.parseString( _dataInput )
        elif ( isinstance( _dataInput, XSDataRaddosev01Input ) ):
            self.m_oxsDataInput = _dataInput
        else:
            errorMessage = EDMessage.ERROR_WRONG_DATA_TYPE_02 % ('EDPluginRaddosev01.setDataInput', "XMLString or XSDataRaddosev01Input expected" )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage ) 
            raise RuntimeError, errorMessage
        
        
    def singleValueToCommand( self, _xsSimpleObject, _strKeyword ):
        """
        """
        command = None   
        if( _xsSimpleObject is not None ):
            command = _strKeyword
            strValue = EDString( _xsSimpleObject.getValue() )
            command = EDString( command + " " + strValue )
            
        return command
    
    
    def atomCompositionToCommand( self, _xsAtomComposition, _strKeyword ):
        """
        """
        command = None 
        if( _xsAtomComposition is not None ):
            command = _strKeyword
            atomList = _xsAtomComposition.getAtom()
            
            for atom in atomList:
                strAtomSymbol = EDString( atom.getSymbol().getValue() )
                strAtomAmount = None
                if( atom.getNumberOf() is not None ):
                    strAtomAmount = EDString( atom.getNumberOf().getValue() )
                elif( atom.getConcentration() is not None ):
                    strAtomAmount = EDString( atom.getConcentration().getValue() )
                command = EDString( command + " " + strAtomSymbol + " " + strAtomAmount )
            
        return command
    
    
    def sizeToCommand( self, _xsSize, _strKeyword ):
        """
        """
        command = None 
        if( _xsSize is not None ):
            command = _strKeyword
            strX = EDString( _xsSize.getX().getValue() )
            strY = EDString( _xsSize.getY().getValue() )
            
            strZ = None
            # if it is a 3D object
            if( _xsSize.getZ() is not None ):
                strZ = EDString( _xsSize.getZ().getValue() )
            
            command = EDString( command + " " + strX + " " + strY )
            
            if( strZ is not None ):
                command = EDString( command + " " + strZ )
            
        return command
    
    
    def cellToCommand( self, _xsCell, _strKeyword ):
        """
        """
        command = None 
        if( _xsCell is not None ):
            command = _strKeyword
            strA = EDString( _xsCell.getLength_a().getValue() )
            strB = EDString( _xsCell.getLength_b().getValue() )
            strC = EDString( _xsCell.getLength_c().getValue() )
            strAlpha = EDString( _xsCell.getAngle_alpha().getValue() )
            strBeta = EDString( _xsCell.getAngle_beta().getValue() )
            strGamma = EDString( _xsCell.getAngle_gamma().getValue() )

            command = EDString( command + " " + strA + " " + strB + " " + strC + " " + strAlpha + " " + strBeta + " " + strGamma )
            
        return command
    
    
    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginRaddosev01.generateExecutiveSummary")
        if ( self.getStringVersion() is not None ):
            self.addExecutiveSummaryLine( self.getStringVersion() )
        xsDataRaddoseInput  = self.getDataInput()
        xsDataRaddoseOutput = self.getDataOutput()
        self.addExecutiveSummaryLine( "Execution of radiation dose estimation successful." )
        self.addExecutiveSummaryLine( "" )
        self.addExecutiveSummaryLine( "Flux                            : %21.2g [photons/s]" % xsDataRaddoseInput.getBeamFlux().getValue() )
        if( xsDataRaddoseInput.getCrystalSize() is not None ):
            self.addExecutiveSummaryLine( "Crystal size                    : %.3f x %.3f x %.3f [mm]" % ( xsDataRaddoseInput.getCrystalSize().getX().getValue(), \
                                                                                              xsDataRaddoseInput.getCrystalSize().getY().getValue(), \
                                                                                              xsDataRaddoseInput.getCrystalSize().getZ().getValue() ))
        else:
            self.addExecutiveSummaryLine( "Crystal size                    : 0.100 x 0.100 x 0.100 [mm] (not set, taking default)" )
        self.addExecutiveSummaryLine( "Beam size                       :         %.3f x %.3f [mm]" % ( xsDataRaddoseInput.getBeamSize().getX().getValue(), \
                                                                                   xsDataRaddoseInput.getBeamSize().getY().getValue() ))
        self.addExecutiveSummaryLine( "Solvent                         : %21.2f [%%]" % self.m_fSolvent )
        self.addExecutiveSummaryLine( "Dose rate                       : %21.2g [Grays/s]" % xsDataRaddoseOutput.getAbsorbedDoseRate().getValue() )
        self.addExecutiveSummaryLine( "Time to reach Henderson limit   : %21.2g [s]" % self.m_fTimeToReachHendersonLimit )

    