#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginReadImageHeaderv01.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDPlugin
from EDImport import EDPluginExec
from EDImport import EDVerbose
from EDImport import EDUtilsImage
from EDImport import EDString
from EDImport import EDDiskExplorer
from EDImport import EDUtilsImage
from EDImport import EDMessage

from XSDataCommonv01 import XSDataFile
from XSDataCommonv01 import XSDataDetector
from XSDataCommonv01 import XSDataBeam
from XSDataCommonv01 import XSDataGoniostat
from XSDataCommonv01 import XSDataWavelength
from XSDataCommonv01 import XSDataImage
from XSDataCommonv01 import XSDataAngle
from XSDataCommonv01 import XSDataLength
from XSDataCommonv01 import XSDataTime
from XSDataCommonv01 import XSDataString
from XSDataCommonv01 import XSDataInteger
from XSDataCommonv01 import XSDataExperimentalCondition
from XSDataCommonv01 import XSDataSubWedge



class EDPluginReadImageHeaderv01( EDPluginExec ):

    def __init__( self ):
        """
        """
        EDPluginExec.__init__( self )
        self.m_xsDataImage = None
    

    def configure( self ):
        """
        """
        EDPlugin.configure( self )
        EDVerbose.DEBUG( "*** EDPluginReadImageHeaderv01.configure")


    def checkParameters( self ):
        """
        Checks the mandatory parameters.
        """
        EDVerbose.DEBUG( "*** EDPluginReadImageHeaderv01.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                 "Data Input is None" )


    def preProcess( self, _oalObject = None ):
        """
        """
        EDPlugin.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginReadImageHeaderv01.preProcess")
        
        
    def process( self, _oalObject = None ):
        """
        """
        EDPlugin.process( self )
        EDVerbose.DEBUG( "*** EDPluginReadImageHeaderv01.process")
        xsDataFile = self.getDataInput()
        oedStringPath = EDString( xsDataFile.getPath().getValue() )
        oedStringAbsolutePath = EDDiskExplorer.getAbsolutePath( oedStringPath )
        oedDictHeader = EDUtilsImage.readHeaderADSC( oedStringPath )
        if ( oedDictHeader is None ):
            errorMessage = EDMessage.ERROR_CANNOT_READ_FILE_02 % ( "EDPluginReadImageHeaderv01.process", oedStringAbsolutePath )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage          
        else:
            xsDataExperimentalCondition = XSDataExperimentalCondition()
            xsDataDetector = XSDataDetector()  

            xsDataDetector.setBeamPositionX(       XSDataLength(  EDString( oedDictHeader[ "BEAM_CENTER_X" ] ).atof() ) )
            xsDataDetector.setBeamPositionY(       XSDataLength(  EDString( oedDictHeader[ "BEAM_CENTER_Y" ] ).atof() ) )
            xsDataDetector.setDistance(            XSDataLength(  EDString( oedDictHeader[ "DISTANCE"   ] ).atof() ) )
            dPixelSize = EDString( oedDictHeader[ "PIXEL_SIZE"   ] ).atof()
            xsDataDetector.setPixelSizeX(          XSDataLength(  dPixelSize ) )
            xsDataDetector.setPixelSizeY(          XSDataLength(  dPixelSize ) )
            if "TWOTHETA" in oedDictHeader.keys():
                xsDataDetector.setTwoTheta(            XSDataAngle(   EDString( oedDictHeader[ "TWOTHETA"   ] ).atof() ) )
            xsDataDetector.setNumberBytesInHeader( XSDataInteger( EDString( oedDictHeader[ "HEADER_BYTES"   ] ).atof()  ) )
            xsDataDetector.setSerialNumber(        XSDataInteger( EDString( oedDictHeader[ "DETECTOR_SN"   ] ).atof() ) )
            xsDataDetector.setNumberPixelX(        XSDataInteger( EDString( oedDictHeader[ "SIZE1"   ] ).atof() ) )
            xsDataDetector.setNumberPixelY(        XSDataInteger( EDString( oedDictHeader[ "SIZE2"   ] ).atof() ) )
            xsDataDetector.setBin(                 XSDataString(  EDString( oedDictHeader[ "BIN" ] ) ) )
            xsDataDetector.setDataType(            XSDataString(  EDString( oedDictHeader[ "TYPE" ] ) ) )
            xsDataDetector.setByteOrder(           XSDataString(  EDString( oedDictHeader[ "BYTE_ORDER" ] ) ) )
            xsDataDetector.setImageSaturation(     XSDataString(  EDString( oedDictHeader[ "CCD_IMAGE_SATURATION" ] ) ) )

            # Determine type of detector...
            iNoPixelsX = xsDataDetector.getNumberPixelX().getValue()
            iNoPixelsY = xsDataDetector.getNumberPixelY().getValue()
            if ( iNoPixelsX == 2304 and iNoPixelsY == 2304 ):
                xsDataDetector.setName( XSDataString( "ADSC Q4" ) )
                xsDataDetector.setType( XSDataString( "q4" ) )
            elif ( iNoPixelsX == 1152 and iNoPixelsY == 1152 ):
                xsDataDetector.setName( XSDataString( "ADSC Q4 bin 2x2" ) )
                xsDataDetector.setType( XSDataString( "q4-2x" ) )
            elif ( iNoPixelsX == 4096 and iNoPixelsY == 4096 ):
                xsDataDetector.setName( XSDataString( "ADSC Q210" ) )
                xsDataDetector.setType( XSDataString( "q210" ) )
            elif ( iNoPixelsX == 2048 and iNoPixelsY == 2048 ):
                xsDataDetector.setName( XSDataString( "ADSC Q210 bin 2x2" ) )
                xsDataDetector.setType( XSDataString( "q210-2x" ) )
            elif ( iNoPixelsX == 6144 and iNoPixelsY == 6144 ):
                xsDataDetector.setName( XSDataString( "ADSC Q315" ) )
                xsDataDetector.setType( XSDataString( "q315" ) )
            elif ( iNoPixelsX == 3072 and iNoPixelsY == 3072 ):
                xsDataDetector.setName( XSDataString( "ADSC Q315 bin 2x2" ) )
                xsDataDetector.setType( XSDataString( "q315-2x" ) )
            else:
                errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ("EDPluginReadImageHeaderv01.process", "Unknown detector type" )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
                raise RuntimeError, errorMessage

            xsDataExperimentalCondition.setDetector( xsDataDetector )

            # Beam object
            
            xsDataBeam = XSDataBeam()
            xsDataBeam.setWavelength(   XSDataWavelength( EDString( oedDictHeader[ "WAVELENGTH" ] ).atof() ) )
            xsDataBeam.setExposureTime( XSDataTime(       EDString( oedDictHeader[ "TIME" ] ).atof() ) )               
            xsDataExperimentalCondition.setBeam( xsDataBeam )
            
            # Goniostat object
            xsDataGoniostat = XSDataGoniostat()
            dRotationAxisStart = EDString( oedDictHeader[ "OSC_START" ] ).atof()
            dOscillationWidth = EDString( oedDictHeader[ "OSC_RANGE" ] ).atof()
            xsDataGoniostat.setRotationAxisStart( XSDataAngle( dRotationAxisStart ) )
            xsDataGoniostat.setRotationAxisEnd(   XSDataAngle( dRotationAxisStart + dOscillationWidth ) )
            xsDataGoniostat.setOscillationWidth(  XSDataAngle( dOscillationWidth ) )
            xsDataGoniostat.setRotationAxis(              XSDataString( EDString( oedDictHeader[ "AXIS" ] ) ) )
            xsDataExperimentalCondition.setGoniostat( xsDataGoniostat )
            
            # Create the image object
            xsDataImage = XSDataImage()
            xsDataImage.setPath( XSDataString( oedStringAbsolutePath ))
            xsDataImage.setDate( XSDataString( EDString( oedDictHeader[ "DATE" ] ) ) )
            oedStringFileName = EDDiskExplorer.getFileName( oedStringPath )
            iImageNumber = EDUtilsImage.getImageNumber( oedStringFileName )
            xsDataImage.setNumber( XSDataInteger( iImageNumber ) ) 
            
            self.m_xsDataSubWedge = XSDataSubWedge()
            self.m_xsDataSubWedge.setExperimentalCondition( xsDataExperimentalCondition )
            self.m_xsDataSubWedge.addImage( xsDataImage )



    def postProcess( self, _oalObject = None ):
        """
        """
        EDPlugin.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginReadImageHeaderv01.postProcess")
        if ( self.m_xsDataSubWedge is not None ):
            self.setDataOutput( self.m_xsDataSubWedge )

    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSDataFile object
        """
        EDVerbose.DEBUG( "*** EDPluginReadImageHeaderv01.setDataInput")
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginReadImageHeaderv01.setDataInput: Input Data is of EDString " )
            self.m_oxsDataInput = XSDataFile.parseString( _dataInput )
        elif ( isinstance( _dataInput, XSDataFile ) ):
            self.m_oxsDataInput = _dataInput
        else:
            errorMessage = EDMessage.ERROR_WRONG_DATA_TYPE_02 % ('EDPluginReadImageHeaderv01.setDataInput', "XMLString or XSDataFile expected" )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage
        
