#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDApplication.py 477 2008-07-31 08:44:38Z incardon $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from ALDefinition               import *
from ALApplication              import ALApplication
from ALAppDefinitionInit        import ALAppDefinitionInit
from ALAppDefinitionEnd         import ALAppDefinitionEnd
from ALApplicationActionControl import ALApplicationActionControl
from ALDebug                    import ALDebug
from ALSystem                   import ALSystem

from EDImport import EDVerbose
from EDImport import EDConfiguration
from EDImport import EDString
from EDImport import EDTime


class EDApplication( ALApplication ):

    m_oedStrCONFIGURATION_PARAM_LABEL = "--conf"
    m_oedConfiguration = None

    
    """
    This is a Configurable ALApplication
    """
    def __init__( self, _strName, _strVersion, _strConfigurationFileName = None ):
        """
        """
        ALApplication.__init__( self, _strName, _strVersion )
        # If _strConfigurationFileName is None, this means that it has not been given to the constructor\
        # It has been given by the command line\
        if( _strConfigurationFileName == None ):
            self.m_oedStrConfigurationFileName = self.getCommandLineArgument( EDApplication.m_oedStrCONFIGURATION_PARAM_LABEL )
        else:
            self.m_oedStrConfigurationFileName = _strConfigurationFileName
        
    
    def setConfiguration( self, _oedConfiguration ):
        """
        """
        EDVerbose.DEBUG( "*** EDApplication.setConfiguration" )
        EDApplication.synchronizeOn()
        if ( _oedConfiguration == None ):
            EDVerbose.warning( "*** EDApplication.setConfiguration: Configuration is None!" )
        else:             
            EDApplication.m_oedConfiguration = _oedConfiguration
        EDApplication.synchronizeOff()
        
        
    def getPluginConfiguration( _pluginName ):
        """
        """
        EDVerbose.DEBUG( "*** EDApplication.getPluginConfiguration" )
        EDApplication.synchronizeOn()
        pluginConfiguration = None
        if ( EDApplication.m_oedConfiguration != None ):
            pluginConfiguration = EDApplication.m_oedConfiguration.getPluginItem( _pluginName )
        else:
            EDVerbose.warning( "*** EDApplication.getPluginConfiguration: No configuration at all!" ) 
        EDApplication.synchronizeOff()
        return pluginConfiguration
    getPluginConfiguration = staticmethod( getPluginConfiguration )
        
    
    def preProcess( self ):
        """
        Reads the command line and retrieves the configuration file path, the plugin name and the dataset file name
        """
        #EDVerbose.DEBUG( "*** EDApplication.preProcess: Getting Configuration File Path from command..." )        
        #oedStrConfigurationFile = self.getCommandLineArgument( EDApplication.m_oedStrCONFIGURATION_PARAM_LABEL )
        #EDVerbose.DEBUG( "*** EDApplication.preProcess: Configuration File Path: " + oedStrConfigurationFile)

        EDVerbose.DEBUG( "*** EDApplication.preProcess: Loading Configuration if not already loaded..." )
        oedConfiguration = EDConfiguration( self.m_oedStrConfigurationFileName )
        self.setConfiguration( oedConfiguration )
        self.loadConfiguration()
        
        EDVerbose.DEBUG( "*** EDApplication.preProcess: Checking... Number of plugins...: " + EDString( EDApplication.m_oedConfiguration.getPluginListSize()) )
        
        
    def process( self ):
        """
        Calls the Plugin to be executed
        """
        EDVerbose.DEBUG( "*** EDApplication.process: I'm processing" )            
        
        
    def loadConfiguration( self ):
        """
        Loads the configuration file if not already loaded
        """
        if(( EDApplication.m_oedConfiguration != None) & (EDApplication.m_oedConfiguration.isLoaded() == False )):
            
            EDVerbose.DEBUG( "*** EDApplication.loadConfiguration: Loading Configuration File..." )
            EDApplication.m_oedConfiguration.load()
            
    def init( self, _bInit = True ):
        """
        This is the Init method in the ALAppDefinitionInit.
        """
        EDVerbose.DEBUG( "Execution: ALAppDefinitionInit.init()" )
        oBoolStatus = True
        oedString = EDString( ALAppDefinitionInit.m_oalCommandLine.getCommandLine() )
        EDVerbose.log( oedString )
        EDVerbose.setVerboseOff()


        if (ALDebug.isDebugModeVerboseOn()==True):
            EDVerbose.setVerboseOn()


        if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--quiet" )):
            EDVerbose.setVerboseOff()


        if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--profile" )):
            ALAppDefinitionInit.setDebugProfileOn()


        if (ALDebug.isDebugModeOn()==True or ALDebug.isDebugModeForceOn()==True):
            ALAppDefinitionInit.setDebugMode()
            ALAppDefinitionInit.setDebugProfileOn()
            EDVerbose.DEBUG( "Debug Mode [ON]" )


        if (ALDebug.isDebugModeAssertOn()==True ):
            ALAssert.setAssertionModeOn()


        if (_bInit==True):
            EDVerbose.screen( self.m_oalApplicationName + "::Starting" )
            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--help" )
                or ALAppDefinitionInit.m_oalCommandLine.existCommand( "-h" )):
                EDVerbose.setVerboseOn()
                #ALAppDefinitionInit.outputHelp()
                self.usage()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--version" ) or
                ALAppDefinitionInit.m_oalCommandLine.existCommand( "-v" )):
                EDVerbose.setVerboseOn()
                ALAppDefinitionInit.outputVersion()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--man" ) or
                ALAppDefinitionInit.m_oalCommandLine.existCommand( "-m" )):
                EDVerbose.setVerboseOn()
                ALAppDefinitionInit.outputMan()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--no-log" )):
                EDVerbose.setLogFileOff()

            # workaround to avoid problem of the ALString.toXml cleaning function of jython
            if (ALSystem.getOSName()=="Linux" and ALSystem.getMachineVirtuel()=="Java"):
                pass
            else:
                oalObject = ALAppDefinitionInit.getResource( AL_FRAMEWORK_LAST_RUN )
                if (oalObject != None):
                    oedStringLastRun = EDString( oalObject );
                    EDVerbose.log( "Last Run: " + oedStringLastRun )

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--import-resource" )):
                oalStrResourceFileName = ALAppDefinitionInit.m_oalCommandLine.getCommandArgument( "--import-resource" )
                ALAppDefinitionInit.importResource( oalStrResourceFileName )

        # The OK to continue the application
        return oBoolStatus
    

    def end( self ):
        """
        End The application framework...closing and registering the resources
        """
        self.__forceCloseActions()
        self.__forceCloseTimerActions()
        ALAppDefinitionEnd.setResource( AL_FRAMEWORK_LAST_RUN, EDTime.timeString() )
        ALAppDefinitionEnd.outputResource()
        self.__getActionLog()


        ofAppRunTime     = self.getRunTime()
        oalStrAppName    = ALAppDefinitionEnd.getApplicationName()
        oedStrAppRunTime = EDString( ofAppRunTime )
        EDVerbose.outputActionLog()
        EDVerbose.log( oalStrAppName + "::Closing" )
        EDVerbose.screen( oalStrAppName + "::Runtime: " + oedStrAppRunTime + " [s]" )
        EDVerbose.log( oalStrAppName + "::Runtime: " + EDTime.getTimeStringPSP( ofAppRunTime ) )


        # Used to generate a XML profile log file to analise the performance
        ALAppDefinitionEnd.outputProfile()

        # To be safety and close external pipes on remote control
        #ALSystem.exit( 0 ) #Bug in Python 2.4 - it raises an exception !!!!!!!!!!


    def __forceCloseTimerActions( self ):
        """
        This ALAppDefinitionEnd force Close Timer Actions.
        """
        EDVerbose.DEBUG( "Execution: ALAppDefinitionEnd.__forceCloseTimerActions()" )
        ALAppDefinitionEnd.synchronizeOn()
        for iAction in ALAppDefinitionEnd.m_oalListTimerActions:
            iAction.close()
        ALAppDefinitionEnd.synchronizeOff()


    def __forceCloseActions( self ):
        """
        This ALAppDefinitionEnd force Close All Actions.
        """
        EDVerbose.DEBUG( "Execution: ALAppDefinitionEnd.__forceCloseActions()" )
        ALAppDefinitionEnd.synchronizeOn()
        ALApplicationActionControl.forceClose()
        ALAppDefinitionEnd.synchronizeOff()


    def __getActionLog( self ):
        ALApplicationActionControl.getActionLogInformation()
    