#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControl.py 474 2008-07-30 15:51:54Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDPlugin
from EDImport import EDList
from EDImport import EDVerbose
from EDImport import EDMessage
from EDImport import EDString
 

class EDPluginControl( EDPlugin ):
    """
    An EDPluginControl is a plugin that is responsible for a EDPluginExec plugin
    """
    
    def __init__ ( self ):
        """
        """
        EDPlugin.__init__( self )
        
        self.m_oedListErrorMessages = EDList()
        
        
    def addWarningMessages( self, _oedListWarningMessages ):
        """
        Flatten the messages in the list
        """
        EDVerbose.DEBUG( "*** EDPluginControl.addWarningMessages")
        self.synchronizeOn()
        for oedStrWarningMessage in _oedListWarningMessages:
            self.addWarningMessage( oedStrWarningMessage )
        self.synchronizeOff()
        
        
    def addErrorMessages( self, _oedListErrorMessages ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControl.addErrorMessages")
        self.synchronizeOn()
        for oedStrErrorMessage in _oedListErrorMessages:
            self.addErrorMessage( oedStrErrorMessage )
        self.synchronizeOff()
        
        
    def retrieveFailureMessages( self, _oedPlugin, _oedStrMethodCaller ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControl.retrieveFailureMessages")
        self.retrieveWarningMessages( _oedPlugin )
        self.retrieveErrorMessages( _oedPlugin, _oedStrMethodCaller, True )
        
        
    def retrieveSuccessMessages( self, _oedPlugin, _oedStrMethodCaller ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControl.retrieveSuccessMessages")
        self.retrieveWarningMessages( _oedPlugin )
        self.retrieveErrorMessages( _oedPlugin, _oedStrMethodCaller, False )
        
        
    def retrieveErrorMessages( self, _oedPlugin, _oedStrMethodCaller, _oedBooleanFailure ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControl.retrieveErrorMessages")
        errorMessages = _oedPlugin.getErrorMessages()
        if(( errorMessages.getNumberObjects() == 0 ) and ( _oedBooleanFailure is True)):
            EDVerbose.DEBUG( "************** " + _oedStrMethodCaller + ": Adding Unexpected error" )
            errorMessages.add( EDMessage.ERROR_UNEXPECTED_01 % ( _oedStrMethodCaller ) )
        self.addErrorMessages( errorMessages )
            
        
    def retrieveWarningMessages( self, _oedPlugin ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControl.retrieveWarningMessages")
        self.addWarningMessages( _oedPlugin.getWarningMessages() )
        

    def appendExecutiveSummary( self, _oedPlugin, _oedStringPrefix="", _bAddSeparator=True ):
        """
        Appends the executive summary from a plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControl.appendExecutiveSummary" )
        if ( _bAddSeparator ):
            self.addExecutiveSummarySeparator()
        for line in _oedPlugin.getListExecutiveSummaryLines():
            oedStringLine = EDString( line )
            if ( oedStringLine.equals( self.m_oedStringExecutiveSummarySeparator ) ):
                 oedStringLine = oedStringLine[ :-len( _oedStringPrefix ) ]
            self.addExecutiveSummaryLine( _oedStringPrefix + oedStringLine )


    def addErrorWarningMessagesToExecutiveSummary( self, _oedStringErrorMessage="Error messages:", _oedStringWarningMessage= "Warning messages:" ):
        """
        Adds error and warning messages (if any)
        """
        if ( self.getErrorMessages().getNumberObjects() != 0 ):
            self.addExecutiveSummarySeparator()
            self.addExecutiveSummaryLine( _oedStringErrorMessage )
            for errorMessage in self.getErrorMessages():
                self.addExecutiveSummaryLine( errorMessage )
            self.addExecutiveSummarySeparator()
        if ( self.getWarningMessages().getNumberObjects() != 0 ):
            self.addExecutiveSummarySeparator()
            self.addExecutiveSummaryLine( _oedStringWarningMessage )
            for warningMessage in self.getWarningMessages():
                self.addExecutiveSummaryLine( warningMessage )
            self.addExecutiveSummarySeparator()
    
