#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginExecProcess.py 474 2008-07-30 15:51:54Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDPluginExec
from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDActionProcess
from EDImport import EDConfiguration
from EDImport import EDDiskExplorer
from EDImport import EDMessage

from XSConfiguration import XSPluginItem

class EDPluginExecProcess( EDPluginExec ):
    """
    The super class for all pluigns who executes a process
    """
    
    def __init__ ( self ):
        """
        Constructor of EDPluginExecProcess with default values.
        """
        EDPluginExec.__init__( self )
        self.m_oedStrCONF_EXEC_PROCESS_TIME_OUT           = "execProcessTimeOut"
        self.m_oedStringConfigTimeOut                     = EDString( "100" )
        self.m_oedStrCONF_EXEC_PROCESS_EXECUTABLE         = "execProcessExecutable"
        self.m_oedStringConfigExecutable                  = EDString( "cat" )
        self.m_oedStringConfigCommandline                 = ""

        
    def preProcess( self, _oalObject = None ):
        """
        """
        EDPluginExec.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.preProcess")

        
    def process( self, _oalObject = None ):
        """
        """
        EDPluginExec.process( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.process starting" )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.process executing: " + self.m_oedStringConfigExecutable )
        self.synchronizeOn()
        oedStringCommand = EDString( self.m_oedStringConfigExecutable + " " + self.m_oedStringConfigCommandline ) 
        oedActionProcess = EDActionProcess( oedStringCommand ) 
        oedActionProcess.setTimeOut( self.m_oedStringConfigTimeOut )
        oedActionProcess.setWorkingFolder( self.getWorkingDirectory() )
        try:
            EDVerbose.screen( self.getBaseName() + ": Processing" )
            oedActionProcess.executeSynchronous()            
        except Exception, detail:
            EDVerbose.DEBUG( "*** EDPluginExecProcess.process ========================================= ERROR! ================" )
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcess.process', self.getClassName(), "cannot execute " + oedStringCommand + " : " + EDString( detail ) ) 
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 
        EDVerbose.DEBUG( "*** EDPluginExecProcess.process finished " )
        self.synchronizeOff()
       
    
    def postProcess( self, _oalObject = None ):
        """
        """
        EDPluginExec.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.postProcess")
        
        
    def configure( self ):
        """
        """
        EDPluginExec.configure( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.configure")
        oxsPluginItem = self.getConfiguration()
        if ( oxsPluginItem == None ):
            EDVerbose.warning( "*** EDPluginExecProcess.configure: No plugin item defined." )
            oxsPluginItem = XSPluginItem()
        oedStringTimeOut = EDConfiguration.getStringParamValue( oxsPluginItem, self.m_oedStrCONF_EXEC_PROCESS_TIME_OUT )
        if( oedStringTimeOut == None ):
            EDVerbose.DEBUG( "*** EDPluginExecProcess.configure: No configuration parameter found for: " + self.m_oedStrCONF_EXEC_PROCESS_TIME_OUT + ", using default value:" +  self.getTimeOut())
        else:
            self.setTimeOut( oedStringTimeOut )
        oedStringExecutable = EDConfiguration.getStringParamValue( oxsPluginItem, self.m_oedStrCONF_EXEC_PROCESS_EXECUTABLE )
        if( oedStringExecutable == None ):
            pass
            #EDVerbose.DEBUG( "*** EDPluginExecProcess.configure: No configuration parameter found for: " + self.m_oedStrCONF_EXEC_PROCESS_EXECUTABLE + ", using default value:" + self.getExecutable() )
        else:
            self.setExecutable( oedStringExecutable )
        
        


    def setExecutable( self, _oedStringExecutable ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringConfigExecutable = EDString( _oedStringExecutable )
        self.synchronizeOff()
        

    def getExecutable( self ):
        """
        """
        return EDString( self.m_oedStringConfigExecutable )

    
    def setCommandline( self, _oedStringCommandline ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringConfigCommandline = EDString( _oedStringCommandline )
        self.synchronizeOff()
        

    def getCommandline( self ):
        """
        """
        return EDString( self.m_oedStringConfigCommandline )


    def setTimeOut( self, _oedStringTimeOut ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringConfigTimeOut = EDString( _oedStringTimeOut )
        #self.m_oedStringConfigTimeOut = float( _oedStringTimeOut )
        self.synchronizeOff()
        

    def getTimeOut( self ):
        """
        """
        return EDString( self.m_oedStringConfigTimeOut )
        #return float( self.m_oedStringConfigTimeOut )
