#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDUtilsImage.py 474 2008-07-30 15:51:54Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDFile
from EDImport import EDString
from EDImport import EDList
from EDImport import EDVerbose
from EDImport import EDDictionary
#from EDImport import PyRe
import re as PyRe


class EDUtilsImage:
    

    def __compileAndMatchRegexpTemplate( _oedStringPathToImage ):
        """
        """
        oedListResult = None
        opyStrBaseImageName = EDFile.getFileName( _oedStringPathToImage )
        opyRegexp           = PyRe.compile(  r'(.*)([^0^1^2^3^4^5^6^7^8^9])([0-9]*)\.(.*)' )
        opyMatch            = opyRegexp.match( opyStrBaseImageName )
        if ( opyMatch != None ):   
            oedListResult = EDList( [ EDString( opyMatch.group(0) ), 
                                      EDString( opyMatch.group(1) ), 
                                      EDString( opyMatch.group(2) ), 
                                      EDString( opyMatch.group(3) ), 
                                      EDString( opyMatch.group(4) ),
                                       ] )
        return oedListResult
    __compileAndMatchRegexpTemplate = staticmethod( __compileAndMatchRegexpTemplate )


    def getImageNumber( _oedStringPathToImage ):
        """
        """
        iImageNumber = None
        oedListResult = EDUtilsImage.__compileAndMatchRegexpTemplate( _oedStringPathToImage )
        if ( oedListResult != None ):
            iImageNumber = oedListResult[3].atoi()
        return iImageNumber
    getImageNumber = staticmethod( getImageNumber )

        
    def getTemplate( _oedStringPathToImage, _oedStringSymbol = "#" ):
        """
        """
        oedStringTemplate = None
        oedListResult = EDUtilsImage.__compileAndMatchRegexpTemplate( _oedStringPathToImage )
        if ( oedListResult != None ):
            oedStringPrefix      = oedListResult[1]
            oedStringSeparator   = oedListResult[2]
            oedStringImageNumber = oedListResult[3]
            oedStringSuffix      = oedListResult[4]
            oedStringHashes = ""
            for i in range( len( oedStringImageNumber ) ):
                oedStringHashes += _oedStringSymbol
            oedStringTemplate = EDString( oedStringPrefix + oedStringSeparator + oedStringHashes + "." + oedStringSuffix )
        return oedStringTemplate
    getTemplate = staticmethod( getTemplate )


    def readHeaderADSC( _oedStringImageFileName ):
        """
        This method reads the header from an ADSC image and returns a dictionary.
        Ideally this method should be a part of a plugin.
        """
        pyFile = None
        oedDictionary = None
        try:
            pyFile = open( _oedStringImageFileName, "r" )
        except:
            EDVerbose.warning( "**** EDUtilsImage.readHeaderADSC: couldn't open file: " + _oedStringImageFileName )

        if ( pyFile != None ):
            EDVerbose.screen( "EDUtilsImage.readHeaderADSC: Reading header from image " + _oedStringImageFileName )
            pyFile.seek( 0, 0 )
            pyStrLine = pyFile.readline()        
            if ( EDString( pyStrLine[0] ).equals( "{" ) ):
                oedDictionary = EDDictionary()
                pyStrLine = pyFile.readline()        
                while ( pyStrLine[0] != "}" ) :
                    pyListSplit = EDString( pyStrLine ).split( "=" )
                    edStringKeyword = EDString( pyListSplit[0] )
                    pyListSplitValue = EDString( pyListSplit[1] ).split( ";" )
                    edStringValue = EDString( pyListSplitValue[0] )
                    oedDictionary.add( edStringKeyword, edStringValue)
                    pyStrLine = pyFile.readline()                
            pyFile.close()
            
        return oedDictionary        
    readHeaderADSC = staticmethod( readHeaderADSC )
