#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDApplicationCheckPlugin.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                 import EDApplication
from EDImport                 import EDVerbose
from EDImport                 import EDConfiguration
from EDImport                 import EDCompiler
from EDImport                 import EDString
from EDImport                 import EDList
from EDImport                 import EDMessage
from EDImport                 import EDCommandLine
# For Bug#10 workaround
from EDImportTest             import EDCommandLineArgument
from EDImportTest             import EDAppDefinition
from EDImportTest             import EDResource
from EDImportTest             import EDTime
from EDImportTest             import EDLogFile
from EDImportTest             import EDSlot
from EDImportTest             import EDSignal
from EDImportTest             import EDDebug
from EDImportTest             import EDDiskExplorer
from ALDefinition             import *
# End For Bug#10 workaround
from EDImportTest             import EDUtilsPath
from EDImportTest             import EDUtilsTest


class EDApplicationCheckPlugin( EDApplication ):

    m_oedStrPLUGIN_PARAM_LABEL = "--execute"
    m_oedStrDATASET_PARAM_LABEL = "--data"

    
    """
    Derives from EDApplication in order to add a Configuration Feature
    """
    def __init__( self, _oedStrPluginName = None, _strConfigurationFileName = None, _oedStrDatasetFileName = None, _oedLogFile = None ):
        """
        """
        
        self.m_oedLogFile = _oedLogFile
        
        EDApplication.__init__( self, "EDApplicationCheckPlugin", "0.1", _strConfigurationFileName )
        
        if( _oedStrPluginName == None ):
            self.m_oedStrPluginName = self.getCommandLineArgument( EDApplicationCheckPlugin.m_oedStrPLUGIN_PARAM_LABEL )
        else:
            self.m_oedStrPluginName = _oedStrPluginName
            
        self.m_oedStrPlugin = None
            
        self.addPluginFolder( EDUtilsPath.getPluginsHome() )
        
        if( _oedStrDatasetFileName == None ):
            self.m_oedStrDatasetFileName = self.getCommandLineArgument( EDApplicationCheckPlugin.m_oedStrDATASET_PARAM_LABEL )
        else:
            self.m_oedStrDatasetFileName = _oedStrDatasetFileName

        self.m_oedStrXMLData = None

        self.m_oedListErrorMessages = EDList()
        self.m_oedListWarningMessages = EDList()
        
        self.m_xsDataOutput = None
        
    
    def preProcess( self ):
        """
        Reads the command line and retrieves the configuration file path, the plugin name and the dataset file name
        """
        EDApplication.preProcess( self )
        strXMLData = EDUtilsTest.readAndParseFile( self.m_oedStrDatasetFileName )

        self.m_oedStrXMLData = EDString( strXMLData )
        
        
    def process( self ):
        """
        Calls the Plugin to be executed
        """
        EDVerbose.DEBUG( "*** EDApplicationCheckPlugin.process: I'm processing" )            
        
        self.m_oedStrPlugin = EDApplicationCheckPlugin.getPluginObject( self.m_oedStrPluginName )
        
        if( self.m_oedStrPlugin != None ):
            
            self.m_oedStrPlugin.connectSUCCESS( self.doSuccessActionPlugin ) 
            self.m_oedStrPlugin.connectFAILURE( self.doFailureActionPlugin )
            
            self.m_oedStrPlugin.setDataInput( self.m_oedStrXMLData )
            
            EDVerbose.DEBUG( "*** EDApplicationCheckPlugin.process: Connecting " + self.m_oedStrPluginName + " execution..." )
            self.m_oedStrPlugin.execute( )
            self.m_oedStrPlugin.synchronize()
        else:
            EDVerbose.error( EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ('EDApplicationCheckPlugin.process', self.m_oedStrPluginName ) )
            
        
    def doSuccessActionPlugin( self, _oedPlugin ):
        """
        """
        EDVerbose.DEBUG("*********** EDApplicationCheckPlugin.doSuccessActionPlugin" )
        self.retrievePluginOutputData( _oedPlugin )
        
        # Print the potential Warnings and Errors
        self.m_oedListWarningMessages = _oedPlugin.getWarningMessages()
        EDVerbose.screen( "*** EDApplicationCheckPlugin.doSuccessActionPlugin: Plugin " + _oedPlugin.getPluginName() + " Successful with : " + EDString( self.m_oedListWarningMessages.getNumberObjects() ) + " Warnings: " )
        for warningMessage in self.m_oedListWarningMessages:
            EDVerbose.screen( warningMessage )
         
        self.m_oedListErrorMessages = _oedPlugin.getErrorMessages()   
        EDVerbose.screen( "*** EDApplicationCheckPlugin.doSuccessActionPlugin: Plugin " + _oedPlugin.getPluginName() + " Successful with : " + EDString( self.m_oedListErrorMessages.getNumberObjects() ) + " Errors: " )
        for errorMessage in self.m_oedListErrorMessages:
            EDVerbose.screen( errorMessage )
            
        #return outputData


    def doFailureActionPlugin( self, _oedPlugin ):
        """
        """
        EDVerbose.DEBUG("*********** EDApplicationCheckPlugin.doFailureActionPlugin" )
        
        # Print the potential Warnings and Errors
        self.m_oedListWarningMessages = _oedPlugin.getWarningMessages()
        EDVerbose.screen( "*** EDApplicationCheckPlugin.doFailureActionPlugin: Plugin " + _oedPlugin.getPluginName() + " Failed with : " + EDString( self.m_oedListWarningMessages.getNumberObjects() ) + " Warnings: " )
        for warningMessage in self.m_oedListWarningMessages:
            EDVerbose.screen( warningMessage )
        
        self.m_oedListErrorMessages = _oedPlugin.getErrorMessages()
        EDVerbose.screen( "*** EDApplicationCheckPlugin.doFailureActionPlugin: Plugin " + _oedPlugin.getPluginName() + " Failed with : " + EDString( self.m_oedListErrorMessages.getNumberObjects() ) + " Errors: \n" )
        for errorMessage in self.m_oedListErrorMessages:
            EDVerbose.screen( errorMessage )
      
            
    def retrievePluginOutputData( self, _oedPlugin ):
        """
        """
        outputDataFile = self.m_oedStrPluginName + "_output.xml"
        EDVerbose.DEBUG("*** EDApplicationCheckPlugin.retrievePluginOutputData: Getting Output Data of : " + _oedPlugin.getPluginName() )
        self.m_xsDataOutput = _oedPlugin.getDataOutput()

        if ( self.m_xsDataOutput != None):
            self.m_xsDataOutput.outputFile( outputDataFile )
            strOutput = EDUtilsTest.readAndParseFile ( outputDataFile )
            #EDVerbose.screen( "*** EDApplicationCheckPlugin.retrievePluginOutputData: Output Data: " + strOutput )
        else:
            EDVerbose.error( "*** EDApplicationCheckPlugin.retrievePluginOutputData: Output Data is None" )
            
            
    def getPlugin( self ):
        """
        """
        return self.m_oedStrPlugin
        
        
    def getPluginOutputData( self ):
        """
        """
        return self.m_xsDataOutput
    
    
    def getWarningMessages( self ):
        """
        """
        return self.m_oedListWarningMessages
        

    def getErrorMessages( self ):
        """
        """
        return self.m_oedListErrorMessages      
    
    
    def create( self, _obPlugin = True ):
        """
        The following code is related to bug#10 workaround
        """
        
        EDAppDefinition.m_oalCommandLine          = EDCommandLine( EDCommandLineArgument() )
        EDAppDefinition.m_oalApplicationFolder    = EDAppDefinition.m_oalCommandLine.getApplicationFolder()
        
        if( self.m_oedLogFile == None ):
            EDVerbose.m_oalLogFile                    = EDLogFile( EDAppDefinition.m_oalApplicationName + ".log" )
        else:
            EDVerbose.m_oalLogFile                    = self.m_oedLogFile
            
        EDAppDefinition.m_oalResource             = EDResource( EDAppDefinition.m_oalApplicationName + ".res" )
        self.m_oalTime = EDTime()
        self.m_ofTimeOut = AL_TIME_INFINITE # It is: 11d:13h:46m:39s:000ms

        # Only for static methods
        EDTime.init()

        self.m_oalSlotInit          = EDSlot()
        self.m_oalSlotPreProcess    = EDSlot()
        self.m_oalSlotProcess       = EDSlot()
        self.m_oalSlotPostProcess   = EDSlot()
        self.m_oalSlotEnd           = EDSlot()

        EDAppDefinition.m_oalSlotSignal.connect( self.exit, EDSignal.error() )
        EDAppDefinition.m_oalSlotSignal.connect( self.exit, EDSignal.exception() )
        EDAppDefinition.m_oalSlotSignal.connect( self.exit, EDSignal.exit() )
        EDAppDefinition.m_oalCommandLine.m_oalSlotException.connect( self.exit, EDSignal.exception() ) # not used??

        if (_obPlugin):
            self.__connectPluginFolder()
            
            
    def __connectPluginFolder( self ):
        if (EDDebug.isDebugModeOn()==True or EDDebug.isDebugModeForceOn()==True):
            EDAppDefinition.m_oalDebug = True
            EDVerbose.setVerboseDebugOn()

        # First Standart Plugin Folder
        oalStringFolder = EDString( EDAppDefinition.m_oalApplicationFolder ) + "/plugins"
        if ( EDAppDefinition.m_oalFactoryPlugin.addPluginFolder( oalStringFolder )==False ):
            # Second Standart Plugin Folder
            oalStringFolder = EDDiskExplorer.mergePath( EDAppDefinition.m_oalApplicationFolder, "../" )
            oalStringFolder = oalStringFolder + "/plugins"
            oalStringFolder = EDDiskExplorer.getAbsolutePath( oalStringFolder )
            if ( EDAppDefinition.m_oalFactoryPlugin.addPluginFolder( oalStringFolder )==False ):
                oalStringFolder = EDDiskExplorer.getFolderName( EDAppDefinition.m_oalCommandLine.getArgumentNumber( 0 ) )
                EDAppDefinition.m_oalFactoryPlugin.addPluginFolderFromPath( oalStringFolder )
        EDAppDefinition.m_oalApplicationPluginFolder = oalStringFolder
        self.__connectCommandLineInterfaceFile( oalStringFolder )


    def __connectCommandLineInterfaceFile( self, _oalStringFolder ):
        oalStringFolder = _oalStringFolder + "/commandLineInterface.xml"
        self.setCommandLineInterfaceFile( oalStringFolder  )

        



if __name__ == '__main__':
     
    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    applicationCheckPlugin = EDApplicationCheckPlugin()
    applicationCheckPlugin.execute()
    
    