#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCase.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from ALTestCase                       import ALTestCase
from EDImport                         import EDVerbose
from EDImport                         import EDDebug
from EDImport                         import EDString
from EDImport                         import EDLogFile
from EDImport                         import EDDiskExplorer
from EDImportTest                     import EDUtilsPath
from EDImport                         import EDAssert



class EDTestCase( ALTestCase ):
    """
    """
    def __init__( self, _oalStringTestName = None ):
        """
        """
        ALTestCase.__init__( self, _oalStringTestName )
        
        self.m_oLogFile = None
        if (_oalStringTestName != None ):
            oalStringNameLog = self.m_oalStringTestName + EDString( ".log" )
            self.m_oLogFile = EDLogFile( oalStringNameLog )
            
        EDVerbose.setVerboseOn()
        EDAssert.setAssertionModeOn()

        self.m_oedStrTestsHome = EDUtilsPath.getTestsHome()
        self.m_oedStrTestsDataHome = EDUtilsPath.getTestsDataHome()
        
    
    def getTestsHome( self ):
        """
        """
        return self.m_oedStrTestsHome
    
    
    def getTestsDataHome( self ):
        """
        """
        return self.m_oedStrTestsDataHome
    
    
    
    def processKernel( self ):

        oboolAssertion = False
        oboolException = False

        

        ointNumberMethods = self.m_oalListTest.getNumberObjects( )
        EDVerbose.unitTest( "===================================================================" )
        EDVerbose.unitTest( "TestCase Total Number of Tests = [ " + EDString( ointNumberMethods ) + " ]" )
        EDVerbose.unitTest( " " )
        ointNumber = 0
        for oTestMethod in self.m_oalListTest:
            ointNumber = ointNumber + 1
            methodName = self.patchMethodName( oTestMethod )

            EDVerbose.unitTest( "===================================================================" )
            EDVerbose.unitTest( "TestCase Method [ " + methodName  + " ]" )
            EDVerbose.unitTest( "TestCase Method [ " + EDString( ointNumber )  + " ]" )
            EDVerbose.unitTest( "-------------------------------------------------------------------" )

            try:
                oTestMethod()
                EDVerbose.unitTest( "[ " + methodName + " ] : [SUCCESS]" )
                EDVerbose.unitTest( " " )
            except AssertionError:
                EDVerbose.unitTest( "TestCase = Assertion Error Raised"  )
                EDVerbose.unitTest( "[ " + methodName + " ] : [FAILURE]" )
                EDVerbose.unitTest( " " )
                oboolAssertion = True
                EDDebug.controlDebug( self )

            except:
                EDVerbose.unitTest( "TestCase = Unexpected Error Raised"  )
                EDVerbose.unitTest( "[ " + methodName + " ] : [FAILURE]" )
                EDVerbose.unitTest( " " )
                oboolException = True
                EDDebug.controlDebug( self )

        
        if(( oboolAssertion == True ) or ( oboolException == True )):
            raise
    
#        if( oboolAssertion == True ):
#            raise AssertionError
#
#        if ( oboolException == True ):
#            raise Exception
        
    def execute( self ):
        """
        This method overrides the ALTest execute method to raise the exception to the TestSuite
        """
        EDVerbose.DEBUG( "Execution: EDTestCase.execute()" )
        try:
            self.executeKernel()
        except:
            raise
        