#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecute.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDDiskExplorer
from EDImport                         import EDAssert
from EDImportTest                     import EDUtilsPath
from EDImportTest                     import EDUtilsTest
from EDImportTest                     import EDTestCasePlugin
from EDImportTest                     import EDApplicationCheckPlugin



class EDTestCasePluginExecute( EDTestCasePlugin ):
    """
    """
    
    def __init__( self, _oedStringPluginName, _oedStrPluginDir = None, _oedStringTestName = None ):
        """
        """
        EDTestCasePlugin.__init__( self, _oedStringPluginName, _oedStrPluginDir, _oedStringTestName )
        
        self.m_oedStrEdnaSite = EDUtilsPath.getEdnaSite()
        
        oedStrRefConfFile = None
        if( self.m_oedStrEdnaSite != None ):
            oedStrRefConfFile = "XSConfiguration_" + self.getEdnaSite() + ".xml"
            
        self.m_oedStrRefConfigFile = EDDiskExplorer.mergePath( EDUtilsPath.getConfigurationHome(), oedStrRefConfFile )
                        
        self.m_oedStrConfigurationFile = None
        self.m_oedStrDataInputFile = None
        self.m_oedApplicationCheckPlugin = None
        
        
    def preProcess( self ):
        """
        """
        
        if( self.m_oedStrEdnaSite == None ):
            raise RuntimeError, "EDNA_SITE must be set"
        
        self.m_oedApplicationCheckPlugin = EDApplicationCheckPlugin( self.getPluginName(), self.getConfigurationFile(), self.getDataInputFile(), EDVerbose.m_oalLogFile )
        self.m_oedApplicationCheckPlugin.addPluginFolder( self.getPluginHome() )
            
            
    def getPlugin( self ):
        """
        """
        return self.getPluginChecker().getPlugin()
    
    
    def getErrorMessages( self ):
        """
        """
        return self.getPluginChecker().getErrorMessages()
    
    
    def getWarningMessages( self ):
        """
        """
        return self.getPluginChecker().getWarningMessages()
            
            
    def setConfigurationFile( self, _oedStrConfigurationFile ):
        """
        """
        self.m_oedStrConfigurationFile = _oedStrConfigurationFile
        
    
    def getConfigurationFile( self ):
        """
        """
        return self.m_oedStrConfigurationFile
    
    
    def getRefConfigFile( self ):
        """
        """
        return self.m_oedStrRefConfigFile
        
        
    def setDataInputFile( self, _oedStrDataInputFile):  
        """
        """
        self.m_oedStrDataInputFile = _oedStrDataInputFile
        
        
    def getDataInputFile( self ):
        """
        """
        return self.m_oedStrDataInputFile
    
    
    def setReferenceDataOutputFile( self, _oedStrReferenceDataOutputFile ):
        """
        """
        self.m_oedStrReferenceDataOutputFile = _oedStrReferenceDataOutputFile
    
    
    def getReferenceDataOutputFile( self ):
        """
        """
        return self.m_oedStrReferenceDataOutputFile
    
    
    def getEdnaSite( self ):
        """
        """
        return self.m_oedStrEdnaSite
    
    
    def getPluginChecker( self ):
        """
        """
        return self.m_oedApplicationCheckPlugin
    
    
    def run( self ):
        """
        """
        self.m_oedApplicationCheckPlugin.execute()
        dataOutputIsNone = True
        dataOutput = self.getPlugin().getDataOutput()
        if( dataOutput is not None ):
            dataOutputIsNone = False
            
        EDAssert.equal( False, dataOutputIsNone )
           
    

            
                

    

                
