#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnit.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDConfiguration
from EDImport                         import EDDiskExplorer
from EDImportTest                     import EDFactoryPlugin
from EDImportTest                     import EDTestCasePlugin



class EDTestCasePluginUnit( EDTestCasePlugin ):
    """
    """
    
    def __init__( self, _oedStringPluginName, _oedStrPluginDir = None, _oedStringTestName = None ):
        """
        """
        EDTestCasePlugin.__init__( self, _oedStringTestName, _oedStrPluginDir, _oedStringTestName )
        oedFactoryPlugin = EDFactoryPlugin()
        oedFactoryPlugin.addPluginFolder( self.getPluginHome() )
        self.m_oedStringPluginName = _oedStringPluginName
        self.m_oedPlugin = oedFactoryPlugin.getPlugin( _oedStringPluginName )
        if ( self.m_oedPlugin == None ):
            EDVerbose.error("*** EDTestCaseEDPlugin.__init__: Could not create plugin: " + _oedStringPluginName)
        else:
            self.m_oedPlugin.setExecutionContext( "EDTestCaseEDPlugin" )
            
            
    def getPlugin( self ):
        """
        """
        return self.m_oedPlugin
    
    
    def createPlugin( self ):
        """
        """
        oedFactoryPlugin = EDFactoryPlugin()
        oedFactoryPlugin.addPluginFolder( self.getPluginHome() )
        oedPlugin = oedFactoryPlugin.getPlugin( self.m_oedStringPluginName )
        if ( oedPlugin == None ):
            EDVerbose.error("*** EDTestCaseEDPlugin.__init__: Could not create plugin: " + self.m_oedStringPluginName )
        else:
            oedPlugin.setExecutionContext( "EDTestCaseEDPlugin" )
            
        return oedPlugin     
        
        
    def getPluginConfiguration( self, _oedStrConfigurationFileName, _oedPlugin = None ):
        """
        """
        oedPlugin = None
        # By default, this method returns the configuration of the plugin attribute member
        # If a plugin is given in parameter, it returns the configuration of the parameter plugin
        if( _oedPlugin == None ):
            oedPlugin = self.m_oedPlugin
        else:
            oedPlugin = _oedPlugin
        
        xsPluginItem = None
        
        oedConfiguration = EDConfiguration( _oedStrConfigurationFileName )
        if( oedConfiguration != None ):
            oedConfiguration.load()
            xsPluginItem = oedConfiguration.getPluginItem( oedPlugin.getPluginName() )
            if( xsPluginItem == None ):
                EDVerbose.warning( "*** EDTestCaseEDPlugin.getPluginConfiguration: Could not get XSPluginItem for: " + self.m_oedPlugin.getPluginName() )
            
        else:
            EDVerbose.warning( "*** EDTestCaseEDPlugin.getPluginConfiguration: Could not load Configuration: " + _oedStrConfigurationFileName )
        
        return xsPluginItem
    
    
    def cleanUp( self, _oedPlugin ):
        """
        """
        workingDirectory = _oedPlugin.getWorkingDirectory()
        if( workingDirectory is not None ):
            fileList = EDDiskExplorer.getFileList( workingDirectory, "*" )
            if( len( fileList ) == 0 ):
                EDVerbose.DEBUG( "Deleting " + workingDirectory + " ..." )
                EDDiskExplorer.deleteFolder( workingDirectory )

                
