#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDUtilsPath.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDObject
from EDImport                         import EDVerbose
from EDImport                         import EDDiskExplorer
from EDImportTest                     import PyOS


class EDUtilsPath (EDObject):
    
    m_oedStrEDNA_HOME = None
    m_oedStrEDNA_SITE = None
    
    def getEdnaHome():
        """
        """
        # EDNA Home
        
        if( EDUtilsPath.m_oedStrEDNA_HOME == None ):
            oedStrEdnaHome = PyOS.environ.get("EDNA_HOME")
            if( oedStrEdnaHome != None):
                EDVerbose.DEBUG( "*** EDUtilsEnv.getEdnaHome: EDNA_HOME is set: " + oedStrEdnaHome )
            else:
                # if EDNA_HOME is not set, let's assume that the tests are lanched from <edna_home>/prototype/tmp
                oedStrEdnaHome = "../../../edna"
                EDVerbose.DEBUG( "*** EDUtilsEnv.getEdnaHome: EDNA_HOME not set: take relative path... " )
            
            EDUtilsPath.m_oedStrEDNA_HOME = EDDiskExplorer.getAbsolutePath( oedStrEdnaHome )
            EDVerbose.DEBUG( "*** EDUtilsEnv.getEdnaHome: EDNA_HOME: " + EDUtilsPath.m_oedStrEDNA_HOME )
        
            if ( EDDiskExplorer.existFolder( EDUtilsPath.m_oedStrEDNA_HOME ) != True ):
                EDVerbose.error( "!!!" + EDUtilsPath.m_oedStrEDNA_HOME + " Not Found!!!" )
            
        return EDUtilsPath.m_oedStrEDNA_HOME
    getEdnaHome = staticmethod( getEdnaHome )
    
    
    def getEdnaSite():
        """
        """    
        # EDNA Site
        if( EDUtilsPath.m_oedStrEDNA_SITE == None ):
            EDUtilsPath.m_oedStrEDNA_SITE = PyOS.environ.get("EDNA_SITE")
            if( EDUtilsPath.m_oedStrEDNA_SITE != None):
                EDVerbose.DEBUG( "*** EDUtilsEnv.getEdnaSite: EDNA_SITE is set: " + EDUtilsPath.m_oedStrEDNA_SITE )
            else:
                EDVerbose.warning( "*** EDUtilsEnv.getEdnaSite: EDNA_SITE not set!!" )

        return EDUtilsPath.m_oedStrEDNA_SITE
    getEdnaSite = staticmethod( getEdnaSite )
    
    
    def getEdnaRoot():
        """
        """
        oedStrEdnaRoot = EDDiskExplorer.mergePath( EDUtilsPath.getEdnaHome(), "prototype" )
        return oedStrEdnaRoot
    getEdnaRoot = staticmethod( getEdnaRoot )
    
    
    def getConfigurationHome():
        """
        """
        oedStrConfigurationHome = EDDiskExplorer.mergePath( EDUtilsPath.getEdnaRoot(), "conf" )
        return oedStrConfigurationHome
    getConfigurationHome = staticmethod( getConfigurationHome )
    
    
    def getPluginsHome():
        """
        """
        oedStrPluginsHome = EDDiskExplorer.mergePath( EDUtilsPath.getEdnaRoot(), "plugins" )
        return oedStrPluginsHome
    getPluginsHome = staticmethod( getPluginsHome )
    
    
    def getTestsHome():
        """
        """
        oedStrTestsHome = EDDiskExplorer.mergePath( EDUtilsPath.getEdnaRoot(), "tests" )
        return oedStrTestsHome
    getTestsHome = staticmethod( getTestsHome )
    
    
    def getTestsDataHome():
        """
        """
        oedStrTestsDataHome = EDDiskExplorer.mergePath( EDUtilsPath.getTestsHome(), "data" )
        return oedStrTestsDataHome
    getTestsDataHome = staticmethod( getTestsDataHome )
   