#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCaseEDConfiguration.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport             import EDCompiler
from EDImport             import EDVerbose
from EDImport             import EDConfiguration
from EDImport             import EDDiskExplorer
from EDImport             import EDAssert
from EDImportTest         import EDTestCase

class EDTestCaseEDConfiguration( EDTestCase ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCase.__init__( self, _oalStringTestName )
        self.m_oedConfiguration = None
        oedStrDataDir = "EDConfiguration"
        self.m_oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
    
    
    def preProcess( self ):
        """
        Constructs the utilitary EDConfiguration class
        """
        #Generates py module with xsd
#        oedStrXsdPath = EDDiskExplorer.mergePath( self.getEdnaRoot(), "datamodel" )
#        self.m_oedConfiguration = EDConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration.xml" ),
#                                                   EDDiskExplorer.mergePath( oedStrXsdPath, "XSConfiguration.xsd" ), 
#                                                   True )
        #Loads py module directly using xml configuration file
        self.m_oedConfiguration = EDConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration.xml" ))
        self.m_oedConfiguration.load()
    
        
    def testGetPluginList( self ):
        """
        Testing the retrieved XSPluginList from configuration
        """
        oedPluginList = self.m_oedConfiguration.getPluginList()
        EDAssert.equal(2, self.m_oedConfiguration.getPluginListSize())
        
        
    def testGetPluginItem( self ):
        """
        Testing Plugin indexingMosflm Configuration
        """
        oxsPluginItem = self.m_oedConfiguration.getPluginItem( "indexingMosflm" )        
        EDAssert.equal("indexingMosflm", oxsPluginItem.getName())
        
        paramList = oxsPluginItem.getXSParamList()
        paramItems = paramList.getXSParamItem()
        
        EDAssert.equal( "workingDir", paramItems[0].getName())
        EDAssert.equal( "/path/to/working/dir", paramItems[0].getValue())
        EDAssert.equal( "number", paramItems[1].getName())
        EDAssert.equal( "3", paramItems[1].getValue())
        
    def testGetPluginItemError( self ):
        """
        Testing the retrieval of an absent plugin
        """
        EDAssert.equal( None, self.m_oedConfiguration.getPluginItem( "toto" ))
            
        
    def testGetParamItem( self ):
        """
        Testing the XSParamItem inside an XSPluginItem
        """
        oxsPluginItem = self.m_oedConfiguration.getPluginItem( "indexingMosflm" )
        oxsParamItem = self.m_oedConfiguration.getParamItem( oxsPluginItem, "workingDir" )
        EDAssert.equal( "workingDir", oxsParamItem.getName() )
        
        
    def testGetParamValue( self ):
        """
        Testing the XSParamItem Value convertion from string to different formats
        """
        oxsPluginItem = self.m_oedConfiguration.getPluginItem( "indexingMosflm" )
        EDAssert.equal( "/path/to/working/dir", self.m_oedConfiguration.getStringParamValue( oxsPluginItem, "workingDir" ) )
        EDAssert.equal( "/path/to/working/dir", EDConfiguration.getStringParamValue( oxsPluginItem, "workingDir" ) )
        EDAssert.equal( 3, self.m_oedConfiguration.getIntegerParamValue( oxsPluginItem, "number" ) )
        EDAssert.equal( 3, EDConfiguration.getIntegerParamValue( oxsPluginItem, "number" ) )
        
            
    def testGetOptionItem( self ):
        """
        Testing the XSOptionItem inside an XSPluginItem
        """
        oxsPluginItem = self.m_oedConfiguration.getPluginItem( "indexing" )
        oxsOptionItemMosflm = self.m_oedConfiguration.getOptionItem( oxsPluginItem, "indexingMosflm" )
        EDAssert.equal( True, oxsOptionItemMosflm.getEnabled() )
        
        oxsOptionItemXds = self.m_oedConfiguration.getOptionItem( oxsPluginItem, "indexingXds" )
        EDAssert.equal( False, oxsOptionItemXds.getEnabled() )
        
        oxsOptionItemLabelit = self.m_oedConfiguration.getOptionItem( oxsPluginItem, "indexingLabelit" )
        EDAssert.equal( False, oxsOptionItemLabelit.getEnabled() )
        

                

##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testGetPluginList )
        self.addTestMethod( self.testGetPluginItem )
        self.addTestMethod( self.testGetPluginItemError )
        self.addTestMethod( self.testGetParamItem )
        self.addTestMethod( self.testGetParamValue )
        self.addTestMethod( self.testGetOptionItem )



##############################################################################

if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCaseEDConfiguration = EDTestCaseEDConfiguration( "EDTestCaseEDConfiguration" )
    edTestCaseEDConfiguration.execute()
