#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCaseEDPluginExecProcessScript.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport         import EDCompiler
from EDImport         import EDVerbose
from EDImport         import EDAssert
from EDImport         import EDPluginExecProcessScript
from EDImport         import EDList
from EDImport         import EDStreamFile
from EDImport         import EDDiskExplorer
from EDImportTest     import EDTestCase


class EDTestCaseEDPluginExecProcessScript( EDTestCase ):
    """
    """
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCase.__init__( self, _oalStringTestName )
    
    
    def testGenerateScript( self ):
        """
        """
        oedListCommandPreExecution = EDList( ['PRE-EXECUTION1', 'PRE-EXECUTION2 PATATE'] )
        oedListCommandExecution = EDList( ['COMMAND1', 'COMMAND2 BANANA'] )
        oedListCommandPostExecution = EDList( ['POST-EXECUTION1', 'POST-EXECUTION2 FRAISE'] )
        oedPluginExecProcessScript = EDPluginExecProcessScript()
        oedPluginExecProcessScript.setShell( "/bin/bash" )
        oedPluginExecProcessScript.setScriptBaseName( "TestCaseGenerateScript" )
        oedPluginExecProcessScript.setListCommandPreExecution( oedListCommandPreExecution )
        oedPluginExecProcessScript.setListCommandExecution( oedListCommandExecution )
        oedPluginExecProcessScript.setListCommandPostExecution( oedListCommandPostExecution )
        oedPluginExecProcessScript.setRequireCCP4( True )
        oedPluginExecProcessScript.setSetupCCP4( "/usr/local/xtal/ccp4-6.0.2/include/ccp4.setup-bash" )
        oedPluginExecProcessScript.configure()
        oedStringScript = oedPluginExecProcessScript.prepareScript()
        oedStringScriptReference = """#!/bin/bash
cd %s
. /usr/local/xtal/ccp4-6.0.2/include/ccp4.setup-bash
PRE-EXECUTION1
PRE-EXECUTION2 PATATE
cat  > TestCaseGenerateScript.log 2> TestCaseGenerateScript.err << EOF-EDPluginExecProcessScript
COMMAND1
COMMAND2 BANANA
EOF-EDPluginExecProcessScript
POST-EXECUTION1
POST-EXECUTION2 FRAISE
""" % ( oedPluginExecProcessScript.getWorkingDirectory() )
        EDAssert.equal( oedStringScriptReference, oedStringScript )
        
        
        
    def testGenerateExecutableScript( self ):
        oedListCommands = EDList( ['COMMAND1', 'COMMAND2 BANANA'] )
        oedPluginExecProcessScript = EDPluginExecProcessScript()
        oedPluginExecProcessScript.setShell( "/bin/bash" )
        oedPluginExecProcessScript.setScriptBaseName( "TestCaseGenerateScript" )
        oedPluginExecProcessScript.setListCommandExecution( oedListCommands )
        oedPluginExecProcessScript.setRequireCCP4( True )
        oedPluginExecProcessScript.setSetupCCP4( "/usr/local/xtal/ccp4-6.0.2/include/ccp4.setup-bash" )
        oedPluginExecProcessScript.configure()
        oedStringScript = oedPluginExecProcessScript.prepareScript()
        oedPluginExecProcessScript.writeExecutableScript( oedStringScript )
        oedStringScriptFileName = oedPluginExecProcessScript.getScriptFileName()
        oedStringScriptFromFile = oedPluginExecProcessScript.readProcessFile( oedStringScriptFileName )
        oedStringScriptReference = """#!/bin/bash
cd %s
. /usr/local/xtal/ccp4-6.0.2/include/ccp4.setup-bash
cat  > TestCaseGenerateScript.log 2> TestCaseGenerateScript.err << EOF-EDPluginExecProcessScript
COMMAND1
COMMAND2 BANANA
EOF-EDPluginExecProcessScript
""" % ( oedPluginExecProcessScript.getWorkingDirectory() )
        EDAssert.equal( oedStringScriptFromFile, oedStringScriptReference )


    def testExecuteScript( self ):
        oedListCommandExecution = EDList( ['COMMAND1', 'COMMAND2 BANANA'] )
        oedStringLogReference = """COMMAND1
COMMAND2 BANANA
"""
        oedPluginExecProcessScript = EDPluginExecProcessScript()
        oedPluginExecProcessScript.setListCommandExecution( oedListCommandExecution )
        oedPluginExecProcessScript.setRequireCCP4( False )
        oedPluginExecProcessScript.execute()      
        oedPluginExecProcessScript.synchronize()
        oedStringScriptLogFileName = oedPluginExecProcessScript.getScriptLogFileName()
        oedStringLogFromFile = oedPluginExecProcessScript.readProcessLogFile()
        EDAssert.equal( oedStringLogFromFile, oedStringLogReference )

        
    def process( self ):
        """
        """
        self.addTestMethod( self.testGenerateScript )
        self.addTestMethod( self.testGenerateExecutableScript )
        self.addTestMethod( self.testExecuteScript )

##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    oedTestCaseEDPluginExecProcessScript = EDTestCaseEDPluginExecProcessScript( "TestCase EDPluginExecProcessScript" )
    oedTestCaseEDPluginExecProcessScript.execute()
