#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCaseEDUtilsImage.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDCompiler
from EDImport import EDVerbose
from EDImport import EDUtilsImage
from EDImport import EDDiskExplorer
from EDImport import EDAssert
from EDImport import EDList
from EDImport import EDDictionary
from EDImport import EDString 

from EDImportTest         import EDTestCase


class EDTestCaseEDUtilsImage( EDTestCase ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCase.__init__( self, _oalStringTestName )
        self.m_oedConfiguration = None
        oedStrDataDir = "images"
        self.m_oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        self.m_oedStringFileName = "ref-testscale_1_001.img"
        self.m_oedStringFilePath = EDDiskExplorer.mergePath( self.m_oedStrDataPath, self.m_oedStringFileName )
        
        
    def testGetImageNumber( self ):
        """
        Testing retrieving the image number from a image filename
        """
        iImageNumber = EDUtilsImage.getImageNumber( self.m_oedStringFileName )
        iReference = 1
        EDAssert.equal( iReference, iImageNumber )
                

    def testGetTemplateHash( self ):
        """
        Testing retrieving the image number from a image filename
        """
        oedStringSymbol = "#"
        oedStringTemplate = EDUtilsImage.getTemplate( self.m_oedStringFileName )
        oedStringTemplateReference = EDString( "ref-testscale_1_###.img" )
        EDAssert.equal( oedStringTemplateReference, oedStringTemplate )


    def testGetTemplateQuestionMark( self ):
        """
        Testing retrieving the image number from a image filename
        """
        oedStringSymbol = "?"
        oedStringTemplate = EDUtilsImage.getTemplate( self.m_oedStringFileName, oedStringSymbol )
        oedStringTemplateReference = EDString( "ref-testscale_1_???.img" )
        EDAssert.equal( oedStringTemplateReference, oedStringTemplate )


    def testReadHeaderADSC( self ):
        """
        Testing reading an ADSC image header.
        """
        oedDictionaryHeader = EDUtilsImage.readHeaderADSC( self.m_oedStringFilePath )
        oedDictionaryHeaderReference = EDDictionary( {'DIM': '2', 'BEAM_CENTER_Y': '104.738', 'BEAM_CENTER_X': '102.459', 'DATE': 'Mon Mar 20 12:38:23 2006', 'WAVELENGTH': '0.9340', 'OSC_START': '0.000', 'AXIS': 'phi', 'BIN': '2x2', 'BYTE_ORDER': 'little_endian', 'OSC_RANGE': '1.000', 'CCD_IMAGE_SATURATION': '65535', 'TWOTHETA': '0.000', 'UNIF_PED': '1500', 'ADC': 'fast', 'HEADER_BYTES': '  512', 'DETECTOR_SN': '444', 'DISTANCE': '198.441', 'PHI': '0.000', 'SIZE1': '2048', 'SIZE2': '2048', 'TIME': '1.00', 'TYPE': 'unsigned_short', 'PIXEL_SIZE': '0.1024'} )
        EDAssert.equal( oedDictionaryHeaderReference, oedDictionaryHeader )
        
##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testGetImageNumber )
        self.addTestMethod( self.testGetTemplateHash )
        self.addTestMethod( self.testGetTemplateQuestionMark )
        self.addTestMethod( self.testReadHeaderADSC )



##############################################################################

if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCaseEDUtilsImage = EDTestCaseEDUtilsImage( "TestCase EDTestCaseEDUtilsImage" )
    edTestCaseEDUtilsImage.execute()
