#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCaseEDUtilsTable.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport         import EDCompiler
from EDImport         import EDVerbose
from EDImport         import EDDiskExplorer
from EDImport         import EDAssert
from EDImport         import EDString
from EDImport         import EDUtilsFile
from EDImport         import EDUtilsTable
from XSDataDnaTables  import dna_tables
from EDImportTest     import EDTestCase


class EDTestCaseEDUtilsTable( EDTestCase ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCase.__init__( self, _oalStringTestName )
        self.m_oedConfiguration = None
        oedStrDataDir = "EDUtilsTable"
        self.m_oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        self.m_oedStringFileName = "EDPluginBestv01_dnaTables.xml"
        self.m_oedStringFilePath = EDDiskExplorer.mergePath( self.m_oedStrDataPath, self.m_oedStringFileName )
        
        
    def testTableListItem( self ):
        """
        """
        xsItem = None
        xsItem2 = None
        oedStringDnaTablesXML = EDUtilsFile.readFile( self.m_oedStringFilePath )
        xsDataDnaTables = dna_tables.parseString( oedStringDnaTablesXML )
        xsTable = EDUtilsTable.getTableFromTables( xsDataDnaTables, "input" )
        xsLists = EDUtilsTable.getListsFromTable( xsTable, "select_task" )
        for xsList in xsLists:
            xsItem = EDUtilsTable.getItemFromList( xsList, "task" )  
            xsItem2 = EDUtilsTable.getItemFromList( xsList, "tata" )  
        EDAssert.equal( "optimize", xsItem.getValueOf_() )
        
        EDAssert.equal( None, xsItem2 )
        
        xsLists = EDUtilsTable.getListsFromTable( xsTable, "select_tata" )
        EDAssert.equal( [], xsLists )
        
        xsTable = EDUtilsTable.getTableFromTables( xsDataDnaTables, "toto" )
        EDAssert.equal( None, xsTable )
        
        EDVerbose.DEBUG( "Test don..." )
        
        
                
        
##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testTableListItem )



##############################################################################

if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCaseEDUtilsTable = EDTestCaseEDUtilsTable( "TestCase EDTestCaseEDUtilsTable" )
    edTestCaseEDUtilsTable.execute()
