#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteControlIndexingv01.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDCompiler
from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDAssert
from EDImport                         import EDDiskExplorer
from EDImportTest                     import EDTestCasePluginExecute
from EDImportTest                     import EDUtilsTest




class EDTestCasePluginExecuteControlIndexingv01( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginControlIndexingv01", "EDPluginControlIndexing-v0.1", _oalStringTestName )

        oedStrDataDir = "EDPluginControlIndexing"
        oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        self.setPluginTestsDataHome( oedStrDataPath )
        
        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataIndexingInput_reference.xml" ) )
        
        self.setReferenceDataOutputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataIndexingResult_reference.xml"))
        self.m_oedObtainedOutputDataFile = self.getPluginName() + "_output.xml"
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginExecute.preProcess( self )
        
        
    def testExecute( self ):
        """
        """ 
        self.run()
        
        # Checks that there are no error messages
        
        plugin = self.getPlugin()

        EDVerbose.DEBUG("Checking error messages...")
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
            
        EDVerbose.DEBUG("Checking warning messages...")
        EDAssert.equal( 0, self.getWarningMessages().getNumberObjects() )
        EDVerbose.DEBUG("Checking obtained result...")         
        # Checks the expected result
        strExpectedOutput = EDUtilsTest.readAndParseFile ( self.getReferenceDataOutputFile() )
        strObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedOutputDataFile )  
        # Check that all the paths in the prediction results actaully exists
        from XSDataCharacterisationv01 import XSDataIndexingResult
        xsDataIndexingResult = XSDataIndexingResult.parseString( strObtainedOutput )
        xsDataGeneratePredictionResult = xsDataIndexingResult.getPredictionResult()
        for predictionImage in xsDataGeneratePredictionResult.getPredictionImage():
            oedStringImagePath = EDString( predictionImage.getPath().getValue() )
            bImageExists = EDDiskExplorer.isFile( oedStringImagePath )
            EDAssert.equal( True, bImageExists)            
        # Remove the prediction results since it contains absolute paths  
        xsDataIndexingResult.setPredictionResult( None )
        EDAssert.equal( strExpectedOutput, xsDataIndexingResult.marshal() )
        EDDiskExplorer.deleteFile( self.m_oedObtainedOutputDataFile )



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteControlIndexingv01 = EDTestCasePluginExecuteControlIndexingv01( "EDTestCasePluginExecuteControlIndexingv01" )
    edTestCasePluginExecuteControlIndexingv01.execute()