#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteMOSFLMv01GeneratePrediction.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDCompiler
from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDAssert
from EDImport                         import EDDiskExplorer
from EDImportTest                     import EDTestCasePluginExecute
from EDImportTest                     import EDUtilsTest




class EDTestCasePluginExecuteMOSFLMv01GeneratePrediction( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginMOSFLMv01GeneratePrediction", "EDPluginGroupMOSFLM-v0.1", _oalStringTestName )

        oedStrDataDir = "EDPluginGroupMOSFLM"
        oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        self.setPluginTestsDataHome( oedStrDataPath )
        
        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataMOSFLMInputGeneratePrediction_reference.xml" ) )
        
        self.setReferenceDataOutputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataMOSFLMOutputGeneratePrediction_reference.xml"))
        self.m_oedObtainedOutputDataFile = self.getPluginName() + "_output.xml"
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginExecute.preProcess( self )
        EDDiskExplorer.deleteFile( self.m_oedObtainedOutputDataFile )
        
        
    def testExecute( self ):
        """
        """ 
        self.run()
        
        # Checks that there are no error messages
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
        
        # Checks the expected result
        strObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedOutputDataFile )     
        from XSDataMOSFLMv01 import XSDataMOSFLMOutputGeneratePrediction
        xsDataMOSFLMOutputGeneratePrediction = XSDataMOSFLMOutputGeneratePrediction.parseString( strObtainedOutput )
        oedStringImagePath = EDString( xsDataMOSFLMOutputGeneratePrediction.getPredictionImage().getPath().getValue() )
        bImageExists = EDDiskExplorer.isFile( oedStringImagePath )
        EDAssert.equal( True, bImageExists)
        EDDiskExplorer.deleteFile( self.m_oedObtainedOutputDataFile )



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteMOSFLMv01GeneratePrediction = EDTestCasePluginExecuteMOSFLMv01GeneratePrediction( "EDTestCasePluginExecuteMOSFLMv01GeneratePrediction" )
    edTestCasePluginExecuteMOSFLMv01GeneratePrediction.execute()