#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitBestv01.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDCompiler
from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDAssert
from EDImport                         import EDDiskExplorer
from EDImport                         import EDDict
from EDImport                         import EDDictionary
from EDImport                         import EDUtilsFile
from EDImport                         import EDList
from EDImportTest                     import EDTestCasePluginUnit
from EDImportTest                     import EDUtilsTest


class EDTestCasePluginUnitBestv01( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginBestv01", "EDPluginBest-v0.1", _oalStringTestName )
        oedStrDataDir = "EDPluginBest"
        self.m_oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        
        self.m_oedObtainedInputFile = "XSDataInputBestv01.xml"
        self.m_oedObtainedInputFile2 = "XSDataInputBestv01FromObject_02.xml"
        self.m_oedReferenceInputFile = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSDataBestv01Input_reference.xml")
        self.m_oedReferenceInputFile2 = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSDataBestv01Input_reference_02.xml")
        
        self.m_oedReferenceScriptLogFileName = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "EDPluginBestv01.log" )
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginUnit.preProcess( self )
        EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile )
        EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile2 )
        
        
    def testConfigureOK( self ):
        """
        """
        oedPluginBest = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration_ESRF.xml" ))
        oedPluginBest.setConfiguration( xsPluginItemGood01 )
        oedPluginBest.configure()
        EDAssert.equal("/bin/bash", oedPluginBest.getShell())
        EDAssert.equal("/opt/pxsoft/DNA/v20080304/redhate4-i686/third_party/best_v3.1.0.d/best", oedPluginBest.getScriptExecutable())
        EDAssert.equal("/opt/pxsoft/ccp4-6.0.2/include/ccp4.setup-bash.orig", oedPluginBest.getSetupCCP4())
        EDAssert.equal("Verion of Best to be tested", oedPluginBest.getStringVersion())
        #EDAssert.equal(600, oedPluginMessage.getTimeOut())
        EDAssert.equal("/opt/pxsoft/DNA/v20080304/redhate4-i686/third_party/best_v3.1.0.d", oedPluginBest.getBestHome())
        EDAssert.equal("export besthome=/opt/pxsoft/DNA/v20080304/redhate4-i686/third_party/best_v3.1.0.d", oedPluginBest.getCommandBestHome())
        self.cleanUp( oedPluginBest )
        
        
    def testSetDataModelInput( self ):
        """
        """
        oedPluginBest = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration_ESRF.xml" ))
        oedPluginBest.setConfiguration( xsPluginItemGood01 )
        oedPluginBest.configure()
             
        from XSDataBestv01 import XSDataBestv01Input
        xsDataBestInput = XSDataBestv01Input()
        
        from XSDataBestv01 import XSDataAbsorbedDoseRate
        from XSDataBestv01 import XSDataFloat
        from XSDataBestv01 import XSDataString
        from XSDataBestv01 import XSDataTime
        from XSDataBestv01 import XSDataFile
        from XSDataBestv01 import XSDataSpeed
        from XSDataBestv01 import XSDataString
        from XSDataBestv01 import XSDataAngle
        
        xsDataBestInput.setCrystalAbsorbedDoseRate( XSDataAbsorbedDoseRate( 0.22E+06 ))
        xsDataBestInput.setCrystalShape( XSDataFloat(1) )
        xsDataBestInput.setCrystalSusceptibility( XSDataFloat(1.5) )
        xsDataBestInput.setDetectorType( XSDataString("q210-2x") )
        xsDataBestInput.setBeamExposureTime( XSDataTime(1) )
        xsDataBestInput.setBeamMaxExposureTime( XSDataTime(10000) )
        xsDataBestInput.setBeamMinExposureTime( XSDataTime(0.1) )
        xsDataBestInput.setGoniostatMinRotationWidth( XSDataAngle(0.1) )
        xsDataBestInput.setGoniostatMaxRotationSpeed( XSDataSpeed(10) )
        xsDataBestInput.setAimedResolution( XSDataFloat(2) )
        xsDataBestInput.setAimedRedundancy( XSDataFloat(6.5) )
        xsDataBestInput.setAimedCompleteness( XSDataFloat(0.9) )
        xsDataBestInput.setAimedIOverSigma( XSDataFloat(3) )
        fileDirectory = oedPluginBest.getWorkingDirectory()
        
        bestFileContentDat = EDUtilsFile.readFile( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "bestfile.dat" ))
        xsDataBestInput.setBestFileContentDat( XSDataString( bestFileContentDat) )
        
        bestFileContentPar = EDUtilsFile.readFile( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "bestfile.par" ))
        xsDataBestInput.setBestFileContentPar( XSDataString( bestFileContentPar ))

        bestFileContentHKL = EDUtilsFile.readFile( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "bestfile1.hkl" ))
        listBestFileContentHKL = EDList()
        listBestFileContentHKL.add( XSDataString( bestFileContentHKL ))
        xsDataBestInput.setBestFileContentHKL( listBestFileContentHKL )
        
        xsDataBestInput.outputFile( self.m_oedObtainedInputFile )
        
        strExpectedInput = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile )
        strObtainedInput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedInputFile )  
        
        EDAssert.equal( strExpectedInput, strObtainedInput )
        
        self.cleanUp( oedPluginBest )

##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testConfigureOK )
        self.addTestMethod( self.testSetDataModelInput )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginUnitBestv01 = EDTestCasePluginUnitBestv01( "EDTestCasePluginUnitBestv01" )
    edTestCasePluginUnitBestv01.execute()

