#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitControlIntegrationv01.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDCompiler
from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDAssert
from EDImport                         import EDDiskExplorer
from EDImport                         import EDDict
from EDImport                         import EDDictionary
from EDImport                         import EDUtilsFile
from EDImport                         import EDList
from EDImportTest                     import EDTestCasePluginUnit
from EDImportTest                     import EDUtilsTest


class EDTestCasePluginUnitControlIntegrationv01( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginControlIntegrationv01", "EDPluginControlIntegration-v0.1", _oalStringTestName )
        oedStrDataDir = "EDPluginControlIntegration"
        self.m_oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        
        self.m_oedReferenceInputFile = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSDataIntegrationInput_reference.xml")
#        self.m_oedReferenceInputFile2 = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSDataIntegrationv01Input_reference_02.xml")
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginUnit.preProcess( self )
        #EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile )
        
        
    def testConfigureOK( self ):
        """
        """
        oedPluginControlIntegrationv01 = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration_ESRF.xml" ))
        oedPluginControlIntegrationv01.setConfiguration( xsPluginItemGood01 )
        oedPluginControlIntegrationv01.configure()
        #EDAssert.equal("/opt/pxsoft/ccp4-6.0.2/lib/data", oedPluginControlIntegrationv01.getSymopHome())
    
        self.cleanUp( oedPluginControlIntegrationv01 )

        
        
    def testSetDataInput( self ):
        """
        """
        oedPluginControlIntegrationv01 = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration_ESRF.xml" ))
        oedPluginControlIntegrationv01.setConfiguration( xsPluginItemGood01 )
        oedPluginControlIntegrationv01.configure()

        xmlInput = EDUtilsTest.readAndParseFile( self.m_oedReferenceInputFile )
        oedPluginControlIntegrationv01.setDataInput( xmlInput )

        xsDataIntegrationv01Input = oedPluginControlIntegrationv01.getDataInput()
        xsDataIntegrationv01Input.outputFile( "test.xml" )
        EDAssert.equal( xmlInput, xsDataIntegrationv01Input.marshal() )
        
        self.cleanUp( oedPluginControlIntegrationv01 )
        
##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testConfigureOK )
        #self.addTestMethod( self.testSetDataModelInput )
        self.addTestMethod( self.testSetDataInput )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    EDTestCasePluginUnitControlIntegrationv01 = EDTestCasePluginUnitControlIntegrationv01( "EDTestCasePluginUnitControlIntegrationv01" )
    EDTestCasePluginUnitControlIntegrationv01.execute()

