#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMOSFLMv01.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDCompiler
from EDImport import EDString
from EDImport import EDVerbose
from EDImport import EDDiskExplorer
from EDImport import EDList
from EDImport import EDAssert

from EDImportTest import EDTestCasePluginUnit
from EDImportTest import EDSystem
from EDImportTest import EDUtilsTest
import StringIO as PyStringIO

from XSDataCommonv01 import XSDataAngle
from XSDataCommonv01 import XSDataLength
from XSDataCommonv01 import XSDataInteger
from XSDataCommonv01 import XSDataString
from XSDataCommonv01 import XSDataFloat


from XSDataCharacterisationv01 import XSDataIndexingInput



class EDTestCasePluginUnitMOSFLMv01( EDTestCasePluginUnit ):
    """
    """

    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMOSFLMv01", _oalStringTestName )        
        self.m_oedStrDataPath = EDDiskExplorer.mergePath(self.getTestsDataHome(), "EDPluginGroupMOSFLM" )

    
    def getReferenceDataMOSFLMNewmat( self ):
        from XSDataCommonv01 import XSDataMatrix
        from XSDataCommonv01 import XSDataCell
        from XSDataMOSFLMv01 import XSDataMOSFLMNewmat
        from XSDataMOSFLMv01 import XSDataMOSFLMMissettingsAngles

        oxsDataMOSFLMNewmat = XSDataMOSFLMNewmat()

        oxsDataMatrixA = XSDataMatrix()
        oxsDataMatrixA.setM11(  -0.00826416 )
        oxsDataMatrixA.setM12( 0.00885073 )
        oxsDataMatrixA.setM13( 0.00937013 )
        oxsDataMatrixA.setM21( 0.00126554 )
        oxsDataMatrixA.setM22( 0.01251971 )
        oxsDataMatrixA.setM23( -0.00845341 )
        oxsDataMatrixA.setM31( -0.01484956 )
        oxsDataMatrixA.setM32( -0.00385867 )
        oxsDataMatrixA.setM33( -0.00593515 )
        oxsDataMOSFLMNewmat.setAMatrix( oxsDataMatrixA )
        
        oxsDataMOSFLMMissettingsAngles = XSDataMOSFLMMissettingsAngles()
        oxsDataMOSFLMMissettingsAngles.setPhix( XSDataAngle( 1.000 ) )
        oxsDataMOSFLMMissettingsAngles.setPhiy( XSDataAngle( 2.000 ) )
        oxsDataMOSFLMMissettingsAngles.setPhiz( XSDataAngle( 3.000 ) )
        oxsDataMOSFLMNewmat.setMissettingAngles( oxsDataMOSFLMMissettingsAngles )

        oxsDataMatrixU = XSDataMatrix()
        oxsDataMatrixU.setM11( -0.4849475 )
        oxsDataMatrixU.setM12( 0.5598049 )
        oxsDataMatrixU.setM13( 0.6718960 )
        oxsDataMatrixU.setM21( 0.0742629 )
        oxsDataMatrixU.setM22( 0.7918670 )
        oxsDataMatrixU.setM23( -0.6061614 )
        oxsDataMatrixU.setM31( -0.8713845 )
        oxsDataMatrixU.setM32( -0.2440595 )
        oxsDataMatrixU.setM33( -0.4255866 )
        oxsDataMOSFLMNewmat.setUMatrix( oxsDataMatrixU )
        
        oxsDataCellRefined = XSDataCell()
        oxsDataCellRefined.setLength_a(    XSDataLength( 54.8079 ) )
        oxsDataCellRefined.setLength_b(    XSDataLength(59.0751 ) )
        oxsDataCellRefined.setLength_c(    XSDataLength(66.9736 ) )
        oxsDataCellRefined.setAngle_alpha( XSDataAngle( 91.0000 ) )
        oxsDataCellRefined.setAngle_beta(  XSDataAngle( 92.0000 ) )
        oxsDataCellRefined.setAngle_gamma( XSDataAngle( 93.0000 ) )
        oxsDataMOSFLMNewmat.setRefinedCell( oxsDataCellRefined )

        return oxsDataMOSFLMNewmat
    
        
    def testGetDataMOSFLMNewmat( self ):
        oedPluginMOSFLMv01 = self.createPlugin()
        oedPluginMOSFLMv01.configure()
        oesStringFilename = EDDiskExplorer.mergePath( self.m_oedStrDataPath,  "EDPluginMOSFLMv01_autoindexMat_test.txt" )
        xsDataMOSFLMNewmat = oedPluginMOSFLMv01.getDataMOSFLMNewmat( oesStringFilename )
        xsDataMOSFLMNewmatReference = self.getReferenceDataMOSFLMNewmat()
        xsDataMOSFLMNewmatReference.outputFile( "test_reference.xml" )
        xsDataMOSFLMNewmat.outputFile( "test1.xml" )
        EDAssert.equal( xsDataMOSFLMNewmatReference.marshal(), xsDataMOSFLMNewmat.marshal() )
        self.cleanUp( oedPluginMOSFLMv01 )


    def testWriteDataMOSFLMNewmat( self ):
        oedPluginMOSFLMv01 = self.createPlugin()
        oedPluginMOSFLMv01.configure()
        oxsDataMOSFLMNewmatReference = self.getReferenceDataMOSFLMNewmat()
        oedStringNewmatFilename = oedPluginMOSFLMv01.getScriptBaseName() + "_testWriteDataMOSFLMNewmat.mat"
        oedPluginMOSFLMv01.writeDataMOSFLMNewmat( oxsDataMOSFLMNewmatReference, oedStringNewmatFilename )
        self.cleanUp( oedPluginMOSFLMv01 )


    def testGenerateMOSFLMCommands( self ):
        from XSDataMOSFLMv01 import XSDataMOSFLMInput
        from XSDataMOSFLMv01 import XSDataMOSFLMBeamPosition
        from XSDataMOSFLMv01 import XSDataDetector        
        oxsDataMOSFLMInput = XSDataMOSFLMInput()
        oxsDataMOSFLMBeam = XSDataMOSFLMBeamPosition()
        oxsDataMOSFLMBeam.setX( XSDataLength( 1.0 ) )
        oxsDataMOSFLMBeam.setY( XSDataLength( 2.0 ) )
        oxsDataMOSFLMInput.setBeam( oxsDataMOSFLMBeam )
        oxsDataDetector = XSDataDetector()
        oxsDataDetector.setType( XSDataString( "ADSC" ) )
        oxsDataMOSFLMInput.setDetector( oxsDataDetector )
        oxsDataMOSFLMInput.setDirectory( XSDataString( "/tmp" ) )
        oxsDataMOSFLMInput.setTemplate( XSDataString( "testdata_1_###.img" ) )
        oxsDataMOSFLMInput.setWavelength( XSDataFloat( 1.1111 ) )
        oxsDataMOSFLMInput.setDistance( XSDataLength( 222.22 ) )
        oedPluginMOSFLMv01 = self.createPlugin()
        oedPluginMOSFLMv01.configure()
        oedPluginMOSFLMv01.m_oxsDataInput = oxsDataMOSFLMInput
        oedPluginMOSFLMv01.generateMOSFLMCommands()
        oedListCommands = oedPluginMOSFLMv01.getListCommandExecution()
        oedListCommandsReference = ['WAVELENGTH 1.1111', 'DISTANCE 222.22', 'BEAM 1.0 2.0', 'DETECTOR ADSC', 'DIRECTORY /tmp', 'TEMPLATE testdata_1_###.img']
        EDAssert.equal( oedListCommandsReference, oedListCommands )
        self.cleanUp( oedPluginMOSFLMv01 )

    
    def testEDHandlerXSDataMOSFLMv01_generateXSDataMOSFLMInputIndexing( self ):
        oedStringFilename = "XSDataIndexingInput_reference.xml"
        oedStringPath = EDDiskExplorer.mergePath( self.m_oedStrDataPath,  oedStringFilename )
        oedStringXMLIndexingInput = EDUtilsTest.readAndParseFile( oedStringPath )
        from XSDataCharacterisationv01 import XSDataIndexingInput
        xsDataIndexingInput = XSDataIndexingInput.parseString( oedStringXMLIndexingInput  )

        from EDHandlerXSDataMOSFLMv01 import EDHandlerXSDataMOSFLMv01
        xsDataMOSFLMInputIndexing = EDHandlerXSDataMOSFLMv01.generateXSDataMOSFLMInputIndexing( xsDataIndexingInput )
          
        xsDataMOSFLMInputIndexing.outputFile( "XSDataMOSFLMInputIndexing_reference.xml" )
                
        oedStringReferenceFilename = "XSDataMOSFLMInputIndexing_reference.xml"
        oedStringReferencePath = EDDiskExplorer.mergePath( self.m_oedStrDataPath,  oedStringReferenceFilename )
        #xsDataMOSFLMInputIndexing.outputFile( oedStringReferencePath )
        oedStringXMLMOSFLMInputIndexingReference = EDUtilsTest.readAndParseFile( oedStringReferencePath )
        EDAssert.equal( oedStringXMLMOSFLMInputIndexingReference, xsDataMOSFLMInputIndexing.marshal() )


    def testEDHandlerXSDataMOSFLMv01_generateXSDataIndexingResult( self ):
        oedStringFilename = "XSDataMOSFLMOutputIndexing_reference.xml"
        oedStringPath = EDDiskExplorer.mergePath( self.m_oedStrDataPath,  oedStringFilename )
        oedStringXMLMOSFLMOutputIndexing = EDUtilsTest.readAndParseFile( oedStringPath )
        from XSDataMOSFLMv01 import XSDataMOSFLMOutputIndexing
        xsDataMOSFLMOutputIndexing = XSDataMOSFLMOutputIndexing.parseString(oedStringXMLMOSFLMOutputIndexing  )

        from EDHandlerXSDataMOSFLMv01 import EDHandlerXSDataMOSFLMv01
        xsDataIndexingResult = EDHandlerXSDataMOSFLMv01.generateXSDataIndexingResult( xsDataMOSFLMOutputIndexing )
        #print      xsDataIndexingResult.marshal()   
        oedStringReferenceFilename = "XSDataIndexingResult_reference.xml"
        oedStringReferencePath = EDDiskExplorer.mergePath( self.m_oedStrDataPath,  oedStringReferenceFilename )
        #xsDataIndexingResult.outputFile( oedStringReferenceFilename )
        #xsDataIndexingResult.outputFile( oedStringReferencePath )
        oedStringXMLIndexingOutputReference = EDUtilsTest.readAndParseFile( oedStringReferencePath )
        EDAssert.equal( oedStringXMLIndexingOutputReference, xsDataIndexingResult.marshal() )

        
    def testEDHandlerXSDataMOSFLMv01_generateXSDataMOSFLMInputIntegration( self ):
        oedStringFilename = "XSDataIntegrationInput_reference.xml"
        oedStringPath = EDDiskExplorer.mergePath( self.m_oedStrDataPath,  oedStringFilename )
        oedStringXMLIntegrationInput = EDUtilsTest.readAndParseFile( oedStringPath )
        from XSDataCharacterisationv01 import XSDataIntegrationInput
        xsDataIntegrationInput = XSDataIntegrationInput.parseString( oedStringXMLIntegrationInput  )

        from EDHandlerXSDataMOSFLMv01 import EDHandlerXSDataMOSFLMv01
        xsDataMOSFLMInputIntegration = EDHandlerXSDataMOSFLMv01.generateXSDataMOSFLMInputIntegration( xsDataIntegrationInput )
        oedStringReferenceFilename = "XSDataMOSFLMInputIntegration_reference.xml"
        oedStringReferencePath = EDDiskExplorer.mergePath( self.m_oedStrDataPath,  oedStringReferenceFilename )
        #xsDataMOSFLMInputIntegration.outputFile( oedStringReferenceFilename )
        #xsDataMOSFLMInputIntegration.outputFile( oedStringReferencePath )
        oedStringXMLIntegrationInputReference = EDUtilsTest.readAndParseFile( oedStringReferencePath )
        EDAssert.equal( oedStringXMLIntegrationInputReference, xsDataMOSFLMInputIntegration.marshal() )

    
    def testEDHandlerXSDataMOSFLMv01_generateXSDataIntegrationSubWedgeResult( self ):
        oedStringFilename = "XSDataMOSFLMOutputIntegration_reference.xml"
        oedStringPath = EDDiskExplorer.mergePath( self.m_oedStrDataPath,  oedStringFilename )
        oedStringXMLMOSFLMOutputIntegration = EDUtilsTest.readAndParseFile( oedStringPath )
        from XSDataMOSFLMv01 import XSDataMOSFLMOutputIntegration
        xsDataMOSFLMOutputIntegration = XSDataMOSFLMOutputIntegration.parseString( oedStringXMLMOSFLMOutputIntegration  )

        from EDHandlerXSDataMOSFLMv01 import EDHandlerXSDataMOSFLMv01
        xsDataIntegrationSubWedgeResult = EDHandlerXSDataMOSFLMv01.generateXSDataIntegrationSubWedgeResult( xsDataMOSFLMOutputIntegration )
                
        oedStringReferenceFilename = "XSDataIntegrationSubWedgeResult_reference.xml"
        oedStringReferencePath = EDDiskExplorer.mergePath( self.m_oedStrDataPath,  oedStringReferenceFilename )
        #xsDataIntegrationSubWedgeResult.outputFile( oedStringReferenceFilename )
        #xsDataIntegrationSubWedgeResult.outputFile( oedStringReferencePath )
        oedStringXMLIntegrationSubWedgeReference = EDUtilsTest.readAndParseFile( oedStringReferencePath )
        #EDAssert.equal( oedStringXMLIntegrationSubWedgeReference, xsDataIntegrationSubWedgeResult.marshal() )

    
    
    def process( self ):
        self.addTestMethod( self.testGetDataMOSFLMNewmat )
        self.addTestMethod( self.testWriteDataMOSFLMNewmat )
        self.addTestMethod( self.testGenerateMOSFLMCommands )
        self.addTestMethod( self.testEDHandlerXSDataMOSFLMv01_generateXSDataMOSFLMInputIndexing )
        self.addTestMethod( self.testEDHandlerXSDataMOSFLMv01_generateXSDataIndexingResult )
        self.addTestMethod( self.testEDHandlerXSDataMOSFLMv01_generateXSDataMOSFLMInputIntegration )
        self.addTestMethod( self.testEDHandlerXSDataMOSFLMv01_generateXSDataIntegrationSubWedgeResult )

    

##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCasePluginUnitMOSFLMv01 = EDTestCasePluginUnitMOSFLMv01( "EDTestCasePluginUnitMOSFLMv01" )
    edTestCasePluginUnitMOSFLMv01.execute()
