#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMOSFLMv01GeneratePrediction.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDCompiler
from EDImport import EDString
from EDImport import EDVerbose
from EDImport import EDDiskExplorer
from EDImport import EDList
from EDImport import EDAssert

from EDImportTest     import EDTestCasePluginUnit
from EDImportTest     import EDSystem
from EDImportTest     import EDUtilsTest
import sys

from XSDataCommonv01 import XSDataAngle
from XSDataCommonv01 import XSDataLength
from XSDataCommonv01 import XSDataInteger
from XSDataCommonv01 import XSDataString
from XSDataCommonv01 import XSDataFloat
from XSDataCommonv01 import XSDataWavelength
from XSDataCommonv01 import XSDataFile


class EDTestCasePluginUnitMOSFLMv01GeneratePrediction( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMOSFLMv01GeneratePrediction", "EDPluginGroupMOSFLM-v0.1", _oalStringTestName )

        oedStrDataDir = "EDPluginGroupMOSFLM"
        self.m_oedStringDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        
        oedStrDataImageDir = "images"
        self.m_oedStringDataImagePath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataImageDir )
        oedStrDataDir = "EDPluginGroupMOSFLM"

        self.m_oedStringReferenceInputFile = EDDiskExplorer.mergePath( self.m_oedStringDataPath, "XSDataMOSFLMv01InputGeneratePrediction_reference.xml")
        self.m_oedStringReferenceDataOutputFile = EDDiskExplorer.mergePath( self.m_oedStringDataPath, "XSDataMOSFLMOutputGeneratePrediction_reference.xml")
        
        oedStrDataImageDir = "images"
        self.m_oedStrDataImagePath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataImageDir )
        
            
    
    def generateDataMOSFLMInputGeneratePrediction( self ):
        """
        """
        from XSDataMOSFLMv01 import XSDataMOSFLMBeamPosition
        oxsDataMOSFLMBeamPosition = XSDataMOSFLMBeamPosition()
        oxsDataMOSFLMBeamPosition.setX( XSDataLength( 102.5 ) )
        oxsDataMOSFLMBeamPosition.setY( XSDataLength( 104.7 ) )
        
        from XSDataMOSFLMv01 import XSDataMOSFLMDetector
        oxsDataMOSFLMDetector = XSDataMOSFLMDetector()
        oxsDataMOSFLMDetector.setType( XSDataString( "ADSC" ) )
        
        from XSDataMOSFLMv01 import XSDataMOSFLMImage
        oxsDataMOSFLMImage1 = XSDataMOSFLMImage()
        oxsDataMOSFLMImage1.setNumber( XSDataInteger( 1 ) )
        oxsDataMOSFLMImage1.setRotationAxisStart( XSDataAngle( 0.0 ) )
        oxsDataMOSFLMImage1.setRotationAxisEnd( XSDataAngle( 1.0 ) )
                
        from XSDataMOSFLMv01 import XSDataMOSFLMInputGeneratePrediction
        oxsDataMOSFLMInputGeneratePrediction = XSDataMOSFLMInputGeneratePrediction()
        oxsDataMOSFLMInputGeneratePrediction.setDistance( XSDataLength( 198.4 ) )
        oxsDataMOSFLMInputGeneratePrediction.setWavelength( XSDataWavelength( 0.9340 ) )
        oxsDataMOSFLMInputGeneratePrediction.setBeam( oxsDataMOSFLMBeamPosition )
        oxsDataMOSFLMInputGeneratePrediction.setDetector( oxsDataMOSFLMDetector )
        oxsDataMOSFLMInputGeneratePrediction.setDirectory( XSDataString( self.m_oedStrDataImagePath ) )
        oxsDataMOSFLMInputGeneratePrediction.setTemplate( XSDataString( "ref-testscale_1_###.img" ) )
        oxsDataMOSFLMInputGeneratePrediction.setImage( oxsDataMOSFLMImage1 )
        
        return oxsDataMOSFLMInputGeneratePrediction
        
        
    def testGenerateMOSFLMCommands( self ):
        """
        """
        pluginGeneratePrediction = self.createPlugin()
        pluginGeneratePrediction.configure()
        oxsDataMOSFLMInputGeneratePrediction = self.generateDataMOSFLMInputGeneratePrediction()
        pluginGeneratePrediction.setDataInput( oxsDataMOSFLMInputGeneratePrediction )
        pluginGeneratePrediction.generateMOSFLMCommands()
        oedListCommandsReference = EDList( ['WAVELENGTH 0.934', 'DISTANCE 198.4', 'BEAM 102.5 104.7', 'DETECTOR ADSC', 'DIRECTORY ' + self.m_oedStrDataImagePath, 'TEMPLATE ref-testscale_1_###.img', 'XGUI ON', 'IMAGE 1 PHI 0.000000 TO 1.000000', 'GO', 'PREDICT_SPOTS', 'CREATE_IMAGE PREDICTION ON BINARY TRUE FILENAME ' + pluginGeneratePrediction.getPredictionImageFileName(), 'RETURN', 'EXIT'] )
        oedListCommands = pluginGeneratePrediction.getListCommandExecution()
        print oedListCommandsReference
        print oedListCommands
        EDAssert.equal( oedListCommandsReference, oedListCommands )
        
        self.cleanUp( pluginGeneratePrediction )

    def testCreateDataMOSFLMOutputGeneratePrediction( self ):
        """
        """
        pluginGeneratePrediction = self.createPlugin()
        pluginGeneratePrediction.configure()
        pluginGeneratePrediction.setPredictionImageFileName( "prediction.jpg" )
        oedStringBaseName = pluginGeneratePrediction.createBaseName()
        pluginGeneratePrediction.setScriptBaseName( oedStringBaseName )
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_oedStringDataPath, "EDPluginMOSFLMv01GeneratePrediction_outputDnaTables_ok.xml"), EDDiskExplorer.mergePath( pluginGeneratePrediction.getWorkingDirectory(), oedStringBaseName + "_dnaTables.xml" ) ) 
        oedStringNewmatFile = EDDiskExplorer.mergePath( self.m_oedStringDataPath, "EDPluginMOSFLMv01_autoindexMat_ok.txt" )
        pluginGeneratePrediction.setNewmatFileName( oedStringNewmatFile )
        from XSDataMOSFLMv01 import XSDataMOSFLMInputGeneratePrediction
        xsDataMOSFLMInputGeneratePrediction = XSDataMOSFLMInputGeneratePrediction.parseFile( EDDiskExplorer.mergePath( self.m_oedStringDataPath, "XSDataMOSFLMInputGeneratePrediction_reference.xml" ) )
        pluginGeneratePrediction.setDataInput( xsDataMOSFLMInputGeneratePrediction )
        xsDataMOSFLMOutputGeneratePrediction = pluginGeneratePrediction.createDataMOSFLMOutputGeneratePrediction()
        oedStringImagePath = xsDataMOSFLMOutputGeneratePrediction.getPredictionImage().getPath().getValue()
        #xsDataMOSFLMOutputGeneratePrediction.outputFile( "XSDataMOSFLMOutputGeneratePrediction_reference.xml" )
        oedStringReferenceXML = EDUtilsTest.readAndParseFile( self.m_oedStringReferenceDataOutputFile )
        from XSDataMOSFLMv01 import XSDataMOSFLMOutputGeneratePrediction
        xsDataMOSFLMOutputGeneratePredictionReference = XSDataMOSFLMOutputGeneratePrediction.parseString( oedStringReferenceXML )
        xsDataImage = xsDataMOSFLMOutputGeneratePredictionReference.getPredictionImage()
        xsDataImage.setPath( XSDataString( oedStringImagePath ))
        xsDataMOSFLMOutputGeneratePredictionReference.setPredictionImage( xsDataImage )
        EDAssert.equal( xsDataMOSFLMOutputGeneratePredictionReference.marshal(), xsDataMOSFLMOutputGeneratePrediction.marshal() )
        
        self.cleanUp( pluginGeneratePrediction )

    def process( self ):
        """
        """
        self.addTestMethod( self.testGenerateMOSFLMCommands )
        self.addTestMethod( self.testCreateDataMOSFLMOutputGeneratePrediction )        



##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCasePluginUnitMOSFLMv01GeneratePrediction = EDTestCasePluginUnitMOSFLMv01GeneratePrediction( "EDTestCasePluginUnitMOSFLMv01GeneratePrediction" )
    edTestCasePluginUnitMOSFLMv01GeneratePrediction.execute()
