#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMOSFLMv01Indexing.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDCompiler
from EDImport import EDString
from EDImport import EDVerbose
from EDImport import EDDiskExplorer
from EDImport import EDList
from EDImport import EDAssert

from EDImportTest     import EDTestCasePluginUnit
from EDImportTest     import EDSystem
from EDImportTest     import EDUtilsTest
import sys

from XSDataCommonv01 import XSDataAngle
from XSDataCommonv01 import XSDataLength
from XSDataCommonv01 import XSDataInteger
from XSDataCommonv01 import XSDataString
from XSDataCommonv01 import XSDataFloat
from XSDataCommonv01 import XSDataWavelength


class EDTestCasePluginUnitMOSFLMv01Indexing( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMOSFLMv01Indexing", "EDPluginGroupMOSFLM-v0.1", _oalStringTestName )

        oedStrDataDir = "EDPluginGroupMOSFLM"
        self.m_oedStringDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        
        oedStrDataImageDir = "images"
        self.m_oedStringDataImagePath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataImageDir )
        oedStrDataDir = "EDPluginGroupMOSFLM"

        self.m_oedStringReferenceDataInputFile  = EDDiskExplorer.mergePath( self.m_oedStringDataPath, "XSDataMOSFLMInputIndexing_reference.xml")
        self.m_oedStringReferenceDataOutputFile = EDDiskExplorer.mergePath( self.m_oedStringDataPath, "XSDataMOSFLMOutputIndexing_reference.xml")
        
        oedStrDataImageDir = "images"
        self.m_oedStrDataImagePath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataImageDir )
        
            
    
    def generateDataMOSFLMInputIndexing( self ):
        """
        """
        from XSDataMOSFLMv01 import XSDataMOSFLMBeamPosition
        oxsDataMOSFLMBeamPosition = XSDataMOSFLMBeamPosition()
        oxsDataMOSFLMBeamPosition.setX( XSDataLength( 102.5 ) )
        oxsDataMOSFLMBeamPosition.setY( XSDataLength( 104.7 ) )
        
        from XSDataMOSFLMv01 import XSDataMOSFLMDetector
        oxsDataMOSFLMDetector = XSDataMOSFLMDetector()
        oxsDataMOSFLMDetector.setType( XSDataString( "ADSC" ) )
        
        from XSDataMOSFLMv01 import XSDataMOSFLMImage
        oxsDataMOSFLMImage1 = XSDataMOSFLMImage()
        oxsDataMOSFLMImage1.setNumber( XSDataInteger( 1 ) )
        oxsDataMOSFLMImage1.setRotationAxisStart( XSDataAngle( 0.0 ) )
        oxsDataMOSFLMImage1.setRotationAxisEnd( XSDataAngle( 1.0 ) )
        
        from XSDataMOSFLMv01 import XSDataMOSFLMImage
        oxsDataMOSFLMImage2 = XSDataMOSFLMImage()
        oxsDataMOSFLMImage2.setNumber( XSDataInteger( 2 ) )
        oxsDataMOSFLMImage2.setRotationAxisStart( XSDataInteger( 90.0 ) )
        oxsDataMOSFLMImage2.setRotationAxisEnd( XSDataInteger( 91.0 ) )
        
        from XSDataMOSFLMv01 import XSDataMOSFLMInputIndexing
        oxsDataMOSFLMInputIndexing = XSDataMOSFLMInputIndexing()
        oxsDataMOSFLMInputIndexing.setDistance( XSDataLength( 198.4 ) )
        oxsDataMOSFLMInputIndexing.setWavelength( XSDataWavelength( 0.9340 ) )
        oxsDataMOSFLMInputIndexing.setBeam( oxsDataMOSFLMBeamPosition )
        oxsDataMOSFLMInputIndexing.setDetector( oxsDataMOSFLMDetector )
        oxsDataMOSFLMInputIndexing.setDirectory( XSDataString( self.m_oedStrDataImagePath ) )
        oxsDataMOSFLMInputIndexing.setTemplate( XSDataString( "ref-testscale_1_###.img" ) )
        oxsDataMOSFLMInputIndexing.addImage( oxsDataMOSFLMImage1 )
        oxsDataMOSFLMInputIndexing.addImage( oxsDataMOSFLMImage2 )
        
        return oxsDataMOSFLMInputIndexing
        
        
    def testGenerateMOSFLMCommands( self ):
        """
        """
        pluginIndexing = self.createPlugin()
        pluginIndexing.configure()
        oxsDataMOSFLMInputIndexing = self.generateDataMOSFLMInputIndexing()
        pluginIndexing.setDataInput( oxsDataMOSFLMInputIndexing )
        pluginIndexing.generateMOSFLMCommands()
        oedListCommandsReference = EDList( ['WAVELENGTH 0.934', 'DISTANCE 198.4', 'BEAM 102.5 104.7', 'DETECTOR ADSC', 'DIRECTORY ' + self.m_oedStrDataImagePath, 'TEMPLATE ref-testscale_1_###.img', 'NEWMAT ' + pluginIndexing.getNewmatFileName(), 'AUTOINDEX DPS REFINE IMAGE 1 PHI 0.0 1.0', 'AUTOINDEX DPS REFINE IMAGE 2 PHI 90.0 91.0', 'MOSAIC ESTIMATE', 'GO'] )
        oedListCommands = pluginIndexing.getListCommandExecution()
        #print oedListCommandsReference
        #print oedListCommands
        EDAssert.equal( oedListCommandsReference, oedListCommands )        
        self.cleanUp( pluginIndexing )


    def testCreateDataMOSFLMOutputIndexing( self ):
        """
        """
        pluginIndexing = self.createPlugin()
        pluginIndexing.configure()
        oedStringBaseName = pluginIndexing.createBaseName()
        pluginIndexing.setScriptBaseName( oedStringBaseName )
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_oedStringDataPath, "EDPluginMOSFLMv01Indexing_outputDnaTables_ok.xml"), EDDiskExplorer.mergePath( pluginIndexing.getWorkingDirectory(), oedStringBaseName + "_dnaTables.xml" ) ) 
        oedStringNewmatFile = EDDiskExplorer.mergePath( self.m_oedStringDataPath, "EDPluginMOSFLMv01_autoindexMat_ok.txt" )
        pluginIndexing.setNewmatFileName( oedStringNewmatFile )
        xsDataMOSFLMOutputIndexing = pluginIndexing.createDataMOSFLMOutputIndexing()
        #xsDataMOSFLMOutputIndexing.outputFile( "XSDataMOSFLMOutputIndexing_reference.xml" )
        oedStringReferenceXML = EDUtilsTest.readAndParseFile( self.m_oedStringReferenceDataOutputFile )
        EDAssert.equal( oedStringReferenceXML, xsDataMOSFLMOutputIndexing.marshal() )
        self.cleanUp( pluginIndexing )


    def testGenerateExecutiveSummary( self ):
        """
        """
        pluginIndexing = self.createPlugin()
        pluginIndexing.configure()
        from XSDataMOSFLMv01 import XSDataMOSFLMInputIndexing
        from XSDataMOSFLMv01 import XSDataMOSFLMOutputIndexing
        oedStringMOSFLMInputIndexingXML  = EDUtilsTest.readAndParseFile( self.m_oedStringReferenceDataInputFile )
        oedStringMOSFLMOutputIndexingXML = EDUtilsTest.readAndParseFile( self.m_oedStringReferenceDataOutputFile )
        xsDataMOSFLMInputIndexing  = XSDataMOSFLMInputIndexing.parseString( oedStringMOSFLMInputIndexingXML )
        xsDataMOSFLMOutputIndexing = XSDataMOSFLMOutputIndexing.parseString( oedStringMOSFLMOutputIndexingXML )
        pluginIndexing.setDataInput( xsDataMOSFLMInputIndexing )
        pluginIndexing.setDataOutput( xsDataMOSFLMOutputIndexing )
        pluginIndexing.generateExecutiveSummary( pluginIndexing )
        pluginIndexing.verboseScreenExecutiveSummary()
        self.cleanUp( pluginIndexing )

    def process( self ):
        """
        """
        self.addTestMethod( self.testGenerateMOSFLMCommands )
        self.addTestMethod( self.testCreateDataMOSFLMOutputIndexing )
        self.addTestMethod( self.testGenerateExecutiveSummary )  



##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCasePluginUnitMOSFLMv01Indexing = EDTestCasePluginUnitMOSFLMv01Indexing( "EDTestCasePluginUnitMOSFLMv01Indexing" )
    edTestCasePluginUnitMOSFLMv01Indexing.execute()
