#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMOSFLMv01Integration.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport         import EDCompiler
from EDImport         import EDString
from EDImport         import EDVerbose
from EDImport         import EDDiskExplorer
from EDImport         import EDList
from EDImport         import EDAssert
from EDImportTest     import EDTestCasePluginUnit
from EDImportTest     import EDSystem
from EDImportTest     import EDUtilsTest

from XSDataCommonv01 import XSDataFile
from XSDataCommonv01 import XSDataString

class EDTestCasePluginUnitMOSFLMv01Integration( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMOSFLMv01Integration", "EDPluginGroupMOSFLM-v0.1", _oalStringTestName )

        oedStrDataDir = "EDPluginGroupMOSFLM"
        self.m_oedStringDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        self.m_oedStringReferenceDataInputFile  = EDDiskExplorer.mergePath( self.m_oedStringDataPath, "XSDataMOSFLMInputIntegration_reference.xml")
        self.m_oedStringReferenceDataOutputFile = EDDiskExplorer.mergePath( self.m_oedStringDataPath, "XSDataMOSFLMOutputIntegration_reference.xml")
        
        oedStrDataImageDir = "images"
        self.m_oedStrDataImagePath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataImageDir )
        
        
    def preProcess( self ):
        """
        """  
        EDVerbose.DEBUG ("*** EDTestCaseEDPluginMOSFLMv01Integration.preProcess")
    
    
                
    def testGenerateMOSFLMIntegrationCommands( self ):
        """
        """
        pluginIntegration = self.createPlugin()
        pluginIntegration.configure()
        strXMLInputData = EDUtilsTest.readAndParseFile ( self.m_oedStringReferenceDataInputFile )
        pluginIntegration.setDataInput( strXMLInputData )
        pluginIntegration.generateMOSFLMCommands()
        oedListCommandExecution = pluginIntegration.getListCommandExecution()
        #oedListCommandReference = ['SYMMETRY P222', 'MATRIX ' + EDString( pluginIntegration.getScriptBaseName() + "_newmat.mat" ), 'MOSAIC 0.75', 'WAVELENGTH 0.934', 'DISTANCE 198.441', 'BEAM 102.478996 104.8862', 'DETECTOR ADSC', 'DIRECTORY '+self.m_oedStrDataImagePath, 'TEMPLATE ref-testscale_1_###.img', 'HKLOUT ' + EDString( pluginIntegration.getScriptBaseName() ) + '_process_1_1.mtz', 'PROCESS 1 TO 1 START 0.000000 ANGLE 1.000000', 'BEST ON', 'GO', 'BEST OFF']
        oedListCommandReference = ['WAVELENGTH 0.934', 'DISTANCE 198.440994', 'BEAM 102.478996 104.8862', 'DETECTOR ADSC', 'DIRECTORY '+self.m_oedStrDataImagePath, 'TEMPLATE ref-testscale_1_###.img', 'SYMMETRY P222', 'MATRIX ' + EDString( pluginIntegration.getScriptBaseName() ) + '_matrix.mat', 'MOSAIC 0.75', 'HKLOUT ' + EDString( pluginIntegration.getScriptBaseName() ) + '_process_1_1.mtz', 'PROCESS 1 TO 1 START 0.000000 ANGLE 1.000000', 'BEST ON', 'GO', 'BEST OFF']
        #print oedListCommandReference
        #print oedListCommandExecution
        EDAssert.equal(oedListCommandReference, oedListCommandExecution)
        

    def testCreateDataMOSFLMOutputIntegration( self ):
        """
        """
        pluginIntegration = self.createPlugin()
        pluginIntegration.configure()
        oedStringBaseName = pluginIntegration.getBaseName()
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_oedStringDataPath, "EDPluginMOSFLMv01Integration_bestfileDat_ok.txt"), EDDiskExplorer.mergePath ( pluginIntegration.getWorkingDirectory(), "bestfile.dat") ) 
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_oedStringDataPath, "EDPluginMOSFLMv01Integration_bestfilePar_ok.txt"), EDDiskExplorer.mergePath ( pluginIntegration.getWorkingDirectory(), "bestfile.par") ) 
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_oedStringDataPath, "EDPluginMOSFLMv01Integration_bestfileHKL_ok.txt"), EDDiskExplorer.mergePath ( pluginIntegration.getWorkingDirectory(), "bestfile.hkl") ) 
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_oedStringDataPath, "EDPluginMOSFLMv01Integration_outputDnaTables_ok.xml"), EDDiskExplorer.mergePath( pluginIntegration.getWorkingDirectory(), oedStringBaseName + "_dnaTables.xml" ) ) 
        oedStringMatrixFile = EDDiskExplorer.mergePath( self.m_oedStringDataPath, "EDPluginMOSFLMv01_autoindexMat_ok.txt" )
        pluginIntegration.setMatrixFileName( oedStringMatrixFile )
        xsDataMOSFLMIntegrationOutput = pluginIntegration.createDataMOSFLMOutputIntegration()
        # Fix problem with absolute path by replacing it with a fixed one
        xsDataFile = XSDataFile()
        xsDataFile.setPath( XSDataString( "/tmp/EDPluginMOSFLMv01Integration_process_1_1.mtz" ) )
        xsDataMOSFLMIntegrationOutput.setGeneratedMTZFile( xsDataFile )
        oedStringReferenceXML = EDUtilsTest.readAndParseFile( self.m_oedStringReferenceDataOutputFile )
        xsDataMOSFLMIntegrationOutput.outputFile( "test.xml" )
        EDAssert.equal(oedStringReferenceXML, xsDataMOSFLMIntegrationOutput.marshal() )
        

    def testGenerateExecutiveSummary( self ):
        """
        """
        pluginIntegration = self.createPlugin()
        pluginIntegration.configure()
        from XSDataMOSFLMv01 import XSDataMOSFLMInputIntegration
        from XSDataMOSFLMv01 import XSDataMOSFLMOutputIntegration
        oedStringMOSFLMInputIntegrationXML = EDUtilsTest.readAndParseFile( self.m_oedStringReferenceDataInputFile )
        oedStringMOSFLMOutputIntegrationXML = EDUtilsTest.readAndParseFile( self.m_oedStringReferenceDataOutputFile )
        xsDataMOSFLMInputIntegration = XSDataMOSFLMInputIntegration.parseString( oedStringMOSFLMInputIntegrationXML )
        xsDataMOSFLMOutputIntegration = XSDataMOSFLMOutputIntegration.parseString( oedStringMOSFLMOutputIntegrationXML )
        pluginIntegration.setDataInput( xsDataMOSFLMInputIntegration )
        pluginIntegration.setDataOutput( xsDataMOSFLMOutputIntegration )
        pluginIntegration.generateExecutiveSummary( pluginIntegration )
        pluginIntegration.verboseScreenExecutiveSummary()
        self.cleanUp( pluginIntegration )



    def process( self ):
        """
        """
        self.addTestMethod( self.testGenerateMOSFLMIntegrationCommands )
        self.addTestMethod( self.testCreateDataMOSFLMOutputIntegration )        
        self.addTestMethod( self.testGenerateExecutiveSummary )  
        
        
    def postProcess( self ):
        """
        """
        
        # check if in the next AALib release, the MOSFLM plugin directory is added only once in the PYTHONPATH
        EDVerbose.DEBUG ( "*** EDTestCaseEDPluginMOSFLMv01Integration.postProcess" )



##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCasePluginUnitMOSFLMv01Integration = EDTestCasePluginUnitMOSFLMv01Integration( "EDTestCasePluginUnitMOSFLMv01Integration" )
    edTestCasePluginUnitMOSFLMv01Integration.execute()
