#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMessagePerClass.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDCompiler
from EDImport                         import EDVerbose
from EDImport                         import EDPlugin
from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDAssert
from EDImport                         import EDDiskExplorer
from EDImportTest                     import EDTestCasePluginUnit




class EDTestCasePluginUnitMessagePerClass( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMessage", "EDPluginMessage-v0.1", _oalStringTestName )
        self.m_xsPluginItem = None
        self.m_dataInput = None
        
        oedStrDataDir = "EDPluginMessage"
        self.m_oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        
        
    def preProcess( self, _oedObject = None ):
        """
        """
        self.m_xsPluginItem = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, self.getPlugin().getPluginName() + "_configuration_OK_01.xml" ))
        edStrDataInputFileName = EDDiskExplorer.mergePath( self.m_oedStrDataPath, self.getPlugin().getPluginName() + "_inputData_OK_01.xml" )
        strXMLData = None
        try:
            strXMLData = open( edStrDataInputFileName, "r" ).read()
        except:
            raise
        
        self.m_dataInput = EDString( strXMLData )
    
        
    def testConfigure( self ):
        """
        """
        oedPluginMessage = self.getPlugin()
        oedPluginMessage.setConfiguration( self.m_xsPluginItem )
        oedPluginMessage.configure()
        EDAssert.equal("it/is/here", oedPluginMessage.getWorkingDir())
                    
        
    def testSetDataInput( self ):
        """
        """
        oedPluginMessage = self.getPlugin()
        oedPluginMessage.setDataInput( self.m_dataInput )
        
        EDAssert.equal("This is the secret key", oedPluginMessage.getMessage())
        
        
    def testExecute( self ):
        """
        """
        oedPluginMessage = self.getPlugin()
        oedPluginMessage.execute()
        oedPluginMessage.synchronize()
        
        EDAssert.equal( "OK!!! Got your Message", oedPluginMessage.getResponse() )       
        
        

##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testConfigure )
        self.addTestMethod( self.testSetDataInput )
        self.addTestMethod( self.testExecute )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginUnitMessagePerClass = EDTestCasePluginUnitMessagePerClass( "EDTestCasePluginUnitMessagePerClass" )
    edTestCasePluginUnitMessagePerClass.execute()

