#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMessagePerMethod.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDCompiler
from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDAssert
from EDImport                         import EDDiskExplorer
from EDImportTest                     import EDTestCasePluginUnit




class EDTestCasePluginUnitMessagePerMethod( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMessage", "EDPluginMessage-v0.1", _oalStringTestName )

        oedStrDataDir = "EDPluginMessage"
        self.m_oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
    
        
    def testConfigureOK( self ):
        """
        """
        oedPluginMessage = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, oedPluginMessage.getPluginName() + "_configuration_OK_01.xml" ))
        oedPluginMessage.setConfiguration( xsPluginItemGood01 )
        oedPluginMessage.configure()
        EDAssert.equal("it/is/here", oedPluginMessage.getWorkingDir())
        
        
    def testConfigureBAD01( self ):
        
        oedPluginMessage = self.createPlugin()
        xsPluginItemBad01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, oedPluginMessage.getPluginName() + "_configuration_BAD_01.xml" ))
        oedPluginMessage.setConfiguration( xsPluginItemBad01 )
        oedPluginMessage.configure()
        EDAssert.equal("Default location", oedPluginMessage.getWorkingDir())
        self.cleanUp( oedPluginMessage )
        
        
    def testConfigureBAD02( self ):       

        oedPluginMessage = self.createPlugin()
        xsPluginItemBad02 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, oedPluginMessage.getPluginName() + "_configuration_BAD_02.xml" ))
        oedPluginMessage.setConfiguration( xsPluginItemBad02 )
        oedPluginMessage.configure()
        EDAssert.equal("Default location", oedPluginMessage.getWorkingDir()) 
        self.cleanUp( oedPluginMessage )
        
        
    def testSetDataInputOK( self ):
        """
        """
        oedPluginMessage = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, oedPluginMessage.getPluginName() + "_configuration_OK_01.xml" ))

        oedPluginMessage.setConfiguration( xsPluginItemGood01 )
        oedPluginMessage.configure()
        edStrDataInputFileName = EDDiskExplorer.mergePath( self.m_oedStrDataPath, oedPluginMessage.getPluginName() + "_inputData_OK_01.xml" )
        strXMLData = None
        try:
            strXMLData = open( edStrDataInputFileName, "r" ).read()
        except:
            raise
        
        oedPluginMessage.setDataInput( EDString( strXMLData ) )
        
        EDAssert.equal("This is the secret key", oedPluginMessage.getMessage())
        self.cleanUp( oedPluginMessage )
        
        
    def testExecuteOK( self ):
        """
        """
        oedPluginMessage = self.createPlugin()
        self.checkExecute( oedPluginMessage,
                           EDDiskExplorer.mergePath( self.m_oedStrDataPath, oedPluginMessage.getPluginName() + "_configuration_OK_01.xml" ),
                           EDDiskExplorer.mergePath( self.m_oedStrDataPath, oedPluginMessage.getPluginName() + "_inputData_OK_01.xml" ),
                           "OK!!! Got your Message")
        self.cleanUp( oedPluginMessage )
        
        
    def testExecuteBAD01( self ):
        """
        """
        oedPluginMessage = self.createPlugin()
        self.checkExecute( oedPluginMessage,
                           EDDiskExplorer.mergePath( self.m_oedStrDataPath, oedPluginMessage.getPluginName() + "_configuration_OK_01.xml" ),
                           EDDiskExplorer.mergePath( self.m_oedStrDataPath, oedPluginMessage.getPluginName() + "_inputData_BAD_01.xml" ),
                           "Sorry... Try Again!!!")
        self.cleanUp( oedPluginMessage )
        
        
    def checkExecute(self, _oedPlugin, _oedStrConfigFileName, _oedStrDataInputFileName, _oedStrExpected ):
        """
        """
        xsPluginItem = self.getPluginConfiguration( _oedStrConfigFileName )
        _oedPlugin.setConfiguration( xsPluginItem )
        _oedPlugin.configure()

        edStrDataInputFileName = _oedStrDataInputFileName
        strXMLData = None
        try:
            strXMLData = open( edStrDataInputFileName, "r" ).read()
        except:
            raise
        
        _oedPlugin.setDataInput( EDString( strXMLData ) )

        _oedPlugin.execute()
        _oedPlugin.synchronize()
        
        EDAssert.equal( _oedStrExpected, _oedPlugin.getResponse() )        
        self.cleanUp( _oedPlugin )
        

##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testConfigureOK )
        self.addTestMethod( self.testConfigureBAD01 )
        self.addTestMethod( self.testConfigureBAD02 )
        self.addTestMethod( self.testSetDataInputOK )
        self.addTestMethod( self.testExecuteOK )
        self.addTestMethod( self.testExecuteBAD01 )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginUnitMessagePerMethod = EDTestCasePluginUnitMessagePerMethod( "EDTestCasePluginUnitMessagePerMethod" )
    edTestCasePluginUnitMessagePerMethod.execute()

