#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitRaddosev01.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDCompiler
from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDAssert
from EDImport                         import EDDiskExplorer
from EDImport                         import EDDict
from EDImport                         import EDDictionary
from EDImportTest                     import EDTestCasePluginUnit
from EDImportTest                     import EDUtilsTest


class EDTestCasePluginUnitRaddosev01( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginRaddosev01", "EDPluginRaddose-v0.1", _oalStringTestName )
        oedStrDataDir = "EDPluginRaddose"
        self.m_oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        
        self.m_oedObtainedInputFile = "XSDataInputRaddosev01.xml"
        self.m_oedObtainedInputFile2 = "XSDataInputRaddosev01FromObject_02.xml"
        self.m_oedReferenceInputFile = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSDataRaddosev01Input_reference.xml")
        self.m_oedReferenceInputFile2 = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSDataRaddosev01Input_reference_02.xml")
        
        self.m_oedReferenceScriptLogFileName = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "EDPluginRaddosev01.log" )
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginUnit.preProcess( self )
        EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile )
        EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile2 )
        
        
    def testConfigureOK( self ):
        """
        """
        oedPluginRaddose = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration_ESRF.xml" ))
        oedPluginRaddose.setConfiguration( xsPluginItemGood01 )
        oedPluginRaddose.configure()
        EDAssert.equal("/bin/bash", oedPluginRaddose.getShell())
        EDAssert.equal("/opt/pxsoft/raddose/v20070705/redhate4-i686/bin/raddose_05_july_2007/raddose", oedPluginRaddose.getScriptExecutable())
        EDAssert.equal("/opt/pxsoft/ccp4-6.0.2/include/ccp4.setup-bash.orig", oedPluginRaddose.getSetupCCP4())
        EDAssert.equal("Verion of Raddose to be tested", oedPluginRaddose.getStringVersion())
        #EDAssert.equal(600, oedPluginMessage.getTimeOut())
        
        self.cleanUp( oedPluginRaddose )
        
        
    def testSetDataModelInput( self ):
        """
        """
        # Crystal         
        from XSDataRaddosev01 import XSDataRaddosev01Input
        xsDataRaddoseInput = XSDataRaddosev01Input()
        
        from XSDataRaddosev01 import XSDataString
        from XSDataRaddosev01 import XSDataFloat
        from XSDataRaddosev01 import XSDataAtom
        from XSDataRaddosev01 import XSDataAtomicComposition
        from XSDataRaddosev01 import XSDataAngle
        from XSDataRaddosev01 import XSDataLength
        from XSDataRaddosev01 import XSDataSize
        from XSDataRaddosev01 import XSDataCell
        from XSDataRaddosev01 import XSDataInteger

        xsDataAtomSulfur = XSDataAtom()
        xsDataAtomSulfur.setNumberOf( XSDataInteger(4) )
        xsDataAtomSulfur.setSymbol( XSDataString("S"))
        xsDataAtomSelenium = XSDataAtom()
        xsDataAtomSelenium.setNumberOf( XSDataInteger(4) )
        xsDataAtomSelenium.setSymbol( XSDataString("Se"))
        
        xsDataAtomicComposition = XSDataAtomicComposition()
        xsDataAtomicComposition.addAtom( xsDataAtomSulfur )
        xsDataAtomicComposition.addAtom( xsDataAtomSelenium )
        xsDataRaddoseInput.setCrystalPATM( xsDataAtomicComposition )
        xsDataRaddoseInput.setCrystalNRES( XSDataFloat(295) )
        xsDataRaddoseInput.setCrystalNMON( XSDataFloat(8) )
        xsDataCell = XSDataCell( XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataLength(78.9),
                                 XSDataLength(95.162),
                                 XSDataLength(104.087) )
        
        xsDataSizeCrystal = XSDataSize( XSDataLength(0.1),
                                        XSDataLength(0.1),
                                        XSDataLength(0.1))
        
        xsDataRaddoseInput.setCrystalCell( xsDataCell )
        xsDataRaddoseInput.setCrystalSize( xsDataSizeCrystal )
        
        # Beam
        from XSDataRaddosev01 import XSDataFlux
        from XSDataRaddosev01 import XSDataWavelength
        from XSDataRaddosev01 import XSDataTime
                
        xsDataSize = XSDataSize( XSDataLength(0.1),
                                 XSDataLength(0.1))
        
        xsDataRaddoseInput.setBeamSize( xsDataSize )
        xsDataRaddoseInput.setBeamFlux( XSDataFlux(1e+12) )
        xsDataRaddoseInput.setBeamWavelength( XSDataWavelength(2.41) )
        xsDataRaddoseInput.setBeamExposureTime( XSDataTime(1))
 
        from XSDataRaddosev01 import XSDataInteger

        xsDataRaddoseInput.setNumberOfImages( XSDataInteger(1) )
        
        xsDataRaddoseInput.outputFile( self.m_oedObtainedInputFile )
        
        strExpectedInput = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile )
        strObtainedInput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedInputFile )  
        
        EDAssert.equal( strExpectedInput, strObtainedInput )
             
        
    def testSetDataInput( self ):
        """
        """
        oedPluginRaddose = self.createPlugin()
        
        EDAssert.raiseException( RuntimeError, oedPluginRaddose.setDataInput, EDDict())
        
        strXMLInputData = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile )
        oedPluginRaddose.setDataInput( EDString( strXMLInputData ) )
        oedPluginRaddose.preProcess()
        
        xsDataInput = oedPluginRaddose.getDataInput()
        xsDataInput.outputFile( "XSDataInputRaddosev01FromObject.xml" )
        strExpectedInput = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile )
        strObtainedInput = EDUtilsTest.readAndParseFile ( "XSDataInputRaddosev01FromObject.xml" )
        
        EDAssert.equal( strExpectedInput, strObtainedInput )
        
        EDAssert.equal( "CELL 78.9 95.162 104.087 90.0 90.0 90.0", oedPluginRaddose.getCommandCrystalCell() )
        
        EDAssert.equal( "NRES 295.0", oedPluginRaddose.getCommandCrystalNRES() )
        EDAssert.equal( "NMON 8.0", oedPluginRaddose.getCommandCrystalNMON() )
        EDAssert.equal( None, oedPluginRaddose.getCommandCrystalNDNA() )
        EDAssert.equal( None, oedPluginRaddose.getCommandCrystalNRNA() )
        EDAssert.equal( "S", xsDataInput.getCrystalPATM().getAtom()[0].getSymbol().getValue() )
        EDAssert.equal( 4, xsDataInput.getCrystalPATM().getAtom()[0].getNumberOf().getValue() )
        EDAssert.equal( "Se", xsDataInput.getCrystalPATM().getAtom()[1].getSymbol().getValue() )
        EDAssert.equal( 4, xsDataInput.getCrystalPATM().getAtom()[1].getNumberOf().getValue() )
        EDAssert.equal( "PATM S 4 Se 4", oedPluginRaddose.getCommandCrystalPATM() )
        EDAssert.equal( None, xsDataInput.getCrystalSATM() )
        EDAssert.equal( None, oedPluginRaddose.getCommandCrystalSATM() )
        EDAssert.equal( "CRYSTAL 0.1 0.1 0.1", oedPluginRaddose.getCommandCrystalSize() )
        
        EDAssert.equal( "BEAM 0.1 0.1", oedPluginRaddose.getCommandBeamSize() )
        EDAssert.equal( "PHOSEC 1e+12", oedPluginRaddose.getCommandBeamFlux() )
        EDAssert.equal( "WAVELENGTH 2.41", oedPluginRaddose.getCommandBeamWavelength() )
        
        EDAssert.equal( "EXPOSURE 1.0", oedPluginRaddose.getCommandExposureTime() )
        EDAssert.equal( "IMAGES 1", oedPluginRaddose.getCommandImages() )
        
        self.cleanUp( oedPluginRaddose )
        
        
    def testDictionnaryResults( self ):
        """
        """
        oedPluginRaddose = self.createPlugin()
        expectedDictionnary = EDDict()
        expectedDictionnary.add("Dose in Grays", "0.22E+06")
        expectedDictionnary.add("Solvent Content (%)", "59.2")
        
        obtainedDictionnary = oedPluginRaddose.analyseScriptLogFileName( self.m_oedReferenceScriptLogFileName )
        
        EDAssert.equal( expectedDictionnary, obtainedDictionnary )
        
        
    def testMergeAtomicComposition( self ):
        """
        """
        from XSDataRaddosev01 import XSDataAtomicComposition
        from XSDataRaddosev01 import XSDataAtom
        from XSDataRaddosev01 import XSDataFloat
        from XSDataRaddosev01 import XSDataString
        from XSDataRaddosev01 import XSDataInteger
        
        xsDataAtomicComposition1 = XSDataAtomicComposition()
        xsDataAtom1 = XSDataAtom()
        xsDataAtom1.setSymbol( XSDataString("S") )
        xsDataAtom1.setNumberOf( XSDataInteger(4) )
        xsDataAtomicComposition1.addAtom( xsDataAtom1 )
        
        xsDataAtom2 = XSDataAtom()
        xsDataAtom2.setSymbol( XSDataString("Se") )
        xsDataAtom2.setNumberOf( XSDataInteger(3) )
        xsDataAtomicComposition1.addAtom( xsDataAtom2 )
        
        xsDataAtomicComposition2 = XSDataAtomicComposition()
        xsDataAtom3 = XSDataAtom()
        xsDataAtom3.setSymbol( XSDataString("S") )
        xsDataAtom3.setNumberOf( XSDataInteger(1) )
        xsDataAtomicComposition2.addAtom( xsDataAtom3 )
        
        xsDataAtom4 = XSDataAtom()
        xsDataAtom4.setSymbol( XSDataString("Fe") )
        xsDataAtom4.setNumberOf( XSDataInteger(5) )
        xsDataAtomicComposition2.addAtom( xsDataAtom4 )
    
        
        from EDHandlerXSDataRaddosev01 import EDHandlerXSDataRaddosev01
        xsDataMergedAtomicComposition = EDHandlerXSDataRaddosev01().mergeAtomicComposition( xsDataAtomicComposition1,
                                                                                           xsDataAtomicComposition2 )
        
        EDAssert.equal( 3, len( xsDataMergedAtomicComposition.getAtom() ) )
        
        for atom in xsDataMergedAtomicComposition.getAtom():
            if( atom.getSymbol().getValue() == "S" ):
                EDAssert.equal( 5, atom.getNumberOf().getValue() )
            if( atom.getSymbol().getValue() == "Se" ):
                EDAssert.equal( 3, atom.getNumberOf().getValue() )
            if( atom.getSymbol().getValue() == "Fe" ):
                EDAssert.equal( 5, atom.getNumberOf().getValue() )
                
        xsDataMergedAtomicComposition = EDHandlerXSDataRaddosev01().mergeAtomicComposition( XSDataAtomicComposition(),
                                                                                           xsDataAtomicComposition2 )
        EDAssert.equal( 2, len( xsDataMergedAtomicComposition.getAtom() ) )
        for atom in xsDataMergedAtomicComposition.getAtom():
            if( atom.getSymbol().getValue() == "S" ):
                EDAssert.equal( 1, atom.getNumberOf().getValue() )
            if( atom.getSymbol().getValue() == "Fe" ):
                EDAssert.equal( 5, atom.getNumberOf().getValue() )

        
        
    def testSymmetryTableTest( self ):
        """
        """
        
        import ConfigParser

        config = ConfigParser.ConfigParser()
        config.read("table_symmetry.txt")

        print "id:", config.get("3", "id")
        print "Number of Symmetry Operators:", config.get("3", "Number of Symmetry Operators")
        print "Symmetry:", config.get("3", "symmetry")
        
        from XSDataRaddosev01 import XSDataSymmetry
        from XSDataRaddosev01 import XSDataFloat
        from XSDataRaddosev01 import XSDataString
        
        xsDataSymmetry = XSDataSymmetry( XSDataString("1"), XSDataFloat() )
        
        
    def testGetXSDataRaddosev01Input( self ):
        """
        """
        
        from XSDataRaddosev01 import XSDataBeam
        from XSDataRaddosev01 import XSDataLength
        from XSDataRaddosev01 import XSDataWavelength
        from XSDataRaddosev01 import XSDataFlux
        from XSDataRaddosev01 import XSDataSize
        from XSDataRaddosev01 import XSDataStructure
        from XSDataRaddosev01 import XSDataChain
        from XSDataRaddosev01 import XSDataFloat
        from XSDataRaddosev01 import XSDataAtom
        from XSDataRaddosev01 import XSDataString
        from XSDataRaddosev01 import XSDataLigand
        from XSDataRaddosev01 import XSDataCrystal
        from XSDataRaddosev01 import XSDataSpaceGroup
        from XSDataRaddosev01 import XSDataSampleCrystalMM
        from XSDataRaddosev01 import XSDataChemicalCompositionMM
        from XSDataRaddosev01 import XSDataAtomicComposition
        from XSDataRaddosev01 import XSDataSolvent
        from XSDataRaddosev01 import XSDataCell
        from XSDataRaddosev01 import XSDataAngle
        from XSDataRaddosev01 import XSDataTime
        from XSDataRaddosev01 import XSDataInteger
        #from XSDataRaddosev01 import XSDataSymmetry
        from XSDataRaddosev01 import XSDataString
        from EDHandlerXSDataRaddosev01 import EDHandlerXSDataRaddosev01
        
        xsDataBeam = XSDataBeam()
        xsDataBeam.setSize( XSDataSize( XSDataLength(0.1), XSDataLength(0.1) ) )
        xsDataBeam.setWavelength( XSDataWavelength( 2.41 ) )
        xsDataBeam.setFlux( XSDataFlux(1e+12) )
        
        xsDataSample = XSDataSampleCrystalMM()
        xsDataStructure = XSDataStructure()
        xsDataComposition = XSDataChemicalCompositionMM()
        
        xsDataChain = XSDataChain()
        xsDataChain.setType( XSDataString( "protein" ))
        xsDataChain.setNumberOfCopies( XSDataFloat( 2 ))
        xsDataAtomicComposition = XSDataAtomicComposition()
        xsDataAtom1 = XSDataAtom()
        xsDataAtom1.setSymbol( XSDataString("Se") )
        xsDataAtom1.setNumberOf( XSDataInteger(4) )
        xsDataAtomicComposition.addAtom( xsDataAtom1 )
        xsDataChain.setHeavyAtoms( xsDataAtomicComposition )
        xsDataChain.setNumberOfMonomers( XSDataFloat( 100 ))
        xsDataStructure.addChain( xsDataChain )
        
        xsDataChain2 = XSDataChain()
        xsDataChain2.setType( XSDataString( "rna" ))
        xsDataChain2.setNumberOfCopies( XSDataFloat( 1 ))
        xsDataChain2.setNumberOfMonomers( XSDataFloat( 60 ))
        xsDataStructure.addChain( xsDataChain2 )
        
        xsDataLigand = XSDataLigand()
        xsDataLigand.setNumberOfCopies( XSDataFloat( 2 ))
        xsDataLigand.setNumberOfLightAtoms( XSDataFloat( 42 ))
        xsDataAtomicComposition = XSDataAtomicComposition()
        xsDataAtom2 = XSDataAtom()
        xsDataAtom2.setSymbol( XSDataString("Fe") )
        xsDataAtom2.setNumberOf( XSDataInteger(1) )
        xsDataAtomicComposition.addAtom( xsDataAtom2 )
        xsDataLigand.setHeavyAtoms( xsDataAtomicComposition )
        xsDataStructure.addLigand( xsDataLigand )
        xsDataStructure.setNumberOfCopiesInAsymmetricUnit( XSDataFloat( 0.25 ))
        
        xsDataSolvent = XSDataSolvent()
        xsDataAtomicComposition = XSDataAtomicComposition()
        xsDataAtomNa = XSDataAtom()
        xsDataAtomNa.setSymbol( XSDataString("Na") )
        xsDataAtomNa.setConcentration( XSDataFloat(1000) )
        xsDataAtomicComposition.addAtom( xsDataAtomNa )
        xsDataAtomCl = XSDataAtom()
        xsDataAtomCl.setSymbol( XSDataString("Cl") )
        xsDataAtomCl.setConcentration( XSDataFloat(1000) )
        xsDataAtomicComposition.addAtom( xsDataAtomCl )
        xsDataSolvent.setAtoms( xsDataAtomicComposition )

        xsDataComposition.setStructure( xsDataStructure )
        xsDataComposition.setSolvent( xsDataSolvent )
        xsDataSample.setChemicalComposition( xsDataComposition )
        
        xsDataSample.setSize( XSDataSize( XSDataLength(0.1), XSDataLength(0.1), XSDataLength(0.1) ))
        xsDataCell = XSDataCell( XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataLength(78.9),
                                 XSDataLength(95.162),
                                 XSDataLength(104.087) )
        
        xsDataCrystal = XSDataCrystal()
        xsDataSpaceGroup = XSDataSpaceGroup()
        
        xsDataCrystal.setCell( xsDataCell )
        
        xsDataSpaceGroup.setITNumber( XSDataInteger(16))
        xsDataCrystal.setSpaceGroup( xsDataSpaceGroup )
        
        xsDataSample.setCrystal( xsDataCrystal )
        
        #edStringSymTableFileName = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "symop.lib" )
        
        iNumSymOperators = 4
        
        
        xsDataRaddosev01Input = EDHandlerXSDataRaddosev01().getXSDataRaddosev01Input( xsDataBeam,
                                                                                      xsDataSample,
                                                                                      iNumSymOperators )
        
        
        xsDataRaddosev01Input.outputFile( self.m_oedObtainedInputFile2 )
        strExpectedInput = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile2 )
        strObtainedInput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedInputFile2 )
        EDAssert.equal( strExpectedInput, strObtainedInput )
        
        

##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testConfigureOK )
        self.addTestMethod( self.testSetDataModelInput )
        self.addTestMethod( self.testSetDataInput )
        self.addTestMethod( self.testDictionnaryResults )
        self.addTestMethod( self.testMergeAtomicComposition )
        self.addTestMethod( self.testGetXSDataRaddosev01Input )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginUnitRaddosev01 = EDTestCasePluginUnitRaddosev01( "EDTestCasePluginUnitRaddosev01" )
    edTestCasePluginUnitRaddosev01.execute()

