#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitReadImageHeaderv01.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport         import EDCompiler
from EDImport         import EDVerbose
from EDImport         import EDDiskExplorer
from EDImportTest     import EDUtilsTest
from EDImportTest     import EDTestCasePluginUnit
from EDImport         import EDString 
from EDImport         import EDAssert 
from EDImport         import EDFile 

class EDTestCasePluginUnitReadImageHeaderv01( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None ):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginReadImageHeaderv01", "EDPluginReadImageHeader-v0.1", _oalStringTestName )
        
        oedStrDataDir = "EDPluginReadImageHeader"
        self.m_oedStrDataPathPlugin = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        self.m_oedStrDataPathImages = EDDiskExplorer.mergePath( self.getTestsDataHome(), "images" )

        
    def testExecute( self ):
        """
        """
        oedStrConfigFileName    = EDDiskExplorer.mergePath( self.m_oedStrDataPathPlugin, "EDPluginReadImageHeaderv01_configuration_OK_01.xml" )
        oedStrDataInputFileName = EDDiskExplorer.mergePath( self.m_oedStrDataPathPlugin, "EDPluginReadImageHeaderv01_inputData_OK_01.xml" )
        xsPluginItem = self.getPluginConfiguration( oedStrConfigFileName )
        oEDPluginReadImageHeaderv01 = self.createPlugin()
        oEDPluginReadImageHeaderv01.setConfiguration( xsPluginItem )
        oEDPluginReadImageHeaderv01.configure()

        oedStringXMLInputData = EDUtilsTest.readAndParseFile( oedStrDataInputFileName )
        
        oEDPluginReadImageHeaderv01.setDataInput( oedStringXMLInputData )

        oEDPluginReadImageHeaderv01.execute()
        oEDPluginReadImageHeaderv01.synchronize()    
        oxsDataOutput = oEDPluginReadImageHeaderv01.getDataOutput()
        oedStringXMLDataOutput = None
        if ( oxsDataOutput != None ):
            oedStringXMLDataOutput = oxsDataOutput.marshal()
        
        oedStringOutputDataReferenceFileName = EDDiskExplorer.mergePath( self.m_oedStrDataPathPlugin, "EDPluginReadImageHeaderv01_outputData_OK_01.xml" )

        oedStringXMLOutputDataReference = EDUtilsTest.readAndParseFile( oedStringOutputDataReferenceFileName )

        EDAssert.equal( oedStringXMLOutputDataReference,  oedStringXMLDataOutput )        

        

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

    

##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCasePluginUnitReadImageHeaderv01 = EDTestCasePluginUnitReadImageHeaderv01( "EDTestCasePluginUnitReadImageHeaderv01" )
    edTestCasePluginUnitReadImageHeaderv01.execute()
