#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitStrategyv01.py 476 2008-07-30 16:12:58Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport                         import EDCompiler
from EDImport                         import EDVerbose
from EDImport                         import EDString
from EDImport                         import EDAssert
from EDImport                         import EDDiskExplorer
from EDImport                         import EDDict
from EDImport                         import EDDictionary
from EDImport                         import EDUtilsFile
from EDImport                         import EDList
from EDImportTest                     import EDTestCasePluginUnit
from EDImportTest                     import EDUtilsTest


class EDTestCasePluginUnitStrategyv01( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginControlStrategyv01", "EDPluginControlStrategy-v0.1", _oalStringTestName )
        oedStrDataDir = "EDPluginStrategy"
        self.m_oedStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), oedStrDataDir )
        
        self.m_oedObtainedInputFile = "XSDataInputStrategyv01.xml"
        self.m_oedReferenceInputFile = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSDataStrategyv01Input_reference.xml")
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginUnit.preProcess( self )


    def postProcess( self ):
        """
        """
        EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile )
        
        
    def testConfigureOK( self ):
        """
        """
        oedPluginStrategy = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration_ESRF.xml" ))
        oedPluginStrategy.setConfiguration( xsPluginItemGood01 )
        oedPluginStrategy.configure()
        EDAssert.equal("/opt/pxsoft/ccp4-6.0.2/lib/data", oedPluginStrategy.getSymopHome())
        
        self.cleanUp( oedPluginStrategy )
        
        
    def testSetDataModelInput( self ):
        """
        """
        oedPluginStrategy = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration_ESRF.xml" ))
        oedPluginStrategy.setConfiguration( xsPluginItemGood01 )
        oedPluginStrategy.configure()
             
        from XSDataCharacterisationv01 import XSDataStrategyInput
        xSDataStrategy = XSDataStrategyInput()
        
        # Beam
        from XSDataCharacterisationv01 import XSDataBeam
        from XSDataCharacterisationv01 import XSDataFlux
        from XSDataCharacterisationv01 import XSDataWavelength
        from XSDataCharacterisationv01 import XSDataSize
        from XSDataCharacterisationv01 import XSDataLength
        from XSDataCharacterisationv01 import XSDataTime
        from XSDataCharacterisationv01 import XSDataExperimentalCondition
        
        xsExperimentalCondition = XSDataExperimentalCondition()

        xsBeam = XSDataBeam()
        xsBeam.setFlux( XSDataFlux(1e+12) )
        xsBeam.setWavelength( XSDataWavelength(2.41) )
        xsBeam.setSize( XSDataSize( XSDataLength(0.1), XSDataLength(0.1)) )  
        xsBeam.setExposureTime( XSDataTime(1))
        
        xsExperimentalCondition.setBeam( xsBeam )
        
        # Detector and Exposure Time
        from XSDataCharacterisationv01 import XSDataDetector
        from XSDataCharacterisationv01 import XSDataString
        from XSDataCharacterisationv01 import XSDataGoniostat

        xsDataDetector = XSDataDetector()
        xsDataDetector.setType( XSDataString("q210-2x"))
        xsExperimentalCondition.setDetector( xsDataDetector )
        
        xsDataGoniostat = XSDataGoniostat()
        xsDataGoniostat.setRotationAxis( XSDataString("phi"))
        xsExperimentalCondition.setGoniostat( xsDataGoniostat )
        
        xSDataStrategy.setExperimentalCondition( xsExperimentalCondition )
        
        
        
        
        # Best Files
        bestFileContentDat = EDUtilsFile.readFile( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "bestfile.dat" ))
        xSDataStrategy.setBestFileContentDat( XSDataString( bestFileContentDat ))
        bestFileContentPar = EDUtilsFile.readFile( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "bestfile.par" ))
        xSDataStrategy.setBestFileContentPar( XSDataString( bestFileContentPar ))
        bestFileContentHKL = EDUtilsFile.readFile( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "bestfile1.hkl" ))
        listBestFileContentHKL = EDList()
        listBestFileContentHKL.add( XSDataString( bestFileContentHKL ))
        xSDataStrategy.setBestFileContentHKL( listBestFileContentHKL )
        
        # Crystal
        from XSDataCharacterisationv01 import XSDataCrystal
        from XSDataCharacterisationv01 import XSDataStructure
        from XSDataCharacterisationv01 import XSDataChain
        from XSDataCharacterisationv01 import XSDataFloat
        from XSDataCharacterisationv01 import XSDataAtom
        from XSDataCharacterisationv01 import XSDataString
        from XSDataCharacterisationv01 import XSDataLigand
        from XSDataCharacterisationv01 import XSDataSampleCrystalMM
        from XSDataCharacterisationv01 import XSDataChemicalCompositionMM
        from XSDataCharacterisationv01 import XSDataAtomicComposition
        from XSDataCharacterisationv01 import XSDataSolvent
        from XSDataCharacterisationv01 import XSDataCell
        from XSDataCharacterisationv01 import XSDataSpaceGroup
        from XSDataCharacterisationv01 import XSDataAngle
        from XSDataCharacterisationv01 import XSDataString
        from XSDataCharacterisationv01 import XSDataInteger
        
        xsDataSampleCrystalMM = XSDataSampleCrystalMM()
        xsDataStructure = XSDataStructure()
        xsDataComposition = XSDataChemicalCompositionMM()
        
        xsDataChain = XSDataChain()
        xsDataChain.setType( XSDataString( "protein" ))
        xsDataChain.setNumberOfCopies( XSDataFloat( 2 ))
        xsDataAtomicComposition = XSDataAtomicComposition()
        xsDataAtom1 = XSDataAtom()
        xsDataAtom1.setSymbol( XSDataString("Se") )
        xsDataAtom1.setNumberOf( XSDataInteger(4) )
        xsDataAtomicComposition.addAtom( xsDataAtom1 )
        
        xsDataChain.setHeavyAtoms( xsDataAtomicComposition )
        xsDataChain.setNumberOfMonomers( XSDataFloat( 100 ))
        xsDataStructure.addChain( xsDataChain )
        
        xsDataChain2 = XSDataChain()
        xsDataChain2.setType( XSDataString("rna") )
        xsDataChain2.setNumberOfCopies( XSDataFloat( 1 ))
        xsDataChain2.setNumberOfMonomers( XSDataFloat( 60 ))
        xsDataStructure.addChain( xsDataChain2 )
        
        xsDataLigand = XSDataLigand()
        xsDataLigand.setNumberOfCopies( XSDataFloat( 2 ))
        xsDataLigand.setNumberOfLightAtoms( XSDataFloat( 42 ))
        xsDataAtomicComposition = XSDataAtomicComposition()
        xsDataAtom2 = XSDataAtom()
        xsDataAtom2.setSymbol( XSDataString("Fe") )
        xsDataAtom2.setNumberOf( XSDataInteger(1) )
        xsDataAtomicComposition.addAtom( xsDataAtom2 )
        xsDataLigand.setHeavyAtoms( xsDataAtomicComposition )
        xsDataStructure.addLigand( xsDataLigand )
        xsDataStructure.setNumberOfCopiesInAsymmetricUnit( XSDataFloat( 0.25 ))
        
        xsDataSolvent = XSDataSolvent()
        xsDataAtomicComposition = XSDataAtomicComposition()
        
        xsDataAtom3 = XSDataAtom()
        xsDataAtom3.setSymbol( XSDataString("Na") )
        xsDataAtom3.setConcentration( XSDataFloat(1000) )
        xsDataAtom4 = XSDataAtom()
        xsDataAtom4.setSymbol( XSDataString("Cl") )
        xsDataAtom4.setConcentration( XSDataFloat(1000) )
        
        xsDataAtomicComposition.addAtom( xsDataAtom3 )
        xsDataAtomicComposition.addAtom( xsDataAtom4 )
        xsDataSolvent.setAtoms( xsDataAtomicComposition )

        xsDataComposition.setStructure( xsDataStructure )
        xsDataComposition.setSolvent( xsDataSolvent )
        xsDataSampleCrystalMM.setChemicalComposition( xsDataComposition )
        
        xsDataSampleCrystalMM.setSize( XSDataSize( XSDataLength(0.1), XSDataLength(0.1), XSDataLength(0.1) ))
        
        xsDataCrystal = XSDataCrystal()
        xsDataCell = XSDataCell( XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataLength(78.9),
                                 XSDataLength(95.162),
                                 XSDataLength(104.087) )
        xsDataCrystal.setCell( xsDataCell )
        
        xsDataSpaceGroup = XSDataSpaceGroup()
        xsDataSpaceGroup.setITNumber( XSDataInteger( 16 ))
        xsDataCrystal.setSpaceGroup( xsDataSpaceGroup )
        
        xsDataSampleCrystalMM.setSusceptibility( XSDataFloat( 1.5 ))
        
        xSDataStrategy.setCrystalRefined( xsDataCrystal )
        
        xSDataStrategy.setSample( xsDataSampleCrystalMM )
        
        xSDataStrategy.outputFile( self.m_oedObtainedInputFile )
        
        strExpectedInput = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile )
        strObtainedInput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedInputFile )  
        
        EDAssert.equal( strExpectedInput, strObtainedInput )
        
        self.cleanUp( oedPluginStrategy )
        
        
    def testSymmetryTable( self ):
        """
        """
        oedPluginStrategy = self.createPlugin()
        edStringSymTableFileName = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "symop.lib" )
        strSpaceGroupId = "16"
        iNumOperator = oedPluginStrategy.getNumberOfSymmetryOperators( strSpaceGroupId, edStringSymTableFileName )
        EDAssert.equal( 4, iNumOperator )
        
        
    def testDefaultChemicalComposition( self ):
        """
        """
        oedPluginStrategy = self.createPlugin()
        
        from XSDataCharacterisationv01 import XSDataSampleCrystalMM
        from XSDataCharacterisationv01 import XSDataCrystal
        from XSDataCharacterisationv01 import XSDataCell
        from XSDataCharacterisationv01 import XSDataAngle
        from XSDataCharacterisationv01 import XSDataLength
        
        xsDataSampleCrystalMM = XSDataSampleCrystalMM()
        xsDataCrystal = XSDataCrystal()
        xsDataCell = XSDataCell( XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataLength(78.9),
                                 XSDataLength(95.162),
                                 XSDataLength(104.087) )
        xsDataCrystal.setCell( xsDataCell )
        xsDataSampleCrystalMM.setCrystal( xsDataCrystal )
        inumOperators = 4
        
        xsDataSample2 = oedPluginStrategy.getDefaultChemicalComposition( xsDataSampleCrystalMM, inumOperators )
        strChainType = xsDataSample2.getStructure().getChain()[0].getType()
        EDAssert.equal( "protein", strChainType.getValue() )
        

##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testConfigureOK )
        self.addTestMethod( self.testSetDataModelInput )
        self.addTestMethod( self.testSymmetryTable )
        self.addTestMethod( self.testDefaultChemicalComposition )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    eDTestCasePluginUnitStrategyv01 = EDTestCasePluginUnitStrategyv01( "EDTestCasePluginUnitStrategyv01" )
    eDTestCasePluginUnitStrategyv01.execute()

